#ifndef DATAREFERENCEHANDLER_H
#define DATAREFERENCEHANDLER_H

// Framework include files
#include "POOLCore/Token.h"
#include "StorageSvc/DbDefaultDataHandler.h"

/*
 *    POOL namespace
 */
namespace pool    {

  // Forward declarations
  class IDataSvc;

  /** @class DataReferenceHandler DataReferenceHandler.h
    * 
    * Exampel class to show specialized reference handling
    * depending on the implementation of the data cache.
    *
    * @author  M.Frank
    * @version 1.0
    * @date    10/02/2003
    */
  class DataReferenceHandler : public DbDefaultDataHandler {
    /// reference to data cache
    IDataSvc* m_cache;
  public:

    /// Standard Constructor: Tkes data cache as argument.
    DataReferenceHandler(IDataSvc* cache);

    /// Standard Destructor
    virtual ~DataReferenceHandler();

    /// Internal call on saving references.
    /** Convert transient reference to persistent OID.
      * @param pRef           [IN] Pointer to reference object to be saved
      * @param pOid           [IN] Pointer to OID data for this reference
      *                            The oid is all, what is stored in the
      *                            persistent world.
      *
      * @return DbStatus code indicating success or failure
      */
    virtual DbStatus saveReference(Reference* pRef, Token::OID_t* data);

    /// Internal call on loading references.
    /** Convert persistent OID into a transient reference.
      * @param pRef           [IN] Pointer to reference object to be loaded
      * @param pOid           [IN] Pointer to OID data for this reference
      *                            The oid is all, what is stored in the
      *                            persistent world.
      *
      * @return DbStatus code indicating success or failure
      */
    virtual DbStatus loadReference(Reference* pRef, Token::OID_t* data);
  };
}
#endif // DATAREFERENCEHANDLER_H
