#ifndef POOL_IDATATRANSFORM_H
#define POOL_IDATATRANSFORM_H

// Framework includes
#include "Reflex/Type.h"

/** 
    pool namespace.
*/
namespace pool  {
  class DataCallBack;
  class Guid;

  /** class IDataTransform.
   *  Abstract class for the management of the callback objects
   *  for the transient and persistent shapes.
   */
  class IDataTransform    {
  public:
    /// Destructor
    virtual ~IDataTransform() {};

    /**
      Sets the behaviour for acting when a callback object is not found.
      If set, a missing callback object is created automatically (default behaviour),
      else an exception is thrown.
    */
    virtual void setAutomaticCallBackCreation( bool flag = true ) = 0;

    /// Translation form object type to object-Guid
    virtual const Guid& classID( seal::reflex::Type& theClass ) = 0;

    /// Access Transformation object to convert an object.
    /**
        @param   obj_id    [IN] Class ID of the object
        @param   shape_id  [IN] Shape identifier of the persistent shape.
        @return DataCallBack&   Reference to the callback object.
    */
    virtual DataCallBack& transformation( const Guid& obj_id,
					  const Guid& shape_id ) = 0;

    /// Register a new transformation object
    virtual bool registerTransformation( const Guid& object_typ, 
					 const Guid& shape_typ, 
					 const DataCallBack& callback ) = 0;
  };
}
#endif // POOL_IDATATRANSFORM_H
