#ifndef IDATATRANSFORMINFO_H
#define IDATATRANSFORMINFO_H

#include "PluginManager/PluginInfo.h"

/**
    pool namespace
*/
namespace pool {

  // Forward declarations
  class IDataTransform;

  /** @class IDataTransformInfo IDataTransformInfo.h PersistenctSvc/IDataTransformInfo.h
   * 
   *  IDataTransformInfo is the PluginInfo class needed to define
   *  plugins of type IDataTransform
   *
   */

  class IDataTransformInfo : public seal::PluginInfo
  {
  public:
    /// Constructor of these types must be provided
    IDataTransformInfo( seal::Module* module,
			 const std::string& type );

    IDataTransformInfo( seal::Module* module,
			 seal::ModuleDescriptor* details );

    /// Destructor
    ~IDataTransformInfo();

    /// Factory base class: defines constructor arguments.
    struct Factory
    {
      virtual ~Factory() {};
      virtual IDataTransform* create() = 0;
    };

    /// Concrete #Factory implementation for a specific object type @c T.
    template <class T> struct AutoFactory : public Factory
    {
      virtual IDataTransform* create() {
	return new T();
      }
    };

    /** the actual factory method.
     *  Called with the same argument list as
     *  the constructor of every implementation.
     */
    IDataTransform* create() const {
      Factory* f = factory();
      if ( f ) {
	return m_factory->create();
      }
      else return 0;
    }

    /// What happens when attaching a module
    void      attach( Factory* factory );
    
    /// What happens when detaching a module
    void      detach();
    
    /// The name of the category
    static const char* label();
    
  private:
    Factory*  factory() const;
    Factory*  m_factory;
  };

}
#endif
