#ifndef TESTSCHEMA_H_
#define TESTSCHEMA_H_

#include <typeinfo>
#include <iostream>
#include <iomanip>
#include <vector>
#include <list>



#ifdef SCHEMA_CHANGE
#define SCHEMA_FLIP(x,y) y  
#else
#define SCHEMA_FLIP(x,y) x
#endif

#define IMPLEMENT_DEFAULT(cl) \
  cl() : m_data0(0)  {} \
  virtual ~cl() {} \
  void set(int i, int j)  { m_data0 = (i+1)%100; m_schema_data.set(i,j);  } \
  void dump() const  { _dmp(*this, long(m_data0), m_schema_data) << std::endl;  } 


template <class T1> inline const char* type_name(T1 typ)  {
  try {
    return typeid(typ).name();
  }
  catch(...)  {
    return "Unknown type";
  }
}

template <class T1, class T2, class T3> inline 
std::ostream& _dmp(T1 a, T2 b, T3 c)  {
  std::cout.width(48); std::cout << std::left << type_name(a) << "> ";
  std::cout.width(16); std::cout << b;
  std::cout << " "; std::cout.width(16); std::cout << c;
  return std::cout;
}

class TestAggregate  {
public:
  int    m_data0;
  float  m_data1;
  double m_data2;
  TestAggregate() : m_data0(0), m_data1(0.), m_data2(0.) {}
  virtual ~TestAggregate() {}
  void set(int i, int j)  {
    m_data0 = i+1;
    m_data1 = float(m_data0+1000*j);
    m_data2 = double(m_data1+float(1000000*j));
  }
};
inline std::ostream& operator<<(std::ostream& os, const TestAggregate& t)  {
  os << std::endl << "     ---> ";
  _dmp(t, t.m_data0, t.m_data1) << " " << t.m_data2 << std::endl;
  return os;
}

class SimpleClass {
public:
  float   m_data0;
  SimpleClass() : m_data0(0.)  {  }
  virtual ~SimpleClass() {}
  void set(int i, int j)  {
    m_data0 = float(i+1);
  }
  void dump() const {
    _dmp(*this, m_data0, "") << std::endl;
  }
};
inline std::ostream& operator<<(std::ostream& os, const SimpleClass& t)
{ 
  os << std::endl << "     ---> ";
  return _dmp(t, t.m_data0, "") << std::endl; 
}

class AddBaseClass SCHEMA_FLIP ( {, : public SimpleClass { )
public:
  int m_data1;
  AddBaseClass() : m_data1(0)  {  }
  virtual ~AddBaseClass() {}
  void set(int i, int j)  {
    m_data1 = i+1;
    SCHEMA_FLIP(; , m_data0 = float(m_data1+10000*j));
  }
  void dump() const  {
    _dmp(*this, m_data1,SCHEMA_FLIP( "" , m_data0 )) << std::endl;
  }
};

class AddAggregateMember  {
public:
  int   m_data0;
  SCHEMA_FLIP(; , TestAggregate m_schema_data);
  AddAggregateMember() : m_data0(0)  {  }
  virtual ~AddAggregateMember() {}
  void set(int i, int j)  {
    m_data0 = i+1;
    SCHEMA_FLIP(; , m_schema_data.set(i,j) );
  }
  void dump() const  {
    _dmp(*this, m_data0,SCHEMA_FLIP( "" , m_schema_data )) << std::endl;
  }
};

class AddPrimitiveMemberAtEnd  {
public:
  int   m_data0;
  SCHEMA_FLIP(; , int m_schema_data);
  AddPrimitiveMemberAtEnd() : m_data0(0)  {
    SCHEMA_FLIP(; , m_schema_data = 0);
  }
  virtual ~AddPrimitiveMemberAtEnd() {}
  void set(int i, int j)  {
    m_data0 = i+1;
    SCHEMA_FLIP(; , m_schema_data = m_data0+10000*j);
  }
  void dump() const  {
    _dmp(*this, m_data0, SCHEMA_FLIP( "" , m_schema_data )) << std::endl;
  }
};

class AddPrimitiveMemberAtFront  {
public:
  SCHEMA_FLIP(; , int m_schema_data );
  int   m_data0;
  AddPrimitiveMemberAtFront() : m_data0(0)  {
    SCHEMA_FLIP(; , m_schema_data = 0);
  }
  virtual ~AddPrimitiveMemberAtFront() {}
  void set(int i, int j)  {
    m_data0 = i+1;
    SCHEMA_FLIP(; , m_schema_data = m_data0+10000*j );
  }
  void dump() const  {
    _dmp(*this, m_data0, SCHEMA_FLIP( "" , m_schema_data )) << std::endl;
  }
};

class RemovePrimitiveMemberAtEnd  {
public:
  int   m_data0;
  SCHEMA_FLIP(int m_schema_data, ;);
  RemovePrimitiveMemberAtEnd() : m_data0(0)  {
    SCHEMA_FLIP(m_schema_data = 0, ;);
  }
  virtual ~RemovePrimitiveMemberAtEnd() {}
  void set(int i, int j)  {
    m_data0 = i+1;
    SCHEMA_FLIP(m_schema_data = m_data0+10000*j, ;);
  }
  void dump() const  {
    _dmp(*this, m_data0, SCHEMA_FLIP( m_schema_data, "" )) << std::endl;
  }
};

class RemovePrimitiveMemberAtFront  {
public:
  SCHEMA_FLIP(int m_schema_data, ;);
  int   m_data0;
  RemovePrimitiveMemberAtFront() : m_data0(0)  {
    SCHEMA_FLIP(m_schema_data = 0, ;);
  }
  virtual ~RemovePrimitiveMemberAtFront() {}
  void set(int i, int j)  {
    m_data0 = i+1;
    SCHEMA_FLIP(m_schema_data = m_data0+10000*j, ;);
  }
  void dump() const  {
    _dmp(*this, m_data0, SCHEMA_FLIP( m_schema_data, "" )) << std::endl;
  }
};

class ChangePrimitiveType  {
public:
  int   m_data0;
  SCHEMA_FLIP(float , double) m_schema_data;
  SCHEMA_FLIP(short , int)    m_schema_data2;
  ChangePrimitiveType() : m_data0(0), m_schema_data(0.0), m_schema_data2(0)  {}
  ChangePrimitiveType(const ChangePrimitiveType& c) 
    : m_data0(c.m_data0), m_schema_data(c.m_schema_data), m_schema_data2(c.m_schema_data2) {}
  virtual ~ChangePrimitiveType() {}
  void set(int i, int j)  {
    m_data0 = i+1;
    m_schema_data = float(m_data0+10000*j);
    m_schema_data2 = m_data0+1000*j;
  }
  void dump() const  {
    _dmp(*this, m_data0, SCHEMA_FLIP( m_schema_data, m_schema_data )) 
      << " " << typeid(m_schema_data).name() 
      << " " << m_schema_data2 << " " << typeid(m_schema_data2).name()
      << std::endl;
  }
};

class RenameMember  {
public:
  int SCHEMA_FLIP(m_data0,m_data1);
  RenameMember() : SCHEMA_FLIP(m_data0,m_data1) (0)  {}
  virtual ~RenameMember() {}
  void set(int i, int j)  {
    SCHEMA_FLIP(m_data0,m_data1) = i+1;
  }
  void dump() const  {
    _dmp(*this, SCHEMA_FLIP(m_data0,m_data1), "") << std::endl;
  }
};

class SCHEMA_FLIP(OriginalUnrelatedClass , ChangedUnrelatedClass)  {
public:
  int m_data0;
  SCHEMA_FLIP(OriginalUnrelatedClass , ChangedUnrelatedClass )() : m_data0(0)  {  }
  virtual ~SCHEMA_FLIP(OriginalUnrelatedClass , ChangedUnrelatedClass )() {}
  void set(int i, int j)  {
    m_data0 = i+1;
  }
  void dump() const  {
    _dmp(*this, m_data0, "") << std::endl;
  }
};
inline std::ostream& operator<<(std::ostream& os, const SCHEMA_FLIP(OriginalUnrelatedClass , ChangedUnrelatedClass)& t)  {
  os << std::endl << "     ---> "; t.dump(); return os;
}

namespace OrgSpace  {
  class d_dumOrgSpace {};
}
namespace ChangedSpace  {
  class d_dumOrgSpace {};
}
namespace AddSpace  {
  class d_dumOrgSpace {};
}

namespace SCHEMA_FLIP(OrgSpace , ChangedSpace)  {
  class TestAggregate  {
  public:
    int    m_data0;
    float  m_data1;
    double m_data2;
    TestAggregate() : m_data0(0), m_data1(0.), m_data2(0.) {}
    virtual ~TestAggregate() {}
    void set(int i, int j)  {
      m_data0 = i+1;
      m_data1 = float(m_data0+1000*j);
      m_data2 = double(m_data1+float(1000000*j));
    }
  };

  class SimpleClass2 {
  public:
    float   m_data0;
    SimpleClass2() : m_data0(0.)  {  }
    virtual ~SimpleClass2() {}
    void set(int i, int j)  {
      m_data0 = float(i+1);
    }
    void dump() const  {
      _dmp(*this, m_data0, "") << std::endl;
    }
  };
}
inline std::ostream& operator<<(std::ostream& os, const SCHEMA_FLIP(OrgSpace , ChangedSpace)::TestAggregate& t)  {
  os << std::endl << "     ---> ";
  _dmp(t, t.m_data0, t.m_data1) << " " << t.m_data2 << std::endl;
  return os;
}

SCHEMA_FLIP(class SimpleClass3 , namespace AddSpace { class SimpleClass3)  {
  public:
    float   m_data0;
    SimpleClass3() : m_data0(0.)  {  }
    virtual ~SimpleClass3() {}
    void set(int i, int j)  {
      m_data0 = float(i+1);
    }
    void dump() const  {
      _dmp(*this, m_data0, "") << std::endl;
    }
  }
SCHEMA_FLIP ( ; , ; } )

class RelatedClass : public SimpleClass   {
public:
  SCHEMA_FLIP(char , long) m_data1;
  RelatedClass () : m_data1(0)  {  }
  virtual ~RelatedClass() {}
  void set(int i, int j)  {
    m_data1 = (i+1)%100;
    SimpleClass::set(i,j);
  }
  void dump() const  {
    _dmp(*this, m_data0, m_data1) << std::endl;
  }
};
inline std::ostream& operator<<(std::ostream& os, const RelatedClass& t) { 
  os << std::endl << "     ---> ";
  return _dmp(t, t.m_data0, t.m_data1) << std::endl; 
}

class ChangeMemberSpace  {
public:
  SCHEMA_FLIP(char , long) m_data0;
  SCHEMA_FLIP(OrgSpace , ChangedSpace)::TestAggregate m_schema_data;
  IMPLEMENT_DEFAULT(ChangeMemberSpace);
};

class AddMemberSpace  {
public:
  SCHEMA_FLIP(char , long) m_data0;
  SCHEMA_FLIP(TestAggregate , ChangedSpace::TestAggregate) m_schema_data;
  IMPLEMENT_DEFAULT(AddMemberSpace);
};

class ChangeMemberUnrelated  {
public:
  SCHEMA_FLIP(char , long) m_data0;
  SCHEMA_FLIP(OriginalUnrelatedClass , ChangedUnrelatedClass) m_schema_data;
  IMPLEMENT_DEFAULT(ChangeMemberUnrelated);
};

class ChangeMemberRelated  {
public:
  SCHEMA_FLIP(char , long) m_data0;
  SCHEMA_FLIP(SimpleClass , RelatedClass ) m_schema_data;
  IMPLEMENT_DEFAULT(ChangeMemberRelated);
};

template <class T> class VectorEvolution  {
public:
  std::vector<T> m_data;
  VectorEvolution()  {}
  virtual ~VectorEvolution() { m_data.clear(); }
  void set(int i, int j)  {
    for ( int i = 0; i < 5; ++i )  {
      m_data.push_back(T());
      m_data.back().set(i, j);
    }
  }
  void dump() const  { 
    std::cout << typeid(*this).name() << std::endl;
    for (std::vector<T>::const_iterator i=m_data.begin(); i!=m_data.end(); ++i)  {
      std::cout << "      ";
      (*i).dump();
    }
  }
};

template <class T> class VectorPointerEvolution  {
public:
  std::vector<T*> m_data;
  VectorPointerEvolution()  {}
  virtual ~VectorPointerEvolution() { 
    for ( std::vector<T>::iterator i=m_data.begin(); i != m_data.end(); ++i)  {
      if ( 0 != (*i) )  {
        delete (*i);
        (*i) = 0;
      }
    }
    m_data.clear(); 
  }
};

namespace  {
  struct instantiate {
    VectorEvolution<ChangePrimitiveType> m_vec_ChangePrimitiveType;
    VectorEvolution<ChangePrimitiveType> m_vec_ptr_ChangePrimitiveType;
  };
}
#ifdef G__DICTIONARY
using namespace std;
#endif

#endif // TESTSCHEMA_H_
