#ifndef TEST_H_
#define TEST_H_

// what I can do
#define __use_primitives
#define __use_refs
#define __use_unkeyed
#define __use_arrays

#define __use_keyed
#define __use_recursive_structs
#define __use_full_stl
#define __use_custom_streamer

#define __use_root_pointers
#define __use_embedded_classes
// #define __use_diamonds

#include <string>
#include <vector>
#include <list>
#include <deque>
#include <queue>
#include <stack>
#include "pool/Ref.h"
#include "TNamed.h"
#include "TRef.h"

#include "POOLCore/win32/stl_hash.h"
#ifdef __GNUC__
namespace STDEXT {
  template <> struct hash<std::string>
  {size_t operator()(const std::string& s) const { return __gnu_cxx_hash_string(s.c_str()); }  };
  template <> struct hash<const std::string>
  {size_t operator()(const std::string& s) const { return __gnu_cxx_hash_string(s.c_str()); }  };
  template <> struct hash<std::string&>  
  {size_t operator()(const std::string& s) const { return __gnu_cxx_hash_string(s.c_str()); }  };
  template <> struct hash<const std::string&>
  {size_t operator()(const std::string& s) const { return __gnu_cxx_hash_string(s.c_str()); }  };
}
#endif
#include <set>
#include <map>
#include <memory>

namespace pool  {

  template <class T> class my_allocator : public std::allocator<T>  {
    long   m_tmpBuff[32];   // Change artificially allocator size
  public:
    my_allocator() {}
  	template<class _Other> my_allocator(const my_allocator<_Other>&) {}
    ~my_allocator() {}
	  template<class _Other> struct rebind		{
      // convert an allocator<_Ty> to an allocator <_Other>
	  	typedef my_allocator<_Other> other;
  	};

  };

#define SPECIAL_ALLOCATOR pool::my_allocator

  /**
    * @author  M.Frank
    * @version 1.0
    */
  template<class T> class own_ptr {
  public:
    typedef T value_type;
    own_ptr(T *p = 0) : m_p(p)		            {                 }
    own_ptr(const own_ptr<T>& c) : m_p(c.release()) {                 }
    virtual ~own_ptr()              { delete m_p;                     }
    T& operator*() const            { return *get();                  }
    T *operator->() const           { return get();                   }
    own_ptr<T>& 
    operator=(const own_ptr<T>& c)  { reset(c.release());return *this;}
    bool operator==(const own_ptr<T>& c) const { return m_p == c.m_p; }
    bool operator< (const own_ptr<T>& c) const { return m_p < c.m_p;  }
    T *get() const                  { return m_p;                     }
    T *release() const              { T *tmp = m_p;m_p = 0;return tmp;}
    void reset(T* p = 0)            { if (p != m_p)delete m_p;	m_p=p;}
  private:
    mutable T *m_p;
  };
}

class AbstractBase  {
public:
  float m_float;
  AbstractBase(float f);
  virtual ~AbstractBase();
  virtual long getInt() = 0;
};
class AbstractImp  : public AbstractBase {
public:
  long m_int;
  AbstractImp();
  virtual ~AbstractImp();
  virtual long getInt() { return m_int; }
};

class DiamondBase  {
public:
  long m_baseInt;
  DiamondBase();
  virtual ~DiamondBase();
};

class DiamondDerivedA : virtual public DiamondBase {
public:
  float m_float;
  DiamondDerivedA();
  virtual ~DiamondDerivedA();
};

class DiamondDerivedB : virtual public DiamondBase {
public:
  longlong m_longlong;
  DiamondDerivedB();
  virtual ~DiamondDerivedB();
};

class DiamondClosure 
  : virtual public DiamondDerivedA,
    virtual public DiamondDerivedB
{
public:
  double m_derivDouble;
  DiamondClosure();
  virtual ~DiamondClosure();
};

class NoVirtual  {
public:
  int m_data1;
  NoVirtual() : m_data1(0) {}
  NoVirtual(const NoVirtual& c):m_data1(c.m_data1) {}
  ~NoVirtual() {}
};

class NoVirtualSubClass : public NoVirtual {
public:
  int m_data2;
  NoVirtualSubClass() : NoVirtual(), m_data2(0) {}
  NoVirtualSubClass(const NoVirtualSubClass& c):NoVirtual(c), m_data2(c.m_data2) {}
  ~NoVirtualSubClass() {}
};

/** @class Aggregated
  * A small and stupid class used by the main test.
  * 
  * This simple and stupid class just consist out of two
  * data elements, an integer and a long.
  *
  * @author  M.Frank
  * @version 1.0
  */
class Aggregated  {
public:
  /// integer data member
  int  data1;
  /// long data member
  long data2;
  /// Standard Constructor
  Aggregated();
  /// Initializing constructor
  Aggregated(int d1, long d2);
  /// Copy Constructor
  Aggregated(const Aggregated& c);
  /// Standard Destructor
  virtual ~Aggregated();
  /// Equality check
  bool operator == (const Aggregated& c)  const 
  { return data1 ==c.data1 && data2==c.data2; }
  /// Non-equality check
  bool operator != (const Aggregated& c)  const     
  { return !this->operator==(c); }
  /// Operator less
  bool operator <  (const Aggregated& c)  const
  { return data1 < c.data1 && data2 < c.data2; }
  /// Operator greater
  bool operator >  (const Aggregated& c)  const
  { return !this->operator < (c);              }
};

/** @class SubAggregated
  *
  * Class to test polymorphic pointer handling using normal inheritance.
  * 
  * @author  M.Frank
  * @version 1.0
  */
class SubAggregated : public Aggregated {
public:
  /// integer data member
  int  data3;
  /// Standard Constructor
  SubAggregated();
  /// Initializing Constructor
  SubAggregated(const Aggregated& c);
  /// Initializing Constructor
  SubAggregated(int i, int k, int l);
  /// Standard Destructor
  ~SubAggregated();
};

/** @class PtrAggregated
  *
  * Class to test polymorphic pointer handling.
  * 
  * @author  M.Frank
  * @version 1.0
  */
class PtrAggregated : public Aggregated {
public:
  /// integer data member
  int  data3;
  /// Standard Constructor
  PtrAggregated();
  /// Initializing Constructor
  PtrAggregated(const Aggregated& c);
  /// Initializing Constructor
  PtrAggregated(int i, int k, int l);
  /// Standard Destructor
  ~PtrAggregated();
};

/** @class VirtualAggregated
  *
  * Class to test polymorphic pointer handling using virtual inheritance.
  * 
  * @author  M.Frank
  * @version 1.0
  */
class VirtualAggregated : virtual public Aggregated {
public:
  /// integer data member
  int  data3;
  /// Standard Constructor
  VirtualAggregated();
  /// Initializing Constructor
  VirtualAggregated(const Aggregated& c);
  /// Initializing Constructor
  VirtualAggregated(int i, int k, int l);
  /// Standard Destructor
  ~VirtualAggregated();
};

/** @class NSAggregated
  *
  * Class to test polymorphic pointer handling with classes in namespaces.
  * 
  * @author  M.Frank
  * @version 1.0
  */
namespace A { namespace B { namespace C {
class NSAggregated : virtual public Aggregated {
public:
  /// integer data member
  int  data3;
  /// Standard Constructor
  NSAggregated();
  /// Initializing Constructor
  NSAggregated(const Aggregated& c);
  /// Standard Destructor
  ~NSAggregated();
};
}}}

/** @class CustomObj
  * A small and stupid class used by the main test.
  * 
  * This simple and stupid class just consist out of two
  * data elements, an integer and a long.
  *
  * @author  M.Frank
  * @version 1.0
  */
class CustomObj   {
public:
  /// integer data member
  int  data1;
  /// long data member
  long data2;
  /// Standard Constructor
  CustomObj();
  /// Initializing constructor
  CustomObj(int d1, long d2);
  /// Copy Constructor
  CustomObj(const CustomObj& c);
  /// Standard Destructor
  ~CustomObj();
  /// Equality check
  bool operator == (const CustomObj& c)  const 
  { return data1 ==c.data1 && data2==c.data2; }
  /// Non-equality check
  bool operator != (const CustomObj& c)  const     
  { return !this->operator==(c); }
  /// Operator less
  bool operator <  (const CustomObj& c)  const
  { return data1 < c.data1 && data2 < c.data2; }
  /// Operator greater
  bool operator >  (const CustomObj& c)  const
  { return !this->operator < (c);              }
};

namespace STDEXT {
  template <> struct hash<Aggregated> {
    size_t operator()(const Aggregated& x) const { return x.data1+x.data2; }
  };
  template <> struct hash<Aggregated*> {
    size_t operator()(const Aggregated* x) const { return size_t(x); }
  };
	inline size_t hash_value(const Aggregated& v) {	return hash<Aggregated>().operator()(v);  }
	inline size_t hash_value(const Aggregated* v) {	return hash<Aggregated*>().operator()(v); }
};
typedef unsigned long CLID;

class EventHeaderElement {
public:
  EventHeaderElement(){}
  EventHeaderElement(const EventHeaderElement& e)
    : m_Clid(e.m_Clid), m_Key(e.m_Key), m_Token(e.m_Token)  {}
   EventHeaderElement(CLID clid, const std::string key, const std::string token)
     : m_Clid(clid), m_Key(key), m_Token(token)  {}
   ~EventHeaderElement(){}
public:
   CLID        m_Clid;
   std::string m_Key;
   std::string m_Token;
};

class EventHeaderBase   {
public:
  virtual void addElement(const EventHeaderElement &e) = 0;
};

// original code used direct inheritance - open pool bug 
//class  EventHeader : public std::vector<EventHeaderElement> {
class  EventHeader : public EventHeaderBase {
public:
  virtual void addElement(const EventHeaderElement &e) { m_Elems.push_back(e); }  
  /// make sure class is polymorph (has a vtable)
  virtual ~EventHeader()
    {}
 
private:
    std::vector<EventHeaderElement> m_Elems;
};

// DumbStruct
// A structure holding a single instance of class T
template <class T>
struct DumbStruct {
public:
  DumbStruct() {  }
  DumbStruct(T nui) { i = nui; }
  DumbStruct(const DumbStruct& nuDS) { i = nuDS.i; }
  virtual ~DumbStruct() {}

  std::string msg() { return "[OVAL] Hello there!"; }
  const std::string msg() const { return "[OVAL] Hello there!"; }
 
  T i;
};

/** @class test_base_base
  * 
  * Low level base class for our test.
  *
  * @author  M.Frank
  * @version 1.0
  */
struct test_base_base  {

#ifdef __use_primitives
  /**
    *      use primitive data members
    */
  bool                      m_bool1;
  bool                      m_bool2;
  char                      m_char;
  short                     m_short;
  int                       m_int;
  long                      m_long;
  longlong                  m_int64;
  float                     m_float;
  double                    m_double;
  long double               m_longdouble;
  unsigned char             m_uchar;
  unsigned short            m_ushort;
  unsigned int              m_uint;
  unsigned long             m_ulong;
  ulonglong                 m_uint64;
  char*                     m_text;
  std::string               m_string;

  int                       m_transient_int;

  /**
    *      use custom ROOT streamers
    */
#ifdef __use_root_pointers
  Aggregated* m_transientPtr;
  Aggregated* m_aggPtr;
  Aggregated* m_aggRef;
  TNamed*     m_namPtr;
  TRef        m_namRef;
#endif
#ifdef __use_custom_streamer
  CustomObj   custom;
#endif
#endif
  /**
    *      use of references
    */
#ifdef __use_refs
  pool::Ref<TNamed>         refNamed1;
#endif  // __use_refs

  /// Standard Constructor
  test_base_base();
  /// Initializing constructor
  test_base_base(const std::string& txt);
  /// Standard destructor
  virtual ~test_base_base();
  /// Initialize data
  void test_base_base_initData();

#ifdef __use_refs
  TNamed* named1()  {
    return refNamed1;
  }
  void setTNamed1(TNamed* pObj)    {
    refNamed1 = pObj;
  }
  void setTNamed1(const pool::Ref<TNamed>& refObj)    {
    refNamed1 = refObj;
  }
#endif
};


/** @class test_base
  * 
  * Base class for our test.
  *
  * @author  M.Frank
  * @version 1.0
  */
struct test_base : public test_base_base  {

#ifdef __use_embedded_classes
  /** @class EmbeddedClasses
    *
    * Example of the use of an embedded classes.
    *
    * @author  M.Frank
    * @version 1.0
    */
  struct EmbeddedClasses {
    /** @class Embedded1
      * A nested embedded class.
      * @author M.Frank
      */
    struct Embedded1 {
      /// Integer aggregate
      int i;
      /// Standard Constructor
      Embedded1();
      /// Standard Destructor
      virtual ~Embedded1();
    };
    /** @class Embedded1
      * A nested embedded class with an aggregate.
      * @author M.Frank
      */
    struct Embedded2 {
      /// Double aggregate
      double d;
      /// Aggregate nested class in nested class
      Embedded1 m_embed1;
      /// Standard Constructor
      Embedded2();
      /// Standard Destructor
      virtual ~Embedded2();
    };
    /** @class Embedded1
      * A nested embedded class with an aggregate.
      * @author M.Frank
      */
    struct Embedded3 {
      /// Aggregate nested class in nested class
      Embedded2 m_embed2;
      /// Float aggregate
      float f;
      /// Standard Constructor
      Embedded3();
      /// Standard Destructor
      virtual ~Embedded3();
    };
    /// Aggregate nested class in nested class + pointer to nested class
    Embedded3  /* m_emb3, */ *m_pemb3;
    /// Aggregate nested class in nested class + pointer to nested class
    Embedded2  /* m_emb2, */ *m_pemb2;
    /// Aggregate nested class in nested class + pointer to nested class
    Embedded1  /* m_emb1, */ *m_pemb1;

    /// Standard Constructor
    EmbeddedClasses();
    /// Standard Destructor
    virtual ~EmbeddedClasses();
  };

#endif // __use_embedded_classes

  const bool                m_cbool;
  const char                m_cchar;
  const short               m_cshort;
  const int                 m_cint;
  const long                m_clong;

  mutable char              m_mchar;
  mutable short             m_mshort;
  mutable int               m_mint;
  mutable long              m_mlong;

  TNamed*                   m_cnamPtr;

  /**
    *      use of references
    */
#ifdef __use_refs
  pool::Ref<TNamed>         refNamed2;
#endif  // __use_refs

  /// Standard Constructor
  test_base();
  /// Initializing constructor
  test_base(const std::string& txt);
  /// Standard destructor
  virtual ~test_base();
  test_base& operator=(const test_base& ) { return *this; }
#ifdef __use_refs
  TNamed* named2()  {
    return refNamed2;
  }
  void setTNamed2(TNamed* pObj)    {
    refNamed2 = pObj;
  }
  void setTNamed2(const pool::Ref<TNamed>& refObj)    {
    refNamed2 = refObj;
  }
#endif
};

struct testSimple {
  AbstractBase* m_abstract;
  /// Standard Constructor
  testSimple() : m_abstract(0) {}
  /// Standard destructor
  virtual ~testSimple() { if ( m_abstract ) delete m_abstract; }
};

struct test : public test_base {
  /**
    *      simple and stupid data types
    */
  typedef std::pair<int, int>               LinkH; 
  LinkH                                     m_linkH;
  std::vector<LinkH >                       m_links;
  std::pair<short,short>                    m_shortPair;
  std::pair<unsigned short,unsigned short>  m_ushortPair;
  std::pair<int,int>                        m_intPair;
  std::pair<unsigned int,unsigned int>      m_uintPair;
  std::pair<long,long>                      m_longPair;
  std::pair<unsigned long,unsigned long>    m_ulongPair;

  AbstractBase*                             m_abstract;

  NoVirtual*                                m_noVirtualPtr;
  NoVirtualSubClass*                        m_noVirtualSubClassPtr;
#ifdef __use_diamonds
  DiamondClosure                            m_diam_closed;
  DiamondBase*                              m_diam_baseP;
  DiamondDerivedA*                          m_diam_AP;
  DiamondDerivedB*                          m_diam_BP;
  DiamondClosure*                           m_diam_closedP;
#endif // __use_diamonds
  /**
    *      use of unkeyed containers
    */
#ifdef __use_unkeyed
  std::vector<bool>                         m_bool_vector;
  std::vector<int>                          m_int_vector;
  std::vector<std::string>                  m_str_vector;
  std::vector<ulonglong>                    m_ulonglong_vector;
  std::vector<double, SPECIAL_ALLOCATOR<double> > m_double_vector_myalloc;
  std::list<bool>                           m_bool_list;
  std::list<int>                            m_int_list;
  std::list<std::string>                    m_str_list;
  std::list<longlong>                       m_longlong_list;
  std::list<double, SPECIAL_ALLOCATOR<double> > m_double_list_myalloc;
  std::vector<Aggregated>                   m_transient_agg_vector;
  std::vector<Aggregated*>                  m_transient_agg_vectorP;
  std::vector<std::vector<char> >           m_vector_vector_char;
  std::vector<std::vector<int> >            m_vector_vector_int;
  std::list<std::vector<char> >             m_list_vector_char;
  std::list<std::vector<int> >              m_list_vector_int;
#endif // __use_unkeyed

  /**
    *      use of aggregated objects
    */
  Aggregated                              m_agg;
  // And unkeyed containers of aggregated objects
#ifdef __use_unkeyed
  std::vector<Aggregated>                 m_agg_vector;
  std::vector<Aggregated*>                m_agg_vectorP;
  std::vector<const Aggregated*>          m_agg_const_vectorP;
  std::vector<Aggregated,  SPECIAL_ALLOCATOR<Aggregated> >  m_agg_vector_myalloc;
  std::vector<Aggregated*, SPECIAL_ALLOCATOR<Aggregated*> > m_agg_vectorP_myalloc;
  std::list<Aggregated*>                  m_agg_listP;
  std::list<Aggregated>                   m_agg_list;
  std::list<Aggregated,  SPECIAL_ALLOCATOR<Aggregated> >    m_agg_list_myalloc;
  std::list<Aggregated*, SPECIAL_ALLOCATOR<Aggregated*> >   m_agg_listP_myalloc;
  std::vector<pool::own_ptr<Aggregated> > m_own_ptr_agg_vector;
  std::list<pool::own_ptr<Aggregated> >   m_own_ptr_agg_list;
#endif // __use_unkeyed
#ifdef __use_keyed
  std::map<int, pool::own_ptr<Aggregated> > m_int_own_ptr_agg_map;
#endif // __use_keyed

  /**
    *      use of references
    */
#ifdef __use_refs
  // and unkeyed containers of references
  #ifdef __use_unkeyed
    std::vector<pool::Ref<TNamed> >       m_ref_vector;
    std::list<pool::Ref<TNamed> >         m_ref_list;
    std::deque<pool::Ref<TNamed> >        m_ref_deque;
    std::queue<pool::Ref<TNamed> >        m_ref_queue;
    std::stack<pool::Ref<TNamed> >        m_ref_stack;
  #endif  // __use_unkeyed
  #ifdef __use_keyed
    std::map<std::string, pool::Ref<TNamed> >          m_ref_map;
    std::multimap<std::string, pool::Ref<TNamed> >     m_ref_multimap;
    STDEXT::hash_map<std::string, pool::Ref<TNamed> >  m_ref_hash_map;
  #endif  // __use_keyed
#endif  // __use_refs
#if defined(__use_keyed) && defined(__use_unkeyed)
    std::map<std::string, std::vector<std::string> >   m_str_str_vector_map;
#endif  // __use_keyed

  /**
    *       use of arrays
    */
#ifdef __use_arrays
  int                                     m_int_array[5];
  Aggregated                              m_agg_array[5];
#endif

  /**
    *       use of recursively defined structures
    */
#ifdef __use_recursive_structs
  std::vector<std::vector<char> >         m_char_vector_vector;
  std::list<std::vector<char> >           m_char_vector_list;

  std::vector<std::vector<int> >          m_int_vector_vector;
  std::list<std::vector<int> >            m_int_vector_list;
#endif

  /**
    *      use of keyed containers
    */
#ifdef __use_keyed
  std::map<std::string, std::string>      m_str_str_map;
  STDEXT::hash_map<int, float>            m_int_float_hash_map;
  STDEXT::hash_multimap<int, float>       m_int_float_hash_multimap;
  std::set<int>                           m_int_set;
  std::set< unsigned short >              m_ushort_set;
  STDEXT::hash_set<int>                   m_int_hash_set;
  STDEXT::hash_multiset<int>              m_int_hash_multiset;
  std::multiset<int>                      m_int_multiset;
  std::map<int, Aggregated>               m_int_agg_map;
  std::map<int, Aggregated*>              m_int_agg_mapP;
  std::set<Aggregated>                    m_agg_set;
  std::set<Aggregated*>                   m_agg_setP;
  std::multimap<int, short>               m_int_short_multimap;
  std::multimap<int, Aggregated>          m_int_agg_multimap;
  std::multimap<int, Aggregated*>         m_int_agg_multimapP;
  std::multiset<Aggregated>               m_agg_multiset;
  std::multiset<Aggregated*>              m_agg_multisetP;

  STDEXT::hash_map<int, Aggregated>       m_int_agg_hash_map;
  STDEXT::hash_map<int, Aggregated*>      m_int_agg_hash_mapP;
  STDEXT::hash_multimap<int, Aggregated>  m_int_agg_hash_multimap;
  STDEXT::hash_multimap<int, Aggregated*> m_int_agg_hash_multimapP;
  STDEXT::hash_set<Aggregated>            m_agg_hash_set;
  STDEXT::hash_set<Aggregated*>           m_agg_hash_setP;
  STDEXT::hash_multiset<Aggregated>       m_agg_hash_multiset;
  STDEXT::hash_multiset<Aggregated*>      m_agg_hash_multisetP;

  std::auto_ptr<Aggregated>               m_agg_auto_ptr_0;
  std::auto_ptr<Aggregated>               m_agg_auto_ptr_1;
  std::auto_ptr<Aggregated>               m_agg_auto_ptr_2;
  std::auto_ptr<Aggregated>               m_agg_auto_ptr_3;
  std::auto_ptr<Aggregated>               m_agg_auto_ptr_4;
  std::auto_ptr<Aggregated>               m_agg_auto_ptr_5;

#endif    // __use_keyed
#ifdef __use_full_stl
  std::deque<int>                         m_int_deque;
  std::queue<int>                         m_int_queue;
  std::stack<int>                         m_int_stack;
  std::deque<Aggregated>                  m_agg_deque;
  std::deque<Aggregated*>                 m_agg_dequeP;
  std::queue<Aggregated>                  m_agg_queue;
  std::queue<Aggregated*>                 m_agg_queueP;
  std::stack<Aggregated>                  m_agg_stack;
  std::stack<Aggregated*>                 m_agg_stackP;
#endif // __use_full_stl

#ifdef __use_embedded_classes
  /// Aggregate nested class.
  EmbeddedClasses                         m_embedded;
  EmbeddedClasses::Embedded1              m_emb1;
  EmbeddedClasses::Embedded2              m_emb2;
  EmbeddedClasses::Embedded3              m_emb3;
#endif // __use_embedded_classes

  /**
    *       class implementation
    */
  /// Standard Constructor
  test();
  /// Initializing constructor
  test(const std::string& txt);
  /// Standard destructor
  virtual ~test();

  void dump();
  void set_a(int val);
  void set_b(int val);
  void set_c(int val);
  int  get_a() const;
  int  get_b() const;
  int  get_c() const;
  void fillData(int i);
  void printData(int i, bool print_stacks);
  void test_initData();
  void test_finiData();

};

/** @class AnotherTNamed
  * Test class to check non-rootcint inheritance from ROOT objects.
  *
  * @author  M.Frank
  * @version 1.0
  */
class AnotherTNamed : public TNamed {
  public:
    std::string str;
    pool::Ref<test> refTest;
    AnotherTNamed();
    AnotherTNamed(const char* a1, const char* a2);
    virtual ~AnotherTNamed();
};

/** @class A::B::C::Named
  * Test class to check non-rootcint inheritance 
  * from ROOT objects with name spaces.
  *
  * @author  M.Frank
  * @version 1.0
  */
namespace A { namespace B { namespace C {
class Named : public TNamed {
  public:
    std::string str;
    Named();
    Named(const char* a1, const char* a2);
    virtual ~Named();
};
}}}

#endif
