#include "StorageSvc/DbInstanceCount.h"
#include "pool/IDataSvc.h"
#include "pool/Ref.h"
#include <stdexcept>
#include <iostream>
using namespace pool;

bool s_tokenDbg = false;
static DbInstanceCount::Counter* s_count = 
  DbInstanceCount::getCounter(typeid(RefBase));

void RefBase::debugTokens(bool val)    {
  s_tokenDbg = val;
}

/// Constructor
RefBase::RefBase() : m_token(0), m_cache(0), m_data(0)  {
  s_count->increment();
}

/// Constructor
RefBase::RefBase(IDataSvc* c, Token* p)
: Reference(), m_token(p), m_cache(c), m_data(0)
{
  s_count->increment();
  if ( m_token )  {
    //int cnt = m_token->addRef();
    if ( s_tokenDbg ) {
      //std::cout << "Token[0]:" << m_token << " " << cnt << std::endl;
    }
  }
}

/// Copy constructor
RefBase::RefBase(const RefBase& c)
: Reference(), m_token(c.m_token), m_cache(c.m_cache), m_data(c.m_data)
{     
  s_count->increment();
  if ( m_token )  {
    int cnt = m_token->addRef();
    if ( s_tokenDbg ) {
      std::cout << "Token[1]:" << m_token << " " << cnt << std::endl;
    }
  }
}

/// Standard destructor
RefBase::~RefBase()       {
  s_count->decrement();
  if ( m_token )  {
    int cnt = m_token->release();
    if ( s_tokenDbg ) {
      std::cout << "Token[2]:" << m_token << " " << cnt << std::endl;
    }
  }
}

/// Set token object for this reference
void RefBase::setToken(Token* pToken)  const    {
  if ( pToken  )  {
    int cnt = pToken->addRef();
    if ( s_tokenDbg ) {
      std::cout << "Token[3]:" << pToken << " " << cnt << std::endl;
    }
  }
  if ( m_token )  {
    int cnt = m_token->release();
    if ( s_tokenDbg ) {
      std::cout << "Token[4]:" << m_token << " " << cnt << std::endl;
    }
  }
  m_token = pToken;
}

DbStatus RefBase::load(void*& ptr, bool throw_if)  const  {
  if ( 0 != m_token )   {
    DbStatus sc = m_cache->find_or_load_object(m_token, ptr);
    if ( !sc.isSuccess() )  {
      ptr = 0;
      sc = m_cache->find_or_load_object(m_token, ptr);
      if ( throw_if )  {
        throw std::runtime_error("Ref::load> Cannot load reference!");
      }
    }
    return sc;
  }
  if ( throw_if )  {
    throw std::runtime_error("Ref::load> Cannot load reference!");
  }
  return Error;
}
