#include "StorageSvc/pool.h"
#include "POOLCore/Token.h"
#include "Reflex/Reflex.h"
#include "pool/test.h"
#include <limits>
#include <iostream>
#include <iomanip>

#define __use_counters

#ifdef __use_counters

#include "StorageSvc/DbInstanceCount.h"

static pool::DbInstanceCount::Counter* s_aggCount = 
  pool::DbInstanceCount::getCounter(typeid(Aggregated));
static pool::DbInstanceCount::Counter* s_subaggCount = 
  pool::DbInstanceCount::getCounter(typeid(SubAggregated));
static pool::DbInstanceCount::Counter* s_ptraggCount = 
  pool::DbInstanceCount::getCounter(typeid(PtrAggregated));
static pool::DbInstanceCount::Counter* s_virtaggCount = 
  pool::DbInstanceCount::getCounter(typeid(VirtualAggregated));
static pool::DbInstanceCount::Counter* s_nsaggCount = 
  pool::DbInstanceCount::getCounter(typeid(A::B::C::NSAggregated));
static pool::DbInstanceCount::Counter* s_customCount = 
  pool::DbInstanceCount::getCounter(typeid(CustomObj));
static pool::DbInstanceCount::Counter* s_test_base_base_Count = 
  pool::DbInstanceCount::getCounter(typeid(test_base_base));
static pool::DbInstanceCount::Counter* s_test_base_Count = 
  pool::DbInstanceCount::getCounter(typeid(test_base));
static pool::DbInstanceCount::Counter* s_test_Count = 
  pool::DbInstanceCount::getCounter(typeid(test));
static pool::DbInstanceCount::Counter* sQTNCount = 
  pool::DbInstanceCount::getCounter(typeid(AnotherTNamed));
static pool::DbInstanceCount::Counter* s_nsTNCount = 
  pool::DbInstanceCount::getCounter(typeid(A::B::C::Named));

static pool::DbInstanceCount::Counter* s_diamondBaseCount =
  pool::DbInstanceCount::getCounter(typeid(DiamondBase));
static pool::DbInstanceCount::Counter* s_diamondClosureCount =
  pool::DbInstanceCount::getCounter(typeid(DiamondClosure));
static pool::DbInstanceCount::Counter* s_diamondDerivedACount =
  pool::DbInstanceCount::getCounter(typeid(DiamondDerivedA));
static pool::DbInstanceCount::Counter* s_diamondDerivedBCount =
  pool::DbInstanceCount::getCounter(typeid(DiamondDerivedB));

#ifdef __use_embedded_classes
static pool::DbInstanceCount::Counter* s_EmbeddedClassesCount =
  pool::DbInstanceCount::getCounter(typeid(test_base::EmbeddedClasses));
static pool::DbInstanceCount::Counter* s_Embedded1Count =
  pool::DbInstanceCount::getCounter(typeid(test_base::EmbeddedClasses::Embedded1));
static pool::DbInstanceCount::Counter* s_Embedded2Count =
  pool::DbInstanceCount::getCounter(typeid(test_base::EmbeddedClasses::Embedded2));
static pool::DbInstanceCount::Counter* s_Embedded3Count =
  pool::DbInstanceCount::getCounter(typeid(test_base::EmbeddedClasses::Embedded3));
#endif

#define CONT_SIZE 6

void dumpRef(const pool::Ref<TNamed>& ref)  {
  TNamed* ptr = ref.ptr(false);
  if ( ptr )  {
    std::cout << "'" << ptr->GetTitle() << "'";
  }
  else  {
    std::cout << "'<Bad>'";
  }
}

void dumpDiamond(const std::string& dsc, const DiamondBase* p)  {
  const DiamondDerivedA* a = dynamic_cast<const DiamondDerivedA*>(p);
  const DiamondDerivedB* b = dynamic_cast<const DiamondDerivedB*>(p);
  const DiamondClosure*  c = dynamic_cast<const DiamondClosure*>(p);
  std::cout << dsc;
  if ( p )  {
    std::cout << p->m_baseInt;
  }
  if ( a )  {
    std::cout << "/A:" << a->m_float;
  }
  if ( b )  {
    std::cout << "/B:" << b->m_longlong;
  }
  if ( c )  {
    std::cout << "/C:" << c->m_derivDouble;
  }
  if ( 0 == p ) {
    std::cout << "<Bad>";
  }
  std::cout << " ";
}

Aggregated* createAggregate(int iloop, const Aggregated& master) {
  if ( (iloop-1)%CONT_SIZE == 0 )  {
    return new Aggregated(master);
  }
  else if ( (iloop-1)%CONT_SIZE == 1 )  {
    return new SubAggregated(master);
  }
  else if ( (iloop-1)%CONT_SIZE == 2 )  {
    return new VirtualAggregated(master);
  }
  else if ( (iloop-1)%CONT_SIZE == 3 )  {
    return new A::B::C::NSAggregated(master);
  }
  else if ( (iloop-1)%CONT_SIZE == 4 )  {
    return new PtrAggregated(master);
  }
  return 0;
}
std::ostream& dumpAggregate(const Aggregated* p) {
  try  {
    const SubAggregated*       sub  = dynamic_cast<const SubAggregated*>(p);
    const PtrAggregated*       ptr  = dynamic_cast<const PtrAggregated*>(p);
    const VirtualAggregated*   virt = dynamic_cast<const VirtualAggregated*>(p);
    const A::B::C::NSAggregated *ns = dynamic_cast<const A::B::C::NSAggregated*>(p);
    if ( p )  {
      std::cout << p->data1 << "/" << p->data2;
    }
    if ( sub )  {
      std::cout << "/S:" << sub->data3;
    }
    if ( virt )  {
      std::cout << "/V:" << virt->data3;
    }
    if ( ns )  {
      std::cout << "/N:" << ns->data3;
    }
    if ( ptr )  {
      std::cout << "/P:" << ptr->data3;
    }
    if ( 0 == p ) {
      std::cout << "<Bad>";
    }
  }
  catch (...)  {
    std::cout << "<Exception>";
  }
  std::cout << " ";
  return std::cout;
}

inline std::ostream& operator<<(std::ostream& /* os */, const Aggregated* p) 
{  return dumpAggregate(p); }

inline std::ostream& operator<<(std::ostream& os, const Aggregated& p) 
{  return os << &p;}

inline std::ostream& operator<<(std::ostream& os, const pool::own_ptr<Aggregated>& p) 
{  return os << p.get();}

template <typename T, typename Q> inline
std::ostream& operator <<(std::ostream& os, const std::pair<T,Q>& p) 
{  return os /* << "Key:" << p.first << */ << p.second; }

template <typename T> inline
void dumpContainer(const std::string& dsc, const T& cnt) {
  typedef typename T::const_iterator IT;
  std::cout << dsc;
  for(IT i = cnt.begin(); i != cnt.end(); ++i) 
    std::cout << (*i);
}

template <typename T> inline 
std::ostream& operator<< (std::ostream& os, const std::vector<T>& cnt) {
  typedef typename std::vector<T>::const_iterator IT;
  for (IT i = cnt.begin(); i!=cnt.end(); ++i)  {
    std::cout << (*i);
  }
  return os;
}

template <typename T> inline 
std::ostream& operator<< (std::ostream& os, std::queue<T>& cnt) {
  for (size_t i = 0; i < cnt.size(); ++i)  {
    std::cout << cnt.front();
    cnt.push(cnt.front());
    cnt.pop();
  }
  return os;
}

template <typename T> inline 
std::ostream& operator<< (std::ostream& os, std::stack<T>& cnt) {
  for (size_t i = 0, _n=cnt.size(); i<_n; ++i)  {
    std::cout << cnt.top();
    cnt.pop();
  }
  return os;
}

template <class T> static void __doDelete(T* c) {   delete c; }
template <class T, class Q> static void __doDelete(std::pair<T,Q>& c) 
{   delete c.second; }

template <class T> static void clear_cont(T& c) {
  typedef typename T::iterator _I;
  for (_I i1=c.begin(); i1!= c.end(); ++i1)
    __doDelete( *i1 );
  c.clear();
}
template <class T> static void pop_cont(T& c) {
  while (c.size() > 0 )  {
    delete c.front();
    c.pop();
  }
}
template <class T> static void top_cont(T& c) {
  while (c.size() > 0 )  {
    delete c.top();
    c.pop();
  }
}

AbstractBase::AbstractBase(float f)
{
  m_float = f;
}

AbstractBase::~AbstractBase()
{
}

AbstractImp::AbstractImp() : AbstractBase(-1.0)
{
  m_int = -1;
}

AbstractImp::~AbstractImp()
{
}

#ifdef __use_embedded_classes
test_base::EmbeddedClasses::Embedded1::Embedded1() 
{
  i = 0;
#ifdef __use_counters
  s_Embedded1Count->increment();
#endif // __use_counters
}
test_base::EmbeddedClasses::Embedded1::~Embedded1() 
{
#ifdef __use_counters
  s_Embedded1Count->decrement();
#endif // __use_counters
}

test_base::EmbeddedClasses::Embedded2::Embedded2() 
{
  d = 0.0;
#ifdef __use_counters
  s_Embedded2Count->increment();
#endif // __use_counters
}
test_base::EmbeddedClasses::Embedded2::~Embedded2() 
{
#ifdef __use_counters
  s_Embedded2Count->decrement();
#endif // __use_counters
}

test_base::EmbeddedClasses::Embedded3::Embedded3() 
{
  f = 0.0;
#ifdef __use_counters
  s_Embedded3Count->increment();
#endif // __use_counters
}
test_base::EmbeddedClasses::Embedded3::~Embedded3() 
{
#ifdef __use_counters
  s_Embedded3Count->decrement();
#endif // __use_counters
}
test_base::EmbeddedClasses::EmbeddedClasses() 
{
  m_pemb1 = 0;
  m_pemb2 = 0;
  m_pemb3 = 0;
#ifdef __use_counters
  s_EmbeddedClassesCount->increment();
#endif // __use_counters
}
test_base::EmbeddedClasses::~EmbeddedClasses()  {
  pool::deletePtr(m_pemb1);
  pool::deletePtr(m_pemb2);
  pool::deletePtr(m_pemb3);
#ifdef __use_counters
  s_EmbeddedClassesCount->decrement();
#endif // __use_counters
}
#endif // __use_embedded_classes


A::B::C::Named::Named() : TNamed() 
{
#ifdef __use_counters
  s_nsTNCount->increment();
#endif // __use_counters
}

A::B::C::Named::Named(const char* a1, const char* a2)
: TNamed(a1, a2) 
{
#ifdef __use_counters
  s_nsTNCount->increment();
#endif // __use_counters
}

A::B::C::Named::~Named() 
{
#ifdef __use_counters
  s_nsTNCount->decrement();
#endif // __use_counters
}

AnotherTNamed::AnotherTNamed() : TNamed() 
{
#ifdef __use_counters
  sQTNCount->increment();
#endif // __use_counters
}

AnotherTNamed::AnotherTNamed(const char* a1, const char* a2)
: TNamed(a1, a2) 
{
#ifdef __use_counters
  sQTNCount->increment();
#endif // __use_counters
}

AnotherTNamed::~AnotherTNamed() 
{
#ifdef __use_counters
  sQTNCount->decrement();
#endif // __use_counters
}


Aggregated::Aggregated() 
{
#ifdef __use_counters
  s_aggCount->increment();
#endif // __use_counters
}

Aggregated::Aggregated(int d1, long d2) 
: data1(d1), data2(d2) 
{
#ifdef __use_counters
  s_aggCount->increment();
#endif // __use_counters
}

Aggregated::Aggregated(const Aggregated& c) 
: data1(c.data1), data2(c.data2) 
{
#ifdef __use_counters
  s_aggCount->increment();
#endif // __use_counters
}

Aggregated::~Aggregated() 
{
#ifdef __use_counters
  s_aggCount->decrement();
#endif // __use_counters
}

/// Standard Constructor
SubAggregated::SubAggregated()
{
  data3 = -1;
#ifdef __use_counters
  s_subaggCount->increment();
#endif // __use_counters
}

/// Initializing Constructor
SubAggregated::SubAggregated(const Aggregated& c)
{
  const SubAggregated* s = dynamic_cast<const SubAggregated*>(&c);
  data1 = c.data1;
  data2 = c.data2;
  if ( s ) {
    data3 = s->data3;
  }
  else {
    data3 = 10*data2;
  }
#ifdef __use_counters
  s_subaggCount->increment();
#endif // __use_counters
}

/// Initializing Constructor
SubAggregated::SubAggregated(int i, int k, int j) : Aggregated(i, k), data3(j) 
{
#ifdef __use_counters
  s_subaggCount->increment();
#endif // __use_counters
}

/// Standard Destructor
SubAggregated::~SubAggregated()
{
#ifdef __use_counters
  s_subaggCount->decrement();
#endif // __use_counters
}

/// Standard Constructor
PtrAggregated::PtrAggregated()
{
  data3 = -1;
#ifdef __use_counters
  s_ptraggCount->increment();
#endif // __use_counters
}

/// Initializing Constructor
PtrAggregated::PtrAggregated(const Aggregated& c)
{
  const PtrAggregated* s = dynamic_cast<const PtrAggregated*>(&c);
  data1 = c.data1;
  data2 = c.data2;
  if ( s ) {
    data3 = s->data3;
  }
  else {
    data3 = 10*data2;
  }
#ifdef __use_counters
  s_ptraggCount->increment();
#endif // __use_counters
}

/// Initializing Constructor
PtrAggregated::PtrAggregated(int i, int k, int j) : Aggregated(i, k), data3(j) 
{
#ifdef __use_counters
  s_ptraggCount->increment();
#endif // __use_counters
}

/// Standard Destructor
PtrAggregated::~PtrAggregated()
{
#ifdef __use_counters
  s_ptraggCount->decrement();
#endif // __use_counters
}

/// Standard Constructor
VirtualAggregated::VirtualAggregated()
{
  data3 = -1;
#ifdef __use_counters
  s_virtaggCount->increment();
#endif // __use_counters
}

/// Initializing Constructor
VirtualAggregated::VirtualAggregated(const Aggregated& c)
{
  const VirtualAggregated* s = dynamic_cast<const VirtualAggregated*>(&c);
  data1 = c.data1;
  data2 = c.data2;
  if ( s ) {
    data3 = s->data3;
  }
  else {
    data3 = 10*data2;
  }
#ifdef __use_counters
  s_virtaggCount->increment();
#endif // __use_counters
}

/// Initializing Constructor
VirtualAggregated::VirtualAggregated(int i, int k, int j) : Aggregated(i, k), data3(j) 
{
#ifdef __use_counters
  s_virtaggCount->increment();
#endif // __use_counters
}

/// Standard Destructor
VirtualAggregated::~VirtualAggregated()
{
#ifdef __use_counters
  s_virtaggCount->decrement();
#endif // __use_counters
}

/// Standard Constructor
A::B::C::NSAggregated::NSAggregated()
{
  data3 = -1;
#ifdef __use_counters
  s_nsaggCount->increment();
#endif // __use_counters
}

/// Initializing Constructor
A::B::C::NSAggregated::NSAggregated(const Aggregated& c)
{
  const VirtualAggregated* s = dynamic_cast<const VirtualAggregated*>(&c);
  data1 = c.data1;
  data2 = c.data2;
  if ( s ) {
    data3 = s->data3;
  }
  else {
    data3 = 10*data2;
  }
#ifdef __use_counters
  s_nsaggCount->increment();
#endif // __use_counters
}

/// Standard Destructor
A::B::C::NSAggregated::~NSAggregated()
{
#ifdef __use_counters
  s_nsaggCount->decrement();
#endif // __use_counters
}

CustomObj::CustomObj() 
{
#ifdef __use_counters
  s_customCount->increment();
#endif // __use_counters
}

CustomObj::CustomObj(int d1, long d2) 
: data1(d1), data2(d2) 
{
#ifdef __use_counters
  s_customCount->increment();
#endif // __use_counters
}

CustomObj::CustomObj(const CustomObj& c) 
: data1(c.data1), data2(c.data2) 
{
#ifdef __use_counters
  s_customCount->increment();
#endif // __use_counters
}

CustomObj::~CustomObj() 
{
#ifdef __use_counters
  s_customCount->decrement();
#endif // __use_counters
}


DiamondBase::DiamondBase() : m_baseInt(-1) 
{
#ifdef __use_counters
  s_diamondBaseCount->increment();
#endif // __use_counters
}

DiamondBase::~DiamondBase() {
#ifdef __use_counters
  s_diamondBaseCount->decrement();
#endif // __use_counters
}

DiamondDerivedA::DiamondDerivedA() 
: DiamondBase(), m_float(-1.) 
{
#ifdef __use_counters
  s_diamondDerivedACount->increment();
#endif // __use_counters
}

DiamondDerivedA::~DiamondDerivedA()       
{
#ifdef __use_counters
  s_diamondDerivedACount->decrement();
#endif // __use_counters
}

DiamondDerivedB::DiamondDerivedB() 
: DiamondBase(), m_longlong(-1) 
{
#ifdef __use_counters
  s_diamondDerivedBCount->increment();
#endif // __use_counters
}

DiamondDerivedB::~DiamondDerivedB()
{
#ifdef __use_counters
  s_diamondDerivedBCount->decrement();
#endif // __use_counters
}

DiamondClosure::DiamondClosure()
: m_derivDouble(-1.)
{
#ifdef __use_counters
  s_diamondClosureCount->increment();
#endif // __use_counters
}

DiamondClosure::~DiamondClosure()
{
#ifdef __use_counters
  s_diamondClosureCount->decrement();
#endif // __use_counters
}

test_base_base::test_base_base() {
  test_base_base_initData();
#ifdef __use_counters
  s_test_base_base_Count->increment();
#endif // __use_counters
}

void test_base_base::test_base_base_initData()  {
#ifdef __use_root_pointers
  m_aggPtr  = 0;
  m_aggRef  = 0;
  m_namPtr  = 0;
  m_namRef  = 0;
#endif // __use_root_pointers

#ifdef __use_primitives
  m_bool1       = true;
  m_bool2       = false;
  m_char        = ~0;
  m_short       = ~0;
  m_int         = ~0;
  m_long        = ~0;
  m_int64       = ~0;
  m_float       = -1.;
  m_double      = -1.;
  m_longdouble  = -1.;
  m_uchar       = ~0;
  m_ushort      = ~0;
  m_uint        = ~0;
  m_ulong       = ~0;
  m_uint64      = ~0;
  m_text        = new char[128];
  sprintf(m_text, "<empty>");
  m_string        ="<empty>";
  m_transient_int = 123456789;
  m_transientPtr  = 0;
#endif // __use_primitives

#ifdef __use_custom_streamer
  custom.data1 = -1;
  custom.data2 = -1;
#endif // __use_custom_streamer
}

test_base_base::test_base_base(const std::string& /* t */ ) { 
  test_base_base_initData();
#ifdef __use_counters
  s_test_base_base_Count->increment();
#endif // __use_counters
}

test_base_base::~test_base_base() {
  delete m_text;
#ifdef __use_root_pointers
  if ( m_aggPtr       ) delete m_aggPtr;
  if ( m_aggRef && m_aggRef != m_aggPtr ) delete m_aggRef;
  if ( m_namPtr       ) delete m_namPtr;
  if ( m_transientPtr ) delete m_transientPtr;
#endif // __use_root_pointers
#ifdef __use_counters
  s_test_base_base_Count->decrement();
#endif // __use_counters
}

test_base::test_base() 
: test_base_base(),
  m_cbool(false),
  m_cchar(-1),
  m_cshort(-1),
  m_cint(-1),
  m_clong(-1),
  m_cnamPtr(0)
{
  m_mchar        = -1;
  m_mshort       = -1;
  m_mint         = -1;
  m_mlong        = -1;
#ifdef __use_counters
  s_test_base_Count->increment();
#endif // __use_counters
}

test_base::test_base(const std::string& t) 
: test_base_base(t),
  m_cbool(false),
  m_cchar(-1),
  m_cshort(-1),
  m_cint(-1),
  m_clong(-1),
  m_cnamPtr(0)
{ 
  m_mchar        = ~0;
  m_mshort       = ~0;
  m_mint         = ~0;
  m_mlong        = ~0;
#ifdef __use_counters
  s_test_base_Count->increment();
#endif // __use_counters
}

test_base::~test_base() {
  if ( m_cnamPtr       ) delete const_cast<TNamed*>(m_cnamPtr);  
#ifdef __use_counters
  s_test_base_Count->decrement();
#endif // __use_counters
}

test::test() : test_base() {
#ifdef __use_counters
  s_test_Count->increment();
#endif // __use_counters
  test_initData();
}

test::test(const std::string& t) : test_base(t) { 
#ifdef __use_counters
  s_test_Count->increment();
#endif // __use_counters
  test_initData();
}

test::~test() {
  test_finiData();
}

void test::test_initData() {
#ifdef __use_diamonds
  m_diam_baseP   = 0;
  m_diam_AP      = 0;
  m_diam_BP      = 0;
  m_diam_closedP = 0;
#endif // __use_diamonds
  m_noVirtualPtr = new NoVirtual;
  m_noVirtualPtr->m_data1 = 1;
  m_noVirtualSubClassPtr = 0;
  m_abstract = 0;
  m_int_vector.resize(7,1);
  m_ulonglong_vector.resize(6,9);
  m_double_vector_myalloc.resize(5,9.5);
  m_agg_vector.resize(5, Aggregated(1,10));
}

void test::test_finiData() {
#ifdef __use_counters
  s_test_Count->decrement();
#endif // __use_counters
#ifdef __use_diamonds
  m_diam_baseP   = 0;
  m_diam_AP      = 0;
  m_diam_BP      = 0;
  m_diam_closedP = 0;
#endif // __use_diamonds

  if ( m_abstract ) delete m_abstract;
  m_abstract = 0;
  if ( m_noVirtualPtr ) delete m_noVirtualPtr;
  m_noVirtualPtr = 0;
  if ( m_noVirtualSubClassPtr ) delete m_noVirtualSubClassPtr;
  m_noVirtualSubClassPtr = 0;

#ifdef __use_unkeyed
  m_int_vector.clear();
  m_str_vector.clear();
  m_bool_vector.clear();
  m_agg_vector.clear();
  m_agg_list.clear();
  m_str_list.clear();
  m_bool_list.clear();
  m_double_vector_myalloc.clear();
  m_agg_vector_myalloc.clear();
  m_double_list_myalloc.clear();
  m_agg_list_myalloc.clear();
  
  clear_cont(m_agg_vectorP);
  clear_cont(m_agg_listP);
  clear_cont(m_agg_dequeP);
  clear_cont(m_agg_vectorP_myalloc);
  clear_cont(m_agg_listP_myalloc);

  //clear_cont(*(std::vector<Aggregated*>*)&m_agg_const_vectorP);
#ifdef __use_refs
#ifdef __use_unkeyed
  m_ref_vector.clear();
  m_ref_list.clear();
#endif // __use_unkeyed
#ifdef __use_keyed
  m_ref_map.clear();
#endif // __use_keyed

#endif // __use_refs
#ifdef __use_recursive_structs
  m_char_vector_vector.clear();
  m_char_vector_list.clear();
  m_int_vector_vector.clear();
  m_int_vector_list.clear();
#endif // __use_recursive_structs
#endif // __use_unkeyed

#ifdef __use_keyed
  m_str_str_map.clear();
  m_int_float_hash_map.clear();
  m_int_set.clear();
  m_int_hash_set.clear();
  m_int_agg_map.clear();
  m_agg_set.clear();
  m_str_str_vector_map.clear();
  clear_cont(m_int_agg_mapP);
  clear_cont(m_agg_setP);
  m_agg_deque.clear();
  pop_cont(m_agg_queueP);
  top_cont(m_agg_stackP);

  m_int_agg_multimap.clear();
  m_agg_multiset.clear();
  clear_cont(m_int_agg_multimapP);
  clear_cont(m_agg_multisetP);

  m_int_agg_hash_map.clear();
  m_agg_hash_set.clear();
  m_agg_hash_multiset.clear();
  m_int_agg_hash_multimap.clear();
  clear_cont(m_int_agg_hash_mapP);
  clear_cont(m_agg_hash_setP);
  clear_cont(m_int_agg_hash_multimapP);
  clear_cont(m_agg_hash_multisetP);
#endif // __use_keyed
}

void test::dump() {}
void test::set_a(int val)    {  m_char = val;   }
void test::set_b(int val)    {  m_short = val;  }
void test::set_c(int val)    {  m_int = val;    }
int  test::get_a() const     {  return m_char;  }
int  test::get_b() const     {  return m_short; }
int  test::get_c() const     {  return m_int;   }

void test::fillData(int i)    {
  char txt1[256];
  ::sprintf(txt1,"Hello [test(%d)]", i);
#ifdef __use_primitives
  m_string  = txt1;
  ::sprintf(m_text,"test_ntchar(%d)", i);
  m_bool1       = false;  // Flip values...
  m_bool2       = true;  
  m_char   = i % std::numeric_limits<char>::max();
  m_uchar  = i % std::numeric_limits<unsigned char>::max();
  m_short  = i % std::numeric_limits<short>::max();
  m_ushort = i % std::numeric_limits<unsigned short>::max();
  m_int    = i % std::numeric_limits<int>::max();
  m_uint   = i % std::numeric_limits<unsigned int>::max();
  m_long   = i % std::numeric_limits<long>::max();
  m_ulong  = i % std::numeric_limits<unsigned long>::max();
#ifdef _WIN32
  m_int64  = i + 1234567890000000000;
  m_uint64 = i + 1234567890000000000;
#else
  m_int64  = i + 1234567890000000000LL;
  m_uint64 = i + 1234567890000000000LL;
#endif
  m_float  = float(i);
  m_double = 100.0   * i;
  m_longdouble = 10000.0 * i;
  m_transient_int = 987654321;

  *(const_cast<bool*>(&m_cbool))   = true;
  *(const_cast<char*>(&m_cchar))   = i % std::numeric_limits<char>::max();
  *(const_cast<short*>(&m_cshort)) = i % std::numeric_limits<short>::max();
  *(const_cast<int*>(&m_cint))     = i % std::numeric_limits<int>::max();
  *(const_cast<long*>(&m_clong))   = i % std::numeric_limits<long>::max();

  m_mchar   = i % std::numeric_limits<char>::max();
  m_mshort  = i % std::numeric_limits<short>::max();
  m_mint    = i % std::numeric_limits<int>::max();
  m_mlong   = i % std::numeric_limits<long>::max();

#endif
  AbstractImp* a = new AbstractImp();
  a->m_float = float(i);
  a->m_int   = 10*i;
  m_abstract = a;

  m_agg.data1 = i;
  m_agg.data2 = 10*i;
  if ( m_noVirtualPtr )  {
    m_noVirtualPtr->m_data1 = i;
  }
#ifdef __use_custom_streamer
  custom.data1 = i;
  custom.data2 = 10*i;
#endif // __use_custom_streamer

#ifdef __use_embedded_classes
  test* o = this;
  o->m_emb1.i                     = i;
  o->m_emb2.m_embed1.i            = 10*i;
  o->m_emb2.d                     = double(100*i);
  o->m_emb3.m_embed2.m_embed1.i   = 1000*i;
  o->m_emb3.m_embed2.d            = double(10000*i);
  o->m_emb3.f                     = float( 100000*i);

#ifdef __use_root_pointers
  m_embedded.m_pemb1 = new EmbeddedClasses::Embedded1;
  m_embedded.m_pemb2 = new EmbeddedClasses::Embedded2;
  m_embedded.m_pemb3 = new EmbeddedClasses::Embedded3;
  m_embedded.m_pemb1->i                   = 2*i;
  m_embedded.m_pemb2->m_embed1.i          = 20*i;
  m_embedded.m_pemb2->d                   = double(200*i);
  m_embedded.m_pemb3->m_embed2.m_embed1.i = 2000*i;
  m_embedded.m_pemb3->m_embed2.d          = double(20000*i);
  m_embedded.m_pemb3->f                   = float( 200000*i);
#endif // __use_root_pointers
#endif // __use_embedded_classes

#ifdef __use_diamonds
  m_diam_closed.m_baseInt     = i;
  m_diam_closed.m_float       = float(10.0)*i;
  m_diam_closed.m_longlong    = 100*i;
  m_diam_closed.m_derivDouble = 1000.0*i;
#endif // __use_diamonds

#ifdef __use_root_pointers
  {
    PtrAggregated* ptr = new PtrAggregated();
    ptr->data1 = i;
    ptr->data2 = 10*i;
    ptr->data3 = 100*i;
    m_aggPtr = ptr;
    ptr = new PtrAggregated();
    ptr->data1 = i;
    ptr->data2 = 10*i;
    ptr->data3 = 100*i;
    m_transientPtr = ptr;
  }
  m_aggRef = m_aggPtr;
  m_namPtr  = new TNamed(txt1, "World");
  *(const_cast<TNamed**>(&m_cnamPtr)) = new TNamed(txt1,"Const World");
  m_namRef  = m_namPtr;
#endif // __use_root_pointers

#ifdef __use_arrays
  m_int_array[0] = i;
  m_int_array[1] = 10*i;
  m_int_array[2] = 100*i;
  m_int_array[3] = 1000*i;
  m_int_array[4] = 10000*i;
  m_agg_array[0].data1 = i;
  m_agg_array[1].data1 = 10*i;
  m_agg_array[2].data1 = 100*i;
  m_agg_array[3].data1 = 1000*i;
  m_agg_array[4].data1 = 10000*i;
  m_agg_array[0].data2 = 10000*i;
  m_agg_array[1].data2 = 1000*i;
  m_agg_array[2].data2 = 100*i;
  m_agg_array[3].data2 = 10*i;
  m_agg_array[4].data2 = i;
#endif // __use_arrays

#ifdef __use_unkeyed
#ifdef __use_refs
  m_ref_vector.push_back(refNamed1);
  m_ref_vector.push_back(refNamed1);
  m_ref_list.push_back(refNamed1);
  m_ref_list.push_back(refNamed1);
  m_ref_deque.push_back(refNamed1);
  m_ref_deque.push_back(refNamed1);
  m_ref_queue.push(refNamed1);
  m_ref_queue.push(refNamed1);
  m_ref_stack.push(refNamed1);
  m_ref_stack.push(refNamed1);

#endif // __use_refs
  m_int_vector.clear();
  m_str_vector.clear();
  m_bool_vector.clear();
  m_ulonglong_vector.clear();
  m_double_vector_myalloc.clear();

  m_agg_vector.clear();
  m_agg_vector_myalloc.clear();

  m_int_list.clear();
  m_str_list.clear();
  m_bool_list.clear();
  m_longlong_list.clear();
  m_double_list_myalloc.clear();

  m_agg_list.clear();
  m_agg_list_myalloc.clear();
#endif // __use_unkeyed

#ifdef __use_recursive_structs
  m_vector_vector_char.clear();
  m_list_vector_char.clear();
  m_vector_vector_int.clear();
  m_list_vector_int.clear();
  {
    std::vector<char> tmp_c;
    std::vector<int>  tmp_i;
    for (char ic=1; ic<4; ++ic )   {
      tmp_i.push_back(    ic);
      tmp_c.push_back('0'+ic);      
      m_char_vector_list.push_back(tmp_c);
      m_char_vector_vector.push_back(tmp_c);
      m_int_vector_list.push_back(tmp_i);
      m_int_vector_vector.push_back(tmp_i);
    }
  }
#endif // __use_recursive_structs

  {
    for ( int cnt = 0; cnt < CONT_SIZE; ++cnt)  {
      int iloop = cnt+1;
      Aggregated master(iloop,11*iloop);
      /**
        *  VECTOR & LIST
        */
#ifdef __use_unkeyed
      {
        char key[] = {cnt+'0', 0};
        m_int_vector.push_back(cnt);
        m_bool_vector.push_back(cnt%2==0 ? true : false);
        m_str_vector.push_back(key);
        m_ulonglong_vector.push_back(m_int64+cnt);
        m_double_vector_myalloc.push_back(double(cnt));

        m_agg_vector.push_back(master);
        m_agg_vector_myalloc.push_back(master);

        m_int_list.push_back(cnt);
        m_bool_list.push_back(cnt%2==0 ? true : false);
        m_str_list.push_back(key);
        m_longlong_list.push_back(m_int64+cnt);
        m_double_list_myalloc.push_back(double(cnt));

        m_agg_list.push_back(master);
        m_agg_list_myalloc.push_back(master);
#ifdef __use_root_pointers
        m_own_ptr_agg_vector.push_back(createAggregate(cnt,master));
        m_own_ptr_agg_list.push_back(createAggregate(cnt,master));
        m_agg_vectorP.push_back(createAggregate(cnt,master));
        m_agg_const_vectorP.push_back(m_agg_vectorP.back());
        m_agg_vectorP_myalloc.push_back(createAggregate(cnt,master));
        m_agg_listP.push_back(createAggregate(cnt,master));
        m_agg_listP_myalloc.push_back(createAggregate(cnt,master));
#endif // __use_root_pointers
      }
#endif // __use_unkeyed

      /**
        *  MAPS & SETS
        */
#ifdef __use_keyed
      {
        char key[] = {cnt+'0', 0};
        m_int_set.insert(iloop);
        m_agg_set.insert(master);
        m_int_multiset.insert(cnt%4);
        m_agg_multiset.insert(master);

        m_str_str_map[key] = key;
        m_int_agg_map[iloop] = master;
        m_int_short_multimap.insert(std::make_pair(cnt%4, short(cnt)));
        m_int_agg_multimap.insert(std::make_pair(cnt%4,master));
        m_str_str_vector_map.insert(std::make_pair(key,m_str_vector));
        m_int_hash_set.insert(iloop);
        m_agg_hash_set.insert(master);
        m_int_hash_multiset.insert(cnt%4);
        m_agg_hash_multiset.insert(master);

        m_int_agg_hash_map[iloop] = master;
        m_int_float_hash_map[iloop] = float(iloop);
        m_int_float_hash_multimap.insert(std::make_pair(cnt%4, float(cnt)));
        m_int_agg_hash_multimap.insert(std::make_pair(cnt%4,master));

#ifdef __use_refs
        m_ref_map.insert(std::make_pair(key,refNamed1));
        m_ref_multimap.insert(std::make_pair(key,refNamed1));
        m_ref_hash_map.insert(std::make_pair(key,refNamed1));
#endif // __use_refs
#ifdef __use_root_pointers      
        m_int_agg_mapP[iloop] = createAggregate(cnt, master);
        m_agg_setP.insert(createAggregate(cnt, master));
        m_int_agg_hash_mapP[iloop] = createAggregate(cnt, master);
        m_int_agg_hash_multimapP.insert(std::make_pair(cnt%4,createAggregate(cnt,master)));
        m_agg_hash_setP.insert(createAggregate(cnt, master));
        m_agg_hash_multisetP.insert(createAggregate(cnt, master));
        m_int_agg_multimapP.insert(std::make_pair(cnt%4,createAggregate(cnt,master)));
        m_agg_multisetP.insert(createAggregate(cnt, master));
        m_int_own_ptr_agg_map.insert(std::make_pair(iloop,pool::own_ptr<Aggregated>(createAggregate(cnt,master))));
#endif // __use_root_pointers
      }
#endif // __use_keyed

      /**
        *  DEQUE STACK AND QUEUE
        */
#ifdef __use_full_stl
      m_int_deque.push_back(cnt);
      m_int_queue.push(cnt);
      m_int_stack.push(cnt);
      m_agg_queue.push(master);
      m_agg_stack.push(master);
      m_agg_deque.push_back(master);
#ifdef __use_root_pointers
      m_agg_queueP.push(createAggregate(cnt, master));
      m_agg_stackP.push(createAggregate(cnt, master));
      m_agg_dequeP.push_back(createAggregate(cnt, master));
#endif // __use_root_pointers
#endif // __use_full_stl
    }
  }

#ifdef __use_root_pointers
  m_agg_auto_ptr_0 = std::auto_ptr<Aggregated>(new Aggregated(m_agg));
  m_agg_auto_ptr_1 = std::auto_ptr<Aggregated>(new SubAggregated(m_agg));
  m_agg_auto_ptr_2 = std::auto_ptr<Aggregated>(new VirtualAggregated(m_agg));
  m_agg_auto_ptr_3 = std::auto_ptr<Aggregated>(new PtrAggregated(m_agg));
  m_agg_auto_ptr_4 = std::auto_ptr<Aggregated>(new A::B::C::NSAggregated(m_agg));
#endif // __use_root_pointers
}

void test::printData(int /* i */, bool print_stacks )
{
#ifdef __use_primitives
  std::cout << std::left        << std::setw(13) 
            << " std::string:"  << std::left << std::setw(20) << m_string
            << "  char:"        << std::left << std::setw(3)  << long(m_char)
            << "  short:"       << long(m_short)
            << "  int:"         << long(m_int)
            << "  long:"        << long(m_long)
            << "  int64:"       << m_int64
            << "  t_int:"       << m_transient_int
            << "  float:"       << m_float
            << std::endl
            << std::left        << std::setw(13) 
            << " char*:"        << std::left << std::setw(20) << m_text
            << " uchar:"        << std::left << std::setw(3)  << long(m_uchar)
            << " ushort:"       << long(m_ushort)
            << " uint:"         << long(m_uint)
            << " ulong:"        << long(m_ulong)
            << " uint64:"       << m_uint64
            << "   dble:"       << m_double
            << " l-dble:"       << double(m_longdouble)
            << std::endl
            << " bool:  "       << m_bool1 << "=0 " << m_bool2 << "=1 ";
  std::cout << " CONST:"
            << " char:"         << std::left << std::setw(3)  << long(m_cchar)
            << "  short:"       << long(m_cshort)
            << "    int:"       << long(m_cint)
            << " long:"         << long(m_clong);
  std::cout << "   MUTABLE:"
            << "  char:"        << std::left << std::setw(3)  << long(m_mchar)
            << "  short:"       << long(m_mshort)
            << "  int:"         << long(m_mint)
            << "  long:"        << long(m_mlong);
  std::cout << std::endl;
#endif

  std::cout << "     agg:" << m_agg.data1 << " " << m_agg.data2;
  std::cout << " auto_ptr:";
  dumpAggregate(m_agg_auto_ptr_0.get());
  dumpAggregate(m_agg_auto_ptr_1.get());
  dumpAggregate(m_agg_auto_ptr_2.get());
  dumpAggregate(m_agg_auto_ptr_3.get());
  dumpAggregate(m_agg_auto_ptr_4.get());
  dumpAggregate(m_agg_auto_ptr_5.get());

#ifdef __use_custom_streamer
  std::cout << " custom:" << custom.data1 << " " << custom.data2;
#endif // __use_custom_streamer

#ifdef __use_refs
  std::cout << std::endl;
  TNamed* pNam1 = named1();   // Implicit object read and pointer conversion 
  std::cout << "     Ref1:";
  if ( pNam1 )   {
    std::cout << pNam1->GetName() << " : " << pNam1->GetTitle();
    //std::cout << std::endl << (void*)refNamed1.token() << " " << refNamed1.token()->toString() << std::endl;
  }
  else  {
    std::cout << "<Bad>";
  }
  TNamed* pNam2 = named2();   // Implicit object read and pointer conversion 
  std::cout << " Ref2:";
  if ( pNam2 )   {
    std::cout << pNam2->GetName() << " : " << pNam2->GetTitle();
    //std::cout << std::endl << (void*)refNamed2.token() << " " << refNamed2.token()->toString() << std::endl;
  }
  else  {
    std::cout << "<Bad>";
  }
#endif // __use_refs
#ifdef __use_root_pointers
  std::cout << std::endl << "     CONST TNamed:";
  if ( m_cnamPtr ) {
    std::cout << m_cnamPtr->GetName() << " " << m_cnamPtr->GetTitle();
  }
  else  {
    std::cout << "<No data>";
  }
  std::cout << " Abstract:";
  if ( m_abstract )  {
    AbstractImp* a = dynamic_cast<AbstractImp*>(m_abstract);
    std::cout << a->m_float << " " << a->m_int;
  }
  else  {
    std::cout << "<No data>";
  }
#endif // __use_root_pointers

#ifdef __use_diamonds
  dumpDiamond("     Diamond(closed):", &m_diam_closed);
#endif // __use_diamonds
  std::cout << "  Transient-Ptr:";
  dumpAggregate(m_transientPtr);
  std::cout << std::endl;
  std::cout << "     Non-Virtual    :" << (void*)m_noVirtualPtr;
  if ( m_noVirtualPtr )  std::cout << " Data:" << m_noVirtualPtr->m_data1 << " ";
  std::cout << "  Sub-class  :" << (void*)m_noVirtualSubClassPtr;
  if ( m_noVirtualSubClassPtr )  {
    std::cout << m_noVirtualSubClassPtr->m_data1 << " "
      << m_noVirtualSubClassPtr->m_data2 << " ";
  }
  std::cout << std::endl;

#ifdef __use_embedded_classes
  test* o = this;
  std::cout << "     embedded: ";
  std::cout << "  1.i:"      << o->m_emb1.i;
  std::cout << "  2.1.i:"    << o->m_emb2.m_embed1.i;
  std::cout << "  2.d:"      << o->m_emb2.d;
  std::cout << "  3.2.1.i:"  << o->m_emb3.m_embed2.m_embed1.i;
  std::cout << "  3.2.d:"    << o->m_emb3.m_embed2.d;
  std::cout << "  3.f:"      << o->m_emb3.f;
  std::cout << std::endl;
#ifdef __use_root_pointers
  std::cout << "     embedded*:";
  if ( m_embedded.m_pemb1 )  {
    std::cout << " *1.i:"     << m_embedded.m_pemb1->i;
    std::cout << " *2.1.i:"   << m_embedded.m_pemb2->m_embed1.i;
    std::cout << " *2.d:"     << m_embedded.m_pemb2->d;
    std::cout << " *3.2.1.i:" << m_embedded.m_pemb3->m_embed2.m_embed1.i;
    std::cout << " *3.2.d:"   << m_embedded.m_pemb3->m_embed2.d;
    std::cout << " *3.f:"     << m_embedded.m_pemb3->f;
  }
  else  {
    std::cout << " empty ";
  }
  std::cout << std::endl;
#endif // __use_root_pointers
#endif // __use_embedded_classes

#ifdef __use_root_pointers
  std::cout << "     ROOT-Ptr:";
  if ( m_aggPtr ) {
    dumpAggregate(m_aggPtr);
  }
  else  {
    std::cout << "<No data>";
  }
  std::cout << " Ref:";
  dumpAggregate(m_aggRef);
  std::cout << (char*)((m_aggPtr==m_aggRef) ? "(Eq)" : "(Diff)");

  std::cout << std::endl << "     NamedPtr:";
  if ( m_namPtr ) {
    std::cout << m_namPtr->GetName();
  }
  else  {
    std::cout << "<No data>";
  }
  std::cout << " PtrRef:";
  if ( m_namRef.GetObject() ) {
    TNamed* n = (TNamed*)m_namRef.GetObject();
    std::cout << n->GetName()
              << (char*)((n==m_namPtr) ? " (Eq)" : " (Diff)");
  }
  else  {
    std::cout << "<No data>";
  }
#endif // __use_refs

  /*
   *      Test access to arrays of primitves and objects
   */
#ifdef __use_arrays
  std::cout << " intA:" << m_int_array[0] << " " << m_int_array[4];
  std::cout << " aggA:" << m_agg_array[4].data1 << " " << m_agg_array[4].data2;
#endif // __use_arrays
  /*
   *      Test access to vector of vectors
   */
#ifdef __use_recursive_structs
  std::cout << std::endl;
  dumpContainer("     vec<vec<char>>: ", m_char_vector_vector);
  dumpContainer(" vec<vec<int>>:", m_int_vector_vector);
  dumpContainer(" list<vec<c>>: ", m_char_vector_list);
  dumpContainer(" list<vec<i>>: ", m_int_vector_list);
#endif // __use_recursive_structs

  /*      Test access to vector of references:   */
#ifdef __use_refs
  std::cout << std::endl;
#ifdef __use_unkeyed
  std::cout << "     Ref:Vec:" << m_ref_vector.size() << "->";
  if ( m_ref_vector.size() > 0 )  {
    dumpRef(m_ref_vector.back());
  }
  else  {
    std::cout << "<Bad> ";
  }
  /*      Test access to list of references:   */
  std::cout << " Lst:" << m_ref_list.size() << "->";
  if ( m_ref_list.size() > 0 )  {
    dumpRef(m_ref_list.back());
  }
  else  {
    std::cout << "<Bad> ";
  }
  /*      Test access to deque of references:   */
  std::cout << " Deq:" << m_ref_deque.size() << "->";
  if ( m_ref_deque.size() > 0 )  {
    dumpRef(m_ref_deque.back());
  }
  else  {
    std::cout << "<Bad> ";
  }
  /*      Test access to queue of references:   */
  std::cout << " Que:" << m_ref_queue.size() << "->";
  if ( m_ref_queue.size() > 0 )  {
    dumpRef(m_ref_queue.back());
  }
  else  {
    std::cout << "<Bad> ";
  }
  std::cout << std::endl;
  /*      Test access to stack of references:   */
  std::cout << "     Ref:Stk:" << m_ref_stack.size() << "->";
  if ( m_ref_stack.size() > 0 )  {
    dumpRef(m_ref_stack.top());
  }
  else  {
    std::cout << "<Bad> ";
  }
#endif
#ifdef __use_keyed
  /*      Test access to map of references:   */
  std::cout << " Map:" << m_ref_map.size() << "->";
  if ( m_ref_map.size() > 0 )  {
    dumpRef((*m_ref_map.begin()).second);
  }
  else  {
    std::cout << "<Bad> ";
  }
  /*      Test access to stack of references:   */
  std::cout << " MMp:" << m_ref_multimap.size() << "->";
  if ( m_ref_multimap.size() > 0 )  {
    dumpRef((*m_ref_multimap.begin()).second);
  }
  else  {
    std::cout << "<Bad> ";
  }
  /*      Test access to stack of references:   */
  std::cout << " Hmp:" << m_ref_hash_map.size() << "->";
  if ( m_ref_hash_map.size() > 0 )  {
    dumpRef((*m_ref_hash_map.begin()).second);
  }
  else  {
    std::cout << "<Bad> ";
  }
#endif
#endif

  /**
    *     VECTOR<own_ptr<T> > 
    *     LIST<own_ptr<T> > 
    *     MAP<own_ptr<T>>
    *     LIST<longlong>
    */
  {
#ifdef __use_unkeyed
    std::cout << std::endl;
#ifdef __use_root_pointers
    dumpContainer("     vector<own_ptr>:", m_own_ptr_agg_vector);
    dumpContainer(" vec<string>:  ", m_str_vector);
    dumpContainer(" vec<bool>: ", m_bool_vector);
    std::cout << std::endl;
    dumpContainer("     list<own_ptr>:  ", m_own_ptr_agg_list);
    dumpContainer(" list<string>: ", m_str_list);
    dumpContainer(" list<bool>:", m_bool_list);
    std::cout << std::endl;
#endif  // __use_root_pointers
#endif  // __use_unkeyed
#ifdef __use_keyed
    dumpContainer("     map<own_ptr>:   ", m_int_own_ptr_agg_map);
    dumpContainer(" map<str,v<s>>:", m_str_str_vector_map);
    std::cout << std::endl;
#endif  // __use_keyed
#ifdef __use_unkeyed
    std::vector<ulonglong>::const_iterator j = m_ulonglong_vector.begin();
    std::cout << "     vector<ulonglong>:" << *j++ << " " << *j++ 
              << " " << *j++ << " " << *j++ << std::endl;
    std::list<longlong>::const_iterator i = m_longlong_list.begin();
    std::cout << "     list<longlong>:   " << *i++ << " " << *i++ 
              << " " << *i++ << " " << *i++ << std::endl;
  }
#endif  // __use_unkeyed

#ifdef __use_keyed
  /**
    *     MAP
    */
  dumpContainer("     map<str,str>: ", m_str_str_map);
  dumpContainer(" Aggr: ", m_int_agg_map);
  dumpContainer(" Aggr*:", m_int_agg_mapP);
  std::cout << std::endl;
  /**
    *     MULTIMAP
    */
  dumpContainer("     multimap<i,s>:", m_int_short_multimap);
  dumpContainer(" Aggr: ", m_int_agg_multimap);
  dumpContainer(" Aggr*:", m_int_agg_multimapP);
  std::cout << std::endl;
  /**
    *     HASHMAP
    */
  dumpContainer("     hash_map<i,f>:", m_int_float_hash_map);
  dumpContainer(" Aggr: ", m_int_agg_hash_map);
  dumpContainer(" Aggr*:", m_int_agg_hash_mapP);
  std::cout << std::endl;
  /**
    *     HASHMULTIMAP
    */
  dumpContainer("     hash_multimap:", m_int_float_hash_multimap);
  dumpContainer(" Aggr: ", m_int_agg_hash_multimap);
  dumpContainer(" Aggr*:", m_int_agg_hash_multimapP);
  std::cout << std::endl;
  /**
    *     SET
    */
  dumpContainer("     set<int>:     ", m_int_set);
  dumpContainer(" Aggr: ", m_agg_set);
  dumpContainer(" Aggr*:", m_agg_setP);
  std::cout << std::endl;
  /**
    *     MULTISET
    */
  dumpContainer( "     multiset<int>:", m_int_multiset);
  dumpContainer(" Aggr: ", m_agg_multiset);
  dumpContainer(" Aggr*:", m_agg_multisetP);
  std::cout << std::endl;
  /**
    *     HASHSET
    */
  dumpContainer("     hashset<int>: ", m_int_hash_set);
  dumpContainer(" Aggr: ", m_agg_hash_set);
  dumpContainer(" Aggr*:", m_agg_hash_setP);
  std::cout << std::endl;
  /**
    *     HASHMULTIMAP
    */
  dumpContainer("     hash_multiset:", m_int_hash_multiset);
  dumpContainer(" Aggr: ", m_agg_hash_multiset);
  dumpContainer(" Aggr*:", m_agg_hash_multisetP);
  std::cout << std::endl;
#endif    // __use_keyed

#ifdef __use_full_stl
  /**
    *     DEQUE
    */
  dumpContainer( "     deque<int>:   ", m_int_deque);
  dumpContainer(" Aggr: ", m_agg_deque);
  dumpContainer(" Aggr*:", m_agg_dequeP);
  std::cout << std::endl;
  /**
    *     QUEUE
    */
  std::cout << "     queue<int>:   " << m_int_queue
            << " Aggr: " << m_agg_queue 
            << " Aggr*:" << m_agg_queueP << std::endl;
  /**
    *     STACK
    */
  if ( print_stacks )  {
    std::cout << "     stack<int>:   " << m_int_stack 
              << " Aggr: " << m_agg_stack;
    {
      std::cout << " Aggr*:";
      for (size_t i = 0, _n=m_agg_stackP.size(); i<_n; ++i)  {
        dumpAggregate(m_agg_stackP.top());
        if ( m_agg_stackP.top() ) delete m_agg_stackP.top();
        m_agg_stackP.pop();
      }
    }
    std::cout << std::endl;
  }
#endif // __use_full_stl
  /**
    *     VECTOR & LIST
    */
#ifdef __use_unkeyed
  dumpContainer("     list<int>:    ", m_int_list);
  dumpContainer(" Aggr: ", m_agg_list);
  dumpContainer(" Aggr*:", m_agg_listP);
  std::cout << std::endl;
  dumpContainer( "     vector<int>:  ", m_int_vector);
  dumpContainer(" Aggr: ", m_agg_vector);
  dumpContainer(" Aggr*:", m_agg_vectorP);
  std::cout << std::endl;
  dumpContainer("     vector<CONST>:", m_agg_const_vectorP);
  std::cout << "  Pointers:";
  for (size_t icp=0; icp < m_agg_const_vectorP.size(); ++icp )  {
    std::cout << " " << icp << ":";
    if ( m_agg_const_vectorP[icp] == m_agg_vectorP[icp] )  {
      std::cout << "T";
    }
    else  {
      std::cout << "F";
    }
  }
  std::cout << std::endl;

  dumpContainer( "     vec<myAlloc>: ", m_double_vector_myalloc);
  dumpContainer(" Aggr: ", m_agg_vector_myalloc);
  dumpContainer(" Aggr*:", m_agg_vectorP_myalloc);
  std::cout << std::endl;
  dumpContainer( "     list<myAlloc>:", m_double_list_myalloc);
  dumpContainer(" Aggr: ", m_agg_list_myalloc);
  dumpContainer(" Aggr*:", m_agg_listP_myalloc);
  std::cout << std::endl;

#endif // __use_unkeyed
#endif
}
