#include "pool/Ref.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "ExampleClass2.h"

namespace pool { const std::string typeName(const std::type_info& typ); }
static int s_printoutMod = 100;

template <class T> static void dump_object(pool::Ref<T>& r) {}
namespace pool  {
  template < class T>
  class ATLASAny : public TestBase {
  public: 

    /// Standard Constructor
    ATLASAny() : TestBase(ROOTKEY_StorageType.type(), "test_Any_1.root", "test_Any_2.root") 
    { 
      m_innerLoops = 400; 
    }

    /// Standard Destructor
    virtual ~ATLASAny()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      TransactionH trH1 = 0;

      Guid g1(getGuid(pool::typeName(typeid(T))));
      Placement place1(m_dbName1,"test_cont_1", g1, ROOTKEY_StorageType.type());

      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);

      sc = cache()->startTransaction(dbcH1, trH1);
      checkError(sc,"Cannot start database transactions (1)");
      for ( size_t i = 0; i < m_innerLoops; ++i )   {
        char txt[256];
        ::sprintf(txt,"Hello World [(%d)]", i);
        pool::Ref<ExampleClass2 > r(new ExampleClass2());
        r->s = txt;
        sc = cache()->registerForWrite(trH1, place1, r.ptr());
        if ( !sc.isSuccess() )   {
          checkError(sc,"Error cache.registerForWrite(trH1, place1, r) [1]");
        }
      }
      std::cout << "Commit transaction..." << std::endl;
      sc = cache()->commit(trH1);
      checkError(sc,"Error cache.commit(...) [1]");
      dumpStreamerInfo("ExampleClass2");
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->clear();
      checkError(sc,"Error cache.clear(...)");
      std::cout << "Diconnect from database accesses..." << std::endl;
      //sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      Guid g1(getGuid(pool::typeName(typeid(T))));
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok1 = new Token();
        tok1->setDb(m_dbName1);
        tok1->setCont("test_cont_1");
        tok1->setTechnology(ROOTKEY_StorageType.type());
        tok1->setClassID(g1);
        tok1->oid().first  = poolDb::INVALID;
        tok1->oid().second = i;

        if ( i % s_printoutMod == 0 ) {
          std::cout << "======================= New round....=====================" << std::endl;
        }

        pool::Ref<T> r(cache(), tok1);
        if ( r && i%s_printoutMod == 0 )   {
          dump_object(r);
        }
//        cache()->clear();
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}
ExampleClass2::ExampleClass2(int) {}
static void dump_object(pool::Ref<ExampleClass2>& r) {
  std::cout << "ATLAS Object works:" 
            << typeid(*r.ptr()).name()
            << " Ptr:" << (void*)r.ptr()
            << r->s << "  " << r->a << " " << r->b << " " << r->c
            << std::endl;
}

pool::TestRunner<pool::ATLASAny <ExampleClass2> > s_testATLASAny("ATLASAny");
