#include "pool/TestBase.h"
#include "ObjyMemoryFile.h"
#include "Test0.h"
#include "TNamed.h"

#include "TROOT.h"
#include "TClass.h"
#include "TStreamerInfo.h"
#include "TStreamerElement.h"
#include "StorageSvc/DbType.h"

static int s_printoutMod = 100;

namespace pool  {

  class CMSTypes : public TestBase {
  public: 

    /// Standard Constructor
    CMSTypes() : TestBase(ROOTKEY_StorageType.type(), "test_CMS_1.root", "test_CMS_2.root") 
    {   }

    /// Standard Destructor
    virtual ~CMSTypes()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      TransactionH trH1 = 0, trH2 = 0;

      Placement place1(m_dbName1,"test_cont_1", getGuid("ObjyMemoryFile"), ROOTKEY_StorageType.type());
      Placement place2(m_dbName1,"test_cont_2", getGuid("TNamed"), ROOTKEY_StorageType.type());
      Placement place3(m_dbName1,"test_cont_3", getGuid("TestO::C"), ROOTKEY_StorageType.type());
      Placement place4(m_dbName1,"test_cont_4", getGuid("TestO::E<TestO::F>"), ROOTKEY_StorageType.type());

      sc = unlink(m_dbName1.c_str())==0 ? DbStatus::SUCCESS : DbStatus::ERROR;
      checkError(sc,"Cannot delete file: "+m_dbName1);

      sc = cache()->connect(m_dbName1, poolDb::RECREATE|poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);

      for ( size_t j = 0; j < m_outerLoops; ++j)  {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          char txt1[256], txt2[256];
          ::sprintf(txt1,"Hello [TNamed(%d)]", i);
          ::sprintf(txt2,"World [TNamed(%d)]", i);
          pool::Ref<ObjyMemoryFile > refFile(new ObjyMemoryFile());
          sc = cache()->registerForWrite(trH1, place1, refFile.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place1, refFile) [1]");
          }
          pool::Ref<TNamed> refNamed(new TNamed(txt1, txt2));
          sc = cache()->registerForWrite(trH1, place2, refNamed.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place2, TNamed) [1]");
          }
          refFile->GetNamesInScope()["Named1"] = refFile;
          refFile->GetScopesContainingName().insert(std::make_pair("Named1",refFile.ptr()));

          pool::Ref<TestO::C> refC(new TestO::C());
          refC->i = i;
          refC->jj = 10* i;
          sc = cache()->registerForWrite(trH1, place3, refC.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place2, TNamed) [1]");
          }

          pool::Ref<TestO::E<TestO::F> > refE(new TestO::E<TestO::F>());
          for (int m=0; m<10; ++m)  {
            TestO::F a;
            a.i=m+i*10;
            refE->push_back(a);
          }
          refC->i = i;
          refC->jj = 10* i;
          sc = cache()->registerForWrite(trH1, place4, refE.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place2, TNamed) [1]");
          }
        }
        std::cout << "Commit transaction..." << std::endl;
        sc = cache()->commit(trH1);

        int offset = 0;
        TClass* cl=gROOT->GetClass("TestO::E<TestO::F>");
        TStreamerInfo* info = cl->GetStreamerInfo();
        TStreamerElement* elt1 = info->GetStreamerElement("D", offset);
        TStreamerElement* elt2 = info->GetStreamerElement("vector<TestO::F>", offset);

        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Diconnect from database accesses..." << std::endl;
        sc = cache()->disconnect(0);
        checkError(sc,"Error cache.disconnect(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
        sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
        checkError(sc,"Cannot connect to database "+m_dbName1);
      }
      sc = cache()->disconnect(0);
      dumpStreamerInfo("ObjyMemoryFile");
      dumpStreamerInfo("ObjyFile");
      dumpStreamerInfo("ooRunObj");
      dumpStreamerInfo("ooScopeObj");
      dumpStreamerInfo("ooObj");
      dumpStreamerInfo("TestO::A");
      dumpStreamerInfo("TestO::B");
      dumpStreamerInfo("TestO::C");
      dumpStreamerInfo("TestO::E<TestO::F>");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;

      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok1 = new Token();
        tok1->setDb(m_dbName1);
        tok1->setCont("test_cont_1");
        tok1->setTechnology(ROOTKEY_StorageType.type());
        tok1->setClassID(getGuid("ObjyMemoryFile"));
        tok1->oid().first  = 3; // Need exact token matching for self reference!
        tok1->oid().second = i;

        Token *tok3 = new Token();
        tok3->setDb(m_dbName1);
        tok3->setCont("test_cont_3");
        tok3->setTechnology(ROOTKEY_StorageType.type());
        tok3->setClassID(getGuid("TestO::C"));
        tok3->oid().first  = 5;
        tok3->oid().second = i;

        Token *tok4 = new Token();
        tok4->setDb(m_dbName1);
        tok4->setCont("test_cont_4");
        tok4->setTechnology(ROOTKEY_StorageType.type());
        tok4->setClassID(getGuid("TestO::E<TestO::F>"));
        tok4->oid().first  = 6;
        tok4->oid().second = i;

        if ( i % s_printoutMod == 0 ) {
          std::cout << "======================= New round....=====================" << std::endl;
        }
        pool::Ref<ooScopeObj> refFile(cache(), tok1);
        if ( refFile && i%s_printoutMod == 0 )   {
          typedef std::multimap<std::string, pool::Ref<ooScopeObj> > multimap_t;
          typedef std::map<std::string, pool::Ref<ooScopeObj> > map_t;
          multimap_t& mm = refFile->GetScopesContainingName();
          map_t& m = refFile->GetNamesInScope();
          std::cout << "CMS ObjyMemoryFile works:" 
                    << typeid(*refFile.ptr()).name()
                    << " Ptr:" << (void*)refFile.ptr()
                    << std::endl;
          for(map_t::iterator j=m.begin(); j != m.end(); ++j)  {
            std::cout << "m_t: " << (*j).first << " " << (*j).second.ptr() << std::endl;
          }
          for(multimap_t::iterator k=mm.begin(); k != mm.end(); ++k)  {
            std::cout << "mm_t:" << (*k).first << " " << (*k).second.ptr() << std::endl;
          }
        }
        pool::Ref<TestO::B> refC(cache(), tok3);
        if ( refC && i%s_printoutMod == 0 )   {
          std::cout << "CMS TestO::C works:" 
            << refC->i << " " << refC->j() << std::endl;
        }
        pool::Ref<TestO::E<TestO::F> > refE(cache(), tok4);
        if ( refE && i%s_printoutMod == 0 )   {
          std::cout << "CMS TestO::E works:" 
            << refE->i << " " << refE->size() << " Elts:";
          for (TestO::E<TestO::F>::const_iterator jj=refE->begin(); jj!=refE->end(); ++jj)  {
            std::cout << " " << (*jj).i;
          }
          std::cout << std::endl;
        }
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus update1()        {
      DbStatus sc;
      Guid g1(getGuid("ObjyMemoryFile"));
      ConnectionH  dbcH1 = 0;
      int refC_count = 0;

      Placement place1(m_dbName1,"test_cont_1", g1, ROOTKEY_StorageType.type());
      for (size_t i = 0; i < m_innerLoops; ++i )   {
        TransactionH trH1 = 0;
        sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
        checkError(sc,"Cannot connect to database "+m_dbName1);
        Token *tok1 = new Token();
        tok1->setDb(m_dbName1);
        tok1->setCont("test_cont_1");
        tok1->setTechnology(ROOTKEY_StorageType.type());
        tok1->setClassID(g1);
        tok1->oid().first  = 3; // Need exact token matching for self reference!
        tok1->oid().second = i;
        tok1->addRef();

        if ( i % s_printoutMod == 0 ) {
          std::cout << "======================= New round....=====================" << std::endl;
        }
        char txt[32];
        sprintf(txt,"Nam_%d",i);
        pool::Ref<ooScopeObj> refFile(cache(), tok1);

        refFile->GetNamesInScope()[txt] = refFile;
        refFile->GetScopesContainingName().insert(std::make_pair(txt,refFile.ptr()));

        sc = cache()->disconnect(0);
        sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
        sc = cache()->startTransaction(dbcH1, trH1);
        sc = cache()->update(trH1, refFile.ptr());
        if ( !sc.isSuccess() )   {
          checkError(sc,"Error cache.update(trH1, refFile)");
        }
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(trH1)");

        if ( refFile && i%s_printoutMod == 0 )   {
          typedef std::multimap<std::string, pool::Ref<ooScopeObj> > multimap_t;
          typedef std::map<std::string, pool::Ref<ooScopeObj> > map_t;
          multimap_t& mm = refFile->GetScopesContainingName();
          map_t& m = refFile->GetNamesInScope();
          std::cout << "CMS ObjyMemoryFile works:" 
                    << typeid(*refFile.ptr()).name()
                    << " Ptr:" << (void*)refFile.ptr()
                    << std::endl;
          for(map_t::iterator j=m.begin(); j != m.end(); ++j)  {
            std::cout << "m_t: " << (*j).first << " " << (*j).second.ptr() << std::endl;
          }
          for(multimap_t::iterator k=mm.begin(); k != mm.end(); ++k)  {
            std::cout << "mm_t:" << (*k).first << " " << (*k).second.ptr() << std::endl;
          }
        }
        //setDumpLinkTables(DbLog::VERBOSE);
        sc = cache()->disconnect(0);
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus update2()        {
      DbStatus sc;
      Guid g1(getGuid("ObjyMemoryFile"));
      ConnectionH  dbcH1 = 0;
      int refC_count = 0;

      Placement place1(m_dbName1,"test_cont_1", g1, ROOTKEY_StorageType.type());
      TransactionH trH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      Token *tok1 = new Token();
      tok1->setDb(m_dbName1);
      tok1->setCont("test_cont_1");
      tok1->setTechnology(ROOTKEY_StorageType.type());
      tok1->setClassID(g1);
      tok1->oid().first  = 3; // Need exact token matching for self reference!
      tok1->oid().second = s_printoutMod;
      pool::Ref<ooScopeObj> refFile(cache(), tok1);
      if ( refFile ) {}
      for (size_t i = 0; i < m_innerLoops; ++i )   {
        if ( i % s_printoutMod == 0 ) {
          std::cout << "======================= New round....=====================" << std::endl;
        }
        char txt[32];
        sprintf(txt,"Nam_%d",i);
        typedef std::multimap<std::string, pool::Ref<ooScopeObj> > multimap_t;
        typedef std::map<std::string, pool::Ref<ooScopeObj> > map_t;
        multimap_t& mm = refFile->GetScopesContainingName();
        map_t& m = refFile->GetNamesInScope();

        m[txt] = refFile;
        mm.insert(std::make_pair(txt,refFile));

        sc = cache()->startTransaction(dbcH1, trH1);
        sc = cache()->update(trH1, refFile.ptr());
        if ( !sc.isSuccess() )   {
          checkError(sc,"Error cache.update(trH1, refFile)");
        }
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(trH1)");

        if ( refFile && i%s_printoutMod == 0 )   {
#if 0
          std::cout << "CMS ObjyMemoryFile works:" 
                    << typeid(*refFile.ptr()).name()
                    << " Ptr:" << (void*)refFile.ptr()
                    << std::endl;
          for(map_t::iterator j=m.begin(); j != m.end(); ++j)  {
            std::cout << "m_t: " << (*j).first << " " << (*j).second.ptr() << std::endl;
          }
          for(multimap_t::iterator k=mm.begin(); k != mm.end(); ++k)  {
            std::cout << "mm_t:" << (*k).first << " " << (*k).second.ptr() << std::endl;
          }
        }
        else  {
#endif
          for(map_t::iterator j=m.begin(); j != m.end(); ++j)  {
            if ( (*j).first.empty() || (*j).second.ptr() == 0 )  {
              std::cout << "> Somethings wrong with map m_t!" << std::endl;
            }
          }
          for(multimap_t::iterator k=mm.begin(); k != mm.end(); ++k)  {
            if ( (*k).first.empty() || (*k).second.ptr() == 0 )  {
              std::cout << "> Somethings wrong with map m_tt!" << std::endl;
            }
          }
        }
        //setDumpLinkTables(DbLog::VERBOSE);
      }
      std::cout << " " << refFile->GetNamesInScope().size() << std::endl;
      std::cout << " " << refFile->GetScopesContainingName().size() << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      std::cout << "Clear cache..." << std::endl;
      cache()->clear();
      return sc;
    }
    DbStatus update()        {
      return update2();
    }
  };
}

pool::TestRunner<pool::CMSTypes> s_testCMSTypes("CMSTypes");
