#include "pool/Ref.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"
#include "StringContainer.h"

static int s_printoutMod = 100;

namespace pool_tests {
  StringContainer::StringContainer()  {
  }
  StringContainer::~StringContainer()  {
  }
  void StringContainer::set(int i)  {
    char txt[32];
    sprintf(txt,"Hello_%d", i);
    m_string = txt;
    for (int j = 0; j < 5; ++j )  {
      sprintf(txt,"Hello_%d", i*100+j);
      m_stringList.push_back(txt);
      m_stringVec.push_back(txt);
    }
  }
  void StringContainer::dump()  const {
    std::cout << " m_string=" << m_string;
    std::cout << std::endl << " m_stringList=";
    for (std::list<std::string>::const_iterator i=m_stringList.begin(); i != m_stringList.end(); ++i)
      std::cout << (*i) << " ";
    std::cout << std::endl << " m_stringVec=";
    for (std::vector<std::string>::const_iterator k=m_stringVec.begin(); k != m_stringVec.end(); ++k)
      std::cout << (*k) << " ";
    std::cout << std::endl;
  }
}

namespace pool  {

  class TestStrings : public TestBase {
  public: 

    /// Standard Constructor
    TestStrings() : TestBase(ROOTKEY_StorageType.type(), "test_string_1.root", "") 
    {
      m_innerLoops = 1; m_outerLoops = 400; 
    }

    /// Standard Destructor
    virtual ~TestStrings()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      return write_objects(dbcH1);
    }

    /// write objects to the persistent store
    DbStatus update()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      return write_objects(dbcH1);
    }

    DbStatus write_objects(ConnectionH dbcH1)
    {
      DbStatus sc;
      TransactionH trH1 = 0;
      Placement place1(m_dbName1,"test_cont_1", getGuid("pool_tests::StringContainer"), m_type);
      std::cout << "Number of OUTER loops:" << m_innerLoops << std::endl;
      std::cout << "Number of INNER loops:" << m_outerLoops << std::endl;

      for ( size_t j = 0; j < m_innerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_outerLoops; ++i )   {
          pool::Ref<pool_tests::StringContainer> ref(new pool_tests::StringContainer());
          ref->set(i);

          sc = cache()->registerForWrite(trH1, place1, ref.ptr());
          if ( !sc.isSuccess() )   {
            checkError(sc,"Error cache.registerForWrite(trH1, place1, TNamed) [1]");
          }
        }
        std::cout << "Commit transaction..." << std::endl;
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      dumpStreamerInfo("pool_tests::StringContainer");
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      for (size_t i = 0; i < m_outerLoops; ++i )   {
        Token *tok2 = new Token();
        tok2->setDb(m_dbName1);
        tok2->setCont("test_cont_1");
        tok2->setTechnology(m_type);
        tok2->setClassID(getGuid("pool_tests::StringContainer"));
        tok2->oid().first  = poolDb::INVALID;
        tok2->oid().second = i;

        if ( i % s_printoutMod == 0 ) {
          std::cout << "======================= New round....=====================" << std::endl;
        }
        // Read first object
        pool::Ref<pool_tests::StringContainer> ref(cache(), tok2);
        if ( ref && i%s_printoutMod == 0 )   {
          ref->dump();
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "test MISSING";
        }
        cache()->clear();
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::TestStrings> s_testDataTypes("Strings");
