#include "pool/test.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"

static int s_printoutMod = 100;
namespace pool  {

  class FileUpdate : public TestBase {
  public: 

    /// Standard Constructor
    FileUpdate() : TestBase(ROOTKEY_StorageType.type(),"file_update1.root","file_update2.root") 
    {  m_innerLoops = 400;  }

    /// Standard Destructor
    virtual ~FileUpdate()               {    }
    
    DbStatus update()  {
      DbStatus sc;
      TransactionH trH1 = 0;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      Placement place1(m_dbName1,"cont_1", getGuid("test"),   m_type);
      Placement place2(m_dbName2,"cont_2", getGuid("TNamed"), m_type);
      sc = cache()->connect(m_dbName1, poolDb::UPDATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      sc = cache()->connect(m_dbName2, poolDb::READ, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2);
      sc = cache()->startTransaction(dbcH1, trH1);
      checkError(sc,"Cannot start database Transactions (1)");
      for(size_t i = 0; i < m_innerLoops; ++i )   {
        pool::Ref<test>   refObj(new test);
        Token *tok3 = new Token();
        tok3->setDb(m_dbName2);
        tok3->setCont("cont_2");
        tok3->setTechnology(m_type);
        tok3->setClassID(getGuid("TNamed"));
        tok3->oid().first  = poolDb::INVALID;
        tok3->oid().second = (i%m_innerLoops)*3;
        pool::Ref<TNamed> refNamed(cache(), tok3);
        if ( refNamed )  {
          refObj->setTNamed1(refNamed); // Assign object pointer to reference
        }
        refObj->setTNamed1(refNamed); // Assign object pointer to reference
        refObj->fillData(i+m_innerLoops);
        sc = cache()->registerForWrite(trH1, place1, refObj.ptr());
        if ( !sc.isSuccess() )   {
          checkError(sc,"Error cache.registerForWrite(trH1, place1, refObj) [1]");
        }
      }
      std::cout << "Commit Transaction..." << std::endl;
      sc = cache()->commit(trH1);
      checkError(sc,"Error cache.commit(...) [1]");
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->clear();
      checkError(sc,"Error cache.clear(...)");
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      sc = cache()->disconnect(dbcH2);
      checkError(sc,"Error cache.disconnect(...) [2]");
      return sc;
    }

    /// write objects to the persistent store
    DbStatus write()  {
      DbStatus sc;
      TransactionH trH1 = 0, trH2 = 0;
      ConnectionH  dbcH1 = 0, dbcH2 = 0;
      Placement place1(m_dbName1,"cont_1", getGuid("test"),   m_type);
      Placement place2(m_dbName2,"cont_2", getGuid("TNamed"), m_type);
      sc = unlink(m_dbName1.c_str())==0 ? DbStatus::Success : DbStatus::Error;
      checkError(sc,"Cannot delete file: "+m_dbName1);
      sc = unlink(m_dbName2.c_str())==0 ? DbStatus::Success : DbStatus::Error;
      checkError(sc,"Cannot delete file: "+m_dbName2);
      sc = cache()->connect(m_dbName1, poolDb::CREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      sc = cache()->connect(m_dbName2, poolDb::CREATE, dbcH2);
      checkError(sc,"Cannot connect to database "+m_dbName2);
      sc = cache()->startTransaction(dbcH2, trH2);
      checkError(sc,"Cannot start database transactions (2)");
      sc = cache()->startTransaction(dbcH1, trH1);
      checkError(sc,"Cannot start database Transactions (1)");
      for ( size_t i = 0; i < m_innerLoops; ++i )   {
        pool::Ref<test>   refObj(new test);
        char txt1[256], txt2[256];
        ::sprintf(txt1,"Hello [TNamed(%d)]", int(i));
        ::sprintf(txt2,"World [TNamed(%d)]", int(i));
        pool::Ref<TNamed> refNamed(new TNamed(txt1, txt2));
        refObj->setTNamed1(refNamed);    // Assign object pointer to reference
        sc = cache()->registerForWrite(trH2, place2, refNamed.ptr());
        if ( !sc.isSuccess() )   {
          checkError(sc,"Error cache.registerForWrite(trH2, place2, refNam) [2]");
        }
        refObj->fillData(i);
        sc = cache()->registerForWrite(trH1, place1, refObj.ptr());
        if ( !sc.isSuccess() )   {
          checkError(sc,"Error cache.registerForWrite(trH1, place1, refObj) [1]");
        }
      }
      std::cout << "Commit Transaction..." << std::endl;
      sc = cache()->commit(trH1);
      checkError(sc,"Error cache.commit(...) [1]");
      sc = cache()->commit(trH2);
      checkError(sc,"Error cache.commit(...) [2]");
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->clear();
      checkError(sc,"Error cache.clear(...)");
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      sc = cache()->disconnect(dbcH2);
      checkError(sc,"Error cache.disconnect(...) [2]");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()  {
      for ( int i = 0; ; ++i )   {
        Token *tok2 = new Token();
        tok2->setDb(m_dbName1);
        tok2->setCont("cont_1");
        tok2->setTechnology(m_type);
        tok2->setClassID(getGuid("test"));
        tok2->oid().first  = 1;
        tok2->oid().second = i;
        // Read first object
        pool::Ref<test> refObj(cache(), tok2);
        if ( !refObj )    {
          std::cout << "======================= End....=====================" << std::endl;
          std::cout << "=== Cannot read any more objects...."                 << std::endl;
          std::cout << "=== End of reading after " << i << " Objects."        << std::endl;
          cache()->clear();
          break;
        }
        if ( refObj && i%s_printoutMod == 0 )   {
          std::cout << "======================= New round....=====================" << std::endl;
          refObj->printData(i,true);
        }
        cache()->clear();
      }
      std::cout << "Clear cache..." << std::endl;
      DbStatus sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::FileUpdate> s_testFileUpdate("FileUpdate");
