#include <iomanip>
#include "pool/test.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbInstanceCount.h"

static int s_printoutMod = 100;

template <typename T> inline 
std::ostream& operator<< (std::ostream& os, const std::vector<T>& cnt) {
  typedef typename std::vector<T>::const_iterator IT;
  for (IT i = cnt.begin(); i!=cnt.end(); ++i)  {
    std::cout << (*i);
  }
  return os;
}
template <typename T, typename Q> inline
std::ostream& operator <<(std::ostream& os, const std::pair<T,Q>& p) 
{  return os << " " << p.first << "," << p.second; }

using namespace poolDb;
namespace pool  {

  struct Vectors : public TestBase {

    typedef std::vector<std::pair<float, float> > _T1;
    typedef std::vector<std::vector<char> >       _T2;
    typedef std::vector<NoVirtual>                _T3;
    typedef std::vector<Aggregated*>              _T4;
    std::string m_t1_type;
    std::string m_t2_type;
    std::string m_t3_type;
    std::string m_t4_type;
    std::string m_t5_type;
    std::string m_t6_type;
    std::string m_t7_type;
    std::string m_t8_type;
    std::string m_t9_type;

    /// Standard Constructor
    Vectors() : TestBase(m_type, "vectors_1.root", "vectors_2.root")  {
      m_t1_type = "std::vector<std::pair<float,float> >";
      m_t2_type = "std::vector<std::vector<char> >";
      m_t3_type = "std::vector<NoVirtual>";
      m_t4_type = "std::vector<Aggregated*>";
      m_t5_type = "DumbStruct<std::vector<std::pair<float,float> > >";
      m_t6_type = "DumbStruct<std::vector<std::vector<char> > >";
      m_t7_type = "DumbStruct<std::vector<NoVirtual> >";
      m_t8_type = "DumbStruct<std::vector<Aggregated*> >";
      m_t9_type = m_t4_type;
    }

    /// Standard Destructor
    virtual ~Vectors()    {    }

    void checkObject(const DumbStruct<_T1>* q, const _T1* p, size_t i)  {
      const _T1* r = q ? &q->i : p;
      if ( r )  {
        for ( size_t k = 0; k < i; ++k )  {
          const _T1& p1 = *(r);
          if ( p1[k].first != float(k*10) || p1[k].second != float(k)  )  {
            if ( q ) std::cout << "DumbStruct: " << m_t1_type;
            std::cout << "Bad data:" << p1[k].first << "," << p1[k].second << std::endl;
            checkError(DbStatus::Error,"DumbStruct: "+m_t1_type+": Read __BAD__ data", true);
          }
        }
        if ( i < 10 )  {
          if ( q ) std::cout << "DumbStruct: ";
          std::cout << m_t1_type << ": " << std::setw(2) << i << " :" << *(r) << std::endl;
        }
        else if ( i % s_printoutMod == 0 ) {
          if ( q ) std::cout << "DumbStruct: ";
          std::cout << m_t1_type << ": " << i << " All OK." << std::endl;
        }
      }
      else  {
        if ( q ) std::cout << "DumbStruct: ";
        std::cout << m_t1_type << " Missing!" << std::endl;
        checkError(DbStatus::Error,m_t1_type+": Cannot read data", true);
      }
    }
    void checkObject(const DumbStruct<_T2>* q, const _T2* p, size_t i)  {
      const _T2* r = q ? &q->i : p;
      if ( r )  {
        std::vector<char> buff;
        for ( size_t k = 0; k < i; ++k )  {
          const _T2& p1 = *(r);
          buff.push_back((k%('A'-'a'))+'a');
          if ( p1[k] != buff )  {
            if ( q ) std::cout << "DumbStruct: ";
            std::cout << "Bad data: (" << m_t2_type << ") " << p1[k] << std::endl;
            checkError(DbStatus::Error,m_t2_type+"Read __BAD__ data", true);
          }
        }
        if ( i < 10 )  {
          if ( q ) std::cout << "DumbStruct: ";
          std::cout << m_t2_type << ": " << std::setw(2) << i << " : " << *(r) << std::endl;
        }
        else if ( i % s_printoutMod == 0 ) {
          if ( q ) std::cout << "DumbStruct: ";
          std::cout << m_t2_type << ": " << i << " All OK." << std::endl;
        }
      }
      else  {
        if ( q ) std::cout << "DumbStruct: ";
        std::cout << m_t2_type << " Missing!" << std::endl;
        checkError(DbStatus::Error,"Cannot read data", true);
      }
    }

    
    void checkObject(const DumbStruct<_T3>* q, const _T3* p, size_t i)  {
      const _T3* r = q ? &q->i : p;
      if ( r )  {
        if ( i < 10 )  {
          if ( q ) std::cout << "DumbStruct: ";
          std::cout << m_t3_type << "[" << i << "," << r->size() << "]:";
          for (size_t k=0; k<i; ++k )  {
            std::cout << "  " << r->at(k).m_data1;
          }
          std::cout << std::endl;
        }
        for (size_t k=0; k<i; ++k )  {
          if ( size_t(r->at(k).m_data1) != k ) {
            std::cout << "Bad data: (" << m_t3_type << ") " << r->at(k).m_data1 << std::endl;
            checkError(DbStatus::Error,"Read __BAD__ data", true);
          }
        }
      }
      if ( r && i%s_printoutMod == 0 && i > 0 )   {
        if ( q ) std::cout << "DumbStruct: ";
        std::cout << m_t3_type << ": r valid[" << i << "," << r->size() << "]" << std::endl;
      }
      else if ( !r )   {
        if ( q ) std::cout << "DumbStruct: ";
        checkError(DbStatus::Error,m_t3_type+": MISSING", true);
      }
    }
    void checkObject(const DumbStruct<_T4>* q, const _T4* p, size_t i)  {
      const _T4* r = q ? &q->i : p;
      if ( i < 10 )  {
        if ( q ) std::cout << "DumbStruct: ";
        std::cout << m_t4_type << "[" << i << "," << r->size() << "]:";
        for (size_t k=0; k<i; ++k )  {
          std::cout << "  " << r->at(k)->data1 << "," << r->at(k)->data2;
        }
        std::cout << std::endl;
      }
      if ( r && i%s_printoutMod == 0 && i > 0 )   {
        if ( q ) std::cout << "DumbStruct: ";
        std::cout << m_t4_type << ": r valid[" << i << "," << r->size() << "]:" << std::endl;
      }
      else if ( !r )   {
        checkError(DbStatus::Error,m_t4_type+": MISSING", true);
      }
    }

    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      TransactionH trH1 = 0;
      ConnectionH  dbcH1 = 0;

      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      Placement place1(m_dbName1,"test_cont_1", getGuid(m_t1_type), m_type);
      Placement place2(m_dbName1,"test_cont_2", getGuid(m_t2_type), m_type);
      Placement place3(m_dbName1,"test_cont_3", getGuid(m_t3_type), m_type);
      Placement place4(m_dbName1,"test_cont_4", getGuid(m_t4_type), m_type);
      Placement place5(m_dbName1,"test_cont_5", getGuid(m_t5_type), m_type);
      Placement place6(m_dbName1,"test_cont_6", getGuid(m_t6_type), m_type);
      Placement place7(m_dbName1,"test_cont_7", getGuid(m_t7_type), m_type);
      Placement place8(m_dbName1,"test_cont_8", getGuid(m_t8_type), m_type);
      Placement place9(m_dbName1,"test_cont_9", getGuid(m_t9_type), m_type);

      for ( size_t j = 0; j < m_outerLoops; ++j )   {
        Aggregated* pagg;
        _T4 v_agg;
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          Ref<_T1>              r1(new _T1());
          Ref<_T2>              r2(new _T2());
          Ref<_T3>              r3(new _T3());
          Ref<_T4 >             r4(new _T4());
          Ref<DumbStruct<_T1> > r5(new DumbStruct<_T1>());
          Ref<DumbStruct<_T2> > r6(new DumbStruct<_T2>());
          Ref<DumbStruct<_T3> > r7(new DumbStruct<_T3>());
          Ref<DumbStruct<_T4> > r8(new DumbStruct<_T4>());
          Ref<_T4 >             r9(new _T4()); // This will be an empty vector
          NoVirtual nv;
          std::vector<char> buff;
          for ( size_t k = 0; k < i; ++k )  {
            buff.push_back((k%('A'-'a'))+'a');
            r1->push_back(_T1::value_type(float(k*10), float(k)));
            r2->push_back(buff);
            nv.m_data1 = k;
            r3->push_back(nv);
            switch(k%4) {
              case 0:
                r4->push_back(pagg=new Aggregated(j,k));
                break;
              case 1:
                r4->push_back(pagg=new SubAggregated(j, k, 1000*k));
                break;
              case 2:
                r4->push_back(pagg=new PtrAggregated(j, k, 1000*k));
                break;
              case 3:
                r4->push_back(pagg=new VirtualAggregated(j, k, 1000*k));
                break;
              default:
                break;
            }
            if ( pagg ) v_agg.push_back(pagg);
          }
          (*(r5.ptr())).i = *(r1.ptr());
          (*(r6.ptr())).i = *(r2.ptr());
          (*(r7.ptr())).i = *(r3.ptr());
          (*(r8.ptr())).i = *(r4.ptr());
          _CHECK( cache()->registerForWrite(trH1, place1, r1.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place2, r2.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place3, r3.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place4, r4.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place5, r5.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place6, r6.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place7, r7.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place8, r8.ptr()) );
          _CHECK( cache()->registerForWrite(trH1, place9, r9.ptr()) );
        }
        std::cout << "Commit transaction..." << std::endl;
        _CHECK( cache()->commit(trH1) );
        for( _T4::iterator va=v_agg.begin(); va != v_agg.end(); ++va )  {
          delete (*va);
        }
        std::cout << "Clear cache..." << std::endl;
        _CHECK( cache()->clear() );
      }
      std::cout << "Diconnect from database accesses..." << std::endl;
      _CHECK( cache()->disconnect(dbcH1) );
      return Success;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      size_t i;
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_1",m_type,m_t1_type,3,i);
        checkObject(0, Ref<_T1 >(cache(), t).ptr(), i);
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_2",m_type,m_t2_type,4,i);
        checkObject(0, Ref<_T2 >(cache(), t).ptr(), i);
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_3",m_type,m_t3_type,5,i);
        checkObject(0, Ref<_T3 >(cache(), t).ptr(), i);
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_4",m_type,m_t4_type,6,i);
        Ref<_T4 >  r(cache(), t);
        checkObject(0, r.ptr(), i);
        for( _T4::iterator va=r->begin(); va != r->end(); ++va )  {
          delete (*va);
        }
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_5",m_type,m_t5_type,7,i);
        checkObject(Ref<DumbStruct<_T1> >(cache(), t).ptr(), 0, i);
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_6",m_type,m_t6_type,8,i);
        checkObject(Ref<DumbStruct<_T2> >(cache(), t).ptr(), 0, i);
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_7",m_type,m_t7_type,9,i);
        checkObject(Ref<DumbStruct<_T3> >(cache(), t).ptr(), 0, i);
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_8",m_type,m_t8_type,10,i);
        Ref<DumbStruct<_T4> >  r(cache(), t);
        checkObject(r.ptr(), 0, i);
        for( _T4::iterator va=r->i.begin(); va != r->i.end(); ++va )  {
          delete (*va);
        }
      }
      for (i = 0; i < m_innerLoops; ++i )   {
        Token *t = makeToken(m_dbName1,"test_cont_9",m_type,m_t9_type,11,i);
        checkObject(0, Ref<_T4 >(cache(), t).ptr(), 10);
      }
      std::cout << "Clear cache..." << std::endl;
      cache()->clear();
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::Vectors> s_testVectors("Vectors");
