#include "pool/Ref.h"
#include "pool/simple.h"
#include "pool/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/DbInstanceCount.h"

static int s_printoutMod = 100;

namespace pool  {

  using namespace simple;

  class SimpleTest : public TestBase {
  public: 
    /// Standard Constructor
    SimpleTest() : TestBase(ROOTKEY_StorageType.type(), "simple_1.root", "simple_2.root")  { }

    /// Standard Destructor
    virtual ~SimpleTest()    {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
      DbStatus sc;
      ConnectionH  dbcH1 = 0;
      sc = unlink(m_dbName1.c_str())==0 ? DbStatus::Success : DbStatus::Error;
      checkError(sc,"Cannot delete file: "+m_dbName1);
      sc = cache()->connect(m_dbName1, poolDb::RECREATE, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);
      TransactionH trH1 = 0;
      Placement place2(m_dbName1,"test_cont", getGuid("simple::test"), m_type);
      for ( size_t j = 0; j < m_outerLoops; ++j )   {
        sc = cache()->startTransaction(dbcH1, trH1);
        checkError(sc,"Cannot start database transactions (1)");
        for ( size_t i = 0; i < m_innerLoops; ++i )   {
          pool::Ref<test > r(new test());
          r->data1 = j;
          for ( size_t k = 0; k < j; ++k )  {
            r->data2.push_back(k*10);
          }
          sc = cache()->registerForWrite(trH1, place2, r.ptr());
          checkError(sc,"Error cache.registerForWrite(trH1, place2, simple::test) [6]");
        }
        sc = cache()->commit(trH1);
        checkError(sc,"Error cache.commit(...) [1]");
        std::cout << "Clear cache..." << std::endl;
        sc = cache()->clear();
        checkError(sc,"Error cache.clear(...)");
      }
      std::cout << "Diconnect from database accesses..." << std::endl;
      sc = cache()->disconnect(dbcH1);
      checkError(sc,"Error cache.disconnect(...) [1]");
      return sc;
    }
    /// read objects from the persistent store
    DbStatus read()        {
      DbStatus sc;
      // Guid g6(getGuid("EventHeader"));
      ConnectionH  dbcH1 = 0;
      sc = cache()->connect(m_dbName1, poolDb::READ, dbcH1);
      checkError(sc,"Cannot connect to database "+m_dbName1);

      for (size_t i = 0; i < m_innerLoops; ++i )   {
        Token *tok2 = new Token();

        // Since this example does not use collections,
        // we do it here by faking the tokens to be 
        // returned by the collections
        // 
        tok2->setDb(m_dbName1);
        tok2->setCont("test_cont");
        tok2->setTechnology(m_type);
        tok2->setClassID(getGuid("simple::test"));
        tok2->oid().first  = 3;
        tok2->oid().second = i;
        if ( i % s_printoutMod == 0 ) {
          std::cout << "======================= New round....=====================" << std::endl;
          cache()->setPrintout(1);
        }
        else {
          cache()->setPrintout(0);
        }
        pool::Ref<test> r(cache(), tok2);
        if ( r && i%s_printoutMod == 0 )   {
          std::cout << " r valid";
        }
        else if ( i%s_printoutMod == 0 )   {
          std::cout << "test MISSING";
        }
        if ( i%s_printoutMod == 0 )   {
          std::cout << std::endl;
        }
        cache()->clear();
      }
      cache()->clear();
      std::cout << "Clear cache..." << std::endl;
      sc = cache()->disconnect(0);
      checkError(sc,"cache.disconnect(0)");
      return sc;
    }
  };
}

pool::TestRunner<pool::SimpleTest> s_testSimpleTest("SimpleTest");
