#include "TROOT.h"
#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"
#include "TSystem.h"
#include <iostream>
#include "pool/TestBase.h"

/*

rootcint Vectors_dict.cpp -c Vectors.h Vectors_LinkDef.h
g++ Vectors_dict.cpp Vectors_objects.cpp -L${ROOTSYS}/lib -lCint -lCore -I${ROOTSYS}/include -ldl -fPIC -shared -o libEmb.so

cl.exe /c /DLL /MD /Z7 /GR /GX Vectors_dict.cpp Vectors_objects.cpp /I"%ROOTSYS%/include"
link.exe Vectors_dict.obj Vectors_objects.obj /libpath:"%ROOTSYS%\lib" libCint.lib libCore.lib /DLL -out:libEmb.dll
*/

namespace pool  {
  static int counter = 0;
  struct RootVectors : public TestBase {

    /// Standard Constructor
    RootVectors()    : TestBase(0, "", "")     {    }

    /// Standard Destructor
    virtual ~RootVectors()               {    }
    
    /// write objects to the persistent store
    DbStatus write()       {
//      gSystem->Load("test_dict.dll");
      TFile *f    = new TFile("RootVectors.root", "RECREATE", "Root RootVectors test");
      TTree *tree = new TTree("T","An example of a ROOT tree");
      std::vector<std::pair<float, float> >* obj = new std::vector<std::pair<float, float> >();
      TBranch *b = tree->Branch("B", "vector<pair<float,float>,allocator<pair<float,float> > >", &obj);
      for(size_t i=0; i < 10; ++i) {
        obj->push_back(std::vector<std::pair<float, float> >::value_type(10*i,i));
        printf("%d\n",tree->Fill());
        obj->clear();
      }
      b->Print();
      b->Write();
      tree->Print();
      tree->Write();
      f->Print();
      f->Close();
      delete f;
      return 1;
    }
    /// write objects to the persistent store
    DbStatus read()       {
      TFile *f    = new TFile("RootVectors.root");
      TTree *tree = (TTree*)f->Get("T");
      std::vector<std::pair<float, float> >* obj = new std::vector<std::pair<float, float> >();
      TBranch *branch  = tree->GetBranch("B");
      branch->SetAddress(&obj);
      Int_t nevent = Int_t(tree->GetEntries());
      Int_t nselected = 0;
      Int_t nb = 0;
      for (Int_t i=0;i<nevent;i++) { 
        printf("%d %d\n", tree->GetEvent(i), obj); 
        printf("%f, %d\n", (*obj)[0].first, (*obj)[0].second); 
      }
      return 1;
    }
  };
}

pool::TestRunner<pool::RootVectors> s_testRootVectors("RootVectors");
