#ifndef ATTRIBUTEEXCEPTION_H
#define ATTRIBUTEEXCEPTION_H

/// @file 
/// @author Jakub.Moscicki@cern.ch (kuba)

#include "AttributeTypes.h" // for typename_string_t
#include "POOLCore/Exception.h"

namespace pool {

  const size_t IDX_NOT_FOUND = size_t(-1); // FIXME:

  /// Action failed because attribute type is different than the type expected by the user
  class attribute_bad_type : public pool::Exception
  {
  public:
    /// Constructor  
    attribute_bad_type(std::string name, typename_string_t stored, typename_string_t attempted);
  };

  /// Action failed because attribute was not found in the attribute list or specification.
  class attribute_not_found : public pool::Exception
  {
  public:
    explicit attribute_not_found(std::string name);
  };

  /// Action failed because attribute with the same name already exists.
  class attribute_duplicate : public pool::Exception
  {
  public:
    explicit attribute_duplicate(std::string name);
  };

  /// Report any internal error.
  class attribute_internal_error : public pool::Exception
  {
  public:
    explicit attribute_internal_error(std::string msg);
  };
}


inline
pool::attribute_bad_type::attribute_bad_type(std::string name, typename_string_t stored, typename_string_t attempted) : 
  pool::Exception(std::string("Type error for attribute ")+name+". Type of attribute is "+stored+". Type attempted by user is "+attempted,
                  "pool::attribute_bad_type",
                  "AttributeList" )
{}


inline
pool::attribute_not_found::attribute_not_found(std::string name) : 
  pool::Exception(std::string("attribute <")+name+"> not found", 
                  "pool::attribute_not_found", 
                  "AttributeList" ) {}

inline 
pool::attribute_duplicate::attribute_duplicate(std::string name) : 
  pool::Exception(std::string("attribute <")+name+"> duplicate error", 
                  "pool::attribute_duplicate", 
                  "AttributeList" ) {}

inline
pool::attribute_internal_error::attribute_internal_error(std::string msg) : 
  pool::Exception(std::string("INTERNAL ERROR: ") + msg,
                  "pool::attribute_internal_error",
                  "AttributeList" ) {}

# endif

