# include "AttributeList/AttributeListSpecification.h"

# include <assert.h>

using namespace pool;

bool basic_spec()
{
  AttributeSpecification aint("a","int"), bint("b","int");
  AttributeSpecification astr("a","string");
  AttributeSpecification aint2("a","int");

  assert(aint != bint);
  assert(astr != aint);
  assert(aint == aint);
  assert(aint == aint2);

  return true;
}

// make sure order is not significant for attribute list specification
bool spec_list_order_ignored()
{
  std::vector< std::pair<std::string, pool::typename_string_t> > columns1;

  columns1.push_back( std::make_pair("A","int"));
  columns1.push_back( std::make_pair("B","int"));
  columns1.push_back( std::make_pair("X", "string"));

  std::vector< std::pair<std::string, pool::typename_string_t> > columns2;

  columns2.push_back( std::make_pair("B","int"));
  columns2.push_back( std::make_pair("A","int"));
  columns2.push_back( std::make_pair("X", "string"));

  pool::AttributeListSpecification spec1;

  for(unsigned int i=0; i<columns1.size(); ++i)
    spec1.push_back( columns1[i].first, columns1[i].second );

  pool::AttributeListSpecification spec2;

  for(unsigned int i=0; i<columns2.size(); ++i)
    spec2.push_back( columns2[i].first, columns2[i].second );

  assert(spec1 == spec2);

  return true;
}

bool spec_list_not_equal()
{
  std::vector< std::pair<std::string, pool::typename_string_t> > columns1;

  columns1.push_back( std::make_pair("A","int"));
  columns1.push_back( std::make_pair("B","int"));
  columns1.push_back( std::make_pair("X", "string"));

  std::vector< std::pair<std::string, pool::typename_string_t> > columns2;

  columns2.push_back( std::make_pair("B","int"));
  columns2.push_back( std::make_pair("A","string"));
  columns2.push_back( std::make_pair("X", "string"));

  pool::AttributeListSpecification spec1;

  for(unsigned int i=0; i<columns1.size(); ++i)
    spec1.push_back( columns1[i].first, columns1[i].second );

  pool::AttributeListSpecification spec2;

  for(unsigned int i=0; i<columns2.size(); ++i)
    spec2.push_back( columns2[i].first, columns2[i].second );

  assert(spec1 != spec2);

  return true;
}


bool spec_list_includes()
{

  pool::AttributeListSpecification abx;
  abx.push_back("A","int");
  abx.push_back("B","int");
  abx.push_back("X", "string");

  pool::AttributeListSpecification bax;
  bax.push_back("B","int");
  bax.push_back("A","int");
  bax.push_back("X", "string");

  pool::AttributeListSpecification ba;
  ba.push_back("B","int");
  ba.push_back("A","int");

  pool::AttributeListSpecification ba_string;
  ba_string.push_back("B","string");
  ba_string.push_back("A","string");

  assert(abx.includes(bax));
  assert(abx.includes(ba));
  assert(!ba.includes(abx));
  assert(!ba.includes(bax));

  assert(!abx.includes(ba_string));
  assert(!ba_string.includes(ba));
  assert(!ba.includes(ba_string));

  return true;
}
