# include "AttributeList/AttributeList.h"
# include "AttributeList/AttributeListSpecification.h"
# include "AttributeList/AttributeExceptions.h"

# include "AttributeList/AttributePredefinedTypes.h" 

# include <iostream>

int example1()
{
  // -- create attribute specification
  pool::AttributeListSpecification spec;
  spec.push_back( "A","int" );
  spec.push_back( "B", typeid(long));
  spec.push_back( "X", "string");
  spec.push_back( "Y", "bool");

  try
    {  
      // -- create attribute list for the specification

      pool::AttributeList alist(spec);
      
      alist["A"].setValue( 10 );
      alist["B"].setValue( long( 20 ) );
      alist["X"].setValue(std::string("xxx"));
      alist["Y"].setValue(true);
      // -- read back the attribute values
      
      int A;
      long B;
      std::string X;
      bool Y;

      alist["A"].getValue(A);
      alist["B"].getValue(B);
      alist["X"].getValue(X);
      alist["Y"].getValue(Y);

      std::cout << "A = " << A << " B = " << B << " X = " << X << " Y = " << Y << std::endl;

      alist.attributeListSpecification().print(std::cout); std::cout << std::endl;
      alist.print(std::cout); std::cout << std::endl;

      std::cout << std::endl << "OK" << std::endl;

    }
  catch(pool::attribute_bad_type x)
    {
      std::cerr << "Type mismatch" << std::endl << x.what() << std::endl;
    }
  catch(pool::attribute_not_found x)
    {
      std::cerr << "Not found" << std::endl << x.what() << std::endl;
    }
  
  return 0;
}

int main()
{
  return example1();
}
