# include "AttributeList/AttributeList.h"
# include "AttributeList/AttributeListSpecification.h"
# include "AttributeList/AttributeExceptions.h"

# include "AttributeList/AttributePredefinedTypes.h" 

# include <iostream>

int token_test()
{

  std::vector< std::pair<std::string, pool::typename_string_t> > columns;

  columns.push_back( std::make_pair("A","Token"));
  columns.push_back( std::make_pair("B","Token"));

  // -- create attribute specification

  pool::AttributeListSpecification spec;

  for(unsigned int i=0; i<columns.size(); ++i)
    spec.push_back( columns[i].first, columns[i].second );


  try
    {  
      // -- create attribute list for the specification

      pool::AttributeList alist(spec);
      pool::Token t1, t2;

      alist["A"].setValue(t1);
      alist["B"].setValue(t2);
      
      // -- read back the attribute values
      
      pool::Token v1,v2;

      alist["A"].getValue(v1);
      alist["B"].getValue(v2);

      std::cout << "A = " << t1.toString() << " B = " << t2.toString() << std::endl;

      assert(t1 == v1);
      assert(t2 == v2);

      std::cout << std::endl << "OK" << std::endl;

    }
  catch(pool::attribute_bad_type x)
    {
      std::cerr << "Type mismatch" << std::endl << x.what() << std::endl;
    }
  catch(pool::attribute_not_found x)
    {
      std::cerr << "Not found" << std::endl << x.what() << std::endl;
    }
  
  return 0;
}

int main()
{
  return token_test();
}
