#include "Collection/CollectionSvc.h"
#include "Collection/CollectionProxy.h"

#include "CollectionBase/CollectionFactory.h"
#include "CollectionBase/CollectionDescription.h"

#include "boost/tokenizer.hpp"



pool::CollectionProxy 
pool::CollectionSvc::createAndRegister( pool::IDataSvc* dataSvc, 
					const std::string& type,
					const std::string& connection,
					const std::string& name,
					const std::string& logicalName,
					MetaDataEntry* collectionMetaData,
					bool authorizeReplace,
					pool::IFileCatalog* catalog )
{
   pool::CollectionFactory* factory = pool::CollectionFactory::get();
   pool::ICollection* collection = 0;
   if( factory ) {
      pool::ISession* session = 0;
      if ( dataSvc ) session = &( dataSvc->session() );
      CollectionDescription description( name, type, connection );
      collection = factory->createAndRegister( description,
					       catalog,
					       authorizeReplace,
					       logicalName,
					       collectionMetaData,
					       session );
   }
  return  pool::CollectionProxy( dataSvc, collection );
}



pool::CollectionProxy
pool::CollectionSvc::openWithPhysicalName( pool::IDataSvc* dataSvc, 
					   const std::string& physicalName,
					   bool readOnly,
					   pool::IFileCatalog* catalog)
{
   pool::CollectionFactory* factory = pool::CollectionFactory::get();
   pool::ICollection* collection = 0;
   if( factory ) {
      pool::ISession* session = 0;
      if ( dataSvc ) session = &( dataSvc->session() );
      //MN FIX ? - in Factory, this method does less than before
      collection = factory->openWithPhysicalName( physicalName,
						  catalog,
						  readOnly,
						  session );
  }
   return  pool::CollectionProxy( dataSvc, collection );
}


pool::CollectionProxy 
pool::CollectionSvc::openWithLogicalName( pool::IDataSvc* dataSvc, 
					  const std::string& logicalName,
					  bool readOnly,
					  pool::IFileCatalog* catalog)
{
   pool::CollectionFactory* factory = pool::CollectionFactory::get();
   pool::ICollection* collection = 0;
   if( factory ) {
      pool::ISession* session = 0;
      if ( dataSvc ) session = &( dataSvc->session() );
      collection = factory->openWithLogicalName( logicalName,
						 catalog,
						 readOnly,
						 session);
   }
   return  pool::CollectionProxy( dataSvc, collection );
}  


pool::CollectionProxy 
pool::CollectionSvc::openWithGuid( pool::IDataSvc* dataSvc, 
                                   const FileCatalog::FileID& id,
                                   bool readOnly,
                                   pool::IFileCatalog* catalog)
{
   pool::CollectionFactory* factory = pool::CollectionFactory::get();
   pool::ICollection* collection = 0;
   if( factory ) {
      pool::ISession* session = 0;
      if ( dataSvc ) session = &( dataSvc->session() );
      collection = factory->openWithGuid( id,
					  catalog,
					  readOnly,
					  session);
   }
   return  pool::CollectionProxy( dataSvc, collection );
}    



pool::CollectionProxy::Iterator 
pool::CollectionSvc::select( pool::IDataSvc* dataSvc, 
			     const std::string& objectQuery,
			     const std::string& collectionQuery,
			     const std::string& objectIteratorOptions,
			     pool::IFileCatalog* catalog)
{
   pool::CollectionFactory* factory = pool::CollectionFactory::get();
   pool::ICollectionCursor* cursor =  0;
   if( factory ) {
      pool::ISession* session = 0;
      if ( dataSvc ) session = &( dataSvc->session() );

      std::vector<std::string> outputList;
      typedef boost::tokenizer<boost::char_separator<char> > Tizer;
      boost::char_separator<char> sep(" ,");
      Tizer tizer( objectIteratorOptions, sep);
      for( Tizer::iterator token=tizer.begin(); token!=tizer.end(); ++token) {
	 if( *token != "SELECT" ) 
	    outputList.push_back( *token );
      }
      cursor = factory->getCatalogCollectionCursor( catalog,
						    collectionQuery,
						    objectQuery,
						    &outputList,
						    0,
						    1,
						    session);
   }
   return pool::CollectionProxy::Iterator( 0, cursor, dataSvc );
}
