#include "CollectionBase/TokenListSpecification.h"

#include "POOLCore/Exception.h"

#include <sstream>


pool::TokenListSpecification::TokenListSpecification() : 
  m_refCounter( 1 ),
  m_tokenNames(),
  m_indexForName()
{}


pool::TokenListSpecification::TokenListSpecification( const pool::TokenListSpecification& rhs ):
  m_refCounter( 1 ),
  m_tokenNames(),
  m_indexForName( rhs.m_indexForName )
{
  for ( std::vector< std::string >::const_iterator iTokenName = rhs.m_tokenNames.begin();
    iTokenName != rhs.m_tokenNames.end(); ++iTokenName ) 
  {
    m_tokenNames.push_back( *iTokenName );
  }
}


pool::TokenListSpecification::~TokenListSpecification()
{}


bool
pool::TokenListSpecification::operator==( const pool::TokenListSpecification& rhs ) const
{
  if ( this->size() != rhs.size() ) return false;
  size_t theSize = this->size();
  for ( size_t i = 0; i < theSize; ++i )
    if ( (*this)[i] != rhs[i] )
      return false;
  return true;
}


void
pool::TokenListSpecification::addRef() const
{
  ++m_refCounter;
}


void
pool::TokenListSpecification::release() const
{
  --m_refCounter;
  if ( m_refCounter == 0 ) delete this;
}


void
pool::TokenListSpecification::extend( const std::string& name )
{
  if ( m_indexForName.find( name ) != m_indexForName.end() )
  {
    std::string errorMsg = "Token with name `" + name + "' already exists in Token list specification.";
    throw pool::Exception( errorMsg,
                           "TokenListSpecification::extend",
                           "CollectionBase" );
  }
  m_tokenNames.push_back( name );
  m_indexForName.insert( std::make_pair( name, static_cast<int>( m_tokenNames.size() - 1 ) ) );
}


int
pool::TokenListSpecification::tokenIndex( const std::string& tokenName ) const
{
  std::map<std::string, int>::const_iterator iIndexForName = m_indexForName.find( tokenName );
  if ( iIndexForName == m_indexForName.end() ) 
  {
    return -1;
  }
  else 
  {
    return iIndexForName->second;
  }
}


const std::string&
pool::TokenListSpecification::tokenName( int tokenIndex ) const
{
  if ( tokenIndex < 0 || tokenIndex >= static_cast<int>( m_tokenNames.size() ) )
  {
    std::ostringstream strm;
    strm << tokenIndex;
    std::string errorMsg = "Cannot find a Token with index `" + strm.str() + "'";
    throw pool::Exception ( errorMsg,
                            "TokenListSpecification::tokenName",
                            "CollectionBase" );
  }

  return ( m_tokenNames[tokenIndex] );
}


