#ifndef POOL_ANYPTR_H
#define POOL_ANYPTR_H

#include <utility>
#include "DataSvc/AnyType.h"
#include <typeinfo>
#include "Reflex/Type.h"
#include "RVersion.h"


#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
namespace ROOT {  namespace Reflex {  class Object;  }}
typedef ROOT::Reflex::Type TypeH;
#else
namespace Reflex {  class Object;  }
typedef Reflex::Type TypeH;
#endif


namespace pool{

  class AnyPtr {

  public: 

    AnyPtr();

    template <class T> AnyPtr(const T* aPtr ) : m_ptr(0), m_type(){
      if(aPtr){
        m_type = typeid(*aPtr);
        m_ptr = aPtr;
      }
    }    

    AnyPtr(const void* aPointer, const std::type_info* aType);

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
    AnyPtr(const ROOT::Reflex::Object& object);
#else
    AnyPtr(const Reflex::Object& object);
#endif

    AnyPtr(const std::pair<void*,const TypeH>& objPair);

    AnyPtr(const AnyPtr &aPtr);

    virtual ~AnyPtr();

    AnyPtr& operator=(const AnyPtr& aPtr);

    template <class T> AnyPtr& operator=(T* pointer){
      m_ptr = pointer;
      if(pointer){
        m_type = typeid(*pointer);
      }
      return *this;
    }

    operator bool() const;

    bool operator==(const AnyPtr& anObj) const;

    bool operator!=(const AnyPtr& anObj) const;

    template <class T> friend bool operator==(T* aPtr, const AnyPtr& anObj){
       return anObj.constPtr() == (const void*)aPtr;
    }
    template <class T> friend bool operator==(const AnyPtr& anObj, T* aPtr){
       return  anObj.constPtr() == (const void*)aPtr;
    }
    
    template <class T> friend bool operator!=(T* aPtr, const AnyPtr& anObj){
       return !(aPtr == anObj);
    }
    template <class T> friend bool operator!=(const AnyPtr& anObj, T* aPtr){
       return !(anObj == aPtr);
    }

    const void* constPtr() const;

    operator const void*() const;

    void* ptr() const;

    void* ptr(const AnyType& asType) const;

    bool isOfType(const AnyType& type) const;

    const AnyType& getType() const;

    const TypeH& getClass() const;

    void reset();

    void forceDelete();

  private:
   
    const void* m_ptr;

    AnyType m_type;
  };

}

#endif
