#include "DataSvc/AnyType.h"
#include "Reflex/Reflex.h"
#include "StorageSvc/DbReflex.h"
#include "POOLCore/Guid.h"

pool::AnyType::AnyType(): m_class(), m_className(""){}

pool::AnyType::AnyType(const std::type_info& type) : 
    m_class(DbReflex::forTypeInfo(type)), m_className(m_class.Name()){}

pool::AnyType::AnyType(const TypeH& c): m_class(c),m_className(""){
  if(c)m_className = m_class.Name();
}
                     
pool::AnyType::AnyType(const AnyType& aType): m_class(aType.m_class),m_className(aType.m_className){}

pool::AnyType::~AnyType(){}

pool::AnyType& pool::AnyType::operator=(const AnyType& aType){
  if(this == &aType){
    return *this;
  }
  m_class = aType.m_class;
  m_className = aType.m_className;
  return *this;
}

pool::AnyType& pool::AnyType::operator=(const std::type_info& type){
  m_class = DbReflex::forTypeInfo(type);
  m_className = type.name();
  return *this;
}

pool::AnyType& pool::AnyType::operator=(const TypeH& aClass){
  m_class = aClass;
  if(m_class) m_className = m_class.Name();
  return *this;
}

pool::AnyType::operator bool() const {
  return m_class;
}

const std::type_info* pool::AnyType::getType() const {
  const std::type_info* ret = 0;
  if(m_class){
    ret = &(m_class.TypeInfo());
  }
  return ret;
} 

std::string pool::AnyType::name() const {
  return m_className;
}

const TypeH& pool::AnyType::getClass() const {
  return m_class;
} 

void* pool::AnyType::cast(const AnyType& asType, const void* aPtr) const {
  void* thePtr = 0;
  if( asType.m_class && m_class ) {
    if ( asType.m_class != m_class )  {
      bool check = m_class.HasBase(asType.m_class);
      if(check){
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
        const ROOT::Reflex::Object obj(m_class,const_cast<void*>(aPtr)); 
#else
        const Reflex::Object obj(m_class,const_cast<void*>(aPtr)); 
#endif
        thePtr = obj.CastObject(asType.m_class).Address();
      } 
    } else {
      thePtr = const_cast<void*>(aPtr);
    }
  }
  return thePtr; 
}

bool pool::AnyType::hasBase(const AnyType& aBaseType) const {
  bool ret = false;
  if( aBaseType.m_class && m_class ) {
    if(aBaseType.m_class != m_class){
      ret = m_class.HasBase(aBaseType.m_class);
    } else {
      ret = true;
    }
  }
  return ret;
}

bool pool::AnyType::isBaseFor(const AnyType& forType) const {
  return forType.hasBase(m_class);
}

void pool::AnyType::destroy(void* aPtr){
  if(aPtr) { 
    if( m_class ) {
      m_class.Destruct(const_cast<void*>(aPtr));
    }
  }
}

void pool::AnyType::reset() {
  m_class = 0;
}
