#include "DataSvc/CacheLink.h" 
#include "DataSvc/IDataSvc.h"
#include "DataSvc/ICacheSvc.h"
#include "DataSvc/CacheEntry.h"
#include "DataSvc/ObjectDeletePolicy.h"
#include "DataSvc/AnyPtr.h"

pool::CacheLink::CacheLink():
  m_cacheSvc(0), m_cacheEntry(0){}

pool::CacheLink::CacheLink(IDataSvc* aDataSvc):
  m_cacheSvc(0),m_cacheEntry(0){
  if(aDataSvc){
    m_cacheSvc = &aDataSvc->cacheSvc();
    if(m_cacheSvc)
    m_cacheSvc->registerRef( static_cast< pool::IRegisteredRef* >( this ) );
  }
}

pool::CacheLink::CacheLink(ICacheSvc* aCacheSvc):
  m_cacheSvc(aCacheSvc),m_cacheEntry(0){
  if(m_cacheSvc)
    m_cacheSvc->registerRef( static_cast< pool::IRegisteredRef* >( this ) );
}


pool::CacheLink::CacheLink(const CacheLink& aCacheLink): 
  IRegisteredRef(aCacheLink),
  m_cacheSvc(aCacheLink.m_cacheSvc),
  m_cacheEntry(aCacheLink.m_cacheEntry){
  if(m_cacheSvc){
    m_cacheSvc->registerRef( static_cast< pool::IRegisteredRef* >( this ) );  
    if(m_cacheEntry){
      m_cacheSvc->linkCacheEntry(m_cacheEntry);
    }
  }
}

pool::CacheLink::~CacheLink(){
  disconnect();
}

pool::CacheLink& pool::CacheLink::operator=(const CacheLink& aCacheLink){
  if(this == &aCacheLink){
    return *this;
  }
  ICacheSvc* aCacheSvc = aCacheLink.m_cacheSvc;
  CacheEntry* aCacheEntry = aCacheLink.m_cacheEntry;
  // this has to be done anyhow!
  if(m_cacheSvc && m_cacheEntry){
    m_cacheSvc->unlinkCacheEntry(m_cacheEntry);      
    m_cacheEntry = 0;
  }
  // if the cachesvc is a new one, swap them!
  if(m_cacheSvc != aCacheSvc){
    if(m_cacheSvc){
      m_cacheSvc->unregisterRef( 
                     static_cast< pool::IRegisteredRef* >( this ) );
      m_cacheSvc = 0;
    }
    if(aCacheSvc){
      m_cacheSvc = aCacheSvc;
      m_cacheSvc->registerRef( static_cast< pool::IRegisteredRef* >( this ) );
    }    
  } 
  if(aCacheSvc && aCacheEntry){
    m_cacheSvc->linkCacheEntry(aCacheEntry);
    m_cacheEntry = aCacheEntry;
  }
  return *this;
}

pool::CacheLink& pool::CacheLink::copy(const AnyType& forType,
                                       const CacheLink& aCacheLink){
  if(this == &aCacheLink){
    return *this;
  }
  ICacheSvc* aCacheSvc = aCacheLink.m_cacheSvc;
  CacheEntry* aCacheEntry = aCacheLink.m_cacheEntry;
  // this has to be done anyhow!
  if(m_cacheSvc && m_cacheEntry){
    m_cacheSvc->unlinkCacheEntry(m_cacheEntry);      
    m_cacheEntry = 0;
  }
  // if the cachesvc is a new one, swap them!
  if(m_cacheSvc != aCacheSvc){
    if(m_cacheSvc){
      m_cacheSvc->unregisterRef( 
                     static_cast< pool::IRegisteredRef* >( this ) );
      m_cacheSvc = 0;
    }
    if(aCacheSvc){
      m_cacheSvc = aCacheSvc;
      m_cacheSvc->registerRef( static_cast< pool::IRegisteredRef* >( this ) );
    }    
  } 
  if(aCacheSvc && aCacheEntry){
    if(m_cacheSvc->linkCacheEntry(aCacheEntry,forType.getClass())){
      m_cacheEntry = aCacheEntry;
    }
  }
  return *this;
}

pool::CacheLink::operator bool() const {
  return m_cacheSvc!=0;
}

bool pool::CacheLink::connect(ICacheSvc* aCacheSvc){
  bool ret = true;
  if(m_cacheSvc != aCacheSvc){
    if(m_cacheSvc){
      if(m_cacheEntry){
        m_cacheSvc->unlinkCacheEntry(m_cacheEntry);      
        m_cacheEntry = 0;
      }
      m_cacheSvc->unregisterRef( 
                     static_cast< pool::IRegisteredRef* >( this ) );
      m_cacheSvc = 0;
    }
    if(aCacheSvc){
      m_cacheSvc = aCacheSvc;
      m_cacheSvc->registerRef( static_cast< pool::IRegisteredRef* >( this ) );
    }    
  }
  return ret;
}

bool pool::CacheLink::bindCacheEntry(const SharedAnyPtr& anObject){
  bool ret = false;
  if(m_cacheSvc){
    CacheEntry* entry = m_cacheSvc->registerObject(anObject);
    if(m_cacheEntry){
      if(entry != m_cacheEntry){
        m_cacheSvc->unlinkCacheEntry(m_cacheEntry);
        m_cacheEntry = entry;
        m_cacheSvc->linkCacheEntry(m_cacheEntry);
      } 
    } else {
      m_cacheEntry = entry;
      m_cacheSvc->linkCacheEntry(m_cacheEntry);
    }
    ret = true;
  }  
  return ret;
}

bool pool::CacheLink::bindCacheEntry(const AnyType& forType, const Token& aToken){
  bool ret = false;
  if(m_cacheSvc){
    CacheEntry* entry = m_cacheSvc->registerToken(aToken);  
    if(m_cacheEntry){
      if(entry != m_cacheEntry){
        m_cacheSvc->unlinkCacheEntry(m_cacheEntry);
        ret = m_cacheSvc->linkCacheEntry(entry,forType.getClass());
        if(ret) m_cacheEntry = entry;
      }
    } else {
      ret = m_cacheSvc->linkCacheEntry(entry,forType.getClass());
      if(ret ) m_cacheEntry = entry;
    }
  }  
  return ret;
}

const pool::SharedAnyPtr& pool::CacheLink::getObject() const {
  const SharedAnyPtr* ret = &m_emptyObject;
  if(m_cacheSvc && m_cacheEntry){
    const SharedAnyPtr& theObj = m_cacheEntry->object();
    if(!theObj){
      m_cacheSvc->getObject(m_cacheEntry);
    }
    ret = &theObj;
  }
  return *ret;
}

pool::Token* pool::CacheLink::getToken() const {
  Token* ret = 0;
  if(m_cacheSvc && m_cacheEntry){
    ret = m_cacheSvc->getToken(m_cacheEntry);
  }
  return ret;
}

bool pool::CacheLink::sendWriteRequest(const Placement& place) const {
  bool ret = false;
  if(m_cacheSvc && m_cacheEntry){
    ret = m_cacheSvc->registerForWrite(place,m_cacheEntry);
  }
  return ret;
}

bool pool::CacheLink::sendMultiWriteRequest(const Placement& place) {
  bool ret = false;
  if(m_cacheSvc && m_cacheEntry){
    if(getToken()){
      // if a token is there, a new cache location has to be build  
      m_cacheSvc->getObject(m_cacheEntry);
      
      CacheEntry *loc = m_cacheEntry->cloneObjectContent();
      if(m_cacheSvc->registerCacheEntry(loc)){
        m_cacheSvc->unlinkCacheEntry(m_cacheEntry);
        m_cacheEntry = loc;
        m_cacheSvc->linkCacheEntry(m_cacheEntry);
      } else {
        delete loc;
      }
    }
    ret = m_cacheSvc->registerForWrite(place,m_cacheEntry);
  }
  return ret;
}

bool pool::CacheLink::sendUpdateRequest() const {
  bool ret = false;
  if(m_cacheSvc && m_cacheEntry){
    ret = m_cacheSvc->registerForUpdate(m_cacheEntry);
  }
  return ret;
}

bool pool::CacheLink::sendDeleteRequest() const {
  bool ret = false;
  if(m_cacheSvc && m_cacheEntry){
    ret = m_cacheSvc->registerForDelete(m_cacheEntry);
  }
  return ret;
}

bool pool::CacheLink::resetObject() const {
  bool ret = false;
  if(m_cacheSvc && m_cacheEntry){
    ret = m_cacheSvc->resetObject(m_cacheEntry);
  }
  return ret;
}

bool pool::CacheLink::deleteObjectFlag() const {
  bool ret = true;
  if(m_cacheSvc){
    ret = m_cacheSvc->objectDeletePolicy().onRef();
  }
  return ret;
}

bool pool::CacheLink::isObjectLoaded() const {
  bool ret= false;
  if(m_cacheSvc && m_cacheEntry){
    ret = m_cacheSvc->isObjectLoaded(m_cacheEntry);
  }
  return ret;
}

bool pool::CacheLink::disconnect(){
  bool ret = true;
  if(m_cacheSvc){
    if(m_cacheEntry){
      m_cacheSvc->unlinkCacheEntry(m_cacheEntry);
      m_cacheEntry = 0;
    }
    m_cacheSvc->unregisterRef( 
                static_cast< pool::IRegisteredRef* >( this ) );
    m_cacheSvc = 0;
  }
  return ret;
}

bool pool::CacheLink::unbind(){
  bool ret = true;
  if(m_cacheSvc){
    if(m_cacheEntry){
      m_cacheSvc->unlinkCacheEntry(m_cacheEntry);
      m_cacheEntry = 0;
    }
  }
  return ret;
}

pool::ICacheSvc* pool::CacheLink::cacheSvc() const {
  return m_cacheSvc;
}

pool::CacheEntry* pool::CacheLink::cacheEntry() const {
  return m_cacheEntry;
}

void
pool::CacheLink::unregisterCacheSvc( pool::ICacheSvc* cacheSvc )
{
  if ( m_cacheSvc == cacheSvc ) m_cacheSvc = 0;
  m_cacheEntry = 0;
}


void
pool::CacheLink::unregisterCacheEntry( CacheEntry* cacheEntry){
  if(m_cacheSvc && (m_cacheEntry == cacheEntry)){
    m_cacheEntry = 0;
  }
}
