#define POOL_CACHETABLE_CPP 1

#include "CacheTable.h"
#include "DataSvc/AnyPtr.h"

pool::CacheEntryImp* pool::CacheTable::getEntry(const AnyPtr& aPtr) {
  CacheEntryImp* ret = 0;
  if( 0 != aPtr){
    KeyPtr aKPtr(aPtr);
    KeyPtrTable::Result res = thePtrTable.get(aKPtr);
    if(res.first){
      ret = res.second;
    }
  }
  return ret;
}

pool::CacheEntryImp* pool::CacheTable::getEntry(const Token& aToken) {
  CacheEntryImp* ret = 0;
  KeyTok aKTok(aToken);
  KeyTokenTable::Result res = theTokenTable.get(aKTok);
  if(res.first){
    ret = res.second;
  }
  return ret;
}

bool pool::CacheTable::unsetEntry(const AnyPtr& aPtr){
  bool ret = false;
  if(0 != aPtr){
    KeyPtr aKPtr(aPtr);
    KeyPtrTable::Result res = thePtrTable.del(aKPtr);
    ret =  res.first;
  }
  return ret;
}

bool pool::CacheTable::unsetEntry(const Token& aTok){
  bool ret = false;
  KeyTok aKTok(aTok);
  KeyTokenTable::Result res = theTokenTable.del(aKTok);
  ret = res.first;
  return ret;
}

bool pool::CacheTable::remove(const CacheEntryImp* aCacheEntry){
  bool deleted = false;
  if( 0 != aCacheEntry){
    KeyPtr k(aCacheEntry);
    deleted = theEntryList.del(k);
  }
  return deleted;
}

bool pool::CacheTable::insert(const CacheEntryImp* aCacheEntry){
  bool ret  = false;
  if( 0 != aCacheEntry){
    KeyPtr k(aCacheEntry);
    ret = theEntryList.insert(k);
  }
  return ret;
}

bool pool::CacheTable::setEntry(const Token& aToken, CacheEntryImp* aCacheEntry){
  bool ret = false;
  if( 0 != aCacheEntry ){
    const KeyTok aKTok(aToken);
    theTokenTable.put(aKTok,aCacheEntry);
    ret =  true;
  }
  return ret;
}

bool pool::CacheTable::setEntry(const AnyPtr& aPtr, CacheEntryImp* aCacheEntry){
  bool ret = false;
  if( 0 != aPtr && 0 != aCacheEntry){
    const KeyPtr aKPtr(aPtr);
    thePtrTable.put(aKPtr,aCacheEntry);
    ret = true;
  }
  return ret;
}

bool pool::CacheTable::find(const CacheEntryImp* aCacheEntry){
  bool ret = false;
  if( 0 != aCacheEntry){
    KeyPtr k(aCacheEntry);
    ret = theEntryList.find(k);
  }
  return ret;
}

pool::CacheTable::const_list_iterator pool::CacheTable::list_begin(){
  return theEntryList.begin();
}
     
pool::CacheTable::const_list_iterator pool::CacheTable::list_end(){
  return theEntryList.end();
}

pool::CacheTable::const_ptr_iterator pool::CacheTable::ptr_iter_begin(){
  return thePtrTable.begin();
}
     
pool::CacheTable::const_ptr_iterator pool::CacheTable::ptr_iter_end(){
  return thePtrTable.end();
}

pool::CacheTable::const_token_iterator pool::CacheTable::token_iter_begin(){
  return theTokenTable.begin();
}
     
pool::CacheTable::const_token_iterator pool::CacheTable::token_iter_end(){  return theTokenTable.end();
}      

void pool::CacheTable::clear() {
  thePtrTable.clear();
  theTokenTable.clear();
  theEntryList.clear();
}

unsigned int pool::CacheTable::size(){
  return theEntryList.size();
}





