#ifndef POOL_HSET_LIST_H
#define POOL_HSET_LIST_H 

#ifdef __GNUC__
#include <ext/hash_set>
#define EXTSTD __gnu_cxx

#elif _WIN32
#include "POOLCore/win32/stl_hash.h"
#define EXTSTD stdext

#elif __ICC
#include <hash_set>
#define EXTSTD std

#endif

namespace pool  {

  /** @class Hset_list Hset_list.h
      Templated container class. Designed to be a random access list of object
      @author Giacomo Govi
      @date 25/07/2002
   **/
  template <class K> class Hset_list {    

    public:

      // convenience types
      typedef K Entry;

      typedef EXTSTD::hash_set<K> KList;
	 
      typedef typename KList::const_iterator const_iterator;

      // default constructor
      Hset_list() : m_list() {}

      // constructor with initial capacity
#ifdef __GNUC__
      Hset_list(unsigned int init_size): m_list(init_size){}
#else 
      Hset_list(unsigned int) : m_list(){}
#endif

      // method to insert a specified value on the list. The entry is inserted if not in the
      // list yet (using the ==operator). returns true if the entry is inserted; false otherwise
      bool insert(const K& aK);

      // method to find a specified value on the list. returns true if the entry is found
      bool find(const K& aK);

      // method to remove a specified entry on the list. Returns true if the entry is deleted
      bool del(const K& aK); 

      /** clear - method to erase all the entry of the table
      */
      void clear();

      /** begin iterator
      */
      const_iterator begin();

      /** end iterator
      */
      const_iterator end();

      /** size - method to retrieve the number of entries of the table
	  @return the number of entries
      */
      unsigned int size();

    private:
	
      // private instance of the hash_set container
      KList m_list;

  };  
}

// method to insert a specified value on the list. The entry is inserted if not in the
// list yet (using the ==operator). returns true if the entry is inserted; false otherwise
template <class K> 
bool pool::Hset_list<K>::insert(const K& aK) {
  typedef typename KList::iterator iterator_type;
  std::pair<iterator_type,bool> p = m_list.insert(aK);
  bool inserted = p.second;
  return inserted;
}

// method to find a specified value on the list. returns true if the entry is found
template <class K> 
bool pool::Hset_list<K>::find(const K& aK) {
  bool found = false;
  typename KList::iterator item = m_list.find(aK);
  if(item != m_list.end()){
    found = true;
  }
  return found;
}

// method to remove a specified entry on the list. Returns true if the entry is deleted
template <class K> bool pool::Hset_list<K>::del(const K& aK) {
  bool deleted = false;
  typename KList::iterator item = m_list.find(aK);
  if(item != m_list.end()){
    m_list.erase(item);
    deleted = true;
  }
  return deleted;
}


// method to erase all the entry of the table
template <class K> void pool::Hset_list<K>::clear(){
  m_list.clear();
}

// begin iterator
template <class K> typename pool::Hset_list<K>::const_iterator 
                                    pool::Hset_list<K>::begin() {
  return m_list.begin();
}

// end iterator
template <class K> typename pool::Hset_list<K>::const_iterator 
                                     pool::Hset_list<K>::end() {
  return m_list.end();
}

// method to retrieve the number of entries of the table
template <class K> unsigned int pool::Hset_list<K>::size(){
  return m_list.size();
}

#endif  // POOL_HSET_LIST_H




