#ifndef LFCCATALOG_PARSER_H
#define LFCCATALOG_PARSER_H

#include "FileCatalog/FCException.h"
#include <vector>
#include <string>

namespace pool {

  class LFCQueryException : public FCbackendException {
  public:
    ///constructor
    LFCQueryException(const std::string& from, const std::string& reason);
    ///virtual destructor
    virtual ~LFCQueryException() throw(){};
  };
  
  class LFCQueryPhrase {
  public: 
    ///constructor
    LFCQueryPhrase(const int& type) : m_type(type) { }
    
    std::string getOp() const {return m_op; }
    std::string getValue() const {return m_value;}

    void setOp(const std::string &op) {m_op = op; }
    void setValue(const std::string &value) {m_value = value;}

    bool isType(const int& type) const {
      return (type == m_type);
    } 
    enum PhraseType { PFN, LFN, GUID, JOIN, COMPOSITE };
    
  private:
    int m_type;
    std::string m_op;
    std::string m_value;
  };

  
  class LFCParser {
  public:
      //// Constructor
    LFCParser();
    
    /// destructor
    virtual ~LFCParser();
    
    std::vector<LFCQueryPhrase> parse(const std::string &query) const;
        
  private:
    std::vector<std::string> split_query(const std::string& query) const;
    int compare(const std::string& str1, const std::string& str2) const;
    int do_transition(const int& state, const std::string& word,
		      std::vector<LFCQueryPhrase>& phrases) const;

    enum LFCQueryState { START = 0, NAME = 1,  COMPARATOR = 2,  
			 VALUE = 3,  JOIN = 4, END = 5};
  };
}
#endif
