// $Id: ParserTest.cpp,v 1.6 2008/02/20 09:38:33 witoldp Exp $
/*
  CPPUnit test for the query parser
  Author: James Casey
*/
#include <cppunit/extensions/HelperMacros.h>
#include "LFCCatalog/LFCParser.h"
#include <iostream>

class ParserTest : public CppUnit::TestFixture{
  CPPUNIT_TEST_SUITE( ParserTest );
  CPPUNIT_TEST( testParse );
  CPPUNIT_TEST( testBadQueries );
  CPPUNIT_TEST( testGoodQueries );

  CPPUNIT_TEST_SUITE_END();

public:
  void setUp(){ 

  }
  
  void tearDown(){

  }

  void testParse(){
    std::cout<<"[OVAL]TEST --> testParse"<<std::endl;
    std::string query("pfname = 'srm://foo'");
    pool::LFCParser parser;

    std::vector<pool::LFCQueryPhrase> result = parser.parse(query);
    CPPUNIT_ASSERT_EQUAL_MESSAGE("one phrase", (size_t)1, result.size());
    CPPUNIT_ASSERT_MESSAGE("name correct", result[0].isType(pool::LFCQueryPhrase::PFN));
    CPPUNIT_ASSERT_EQUAL_MESSAGE("op correct", std::string("="), result[0].getOp());
    CPPUNIT_ASSERT_EQUAL_MESSAGE("value correct", 
				 std::string("srm://foo"), result[0].getValue());
  }

  /** test the underlying parse algorithm.  This allows more than we actually support
      in the LFC e.g. OR  */
  void testBadQueries() {
    std::cout<<"[OVAL]TEST --> testBadQueries"<<std::endl;
    expect_parse_exception("BAD = 'foo'"); //  bad name
    expect_parse_exception("pfname unlike 'foo'"); // bad comparator
    expect_parse_exception("pfname = 'foo"); // unbalanced quotes
    expect_parse_exception("pfname = 'foo' AND"); // no second phrase
    expect_parse_exception("pfname = 'foo' AND BAD = 'foo'"); // bad name, second phrase
    expect_parse_exception("pfname = 'foo' also guid = 'foo'"); // bad join operator
  }

  /** check good queries pass the parser */
  void testGoodQueries() {
    std::cout<<"[OVAL]TEST --> testGoodQueries"<<std::endl;
    expect_no_exception("pfname like 'foo'");
    expect_no_exception("pfname = 'bar'");
    expect_no_exception("lfname like 'foo'");
    expect_no_exception("lfname = 'bar'");
    expect_no_exception("guid like 'foo'");
    expect_no_exception("guid = 'bar'");
    // try other and mixed case
    expect_no_exception("pfName LIKE 'foo'");
    expect_no_exception("PFNAME liKe 'bar'");
    expect_no_exception("lfNAme LIKE 'foo'");
    expect_no_exception("LFNAME liKE 'bar'");
    expect_no_exception("GUID LIKE 'foo'");
    expect_no_exception("guiD LiKE 'bar'");

    // try some join phrases
    expect_no_exception("pfname = 'foo' and guid = 'bar'");
    expect_no_exception("pfname = 'foo' AND guid = 'bar'");
    expect_no_exception("pfname = 'foo' OR guid = 'bar'");
    expect_no_exception("pfname = 'foo' oR guid = 'bar'");

    // try no spaces around an equals
    expect_no_exception("pfname='foo'");

    // try no quotes
    expect_no_exception("pfname=foo");
  }

  void expect_parse_exception(const std::string& query) {
    pool::LFCParser parser;
    try {
      std::vector<pool::LFCQueryPhrase> result = parser.parse(query);
    } catch(const pool::LFCQueryException &e) {
      return;
    }catch (const pool::Exception &e) {
      CPPUNIT_FAIL(std::string("Caught Exception : ") + e.what() + 
		   " : " + query);
    }
    CPPUNIT_FAIL("Expected LFCQueryException on bad query : " + query);
  }

  void expect_no_exception(const std::string& query) {
    pool::LFCParser parser;
    try {
      std::vector<pool::LFCQueryPhrase> result = parser.parse(query);
    }catch (const pool::Exception &e) {
      CPPUNIT_FAIL(std::string("Caught Exception : ") + e.what() + " : " + query);
    }
    return;
  }

};

CPPUNIT_TEST_SUITE_REGISTRATION(ParserTest);

#include<CppUnit_testdriver.cpp>
