#ifndef OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPINGSCHEMA_H
#define OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPINGSCHEMA_H

#include <map>
#include <vector>
#include <string>
#include <set>

namespace coral {

  class AttributeList;
  class ISchema;
  class ITableDataEditor;
}

namespace pool 
{
  
  /**
   *
   */
  
  class ObjectRelationalMappingSchema
  {
    public:

      /// Returns the name of the table which holds the versions of the mappings.
      static std::string mappingVersionTable();
      /// Returns the column name which holds the version in the table which holds the versions of the mappings.
      static std::string versionFieldInMappingVersionTable();
      /// Returns the column name which holds the container name in the table which holds the versions of the classes.
      static std::string containerNameFieldInMappingVersionTable();

      /// Returns the name of the table which holds the versions of the classes.
      static std::string classVersionTable();
      /// Returns the column name which holds the class version in the table which holds the versions of the classes.
      static std::string classVersionFieldInClassVersionTable();
      /// Returns the column name which holds the container name in the table which holds the versions of the classes.
      static std::string containerNameFieldInClassVersionTable();
      /// Returns the column name which holds the version in the table which holds the versions of the classes.
      static std::string mappingVersionFieldInClassVersionTable();

      /// Returns the name of the table which holds the mapping elements.
      static std::string mappingElementTable();
      /// Returns the field which holds the version in the table which holds the mapping elements.
      static std::string versionFieldInMappingElementTable();
      /// Returns the field which holds the element id in the table which holds the mapping elements.
      static std::string elementIdFieldInMappingElementTable();
      /// Returns the field which holds the element type in the table which holds the mapping elements.
      static std::string elementTypeFieldInMappingElementTable();
      /// Returns the field which holds the scope name in the table which holds the mapping elements.
      static std::string scopeNameFieldInMappingElementTable();
      /// Returns the field which holds the variable name in the table which holds the mapping elements.
      static std::string variableNameFieldInMappingElementTable();
      /// Returns the field which holds the index of the variable parameter in the table which holds the mapping elements.
      static std::string variableParIndexFieldInMappingElementTable();
      /// Returns the field which holds the variable type in the table which holds the mapping elements.
      static std::string variableTypeFieldInMappingElementTable();
      /// Returns the field which holds the associated table name in the table which holds the mapping elements.
      static std::string tableNameFieldInMappingElementTable();
      /// Returns the field which holds the column name in the table which holds the names of the associated columns in a mapping element.
      static std::string columnNameFieldInMappingElementTable();

      /// Returns the name of the table (obsolete) which holds the names of the associated columns in a mapping element.
      static std::string mappingElementColumnsTable();
      
      /// Temporary tables for data migration
      static std::string mappingVersionTmpTable();
      static std::string classVersionTmpTable();
      static std::string mappingElementTmpTable();

    public:
    
    /// Constructor
    explicit ObjectRelationalMappingSchema( coral::ISchema& schema );

    /// Destructor
    ~ObjectRelationalMappingSchema();

    bool existTables();

    bool existsOldSchema();

    /// Create the mapping tables
    void createTables();

    /// Create the mapping temporary tables (for migration)
    void createTmpTables();

    /// Drop the mapping tables
    void dropTables(bool includingColumnTable=false);

    /// Creates the mapping temporary tables (for migration)
    void dropTmpTables();

    public:

    static std::set<std::string> listTables();

    private:

    void createVersionTable(const std::string& tableName);

    void createClassVersionTable(const std::string& tableName, const std::string& versionTableName, std::string fkName="" );

    void createMappingTable(const std::string& tableName, const std::string& versionTableName, std::string fkName="");

    void dropTable(const std::string& tableName);

    private:

    coral::ISchema& m_schema;
  };

  class ObjectRelationalMappingOldSchema {
    public:

    /// Returns the name of the table which holds the versions of the mappings.
    static std::string mappingVersionTable();
    /// Returns the column name which holds the version in the table which holds the versions of the mappings.
    static std::string versionFieldInMappingVersionTable();
    /// Returns the column name which holds the container name in the table which holds the versions of the classes.
    static std::string containerNameFieldInMappingVersionTable();

    /// Returns the name of the table which holds the versions of the classes.
    static std::string classVersionTable();
    /// Returns the column name which holds the class version in the table which holds the versions of the classes.
    static std::string classVersionFieldInClassVersionTable();
    /// Returns the column name which holds the container name in the table which holds the versions of the classes.
    static std::string containerNameFieldInClassVersionTable();
    /// Returns the column name which holds the version in the table which holds the versions of the classes.
    static std::string mappingVersionFieldInClassVersionTable();

    /// Returns the name of the table which holds the mapping elements.
    static std::string mappingElementTable();
    /// Returns the field which holds the element type in the table which holds the mapping elements.
    static std::string elementTypeFieldInMappingElementTable();
    /// Returns the field which holds the scope name in the table which holds the mapping elements.
    static std::string scopeNameFieldInMappingElementTable();
    /// Returns the field which holds the variable name in the table which holds the mapping elements.
    static std::string variableNameFieldInMappingElementTable();
    /// Returns the field which holds the variable type in the table which holds the mapping elements.
    static std::string variableTypeFieldInMappingElementTable();
    /// Returns the field which holds the associated table name in the table which holds the mapping elements.
    static std::string tableNameFieldInMappingElementTable();
    /// Returns the field which holds the version in the table which holds the mapping elements.
    static std::string versionFieldInMappingElementTable();

    /// Returns the name of the table which holds the names of the associated columns in a mapping element.
    static std::string mappingElementColumnsTable();
    /// Returns the field which holds the mapping version in the table which holds the names of the associated columns in a mapping element.
    static std::string versionFieldInMappingElementColumnsTable();
    /// Returns the field which holds the scope name in the table which holds the names of the associated columns in a mapping element.
    static std::string scopeNameFieldInMappingElementColumnsTable();
    /// Returns the field which holds the variable name in the table which holds the names of the associated columns in a mapping element.
    static std::string variableNameFieldInMappingElementColumnsTable();
    /// Returns the field which holds the column name in the table which holds the names of the associated columns in a mapping element.
    static std::string columnFieldInMappingElementColumnsTable();
    /// Returns the field which holds the position of the column in a mapping element in the table which holds the names of the associated columns in a mapping element.
    static std::string positionFieldInMappingElementColumnsTable();

  };
}

#endif // OBJECTRELATIONALACCESS_OBJECTRELATIONALMAPPINGSCHEMA_H
