#include "ObjectRelationalAccess/ObjectRelationalMapping.h"
#include "ObjectRelationalAccess/ObjectRelationalNames.h"

pool::ObjectRelationalMapping::ObjectRelationalMapping( const std::string& version ):
  m_version( version ),
  m_className(""),
  m_element(),
  m_dependentElements()
{}

std::set<std::string>
pool::ObjectRelationalMapping::involvedTables() const{
  std::set<std::string> involvedTables;
  std::set<std::string> elementTables = m_element.involvedTables();
  for(std::set<std::string>::const_iterator iT=elementTables.begin();
      iT!=elementTables.end();++iT){
    involvedTables.insert(*iT);
  }
  
  for(std::map< std::string, pool::ObjectRelationalMappingElement >::const_iterator iDep = m_dependentElements.begin();
      iDep != m_dependentElements.end(); ++iDep ){
    std::set<std::string> depElementTables = iDep->second.involvedTables();
    for(std::set<std::string>::const_iterator iT=depElementTables.begin();
        iT!=depElementTables.end();++iT){
      involvedTables.insert(*iT);
    }
  }
  return involvedTables;
}

pool::ObjectRelationalMappingElement&
pool::ObjectRelationalMapping::setElement( const std::string& className,
                                           const std::string& tableName )
{
  m_element = pool::ObjectRelationalMappingElement( pool::ObjectRelationalMappingElement::objectMappingElementType(),
                                                    className,
                                                    className,
                                                    tableName );
  m_className = className;
  return m_element;
}

pool::ObjectRelationalMappingElement&
pool::ObjectRelationalMapping::appendDependentElement( const std::string& className,
                                                       const std::string& tableName )
{  
  return m_dependentElements.insert( std::make_pair( className,
                                                     pool::ObjectRelationalMappingElement( pool::ObjectRelationalMappingElement::dependentObjectMappingElementType(),
                                                                                           className,
                                                                                           className,
                                                                                           tableName ) ) ).first->second;
}

void pool::ObjectRelationalMapping::override(const ObjectRelationalMapping& source)
{
  if(m_className == source.m_className ) m_element.override( source.m_element );
  for(std::map< std::string, pool::ObjectRelationalMappingElement >::iterator iDep = m_dependentElements.begin();
      iDep != m_dependentElements.end(); ++iDep ){
    std::map< std::string, pool::ObjectRelationalMappingElement >::const_iterator iTarg = source.m_dependentElements.find(iDep->first); 
    if(iTarg!=source.m_dependentElements.end()){
      iDep->second.override(iTarg->second);
    }
  }  
}

