#include "ObjectRelationalAccess/ObjectRelationalMapping.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingDescriptor.h"
#include <sstream>

pool::ObjectRelationalMappingDescriptor::ObjectRelationalMappingDescriptor(): m_topLevelElement(0){}

pool::ObjectRelationalMappingDescriptor::~ObjectRelationalMappingDescriptor(){
  if(m_topLevelElement){
    delete m_topLevelElement;
  }
}

pool::ObjectRelationalMappingDescriptorElement* pool::ObjectRelationalMappingDescriptor::setTopLevelElement(const std::string& type){
  if(type==ObjectRelationalMappingDescriptorElement::mappingTag()){
    if(m_topLevelElement != 0 ) {
       delete m_topLevelElement;
    }
    m_topLevelElement = new ObjectRelationalMappingDescriptorElement(type);
  }
  return m_topLevelElement;
}

const pool::ObjectRelationalMappingDescriptorElement* pool::ObjectRelationalMappingDescriptor::topLevelElement() const {
  return m_topLevelElement;
}

std::string pool::ObjectRelationalMappingDescriptor::version() const {
  if(m_topLevelElement){
    return m_topLevelElement->attribute(ObjectRelationalMappingDescriptorElement::versionAttr());
  } else {
    return "";
  }
}

std::vector<pool::ObjectRelationalMappingDescriptorElement*> pool::ObjectRelationalMappingDescriptor::classDescriptors() const {
  std::vector<ObjectRelationalMappingDescriptorElement*> vec;
  if(m_topLevelElement){
    vec =  m_topLevelElement->children();
  }
  return vec;
}

void pool::ObjectRelationalMappingDescriptor::import(const pool::ObjectRelationalMapping& mapping){
    if(!m_topLevelElement){
      m_topLevelElement = new ObjectRelationalMappingDescriptorElement(ObjectRelationalMappingDescriptorElement::mappingTag());
    }
    m_topLevelElement->setAttribute(ObjectRelationalMappingDescriptorElement::versionAttr(),ObjectRelationalMappingDescriptorElement::defaultVersionAttrValue());
    ObjectRelationalMappingDescriptorElement* classElement = new ObjectRelationalMappingDescriptorElement(mapping.element());
    classElement->setType(ObjectRelationalMappingDescriptorElement::classTag());
    classElement->setAttribute(ObjectRelationalMappingDescriptorElement::mappingVersionAttr(),mapping.version());
    m_topLevelElement->importChild(classElement);
    // import dependencies
    for(std::map< std::string, pool::ObjectRelationalMappingElement >::const_iterator iDep = mapping.dependentElements().begin();
        iDep != mapping.dependentElements().end(); iDep++){
      ObjectRelationalMappingDescriptorElement* depElement = new ObjectRelationalMappingDescriptorElement(iDep->second);
      depElement->setType(ObjectRelationalMappingDescriptorElement::classTag());
      depElement->setAttribute(ObjectRelationalMappingDescriptorElement::mappingVersionAttr(),mapping.version());
      m_topLevelElement->importChild(depElement);
    }  
}


