#include "ObjectRelationalAccess/ObjectRelationalMappingElement.h"
#include "ObjectRelationalAccess/ObjectRelationalException.h"
#include "ObjectRelationalAccess/ObjectRelationalNames.h"
#include <sstream>

std::string
pool::ObjectRelationalMappingElement::objectMappingElementType()
{
  static std::string s_objectMappingElementType = "Object";
  return s_objectMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::dependentObjectMappingElementType()
{
  static std::string s_dependentObjectMappingElementType = "DependentObject";
  return s_dependentObjectMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::primitiveMappingElementType()
{
  static std::string s_primitiveMappingElementType = "Primitive";
  return s_primitiveMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::arrayMappingElementType()
{
  static std::string s_arrayMappingElementType = "Array";
  return s_arrayMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::CArrayMappingElementType()
{
  static std::string s_CArrayMappingElementType = "CArray";
  return s_CArrayMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::inlineCArrayMappingElementType()
{
  static std::string s_inlineCArrayMappingElementType = "InlineCArray";
  return s_inlineCArrayMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::poolReferenceMappingElementType()
{
  static std::string s_poolReferenceMappingElementType = "PoolReference";
  return s_poolReferenceMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::poolPointerMappingElementType()
{
  static std::string s_poolPointerMappingElementType = "PoolPointer";
  return s_poolPointerMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::poolPolymorphicPointerMappingElementType()
{
  static std::string s_poolPolymorphicPointerMappingElementType = "PoolPolymorphicPointer";
  return s_poolPolymorphicPointerMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::poolArrayMappingElementType()
{
  static std::string s_poolArrayMappingElementType = "PoolArray";
  return s_poolArrayMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::pointerMappingElementType()
{
  static std::string s_pointerMappingElementType = "Pointer";
  return s_pointerMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::referenceMappingElementType()
{
  static std::string s_referenceMappingElementType = "Reference";
  return s_referenceMappingElementType;
}

std::string
pool::ObjectRelationalMappingElement::blobMappingElementType()
{
  static std::string s_blobMappingElementType = "Blob";
  return s_blobMappingElementType;
}

bool
pool::ObjectRelationalMappingElement::isValidMappingElementType( const std::string& elementType )
{
  return ( elementType == objectMappingElementType() ||
           elementType == dependentObjectMappingElementType() ||
           elementType == primitiveMappingElementType() ||
           elementType == arrayMappingElementType() ||
           elementType == CArrayMappingElementType() ||
           elementType == inlineCArrayMappingElementType() ||
           elementType == poolReferenceMappingElementType() ||
           elementType == poolPointerMappingElementType() ||
           elementType == poolPolymorphicPointerMappingElementType() ||
           elementType == poolArrayMappingElementType() ||
           elementType == pointerMappingElementType() ||
           elementType == referenceMappingElementType() ||
           elementType == blobMappingElementType() );
}

std::string
pool::ObjectRelationalMappingElement::elementTypeAsString( pool::ObjectRelationalMappingElement::ElementType elementType )
{
  switch ( elementType ) {
  case Object :
    return  objectMappingElementType();
    break;
  case DependentObject :
    return  dependentObjectMappingElementType();
    break;
  case Primitive :
    return  primitiveMappingElementType();
    break;
  case Array :
    return  arrayMappingElementType();
    break;
  case CArray :
    return  CArrayMappingElementType();
    break;
  case InlineCArray :
    return  inlineCArrayMappingElementType();
    break;
  case PoolReference :
    return  poolReferenceMappingElementType();
    break;
  case PoolPointer :
    return  poolPointerMappingElementType();
    break;
  case PoolPolymorphicPointer :
    return  poolPolymorphicPointerMappingElementType();
    break;
  case PoolArray :
    return  poolArrayMappingElementType();
    break;
  case Reference :
    return  referenceMappingElementType();
    break;
  case Blob :
    return  blobMappingElementType();
    break;
  case Pointer :
    return  pointerMappingElementType();
    break;
  case Undefined :
    // This should never appear
    break;
  };

  throw pool::ObjectRelationalException( "Undefined mapping element type",
                                         "ObjectRelationalMappingElement::elementTypeAsString" );
  return "";
}


pool::ObjectRelationalMappingElement::ElementType
pool::ObjectRelationalMappingElement::elementTypeFromString( const std::string& elementType )
{
  // Check here the element type
  if ( !  isValidMappingElementType( elementType ) ) {
    throw pool::ObjectRelationalException( "\"" + elementType + "\" is not a supported mapping element type",
                                           "ObjectRelationalMappingElement::elementTypeFromString" );
  }

  pool::ObjectRelationalMappingElement::ElementType result = Undefined;

  if ( elementType == objectMappingElementType() ) {
    return Object;
  }
  if ( elementType == dependentObjectMappingElementType() ) {
    return DependentObject;
  }
  if ( elementType == primitiveMappingElementType() ) {
    return Primitive;
  }
  else if ( elementType == arrayMappingElementType() ) {
    return Array;
  }
  else if ( elementType == CArrayMappingElementType() ) {
    return CArray;
  }
  else if ( elementType == inlineCArrayMappingElementType() ) {
    return InlineCArray;
  }
  else if ( elementType == pointerMappingElementType() ) {
    return Pointer;
  }
  else if ( elementType == referenceMappingElementType() ) {
    return Reference;
  }
  else if ( elementType == poolReferenceMappingElementType() ) {
    return PoolReference;
  }
  else if ( elementType == poolPointerMappingElementType() ) {
    return PoolPointer;
  }
  else if ( elementType == poolPolymorphicPointerMappingElementType() ) {
    return PoolPolymorphicPointer;
  }
  else if ( elementType == poolArrayMappingElementType() ) {
    return PoolArray;
  }
  else if ( elementType == blobMappingElementType() ) {
    return Blob;
  }

  return result;
}

pool::ObjectRelationalMappingElement::ObjectRelationalMappingElement( const std::string& elementType,
                                                                      const std::string& variableName,
                                                                      const std::string& variableType,
                                                                      const std::string& tableName ):
  m_elementType( Undefined ),
  m_parentType( Undefined ),
  m_isDependentTree(false),
  m_scopeName( "" ),
  m_variableName( variableName ),
  m_variableNameForSchema( "" ),
  m_variableType( variableType ),
  m_tableName( tableName ),
  m_columnNames(),
  m_subElements(),
  m_mainTableName( tableName ),
  m_parentClassMappingElement(0)
{
  // Check here the element type
  if ( !  isValidMappingElementType( elementType ) ) {
    throw pool::ObjectRelationalException( "\"" + elementType + "\" is not a supported mapping element type",
                                           "ObjectRelationalMappingElement" );
  }
  m_elementType = elementTypeFromString( elementType );
  if(m_elementType == DependentObject) m_isDependentTree = true;
}

pool::ObjectRelationalMappingElement::~ObjectRelationalMappingElement() {
  if(m_parentClassMappingElement) delete m_parentClassMappingElement;
}

std::set<std::string>
pool::ObjectRelationalMappingElement::involvedTables() const{
  std::set<std::string> involvedTables;
  involvedTables.insert(tableName());
  const_iterator iel = begin();
  for(const_iterator iel=begin();
      iel!=end();++iel){
    std::set<std::string> elementTables = iel->second.involvedTables();
    for(std::set<std::string>::const_iterator iT=elementTables.begin();
        iT!=elementTables.end();++iT){
      involvedTables.insert(*iT);
    }
  }
  return involvedTables;
}

void
pool::ObjectRelationalMappingElement::setDependency( const ObjectRelationalMappingElement& parentClassMappingElement )
{
  m_elementType = DependentObject;
  m_isDependentTree = true;
  if(m_parentClassMappingElement) delete m_parentClassMappingElement;
  m_parentClassMappingElement = new ObjectRelationalMappingElement(parentClassMappingElement);
}

pool::ObjectRelationalMappingElement&
pool::ObjectRelationalMappingElement::appendSubElement( const std::string& elementType,
                                                        const std::string& variableName,
                                                        const std::string& variableType,
                                                        const std::string& tableName )
{
  // Check whether the current element type supports other subelements
  if ( ! ( m_elementType == Object ||
           m_elementType == DependentObject ||
           m_elementType == Array ||
           m_elementType == CArray ||
           m_elementType == InlineCArray ||
           m_elementType == PoolReference ||
           m_elementType == PoolPointer ||
           m_elementType == PoolArray ) ) {
    throw pool::ObjectRelationalException( "Attempted to insert a sub-element under an element of type \"" +
                                           pool::ObjectRelationalMappingElement::elementTypeAsString( m_elementType ) + "\"",
                                           "ObjectRelationalMappingElement::appendSubElement" );
  }
  pool::ObjectRelationalMappingElement& newElement =
    m_subElements.insert( std::make_pair( variableName,pool::ObjectRelationalMappingElement( elementType,
                                                                                             variableName,
                                                                                             variableType,
                                                                                             tableName ) ) ).first->second;
  newElement.m_parentType = m_elementType;
  newElement.m_isDependentTree = m_isDependentTree;
  if ( m_scopeName.empty() ) {
    newElement.m_scopeName = m_variableName;
  } else {
    newElement.m_scopeName = m_scopeName + "::" + m_variableName;
  }
  newElement.m_mainTableName = m_mainTableName;
  return newElement;
}

void
pool::ObjectRelationalMappingElement::appendSubElementTree( const ObjectRelationalMappingElement& element ){
  if ( ! ( m_elementType == Object ||
           m_elementType == DependentObject ||
           m_elementType == Array ||
           m_elementType == CArray ||
           m_elementType == InlineCArray ||
           m_elementType == PoolReference ||
           m_elementType == PoolPointer ||
           m_elementType == PoolArray ) ) {
    throw pool::ObjectRelationalException( "Attempted to insert a sub-element under an element of type \"" +
                                           pool::ObjectRelationalMappingElement::elementTypeAsString( m_elementType ) + "\"",
                                           "ObjectRelationalMappingElement::appendSubElement" );
  }
  m_subElements.insert( std::make_pair(element.variableName(),element));
}

void
pool::ObjectRelationalMappingElement::alterType( const std::string& elementType )
{
  // Check here the element type
  if ( !  isValidMappingElementType( elementType ) ) {
    throw pool::ObjectRelationalException( "\"" + elementType + "\" is not a supported mapping element type",
                                           "ObjectRelationalMappingElement::alterType");
  }
  ElementType elementTypeCode = elementTypeFromString( elementType );
  if(m_elementType != elementTypeCode){
    m_elementType = elementTypeCode;      
    // clear sub elements when no supported by the new type specified
    if ( ! ( m_elementType == Object ||
             m_elementType == DependentObject ||
             m_elementType == Array ||
             m_elementType == CArray ||
             m_elementType == PoolReference ||
             m_elementType == PoolPointer ||
             m_elementType == PoolArray ) ) {
      m_subElements.clear();
    }
  }
}

void
pool::ObjectRelationalMappingElement::alterTableName( const std::string& tableName )
{
  if ( tableName == m_tableName ) return;

  for ( iterator iElement = this->begin();
        iElement != this->end(); ++iElement ) {
    pool::ObjectRelationalMappingElement& subElement = iElement->second;
    if ( subElement.elementType() != Array &&
         subElement.elementType() != CArray &&
         subElement.elementType() != PoolPointer &&
         subElement.elementType() != PoolArray ) {
      subElement.alterTableName( tableName );
    }
  }
  m_tableName = tableName;
}

void
pool::ObjectRelationalMappingElement::setVariableNameForSchema(const std::string& scopeName,const std::string& variableName){
  if ( scopeName.empty() ) {
    m_variableNameForSchema = variableName;
  } else {
    m_variableNameForSchema = scopeName + "_" + variableName;
  }
}

void
pool::ObjectRelationalMappingElement::setColumnNames( const std::vector< std::string >& columns )
{
  m_columnNames = columns;
  if ( m_subElements.empty() ) return;

  std::vector< std::string > columnsToPropagate = m_columnNames;
  if ( this->elementType() == PoolReference ) {
    // Remove the OID fields
    columnsToPropagate.pop_back();
    columnsToPropagate.pop_back();
  }

  for ( iterator iElement = this->begin();
        iElement != this->end(); ++iElement ) {
    pool::ObjectRelationalMappingElement& subElement = iElement->second;

    if ( subElement.elementType() == Object &&
         subElement.tableName() == m_tableName ) {
      subElement.setColumnNames( columnsToPropagate );
    }
    else if ( subElement.elementType() == PoolReference &&
              subElement.tableName() == m_tableName ) {
      std::vector< std::string > columns = subElement.columnNames();
      for ( size_t i = 0; i < columns.size() - 2; ++i ) columns[i] = columnsToPropagate[i];
      subElement.setColumnNames( columns );
    }
  }
}

void pool::ObjectRelationalMappingElement::override(const ObjectRelationalMappingElement& source)
{
  if(variableType()==source.variableType() || elementType()==source.elementType())
  {
    alterType(ObjectRelationalMappingElement::elementTypeAsString(source.elementType()));
    alterTableName(source.tableName());
    setColumnNames(source.columnNames());
    for(iterator iel=begin();
        iel!=end();++iel){
      const_iterator iTarg = source.find(iel->first);
      if(iTarg!=source.end()){
        iel->second.override(iTarg->second);
      }
    }
  }  

}

