#include "ObjectRelationalAccess/ObjectRelationalMappingSchema.h"
#include "ObjectRelationalAccess/ObjectRelationalException.h"
#include "ObjectRelationalAccess/ObjectRelationalNames.h"
#include "RelationalAccess/IColumn.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITablePrivilegeManager.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/SchemaException.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/MessageStream.h"

std::string
pool::ObjectRelationalMappingSchema::mappingVersionTable()
{
  static std::string s_mappingVersionTable = "POOL_OR_MAPPING_VERSIONS";
  return s_mappingVersionTable;
}

std::string
pool::ObjectRelationalMappingSchema::versionFieldInMappingVersionTable()
{
  static std::string s_versionFieldInMappingVersionTable = "MAPPING_VERSION";
  return s_versionFieldInMappingVersionTable;
}

std::string
pool::ObjectRelationalMappingSchema::containerNameFieldInMappingVersionTable()
{
  static std::string s_containerNameFieldInMappingVersionTable = "CONTAINER_ID";
  return s_containerNameFieldInMappingVersionTable;
}

std::string
pool::ObjectRelationalMappingSchema::classVersionTable()
{
  static std::string s_classVersionTable = "POOL_OR_CLASS_VERSIONS";
  return s_classVersionTable;
  
}

std::string
pool::ObjectRelationalMappingSchema::classVersionFieldInClassVersionTable()
{
  static std::string s_classVersionFieldInClassVersionTable = "CLASS_VERSION";
  return s_classVersionFieldInClassVersionTable;
}

std::string
pool::ObjectRelationalMappingSchema::containerNameFieldInClassVersionTable()
{
  static std::string s_containerNameFieldInClassVersionTable = "CONTAINER_ID";
  return s_containerNameFieldInClassVersionTable;
}

std::string
pool::ObjectRelationalMappingSchema::mappingVersionFieldInClassVersionTable()
{
  static std::string s_mappingVersionFieldInClassVersionTable = "MAPPING_VERSION";
  return s_mappingVersionFieldInClassVersionTable;
}

std::string
pool::ObjectRelationalMappingSchema::mappingElementTable()
{
  static std::string s_mappingElementTable = "POOL_OR_MAPPING_ELEMENTS";
  return s_mappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::versionFieldInMappingElementTable()
{
  static std::string s_versionFieldInMappingElementTable = "MAPPING_VERSION";
  return s_versionFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::elementIdFieldInMappingElementTable()
{
  static std::string s_elementIdFieldInMappingElementTable = "ELEMENT_ID";
  return s_elementIdFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::elementTypeFieldInMappingElementTable()
{
  static std::string s_elementTypeFieldInMappingElementTable = "ELEMENT_TYPE";
  return s_elementTypeFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::scopeNameFieldInMappingElementTable()
{
  static std::string s_scopeNameFieldInMappingElementTable = "VARIABLE_SCOPE";
  return s_scopeNameFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::variableNameFieldInMappingElementTable()
{
  static std::string s_variableNameFieldInMappingElementTable = "VARIABLE_NAME";
  return s_variableNameFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::variableParIndexFieldInMappingElementTable()
{
  static std::string s_variableParIndexFieldInMappingElementTable = "VARIABLE_PAR_INDEX";
  return s_variableParIndexFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::variableTypeFieldInMappingElementTable()
{
  static std::string s_variableTypeFieldInMappingElementTable = "VARIABLE_TYPE";
  return s_variableTypeFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::tableNameFieldInMappingElementTable()
{
  static std::string s_tableNameFieldInMappingElementTable = "TABLE_NAME";
  return s_tableNameFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::columnNameFieldInMappingElementTable()
{
  static std::string s_columnNameFieldInMappingElementTable = "COLUMN_NAME";
  return s_columnNameFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingSchema::mappingElementColumnsTable()
{
  static std::string s_mappingElementColumnsTable = "POOL_OR_MAPPING_COLUMNS";
  return s_mappingElementColumnsTable;
}

std::string
pool::ObjectRelationalMappingSchema::mappingVersionTmpTable()
{
  static std::string s_mappingVersionTmpTable = "POOL_TMP_MAPPING_VERSIONS";
  return s_mappingVersionTmpTable;
}

std::string
pool::ObjectRelationalMappingSchema::classVersionTmpTable()
{
  static std::string s_classVersionTmpTable = "POOL_TMP_CLASS_VERSIONS";
  return s_classVersionTmpTable;
  
}

std::string
pool::ObjectRelationalMappingSchema::mappingElementTmpTable()
{
  static std::string s_mappingElementTmpTable = "POOL_TMP_MAPPING_ELEMENTS";
  return s_mappingElementTmpTable;
}

pool::ObjectRelationalMappingSchema::ObjectRelationalMappingSchema( coral::ISchema& schema ):
  m_schema( schema )
{}


pool::ObjectRelationalMappingSchema::~ObjectRelationalMappingSchema()
{
}

bool
pool::ObjectRelationalMappingSchema::existTables(){
  bool ret = false;
  std::set<std::string> tableList = listTables();
  std::set<std::string> missingTables;
  coral::MessageStream log( pool::ObjectRelationalAccess::serviceName() );
  try {
    for(std::set<std::string>::const_iterator iT=tableList.begin(); iT!=tableList.end();++iT){
      if(!m_schema.existsTable( *iT )){
        missingTables.insert( *iT );
      }
    }
    if(missingTables.size()==0) {
      if(!m_schema.existsTable( mappingElementColumnsTable() )){
        ret = true;
      } else {
        throw pool::ObjectRelationalException( "Mapping Database found with an osbsolete schema. Please upgrade to the current version.",
                                               "ObjectRelationalMappingSchema::existTables" );
      }
    } else if(missingTables.size()!=tableList.size()){
      std::stringstream message;
      message << "Mapping Database is corrupted. Table(s): ";
      bool start = false;
      for( std::set<std::string>::const_iterator iMt=missingTables.begin(); iMt!=missingTables.end();++iMt ){
        if(start) message << ",";
        message << *iMt;
        start = true;
      }
      message << " have been not found in the schema.";
      throw pool::ObjectRelationalException( message.str(),
                                             "ObjectRelationalMappingSchema::existTables" );
    }    
  } catch (const coral::Exception& e){
    throw pool::ObjectRelationalException( e.what(),
                                           "ObjectRelationalMappingSchema::existTables" );
  }
  return ret;
}

bool
pool::ObjectRelationalMappingSchema::existsOldSchema(){
  bool ret = false;
  std::set<std::string> tableList = listTables();
  std::set<std::string> missingTables;
  tableList.insert( mappingElementColumnsTable() );
  try {
    for(std::set<std::string>::const_iterator iT=tableList.begin(); iT!=tableList.end();++iT){
      if(!m_schema.existsTable( *iT )){
        missingTables.insert( *iT );
      }
    }
    if(missingTables.size()==0) {
        ret = true;
    } else if(missingTables.size()!=tableList.size()){
      std::stringstream message;
      message << "Old schema is invalid. Expected Table(s): ";
      bool start = false;
      for( std::set<std::string>::const_iterator iMt=missingTables.begin(); iMt!=missingTables.end();++iMt ){
        if(start) message << ",";
        message << *iMt;
        start = true;
      }
      message << " have been not found.";
      throw pool::ObjectRelationalException( message.str(),
                                             "ObjectRelationalMappingSchema::existsOldSchema" );
    }    
  } catch (const coral::Exception& e){
    throw pool::ObjectRelationalException( e.what(),
                                           "ObjectRelationalMappingSchema::existsOldSchema" );
  }
  return ret;

}

void
pool::ObjectRelationalMappingSchema::createVersionTable(const std::string& tableName){
 // The versions table.
  coral::MessageStream log( pool::ObjectRelationalAccess::serviceName() );
  log << coral::Warning << "Creating table \"" << tableName << "\"" << coral::MessageStream::endmsg;
  try {
    coral::TableDescription  description0( "ObjectRelationalAccess" );
    description0.setName( tableName );
    description0.insertColumn( versionFieldInMappingVersionTable(),
                               coral::AttributeSpecification::typeNameForType<std::string>(), 1000, false);
    description0.setNotNullConstraint( versionFieldInMappingVersionTable() );
    description0.insertColumn( containerNameFieldInMappingVersionTable(),
                               coral::AttributeSpecification::typeNameForType<std::string>(), 1000, false);
    description0.setNotNullConstraint( containerNameFieldInMappingVersionTable() );
    description0.setPrimaryKey( versionFieldInMappingVersionTable() );
    m_schema.createTable( description0 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );  
  } catch(const coral::Exception& e) {
    throw pool::ObjectRelationalException( e.what(),
                                           "ObjectRelationalMappingSchema::createVersionTable" );
  }
}

void
pool::ObjectRelationalMappingSchema::createClassVersionTable(const std::string& tableName, const std::string& versionTableName, std::string fkName){

 // The class versions table.
  coral::MessageStream log( pool::ObjectRelationalAccess::serviceName() );
  log << coral::Warning << "Creating table \"" << tableName << "\"" << coral::MessageStream::endmsg;
  try {
    coral::TableDescription description1( "ObjectRelationalAccess" );
    description1.setName( tableName );
    description1.insertColumn(  classVersionFieldInClassVersionTable(),
                               coral::AttributeSpecification::typeNameForType<std::string>(), 1000, false);
    description1.setNotNullConstraint(  classVersionFieldInClassVersionTable() );
    description1.insertColumn(  containerNameFieldInClassVersionTable(),
                               coral::AttributeSpecification::typeNameForType<std::string>(), 1000, false);
    description1.setNotNullConstraint(  containerNameFieldInClassVersionTable() );
    description1.insertColumn(  mappingVersionFieldInClassVersionTable(),
                               coral::AttributeSpecification::typeNameForType<std::string>(), 1000, false);
    description1.setNotNullConstraint(  mappingVersionFieldInClassVersionTable() );
    std::vector<std::string> cols1;
    cols1.push_back( classVersionFieldInClassVersionTable());
    cols1.push_back( containerNameFieldInClassVersionTable());
    description1.setPrimaryKey( cols1 );
    if(fkName.empty()) fkName = mappingVersionFieldInClassVersionTable()+"_FK_0";
    description1.createForeignKey(
    fkName,
    mappingVersionFieldInClassVersionTable(),
    versionTableName,
    versionFieldInMappingVersionTable());
    m_schema.createTable( description1 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
  } catch(const coral::Exception& e) {
    throw pool::ObjectRelationalException( e.what(),
                                           "ObjectRelationalMappingSchema::createClassVersionTable" );
  }
}

void
pool::ObjectRelationalMappingSchema::createMappingTable(const std::string& tableName, const std::string& versionTableName, std::string fkName){
  // The table holding the mapping elements.
  coral::MessageStream log( pool::ObjectRelationalAccess::serviceName() );
  log << coral::Warning << "Creating table \"" << tableName << "\"" <<
         coral::MessageStream::endmsg;
  try {
    coral::TableDescription description2( "ObjectRelationalAccess" );
    description2.setName( tableName );
    description2.insertColumn(  versionFieldInMappingElementTable(),
                               coral::AttributeSpecification::typeNameForType<std::string>(), 1000, false );
    description2.setNotNullConstraint(  versionFieldInMappingElementTable() );
    description2.insertColumn(  elementIdFieldInMappingElementTable(),
                               coral::AttributeSpecification::typeNameForType<std::string>(), 1000, false );  
    description2.setNotNullConstraint(  elementIdFieldInMappingElementTable() );
    description2.insertColumn(  elementTypeFieldInMappingElementTable(),
                                coral::AttributeSpecification::typeNameForType<std::string>() );
    description2.setNotNullConstraint(  elementTypeFieldInMappingElementTable() );
    description2.insertColumn(  scopeNameFieldInMappingElementTable(),
                                coral::AttributeSpecification::typeNameForType<std::string>() );
    description2.setNotNullConstraint(  scopeNameFieldInMappingElementTable() );
    description2.insertColumn(  variableNameFieldInMappingElementTable(),
                                coral::AttributeSpecification::typeNameForType<std::string>() );
    description2.setNotNullConstraint(  variableNameFieldInMappingElementTable() );
    description2.insertColumn(  variableParIndexFieldInMappingElementTable(),
                                coral::AttributeSpecification::typeNameForType<int>() );  
    description2.setNotNullConstraint(  variableParIndexFieldInMappingElementTable() );
    description2.insertColumn(  variableTypeFieldInMappingElementTable(),
                                coral::AttributeSpecification::typeNameForType<std::string>() );
    description2.setNotNullConstraint(  variableTypeFieldInMappingElementTable() );
    description2.insertColumn(  tableNameFieldInMappingElementTable(),
                                coral::AttributeSpecification::typeNameForType<std::string>() );
    description2.setNotNullConstraint(  tableNameFieldInMappingElementTable() );
    description2.insertColumn(  columnNameFieldInMappingElementTable(),
                                coral::AttributeSpecification::typeNameForType<std::string>() );
    description2.setNotNullConstraint(  columnNameFieldInMappingElementTable() );
    std::vector<std::string> cols2;
    cols2.push_back(  versionFieldInMappingElementTable());
    cols2.push_back(  elementIdFieldInMappingElementTable());
    description2.setPrimaryKey( cols2 );
    if(fkName.empty()) fkName = versionFieldInMappingElementTable()+"_FK_1";
    description2.createForeignKey(
    fkName,
    versionFieldInMappingElementTable(),
    versionTableName,
    versionFieldInMappingVersionTable());
    m_schema.createTable( description2 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
  } catch(const coral::Exception& e) {
    throw pool::ObjectRelationalException( e.what(),
                                           "ObjectRelationalMappingSchema::createMappingTable" );
  }
}

void
pool::ObjectRelationalMappingSchema::dropTable(const std::string& tableName){
  coral::MessageStream log( pool::ObjectRelationalAccess::serviceName() );
  log << coral::Warning << "Dropping table \"" << tableName << "\"" <<
    coral::MessageStream::endmsg;
  try {
    m_schema.dropIfExistsTable( tableName );
  } catch(const coral::Exception& e) {
    throw pool::ObjectRelationalException( e.what(),
                                           "ObjectRelationalMappingSchema::dropTable" );
  }
}

void
pool::ObjectRelationalMappingSchema::createTables()
{
  createVersionTable( mappingVersionTable() );
  createClassVersionTable( classVersionTable(),mappingVersionTable() );
  createMappingTable( mappingElementTable(), mappingVersionTable());
}

void
pool::ObjectRelationalMappingSchema::createTmpTables()
{
  createVersionTable( mappingVersionTmpTable() );
  std::string fkName0(mappingVersionFieldInClassVersionTable()+"_TMP_FK_0");
  createClassVersionTable( classVersionTmpTable(), mappingVersionTmpTable(), fkName0);
  std::string fkName1(mappingVersionFieldInClassVersionTable()+"_TMP_FK_1");
  createMappingTable( mappingElementTmpTable(), mappingVersionTmpTable(), fkName1);
}

void
pool::ObjectRelationalMappingSchema::dropTables(bool includingColumnTable){
  if( includingColumnTable ) dropTable( mappingElementColumnsTable());
  dropTable( mappingElementTable() );
  dropTable( classVersionTable() );
  dropTable( mappingVersionTable() );
}

void pool::ObjectRelationalMappingSchema::dropTmpTables(){
  dropTable( mappingElementTmpTable() );
  dropTable( classVersionTmpTable() );
  dropTable( mappingVersionTmpTable() );
}

std::set<std::string>
pool::ObjectRelationalMappingSchema::listTables(){
  std::set<std::string> tableList;
  tableList.insert(  mappingVersionTable() );
  tableList.insert(  classVersionTable() );
  tableList.insert(  mappingElementTable() );
  return tableList;
}

std::string
pool::ObjectRelationalMappingOldSchema::mappingVersionTable()
{
  static std::string s_mappingVersionTable = "POOL_OR_MAPPING_VERSIONS";
  return s_mappingVersionTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::versionFieldInMappingVersionTable()
{
  static std::string s_versionFieldInMappingVersionTable = "MAPPING_VERSION";
  return s_versionFieldInMappingVersionTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::containerNameFieldInMappingVersionTable()
{
  static std::string s_containerNameFieldInMappingVersionTable = "CONTAINER_ID";
  return s_containerNameFieldInMappingVersionTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::classVersionTable()
{
  static std::string s_classVersionTable = "POOL_OR_CLASS_VERSIONS";
  return s_classVersionTable;
  
}

std::string
pool::ObjectRelationalMappingOldSchema::classVersionFieldInClassVersionTable()
{
  static std::string s_classVersionFieldInClassVersionTable = "CLASS_VERSION";
  return s_classVersionFieldInClassVersionTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::containerNameFieldInClassVersionTable()
{
  static std::string s_containerNameFieldInClassVersionTable = "CONTAINER_ID";
  return s_containerNameFieldInClassVersionTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::mappingVersionFieldInClassVersionTable()
{
  static std::string s_mappingVersionFieldInClassVersionTable = "MAPPING_VERSION";
  return s_mappingVersionFieldInClassVersionTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::mappingElementTable()
{
  static std::string s_mappingElementTable = "POOL_OR_MAPPING_ELEMENTS";
  return s_mappingElementTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::elementTypeFieldInMappingElementTable()
{
  static std::string s_elementTypeFieldInMappingElementTable = "ELEMENT_TYPE";
  return s_elementTypeFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::scopeNameFieldInMappingElementTable()
{
  static std::string s_scopeNameFieldInMappingElementTable = "SCOPE_NAME";
  return s_scopeNameFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::variableNameFieldInMappingElementTable()
{
  static std::string s_variableNameFieldInMappingElementTable = "VARIABLE_NAME";
  return s_variableNameFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::variableTypeFieldInMappingElementTable()
{
  static std::string s_variableTypeFieldInMappingElementTable = "VARIABLE_TYPE";
  return s_variableTypeFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::tableNameFieldInMappingElementTable()
{
  static std::string s_tableNameFieldInMappingElementTable = "TABLE_NAME";
  return s_tableNameFieldInMappingElementTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::versionFieldInMappingElementTable()
{
  static std::string s_positionFieldInMappingElementTable = "VERSION";
  return s_positionFieldInMappingElementTable;
}


std::string
pool::ObjectRelationalMappingOldSchema::mappingElementColumnsTable()
{
  static std::string s_mappingElementColumnsTable = "POOL_OR_MAPPING_COLUMNS";
  return s_mappingElementColumnsTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::versionFieldInMappingElementColumnsTable()
{
  static std::string s_versionFieldInMappingElementColumnsTable = "VERSION";
  return s_versionFieldInMappingElementColumnsTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::scopeNameFieldInMappingElementColumnsTable()
{
  static std::string s_scopeNameFieldInMappingElementColumnsTable = "SCOPE_NAME";
  return s_scopeNameFieldInMappingElementColumnsTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::variableNameFieldInMappingElementColumnsTable()
{
  static std::string s_variableNameFieldInMappingElementColumnsTable = "VARIABLE_NAME";
  return s_variableNameFieldInMappingElementColumnsTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::columnFieldInMappingElementColumnsTable()
{
  static std::string s_columnFieldInMappingElementColumnsTable = "COLUMN_NAME";
  return s_columnFieldInMappingElementColumnsTable;
}

std::string
pool::ObjectRelationalMappingOldSchema::positionFieldInMappingElementColumnsTable()
{
  static std::string s_positionFieldInMappingElementColumnsTable = "POSITION";
  return s_positionFieldInMappingElementColumnsTable;
}
