#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/Exception.h"
#include "POOLCore/CommandOptions.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingUtilities.h"
#include <iostream>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-v";
  opt.property = "version";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the version label of the mapping to retrieve";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-o";
  opt.property = "outputfile";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the mapping xml output file name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::ObjectRelationalMappingUtilities utility;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                            pool::ObjectRelationalMappingUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                              pool::ObjectRelationalMappingUtilities::infoVerbosityPropertyValue());
        }
      }
      std::string theVersion("");
      iter = ops.find("version");
      if(iter!=ops.end()){
        theVersion = iter->second;
      } 
      std::string theMappingOutFile = "mapping.xml";
      iter = ops.find("outputfile");
      if(iter!=ops.end()){
        theMappingOutFile = iter->second;
      } 
        std::string theConnectionString("");
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = iter->second;
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = iter->second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::ObjectRelationalMappingUtilities::passwordProperty(),thePassword);
      }
      utility.initialize();
      utility.connect(theConnectionString,coral::ReadOnly);
      utility.session()->transaction().start(true);
      try{
        if(!theVersion.empty()){
          bool exists = utility.existsMapping(theVersion);
          if(exists){
            utility.retrieveMapping(theVersion,theMappingOutFile);
            std::cout << "Mapping version " << theVersion << " dumped in file " << theMappingOutFile << std::endl;
          } else {
            std::cout << "Mapping version " << theVersion << " not found in the database." << std::endl;
          }
        }
        /**else {
          std::vector<std::string> vers = utility.listMappings();
          if(vers.size()>0){
            utility.retrieveMapping(theMappingOutFile);
            std::cout << "Available mappings dumped in file " << theMappingOutFile << std::endl;
          } else {
            std::cout << "No Mapping found in the database." << std::endl;
          } 
          }**/
        utility.session()->transaction().commit();
      } catch (const std::exception& e){
        utility.session()->transaction().rollback();
        std::cout << "ERROR: " << e.what() << std::endl;
        return 1;
      }
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

