#include "CoralBase/Exception.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "POOLCore/CommandOptions.h"
#include "ObjectRelationalAccess/ObjectRelationalMappingUtilities.h"
#include <exception>
#include <iostream>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  /**opt.flag = "-dry";
  opt.property = "dryRun";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "only show the schema changes involved in the command ";
  opt.required = false;
  ops.push_back(opt); **/ 
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::ObjectRelationalMappingUtilities mappingUtility;
      iter = ops.find("debug");
      if(iter!=ops.end()){
        mappingUtility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                            pool::ObjectRelationalMappingUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          mappingUtility.setProperty(pool::ObjectRelationalMappingUtilities::verbosityProperty(),
                              pool::ObjectRelationalMappingUtilities::infoVerbosityPropertyValue());
        }
      }
      /**bool dryRun = false;
      iter = ops.find("dryRun");
      if(iter!=ops.end()){
        dryRun = true;
        }**/
      std::string theConnectionString("");
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = iter->second;
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = iter->second;
        mappingUtility.setProperty(pool::ObjectRelationalMappingUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        mappingUtility.setProperty(pool::ObjectRelationalMappingUtilities::passwordProperty(),thePassword);
      }
      mappingUtility.initialize();
      mappingUtility.connect(theConnectionString,coral::Update);
      mappingUtility.session()->transaction().start();
      try {
        mappingUtility.upgradeMappingSchema();
        mappingUtility.session()->transaction().commit();
        std::cout << "Mapping schema successfully migrated." << std::endl;
      } catch (const coral::Exception& e){
        std::cout << "ERROR: " << e.what() << std::endl;
        mappingUtility.session()->transaction().rollback();
        return 1;
      }
      
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}


