#include "TokenIterator.h"
#include "StorageSvc/IStorageExplorer.h"
#include "StorageSvc/DbSelect.h"
#include "StorageSvc/DbContainer.h"


pool::PersistencySvc::TokenIterator::TokenIterator( const FileDescriptor& fileDescriptor,
                                                    const std::string& containerName,
                                                    IStorageExplorer& storageExplorer,
                                                    const std::string& selection ):
  m_storageExplorer( storageExplorer ),
  m_selection( new pool::DbSelect( selection ) )
{
  m_storageExplorer.select( fileDescriptor, containerName, *m_selection );
}

pool::PersistencySvc::TokenIterator::~TokenIterator()
{
  delete m_selection;
}


pool::Token*
pool::PersistencySvc::TokenIterator::next()
{
  pool::Token* objectToken = 0;
  if ( ! m_storageExplorer.next( *m_selection, objectToken ).isSuccess() ) return 0;
  else return objectToken;
}


bool
pool::PersistencySvc::TokenIterator::seek (long long int position)
{
  m_selection->link().second = int(position);
  return true;
}


long long int
pool::PersistencySvc::TokenIterator::size ()
{
  return m_selection->container().size();
}


