#ifndef RelationalCollectionMetadata_h
#define RelationalCollectionMetadata_h

#include "CollectionBase/ICollection.h"
#include "CollectionBase/ICollectionMetadata.h"

#include "CoralBase/AttributeList.h"

#include <string>


namespace coral {
   class ISessionProxy;
   class IQuery;
   class ICursor;
   class ITable;
}


namespace pool {
   namespace RelationalCollection { 

      class RelationalCollectionMetadata : public ICollectionMetadata
      {
     public:

        RelationalCollectionMetadata();
        virtual ~RelationalCollectionMetadata();

        void initialize( const std::string& collectionName,
                         coral::ISessionProxy& session,
                         pool::ICollection::OpenMode mode );
        
        // void readKeys() const;

        bool existsKey( const std::string& key ) const;

        const char* getValueForKey( const std::string& key ) const;
        
        void setValueForKey( const std::string& key,
                             const std::string& val );



        class RelationalCollectionMetadataIterator
           : public ICollectionMetadata::ICollectionMetadataIterator
        {
       friend class RelationalCollectionMetadata;
          
          RelationalCollectionMetadataIterator( coral::IQuery *query );
          virtual ~RelationalCollectionMetadataIterator( );

          bool operator==( const ICollectionMetadataIterator& _rhs ) const;
          bool operator!=( const ICollectionMetadataIterator& rhs ) const {
             return !operator==( rhs );
          } 
          void operator++();
             
          const std::string&        key() const;
          const char *                value() const;

       protected:
          coral::IQuery*                m_query;
          coral::ICursor*                m_cursor;
          bool                                m_valid;
        };


        
        ICollectionMetadata::const_iterator        begin() const;
        ICollectionMetadata::const_iterator        end() const;
           
     private:
        ICollection::OpenMode                 m_mode;

        std::string                        m_collectionName;
        
        /// Reference to the metadata table (do not delete)
        coral::ITable*                        m_metadataTable;
        
        mutable coral::AttributeList        m_whereDataForMetadata;
        mutable coral::AttributeList        m_whereDataForMetadataKey;
        
      };  // end RelationalCollectionMetadata

   } // end namespace
}

#endif





