#include "RelationalFileCatalogImplementation.h"
#include "RelationalFileCatalogNames.h"
#include "UserQueryParser.h"
#include "QueryHolder.h"
//#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/ConnectionService.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/ITableSchemaEditor.h"
#include "RelationalAccess/ITablePrivilegeManager.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/SessionException.h"
#include "RelationalAccess/SchemaException.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/AttributeListException.h"

#include "FileCatalog/FCException.h"
#include "POOLCore/Guid.h"
#include "CoralBase/MessageStream.h"

#include "CoralBase/MessageStream.h"

#include "Reflex/PluginService.h"

#include <sstream>
#include <memory>

#include <cstdlib>

namespace pool{

typedef RelationalFileCatalog::RelationalFileCatalogImplementation relfilecat;

PLUGINSVC_FACTORY_WITH_ID(relfilecat, std::string("relationalcatalog"), FCImpl*())
}

pool::RelationalFileCatalog::RelationalFileCatalogImplementation::RelationalFileCatalogImplementation():
  m_session( 0 ),
  m_fieldNameToIndex(),
  m_queryHolder( 0 ),
  m_metaData( 0 ),
  m_rowData( 0 )
{}


pool::RelationalFileCatalog::RelationalFileCatalogImplementation::~RelationalFileCatalogImplementation()
{
  if ( m_queryHolder ) delete m_queryHolder;
  this->disconnect();
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::connect()
{
  // Check whether a connection is already open.
  if ( m_session ) {
    throw pool::FCconnectionException( "RelationalFileCatalog::connect",
                                       "catalog is already connected" );
  }


 
  // Retrieve an IConnectionService
  //std::vector< seal::IHandle< coral::IConnectionService > > loadedServices;
  //pool::POOLContext::context()->query( loadedServices );
  //if ( loadedServices.empty() ) {
    // Load the ConnectionService
    //pool::POOLContext::loadComponent( "CORAL/Services/ConnectionService" );
    //pool::POOLContext::context()->query( loadedServices );
  //}
  //seal::IHandle< coral::IConnectionService > serviceHandle = loadedServices.front();

  std::auto_ptr<coral::ConnectionService> connectionService( new coral::ConnectionService() );
 
  // Get a connection handle
  try {
    //    m_session = serviceHandle->connect( this->url(),
    //                                        coral::Update );

    m_session = connectionService->connect( this->url(),
					    coral::Update );
  }
  catch ( coral::Exception& ) {
    throw pool::FCconnectionException( "RelationalFileCatalog::connect",
                                       "Failed to connect to the database" );
  }


  m_transactionsta.set( 1 );//set connection flag to 1
}



void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::disconnect() const
{
  if ( m_session ) {
    delete m_session;
    m_session = 0;
    if ( m_metaData ) {
      delete m_rowData;
      delete m_metaData;
      m_metaData = 0;
    }
    m_fieldNameToIndex.clear();
    m_transactionsta.reset( 1 );//set connection flag to 0
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::start() const
{
  if ( ! m_session )
    throw pool::FCTransactionException( "RelationalFileCatalog::start",
                                        "Not yet connected to a database. Cannot start a new transaction." );
  if (  m_transactionsta.test( 0 ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::start",
                                        "A transaction has already started" );
  }
  try {
    m_session->transaction().start();
  }
  catch( coral::TransactionNotStartedException& ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::start",
                                        "Could not start a new transaction." );
  }

  coral::ISchema& nominalSchema = m_session->nominalSchema();

  // Check for the existence of the PFN table
  if ( ! ( nominalSchema.existsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() ) ) ) {
    coral::MessageStream log( "RelationalFileCatalog" );
    log << coral::Warning << "Table \"" << pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() << "\" does not exist. Creating it." << coral::MessageStream::endmsg;

    coral::TableDescription description( "RelationalFileCatalog" );
    description.setName( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
    description.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable(),
                              coral::AttributeSpecification::typeNameForType<std::string>() );
    description.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable(),
                              coral::AttributeSpecification::typeNameForType<std::string>(),
                              pool::Guid::null().toString().size(), true );
    description.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable(),
                              coral::AttributeSpecification::typeNameForType<std::string>() );

    description.setPrimaryKey( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() );
    description.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() );

    description.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() );

    description.createIndex( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() + "_IDX",
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() );

    nominalSchema.createTable( description ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
  }

  // Check for the existence of the LFN table
  if ( ! ( nominalSchema.existsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ) ) ) {
    coral::MessageStream log(  "RelationalFileCatalog" );
    log << coral::Warning << "Table \"" << pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() << "\" does not exist. Creating it." << coral::MessageStream::endmsg;

    coral::TableDescription description( "RelationalFileCatalog" );
    description.setName( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
    description.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable(),
                              coral::AttributeSpecification::typeNameForType<std::string>() );
    description.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable(),
                              coral::AttributeSpecification::typeNameForType<std::string>(),
                              pool::Guid::null().toString().size(), true );

    description.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() );
    description.setPrimaryKey( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() );

    description.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() );

    description.createIndex( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() + "_IDX",
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() );
    nominalSchema.createTable( description ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
  }
  m_transactionsta.set( 0 );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::commit( pool::FileCatalog::CommitMode ) const
{
  if ( ! m_transactionsta.test( 0 ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::commit",
                                        "Not in a transaction state. Cannot commit." );
  }
  m_session->transaction().commit();
  if ( m_metaData ) {
    delete m_rowData;
    delete m_metaData;
    m_metaData = 0;
  }
  m_transactionsta.reset( 0 );
  m_fieldNameToIndex.clear();
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::rollback() const
{
  if ( ! m_transactionsta.test( 0 ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::rollback",
                                        "Not in a transaction state. Cannot roll back." );
  }
  m_session->transaction().rollback();
  if ( m_metaData ) {
    delete m_rowData;
    delete m_metaData;
    m_metaData = 0;
  }
  m_transactionsta.reset( 0 );
  m_fieldNameToIndex.clear();
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::registerPFN( const std::string& pfn,
                                                                               const std::string& filetype,
                                                                               pool::FileCatalog::FileID& fid ) const
{
  // Generate a new guid.
  pool::Guid guid;
  pool::Guid::create( guid );
  fid = guid.toString();
  this->addPFNtoGuid( fid, pfn, filetype );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::registerLFN( const std::string& pfn,
                                                                               const std::string& lfn ) const
{
  pool::FileCatalog::FileID fid;
  std::string filetype;
  this->lookupFileByPFN( pfn, fid, filetype );
  if ( fid.empty() )
    throw pool::FCnonexistentFileException( "RelationalFileCatalog::registerLFN", pfn );
  pool::LFNEntry entry( lfn, fid );
  this->insertLFN( entry);
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::registerMetaData( const pool::FileCatalog::FileID& fid,
                                                                                    pool::MetaDataEntry& meta ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::registerMetaData",
                                        "A transaction has not been started." );
  }

  if ( ! m_metaData ) this->loadMetaDataSpecification();

  if ( m_metaData->size() > 1 ) {
    // First construct the attribute list specification for this entry
    coral::ISchema& nominalSchema = m_session->nominalSchema();
    coral::ITable& metaDataTable = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );

    // Construct and fill the attribute list.
    (*m_metaData)[ "guid" ].data< std::string >() = fid;
    const coral::AttributeList& userData = meta.attrs();
    for ( coral::AttributeList::const_iterator iUserData = userData.begin(); iUserData != userData.end(); ++iUserData ){
      if ( iUserData->specification().name() == "guid" ) continue; // This should be improved !!!
      coral::Attribute& attribute = (*m_metaData)[ m_fieldNameToIndex[iUserData->specification().name() ] ];
      attribute.setValueFromAddress( iUserData->addressOfData() );
    }

    // Write the entry
    try {
      metaDataTable.dataEditor().insertRow( *m_rowData );
    }
    catch ( coral::DuplicateEntryInUniqueKeyException& ) {
      throw pool::FCduplicatemetadataException( "RelationalFileCatalog::registerMetaData" );
    }
  }
  else if ( meta.size() > 0 ) {
    this->createMetaDataSpec( meta );
    this->registerMetaData( fid, meta );
  }
  meta.setGuidValue( fid );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::addReplicaPFN( const std::string& pf,
                                                                                 const std::string& rpf ) const
{
  pool::FileCatalog::FileID fid;
  std::string ftype;
  this->lookupFileByPFN( pf, fid, ftype );
  if ( fid.empty() ) {
    throw pool::FCnonexistentFileException( "RelationalFileCatalog::addReplicaPFN", pf );
  }
  this->addPFNtoGuid( fid, rpf, ftype );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::addPFNtoGuid( const pool::FileCatalog::FileID& guid,
                                                                                const std::string& pf,
                                                                                const std::string& filetype ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::addPFNtoGuid",
                                        "A transaction has not been started." );
  }

  // Construct the new entry
  coral::ISchema& nominalSchema = m_session->nominalSchema();
  coral::ITable& pfnTable = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
  coral::AttributeList data;
  pfnTable.dataEditor().rowBuffer( data );
  data[ pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() ].data<std::string>() = pf;
  data[ pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() ].data<std::string>() = guid;
  data[ pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable() ].data<std::string>() = filetype;

  // Try to insert the new row into the table
  try {
    pfnTable.dataEditor().insertRow( data );
  }
  catch ( coral::DuplicateEntryInUniqueKeyException& ) {
    throw pool::FCduplicatePFNException( "RelationalFileCatalog::addPFNToGuid" );
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::renamePFN( const std::string& pf,
                                                                             const std::string& newpf ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::renamePFN",
                                        "A transaction has not been started." );
  }

  std::string setClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() + " = :newpf";
  std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() + " = :pf";
  coral::AttributeList data;
  data.extend<std::string>( "newpf" );
  data.extend<std::string>( "pf" );
  data[0].data<std::string>() = newpf;
  data[1].data<std::string>() = pf;
  coral::ISchema& nominalSchema = m_session->nominalSchema();
  nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() ).dataEditor().updateRows( setClause,
                                                                                                                                         whereClause,
                                                                                                                                         data );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::lookupFileByPFN( const std::string& pf,
                                                                                   pool::FileCatalog::FileID& fid,
                                                                                   std::string& ftype ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::lookupFileByPFN",
                                        "A transaction has not been started." );
  }

  coral::ISchema& nominalSchema = m_session->nominalSchema();
  coral::ITable& table = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
  std::auto_ptr< coral::IQuery > query( table.newQuery() );
  std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() +
    " = :" + pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable();
  coral::AttributeList whereData;
  whereData.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() );
  whereData.begin()->data<std::string>() = pf;
  query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() );
  query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable() );
  query->setCondition( whereClause, whereData );
  query->limitReturnedRows();
  coral::ICursor& cursor = query->execute();
  fid = ftype = ""; // Reset the output
  if ( cursor.next() ) {
    const coral::AttributeList& row = cursor.currentRow();
    fid = row[0].data< std::string >();
    ftype = row[1].data< std::string >();
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::lookupFileByLFN( const std::string& lf,
                                                                                   pool::FileCatalog::FileID& fid) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::lookupFileByLFN",
                                        "A transaction has not been started." );
  }

  coral::ISchema& nominalSchema = m_session->nominalSchema();
  coral::ITable& table = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
  std::auto_ptr< coral::IQuery > query( table.newQuery() );
  std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() + " = :" +
    pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable();
  coral::AttributeList whereData;
  whereData.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() );
  whereData[0].data<std::string>() = lf;
  query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() );
  query->setCondition( whereClause, whereData );
  query->limitReturnedRows();
  coral::ICursor& cursor = query->execute();
  fid = ""; // Reset the output
  if ( cursor.next() ) {
    fid = cursor.currentRow()[0].data<std::string>();
  }
}

 
void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::lookupBestPFN( const pool::FileCatalog::FileID& fid,
                                                                                 const FileCatalog::FileOpenMode&, 
                                                                                 const FileCatalog::FileAccessPattern&,
                                                                                 std::string& pf,
                                                                                 std::string& filetype ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::lookupFileByPFN",
                                        "A transaction has not been started." );
  }

  coral::ISchema& nominalSchema = m_session->nominalSchema();
  coral::ITable& table = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
  std::auto_ptr< coral::IQuery > query( table.newQuery() );
  std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() + " = :" +
    pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable();
  coral::AttributeList whereData;
  whereData.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() );
  whereData[0].data<std::string>() = fid;
  query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() );
  query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable() );
  query->setCondition( whereClause, whereData );
  query->limitReturnedRows(); // This is a temporary implememtation of this method, since we do not have any PFN selector !!!
  coral::ICursor& cursor = query->execute();
  pf = filetype = ""; // Reset the output
  if ( cursor.next() ) {
    const coral::AttributeList& row = cursor.currentRow();
    pf = row[ 0 ].data< std::string >();
    filetype = row[ 1 ].data< std::string >();
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::createMetaDataSpec( pool::MetaDataEntry& schemadef) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::createMetaDataSpec",
                                        "A transaction has not been started." );
  }
  coral::ISchema& nominalSchema = m_session->nominalSchema();
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() );
  if ( m_metaData ) {
    delete m_rowData;
    delete m_metaData;
    m_metaData = 0;
  }

  // Create the meta data description table
  coral::TableDescription description1( "RelationalFileCatalog" );
  description1.setName( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() );
  description1.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable(),
                             coral::AttributeSpecification::typeNameForType<std::string>() );
  description1.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInMetaDataDescriptionTable(),
                             coral::AttributeSpecification::typeNameForType<std::string>() );
  description1.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable(),
                             coral::AttributeSpecification::typeNameForType<int>() );
  description1.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable() );
  description1.setUniqueConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable() );
  description1.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInMetaDataDescriptionTable() );
  description1.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable() );
  description1.setUniqueConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable() );
  coral::ITable& descriptionTable = nominalSchema.createTable( description1 );
  descriptionTable.privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );

  // Fill in the values
  coral::AttributeList data;
  data.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable() );
  data.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInMetaDataDescriptionTable() );
  data.extend<int>( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable() );
  int position = 0;
  coral::AttributeList spec=schemadef.attrs();
  for ( coral::AttributeList::const_iterator iAt= spec.begin();
        iAt != spec.end(); ++iAt, ++position ) {
    if ( iAt->specification().name() == "guid" ) continue; // This should be improved !!!

    data[ 0 ].data<std::string>() = iAt->specification().name();
    data[ 1 ].data<std::string>() = iAt->specification().typeName();
    data[ 2 ].data<int>() = position;
    descriptionTable.dataEditor().insertRow( data );
  }

  // Refresh the meta data specification from the database.
  this->loadMetaDataSpecification();

  // Create the meta data table
  coral::TableDescription description2( "RelationalFileCatalog" );
  description2.setName( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
  description2.insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable(),
                             coral::AttributeSpecification::typeNameForType<std::string>(),
                             pool::Guid::null().toString().size(), true );
  position = 0;
  for ( coral::AttributeList::const_iterator iAttribute = m_metaData->begin();
        iAttribute != m_metaData->end(); ++iAttribute, ++position ) {
    if ( iAttribute->specification().name() == "guid" ) {
      continue; // This should be improved !!!
    }
    const std::string columnName = pool::RelationalFileCatalog::RelationalFileCatalogNames::fieldInMetaDataTable( position );
    const std::string typeName = iAttribute->specification().typeName();
    description2.insertColumn( columnName,
                               typeName );
  }
  description2.setNotNullConstraint( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable() );
  description2.setPrimaryKey( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable() );
  nominalSchema.createTable( description2 ).privilegeManager().grantToPublic( coral::ITablePrivilegeManager::Select );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::updateMetaDataSpec( pool::MetaDataEntry& schemadef,
                                                                                      const pool::FileCatalog::MetaUpdateMode metamode ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::lookupFileByPFN",
                                        "A transaction has not been started." );
  }

  if ( ! m_metaData ) this->loadMetaDataSpecification();

  if ( m_metaData->size() == 1 ) { // No meta data yet. Create tables from scratch.
    this->createMetaDataSpec( schemadef );
  }
  else {
    coral::ISchema& nominalSchema = m_session->nominalSchema();
    if ( metamode == pool::FileCatalog::DELETE_REDUNDANT ) { // Drop redundant columns first.
      bool droppedOne = false;
      for( coral::AttributeList::const_iterator iAttribute = m_metaData->begin();
           iAttribute != m_metaData->end(); ++iAttribute ) {
        std::string columnName = iAttribute->specification().name();
        try{
	  schemadef.attrs()[columnName];
        }catch(const coral::AttributeListException& er){
          droppedOne = true;
          std::string metaDataColumnName = pool::RelationalFileCatalog::RelationalFileCatalogNames::fieldInMetaDataTable( m_fieldNameToIndex[ columnName ] );
          nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ).schemaEditor().dropColumn( metaDataColumnName );
          std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable() + " = :columnName";
          coral::AttributeList whereData;
          whereData.extend<std::string>( "columnName" );
          whereData[0].data<std::string>() = columnName;
          nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() ).dataEditor().deleteRows( whereClause, whereData );
        }
      }
      if ( droppedOne ) this->loadMetaDataSpecification();
    }

    // Append the new columns.
    int position = 0;
    for ( std::map< std::string, int >::const_iterator iColumn = m_fieldNameToIndex.begin();
          iColumn != m_fieldNameToIndex.end(); ++iColumn )
      if ( iColumn->second > position )
        position = iColumn->second;

    coral::ITable& descriptionTable = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() );
    coral::ITable& table = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
    coral::AttributeList data;
    data.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable() );
    data.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInMetaDataDescriptionTable() );
    data.extend<int>( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable() );
    bool addedOne = false;
    coral::AttributeList spec=schemadef.attrs();
    for( coral::AttributeList::const_iterator iAt = spec.begin();
         iAt != spec.end(); ++iAt ) {
      if ( iAt->specification().name() == "guid" ) continue; // This should be improved !!!
      addedOne = true;
      std::string columnName = iAt->specification().name();
      try {
        (*m_metaData)[ columnName ];
      }
      catch ( coral::AttributeListException& ) {
        std::string columnType = iAt->specification().typeName();
        ++position;
        data[ 0 ].data<std::string>() = columnName;
        data[ 1 ].data<std::string>() = columnType;
        data[ 2 ].data<int>() = position;
        descriptionTable.dataEditor().insertRow( data );
        table.schemaEditor().insertColumn( pool::RelationalFileCatalog::RelationalFileCatalogNames::fieldInMetaDataTable( position ),
                                           columnType );
      }
    }
    if ( addedOne ) this->loadMetaDataSpecification();
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::getMetaDataSpec( pool::MetaDataEntry& schemadef) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::getMetaDataSpec",
                                        "A transaction has not been started." );
  }

  if ( ! m_metaData )
    this->loadMetaDataSpecification();

  pool::MetaDataEntry entry;
  for ( coral::AttributeList::const_iterator iAttribute = m_metaData->begin();
        iAttribute != m_metaData->end(); ++iAttribute ) {
    if ( iAttribute->specification().name() == "guid" ) continue; // This should be improved !!!
    entry.addAttributeSpec( iAttribute->specification().name(), iAttribute->specification().typeName() );
  }
  schemadef = entry;
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::dropMetaDataSpec() const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::lookupFileByPFN",
                                        "A transaction has not been started." );
  }
  coral::ISchema& nominalSchema = m_session->nominalSchema();
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
  nominalSchema.dropIfExistsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() );
  if ( m_metaData ) {
    delete m_rowData;
    delete m_metaData;
    m_metaData = 0;
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::insertPFN( pool::PFNEntry& pentry) const
{
  this->addPFNtoGuid( pentry.guid(), pentry.pfname(), pentry.filetype() );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::insertLFN( pool::LFNEntry& lentry) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::insertLFN",
                                        "A transaction has not been started." );
  }

  // Construct the new entry
  coral::ISchema& nominalSchema = m_session->nominalSchema();
  coral::ITable& lfnTable = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
  coral::AttributeList data;
  lfnTable.dataEditor().rowBuffer( data );
  data[ pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() ].data<std::string>() = lentry.lfname();
  data[ pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() ].data<std::string>() = lentry.guid();

  // Try to insert the new row into the table
  try {
    lfnTable.dataEditor().insertRow( data );
  }
  catch ( coral::DuplicateEntryInUniqueKeyException& ) {
    throw pool::FCduplicateLFNException( "RelationalFileCatalog::insertLFN" );
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::insertMetaData( pool::MetaDataEntry& mentry) const
{
  this->registerMetaData( mentry.getGuidValue(), mentry );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::deletePFN( const std::string& pf ) const
{
  coral::MessageStream log( "RelationalFileCatalog" );
  
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::deletePFN",
                                        "A transaction has not been started." );
  }

  try
  {
    coral::ISchema& nominalSchema = m_session->nominalSchema();
    coral::ITable& pfnTable = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
    coral::IQuery* query = pfnTable.newQuery();
    std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() + " = :pf";
    coral::AttributeList whereData;
    whereData.extend<std::string>( "pf" );
    whereData[0].data<std::string>()= pf;
    query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() );
    query->setCondition( whereClause, whereData );
    query->limitReturnedRows();
    coral::ICursor& cursor = query->execute();
    std::string myguid;
    if ( cursor.next() )
      myguid = cursor.currentRow()[0].data<std::string>();
    else
      return;
    delete query;

    coral::IQuery* pfncounter = pfnTable.newQuery();
    whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() + " = :pf";
    whereData[0].data<std::string>() = myguid;
    pfncounter->setCondition( whereClause, whereData );
    pfncounter->limitReturnedRows();
    coral::ICursor& cursor2 = pfncounter->execute();
    bool thereisresult = cursor2.next();
    delete pfncounter;

    if( thereisresult )
    {
      std::string w1=pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() + " = :pf";
      nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ).dataEditor().deleteRows( w1, whereData );
      
      if ( nominalSchema.existsTable(  pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) )
      {
        std::string w2=pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable() + " = :pf";
        nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ).dataEditor().deleteRows( w2, whereData );
      }
  
      whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() + " = :pf";
      whereData[0].data<std::string>() = pf;
      pfnTable.dataEditor().deleteRows( whereClause, whereData );
    }
  }
  catch( const coral::Exception& re )
  {
    log << coral::Error << "A CORAL exception has been raised: " << re.what() << coral::MessageStream::endmsg;
    throw re;
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::deleteLFN( const std::string& lf ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::deleteLFN",
                                        "A transaction has not been started." );
  }

  std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() + " = :lf";
  coral::AttributeList whereData;
  whereData.extend<std::string>( "lf" );
  whereData[0].data<std::string>() = lf;
  coral::ISchema& nominalSchema = m_session->nominalSchema();
  nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ).dataEditor().deleteRows( whereClause,
                                                                                                                                         whereData );
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::deleteMetaData( const pool::FileCatalog::FileID& guid ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::deleteMetaData",
                                        "A transaction has not been started." );
  }

  coral::ISchema& nominalSchema = m_session->nominalSchema();
  if ( nominalSchema.existsTable(  pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) ) {
    std::string whereClause = pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable() + " = :guid";
    coral::AttributeList whereData;
    whereData.extend<std::string>( "guid" );
    whereData[0].data<std::string>() = guid;
    nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ).dataEditor().deleteRows( whereClause,
                                                                                                                                                whereData );
  }
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::deleteEntry( const pool::FileCatalog::FileID& guid ) const
{
  if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
    throw pool::FCTransactionException( "RelationalFileCatalog::deleteEntry",
                                        "A transaction has not been started." );
  }

  coral::AttributeList whereData;
  whereData.extend<std::string>( "guid" );
  whereData[0].data<std::string>() = guid;

  coral::ISchema& nominalSchema = m_session->nominalSchema();

  std::string whereClause1 = pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() + " = :guid";
  nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() ).dataEditor().deleteRows( whereClause1,
                                                                                                                                         whereData );
  std::string whereClause2 = pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() + " = :guid";
  nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ).dataEditor().deleteRows( whereClause2,
                                                                                                                                         whereData);

  // Delete the meta data entries as well.
  if ( nominalSchema.existsTable(  pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) ) {
    std::string whereClause3 = pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable() + " = :guid";
    nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ).dataEditor().deleteRows( whereClause3,
                                                                                                                                                whereData );
  }
}


bool
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::isReadOnly() const
{
  return false;
}


bool
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::retrievePFN( const std::string& query,
                                                                               pool::FCBuf<pool::PFNEntry>& buf,
                                                                               const size_t& start )
{
  size_t bufferSize = buf.maxsize();

  if ( start == 0 ) {
    if ( m_queryHolder ) {
      delete m_queryHolder;
      m_queryHolder = 0;
    }

    if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
      throw pool::FCTransactionException( "RelationalFileCatalog::retrievePFN",
                                          "A transaction has not been started." );
    }
    
    if ( ! m_metaData ) this->loadMetaDataSpecification();
    
    coral::ISchema& nominalSchema = m_session->nominalSchema();
    coral::IQuery* squery = nominalSchema.newQuery();
    squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() + "." +
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable(),
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() );
    squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() + "." +
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable(),
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() );
    squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() + "." +
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable(),
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable() );
    squery->setRowCacheSize( bufferSize );
    
    if ( ! query.empty() ) {  // Treat the query here.
      std::pair< std::string, std::vector< std::string > >
        parseResult = pool::RelationalFileCatalog::UserQueryParser::parse( query,
                                                                           m_fieldNameToIndex,
                                                                           pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );

      coral::AttributeList emptyAttributeList;
      squery->setCondition( parseResult.first, emptyAttributeList );
      for ( std::vector< std::string >::const_iterator iTable = parseResult.second.begin();
            iTable != parseResult.second.end(); ++iTable ) {
        if ( *iTable == pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() &&
             ! nominalSchema.existsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) ) {
          return false;
        }
        squery->addToTableList( *iTable );
      }
    }
    else {
      squery->addToTableList( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
    }
    
    m_queryHolder = new pool::RelationalFileCatalog::QueryHolder( squery );
  }

  buf.clear();
  if ( ! m_queryHolder ) return false;
  coral::ICursor& cursor = m_queryHolder->cursor();
  for ( size_t i = 0; i < bufferSize; ++i ) {
    if ( cursor.next() ) {
      const coral::AttributeList& currentRow = cursor.currentRow();
      buf.push_back( pool::PFNEntry( currentRow[ pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable() ].data<std::string>(),
                                     currentRow[ pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() ].data<std::string>(),
                                     currentRow[ pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable() ].data<std::string>() ) );
    }
    else {
      delete m_queryHolder;
      m_queryHolder = 0;
      break;
    }
  }

  if ( buf.empty() ) return false;
  else return true;
}


bool
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::retrieveLFN( const std::string& query,
                                                                               pool::FCBuf<pool::LFNEntry>& buf,
                                                                               const size_t& start )
{
  size_t bufferSize = buf.maxsize();

  if ( start == 0 ) {
    if ( m_queryHolder ) {
      delete m_queryHolder;
      m_queryHolder = 0;
    }

    if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
      throw pool::FCTransactionException( "RelationalFileCatalog::retrieveLFN",
                                          "A transaction has not been started." );
    }
    
    if ( ! m_metaData ) this->loadMetaDataSpecification();
    
    coral::ISchema& nominalSchema = m_session->nominalSchema();
    coral::IQuery* squery = nominalSchema.newQuery();
    squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() + "." +
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable(),
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() );
    squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() + "." +
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable(),
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() );
    squery->setRowCacheSize( bufferSize );
  
    coral::AttributeList emptyAttributeList;
    if ( ! query.empty() ) {  // Treat the query here.
      std::pair< std::string, std::vector< std::string > > parseResult = pool::RelationalFileCatalog::UserQueryParser::parse( query,
                                                                                                                              m_fieldNameToIndex,
                                                                                                                              pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
      squery->setCondition( parseResult.first, emptyAttributeList );
      for ( std::vector< std::string >::const_iterator iTable = parseResult.second.begin();
            iTable != parseResult.second.end(); ++iTable ) {
        if ( *iTable == pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() &&
             ! nominalSchema.existsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) ) {
          return false;
        }
        squery->addToTableList( *iTable );
      }
    }
    else {
      squery->addToTableList( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
    }

    m_queryHolder = new pool::RelationalFileCatalog::QueryHolder( squery );
  }

  buf.clear();
  if ( ! m_queryHolder ) return false;
  coral::ICursor& cursor = m_queryHolder->cursor();
  for ( size_t i = 0; i < bufferSize; ++i ) {
    if ( cursor.next() ) {
      const coral::AttributeList& currentRow = cursor.currentRow();
      buf.push_back( pool::LFNEntry( currentRow[ pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable() ].data<std::string>(),
                                     currentRow[ pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() ].data<std::string>() ) );
    }
    else {
      delete m_queryHolder;
      m_queryHolder = 0;
      break;
    }
  }

  if ( buf.empty() ) return false;
  else return true;
}



bool
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::retrieveMetaData( const std::string& query,
                                                                                    pool::FCBuf<pool::MetaDataEntry>& buf,
                                                                                    const size_t& start )
{
  size_t bufferSize = buf.maxsize();

  if ( start == 0 ) {
    if ( m_queryHolder ) {
      delete m_queryHolder;
      m_queryHolder = 0;
    }

    if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
      throw pool::FCTransactionException( "RelationalFileCatalog::retrieveMetaData",
                                          "A transaction has not been started." );
    }

    coral::ISchema& nominalSchema = m_session->nominalSchema();
    if ( ! nominalSchema.existsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) ) {
      return false;
    }

    if ( ! m_metaData ) this->loadMetaDataSpecification();

    coral::IQuery* squery = nominalSchema.newQuery();
    squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() + "." +
                             pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable(),
                             "guid" );
    for ( coral::AttributeList::const_iterator iAttribute = m_metaData->begin();
          iAttribute != m_metaData->end(); ++iAttribute ) {
      std::string name = iAttribute->specification().name();
      if ( name == "guid" ) continue;
      int index = m_fieldNameToIndex[ name ];
      squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() + "." +
                               pool::RelationalFileCatalog::RelationalFileCatalogNames::fieldInMetaDataTable( index ),
                               name );
    }

    coral::AttributeList emptyAttributeList;
    if ( ! query.empty() ) {  // Treat the query here.
      std::pair< std::string, std::vector< std::string > > parseResult = pool::RelationalFileCatalog::UserQueryParser::parse( query,
                                                                                                                              m_fieldNameToIndex,
                                                                                                                              pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
      squery->setCondition( parseResult.first, emptyAttributeList );
      for ( std::vector< std::string >::const_iterator iTable = parseResult.second.begin();
            iTable != parseResult.second.end(); ++iTable )
        squery->addToTableList( *iTable );
    }
    else {
      squery->addToTableList( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
    }
    
    squery->setRowCacheSize( bufferSize );

    m_queryHolder = new pool::RelationalFileCatalog::QueryHolder( squery );
  }

  buf.clear();
  if ( ! m_queryHolder ) return false;
  coral::ICursor& cursor = m_queryHolder->cursor();
  for ( size_t i = 0; i < bufferSize; ++i ) {
    if ( cursor.next() ) {
      const coral::AttributeList& currentRow = cursor.currentRow();
      pool::MetaDataEntry entry;

      for ( coral::AttributeList::const_iterator iAttribute = m_metaData->begin();
            iAttribute != m_metaData->end(); ++iAttribute ) {
        if ( iAttribute->specification().name() == "guid" ) continue;
        entry.addAttributeSpec( iAttribute->specification().name(), iAttribute->specification().typeName() );
      }

      // Copy the data to the user defined row
      coral::AttributeList& userData = entry.attrs();
      for ( coral::AttributeList::const_iterator iAttribute = currentRow.begin();
            iAttribute != currentRow.end(); ++iAttribute ) {
        iAttribute->copyValueToAddress( userData[iAttribute->specification().name()].addressOfData() );
      }

      buf.push_back( entry );
    }
    else {
      delete m_queryHolder;
      m_queryHolder = 0;
      break;
    }
  }

  if ( buf.empty() ) return false;
  else return true;
}


bool
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::retrieveGuid( const std::string& query,
                                                                                pool::FCBuf<pool::FileCatalog::FileID>& buf,
                                                                                const size_t& start )
{
  size_t bufferSize = buf.maxsize();

  if ( start == 0 ) {
    if ( m_queryHolder ) {
      delete m_queryHolder;
      m_queryHolder = 0;
    }

    if ( ! ( m_transactionsta.test( 0 ) && m_transactionsta.test( 1 ) ) ) {
      throw pool::FCTransactionException( "RelationalFileCatalog::retrieveGuid",
                                          "A transaction has not been started." );
    }

    if ( ! m_metaData ) this->loadMetaDataSpecification();
    
    coral::ISchema& nominalSchema = m_session->nominalSchema();
    coral::IQuery* squery = nominalSchema.newQuery();
    std::pair< std::string, std::vector< std::string > > parseResult = pool::RelationalFileCatalog::UserQueryParser::parse( query,
                                                                                                                            m_fieldNameToIndex,
                                                                                                                            "" );
    coral::AttributeList emptyAttributeList;
    if ( ! parseResult.first.empty() ) {
      squery->setCondition( parseResult.first, emptyAttributeList );
    }

    for ( std::vector< std::string >::const_iterator iTable = parseResult.second.begin();
          iTable != parseResult.second.end(); ++iTable ) {
      if ( *iTable == pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() &&
           ! nominalSchema.existsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) ) {
        return false;
      }
      squery->addToTableList( *iTable );
    }

    const std::string& guidTable = parseResult.second.front();
    if ( guidTable == pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() ) {
      squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() + "." +
                               pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable(),
                               pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable() );
    }
    else if ( guidTable == pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ) {
      squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() + "." +
                               pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable(),
                               pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable() );
    }
    else {
      squery->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() + "." +
                               pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable(),
                               pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable() );
    }

    squery->setRowCacheSize( bufferSize );

    m_queryHolder = new pool::RelationalFileCatalog::QueryHolder( squery );
  }

  buf.clear();
  if ( ! m_queryHolder ) return false;
  coral::ICursor& cursor = m_queryHolder->cursor();
  for ( size_t i = 0; i < bufferSize; ++i ) {
    if ( cursor.next() ) {
      buf.push_back( cursor.currentRow()[0].data<std::string>() );
    }
    else {
      delete m_queryHolder;
      m_queryHolder = 0;
      break;
    }
  }

  if ( buf.empty() ) return false;
  else return true;
}


void
pool::RelationalFileCatalog::RelationalFileCatalogImplementation::loadMetaDataSpecification() const
{
  if ( m_metaData ) {
    delete m_rowData;
    delete m_metaData;
  }
  m_fieldNameToIndex.clear();
  m_metaData = new coral::AttributeList;
  m_rowData = new coral::AttributeList;
  m_metaData->extend<std::string>( "guid" );
  m_rowData->extend< std::string >( pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable() );
  coral::ISchema& nominalSchema = m_session->nominalSchema();
  if ( nominalSchema.existsTable( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() ) ) {
    coral::ITable& table = nominalSchema.tableHandle( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataDescriptionTable() );
    std::auto_ptr< coral::IQuery > query( table.newQuery() );
    coral::AttributeList row;
    query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable() );
    row.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::nameFieldInMetaDataDescriptionTable() );
    query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInMetaDataDescriptionTable() );
    row.extend<std::string>( pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInMetaDataDescriptionTable() );
    query->addToOutputList( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable() );
    row.extend<int>( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable() );
    query->defineOutput( row );
    query->addToOrderList( pool::RelationalFileCatalog::RelationalFileCatalogNames::positionFieldInMetaDataDescriptionTable() );
    query->setRowCacheSize( 100 );
    coral::ICursor& cursor = query->execute();
    while ( cursor.next() ) {
      std::string variableName = row[ 0 ].data<std::string>();
      std::string variableType = row[ 1 ].data<std::string>();
      int position = row[ 2 ].data<int>();
      m_metaData->extend( variableName, variableType );
      m_rowData->extend( pool::RelationalFileCatalog::RelationalFileCatalogNames::fieldInMetaDataTable( position ), variableType );
      m_fieldNameToIndex.insert( std::make_pair( variableName, position ) );
    }
  }

  coral::AttributeList::iterator iRowData = m_rowData->begin();
  coral::AttributeList::const_iterator iMetaData = m_metaData->begin();
  while( iRowData != m_rowData->end() ) {
    iRowData->shareData( *iMetaData );
    ++iRowData;
    ++iMetaData;
  }

}
