#ifndef RELATIONALFILECATALOG_RELATIONALFILECATALOGNAMES_H
#define RELATIONALFILECATALOG_RELATIONALFILECATALOGNAMES_H

#include <string>

namespace pool {

  namespace RelationalFileCatalog {

    /**
     * @class RelationalFileCatalogNames RelationalFileCatalogNames.h
     *
     * A class holding all the string constants used in the RelationalFileCatalogPackage
     */
    class RelationalFileCatalogNames {
    public:
      /// Returns the name of the table holding the PFN <-> FID mapping.
      static const std::string& pfnTable() { return s_pfnTable; }

      /// Returns the name of the PFN field in the table holding the PFN <-> FID mapping
      static const std::string& pfnFieldInPfnTable() { return s_pfnFieldInPfnTable; }

      /// Returns the name of the FID field in the table holding the PFN <-> FID mapping
      static const std::string& fidFieldInPfnTable() { return s_fidFieldInPfnTable; }

      /// Returns the name of the type field in the table holding the PFN <-> FID mapping
      static const std::string& typeFieldInPfnTable() { return s_typeFieldInPfnTable; }

      /// Returns the name of the table holding the LFN <-> FID mapping.
      static const std::string& lfnTable() { return s_lfnTable; }

      /// Returns the name of the LFN field in the table holding the LFN <-> FID mapping
      static const std::string& lfnFieldInLfnTable() { return s_lfnFieldInLfnTable; }

      /// Returns the name of the FID field in the table holding the LFN <-> FID mapping
      static const std::string& fidFieldInLfnTable() { return s_fidFieldInLfnTable; }

      /// Returns the name of the table holding the description of the metadata
      static const std::string& metaDataDescriptionTable() { return s_metaDataDescriptionTable; }

      /// Returns the name of the table holding the metadata
      static const std::string& metaDataTable() { return s_metaDataTable; }

      /// Returns the name field in the table holding the description of the metadata
      static const std::string& nameFieldInMetaDataDescriptionTable() { return s_nameFieldInMetaDataDescriptionTable; }

      /// Returns the type field in the table holding the description of the metadata
      static const std::string& typeFieldInMetaDataDescriptionTable() { return s_typeFieldInMetaDataDescriptionTable; }

      /// Returns the position field in the table holding the description of the metadata
      static const std::string& positionFieldInMetaDataDescriptionTable() { return s_positionFieldInMetaDataDescriptionTable; }

      /// Returns the FID field in the table holding the metadata
      static const std::string& fidFieldInMetaDataTable() { return s_fidFieldInMetaDataTable; }

      /// Returns the field in the metadata table which corresponds to an attribute of a given index.
      static std::string fieldInMetaDataTable( int index );


    private:
      /// The name of the table holding the PFN <-> FID mapping.
      static const std::string s_pfnTable;

      /// The name of the PFN field in the table holding the PFN <-> FID mapping
      static const std::string s_pfnFieldInPfnTable;

      /// The name of the FID field in the table holding the PFN <-> FID mapping
      static const std::string s_fidFieldInPfnTable;

      /// The name of the type field in the table holding the PFN <-> FID mapping
      static const std::string s_typeFieldInPfnTable;

      /// The name of the table holding the LFN <-> FID mapping.
      static const std::string s_lfnTable;

      /// The name of the LFN field in the table holding the LFN <-> FID mapping
      static const std::string s_lfnFieldInLfnTable;

      /// The name of the FID field in the table holding the LFN <-> FID mapping
      static const std::string s_fidFieldInLfnTable;

      /// The name of the table holding the description of the metadata
      static const std::string s_metaDataDescriptionTable;

      /// The name of the table holding the metadata
      static const std::string s_metaDataTable;

      /// The name field in the table holding the description of the metadata
      static const std::string s_nameFieldInMetaDataDescriptionTable;

      /// The type field in the table holding the description of the metadata
      static const std::string s_typeFieldInMetaDataDescriptionTable;

      /// The position field in the table holding the description of the metadata
      static const std::string s_positionFieldInMetaDataDescriptionTable;

      /// The FID field in the table holding the metadata
      static const std::string s_fidFieldInMetaDataTable;

    };

  }

}

#endif
