// -*- C++ -*-
// $Id: RelationalMetaDataCatalogImplementation.h,v 1.4 2008/02/20 09:22:51 witoldp Exp $
#ifndef RELATIONALFILECATALOG_RELATIONALMETADATACATALOGIMPLEMENTATION_H
#define RELATIONALFILECATALOG_RELATIONALMETADATACATALOGIMPLEMENTATION_H 1

#include "FileCatalog/FCMetaImpl.h"
#include "FileCatalog/FCBuf.h"

namespace pool
{
  class FCImpl;
  class MetaDataEntry;
  
  namespace RelationalFileCatalog
  {
    class RelationalFileCatalogImplementation;
    
    /**
     * @class RelationalMetaDataCatalogImplementation RelationalMetaDataCatalogImplementation.h
     *
     * Implementation of the pool::FCMetaImpl interface for a relational database
     */
    class RelationalMetaDataCatalogImplementation : public FCMetaImpl
    {
      public:
        /** Default Constructor */
        RelationalMetaDataCatalogImplementation( FCImpl* fc );

        /** Virtual Destructor */
        virtual ~RelationalMetaDataCatalogImplementation();
      
        virtual void insertMetaData(MetaDataEntry& mentry) const;
      
        /**create meta data definition in the catalog
           @param schemadef [IN] schema definition.
        */
        virtual void createMetaDataSpec(MetaDataEntry& schemadef) const;

        /**update meta data definition in the catalog
           @param schemadef [IN] schema definition to add
        */
        virtual void updateMetaDataSpec(MetaDataEntry& newschemadef, const FileCatalog::MetaUpdateMode metamode) const;

        /**get the metadata schema of the catalog
           @param spec [OUT] catalog meta data schema
        */
        virtual void getMetaDataSpec(MetaDataEntry& spec) const;

        /**drop the meta data definition from the catalog
         */
        virtual void dropMetaDataSpec() const;

        virtual void deleteMetaData(const FileCatalog::FileID& guid) const;

        virtual bool retrievePFN(const std::string& query, FCBuf<PFNEntry>& buf, const size_t& start );

        virtual bool retrieveLFN(const std::string& query, FCBuf<LFNEntry>& buf, const size_t& start );

        virtual bool retrieveMetaData(const std::string& query, FCBuf<MetaDataEntry>& buf, const size_t& start );
        
        virtual bool retrieveGuid(const std::string& query, FCBuf<FileCatalog::FileID>& buf, const size_t& start );        

      private:
        mutable RelationalFileCatalogImplementation* m_rfc;
    };
  }
}

#endif // RELATIONALFILECATALOG_RELATIONALMETADATACATALOGIMPLEMENTATION_H
