#include "UserQueryParser.h"
#include "RelationalFileCatalogNames.h"
#include <set>
#include <sstream>


static void
identifyWord( const std::string& lastWord,
              const std::map< std::string, int >& metadataVariables,
              std::set< std::string >& tablesInvolved,
              std::ostream& parsingOutput,
              const std::string& tableForGuid )
{
  std::map< std::string, int >::const_iterator iVariable = metadataVariables.find( lastWord );
  if ( iVariable != metadataVariables.end() ) {
    parsingOutput << pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable()
                  << "."
                  << pool::RelationalFileCatalog::RelationalFileCatalogNames::fieldInMetaDataTable( iVariable->second );
    tablesInvolved.insert( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
  }
  else if ( lastWord == "guid" ) {
    if ( tableForGuid == pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) {
      parsingOutput << pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable()
                    << "."
                    << pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable();
      tablesInvolved.insert( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
    }
    else if ( tableForGuid == pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ) {
      parsingOutput << pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable()
                    << "."
                    << pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable();
      tablesInvolved.insert( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
    }
    else {
      parsingOutput << pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable()
                    << "."
                    << pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable();
      tablesInvolved.insert( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
    }
  }
  else if ( lastWord == "pfname" ) {
    parsingOutput << pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable()
                  << "."
                  << pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnFieldInPfnTable();
    tablesInvolved.insert( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
  }
  else if ( lastWord == "filetype" ) {
    parsingOutput << pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() 
                  << "."
                  << pool::RelationalFileCatalog::RelationalFileCatalogNames::typeFieldInPfnTable();
    tablesInvolved.insert( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
  }
  else if ( lastWord == "lfname" ) {
    parsingOutput << pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable()
                  << "."
                  << pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnFieldInLfnTable();
    tablesInvolved.insert( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
  }
  else {
    parsingOutput << lastWord;
  }

  if ( ! tableForGuid.empty() ) {
    tablesInvolved.insert( tableForGuid );
  }

}


pool::RelationalFileCatalog::UserQueryParser::ParserResult
pool::RelationalFileCatalog::UserQueryParser::parse( const std::string& userQuery,
                                                     const std::map< std::string, int >& metadataVariables,
                                                     const std::string& tableForGuid )
{
  std::ostringstream parsingOutput;
  std::set< std::string > tablesInvolved;

  std::string lastWord = "";
  bool withinQuotes = false;
  for ( std::string::size_type i = 0; i < userQuery.size(); ++i ) {
    char c = userQuery[i];

    if ( c == '\'' ) {
      if ( withinQuotes ) {
        withinQuotes = false;
      }
      else {
        if ( ! lastWord.empty() ) {
          identifyWord( lastWord, metadataVariables, tablesInvolved, parsingOutput, tableForGuid );
          lastWord = "";
        }
        withinQuotes = true;
      }
      parsingOutput << c;
      continue;
    }

    if ( withinQuotes ) {
      parsingOutput << c;
      continue;
    }

    if ( c == ' ' || c == '\t' || c == '\n' || c == '\r' ||
         c == '`' || c == '~' || c == '!' || c == '@' || c == '%' ||
         c == '^' || c == '&' || c == '*' || c == '(' || c == ')' ||
         c == '-' || c == '+' || c == '=' || c == '[' || c == ']' ||
         c == '{' || c == '}' || c == ';' || c == ':' || c == '"' ||
         c == '\\' || c == '|' || c == '<' || c == '>' || c == ',' ||
         c == '.' || c == '?' || c == '/' ) {
      if ( ! lastWord.empty() ) {
        identifyWord( lastWord, metadataVariables, tablesInvolved, parsingOutput, tableForGuid );
        lastWord = "";
      }
      parsingOutput << c;
    }
    else {
      lastWord += c;
    }
  }

  // Add whatever is left over.
  if ( ! lastWord.empty() ) {
    identifyWord( lastWord, metadataVariables, tablesInvolved, parsingOutput, tableForGuid );
  }

  // Deterimine the list of tables
  std::string sparsingOutput = parsingOutput.str();
  std::vector< std::string > vtablesInvolved;
  if ( sparsingOutput.empty() ) {
    vtablesInvolved.push_back( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
  }
  else {
    sparsingOutput = "( " + sparsingOutput + " )";
    if ( tablesInvolved.find( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() ) != tablesInvolved.end() ) {
      vtablesInvolved.push_back( pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() );
      if ( tablesInvolved.find( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ) != tablesInvolved.end() ) {
        vtablesInvolved.push_back( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
        sparsingOutput += " AND " + pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() + "." + pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable();
        sparsingOutput += " = " + pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() + "." + pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable();
      }
      if ( tablesInvolved.find( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() ) != tablesInvolved.end() ) {
        vtablesInvolved.push_back( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
        sparsingOutput += " AND " + pool::RelationalFileCatalog::RelationalFileCatalogNames::metaDataTable() + "." + pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInMetaDataTable();
        sparsingOutput += " = " + pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() + "." + pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable();
      }
    }
    else {
      if ( tablesInvolved.find( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() ) != tablesInvolved.end() ) {
        vtablesInvolved.push_back( pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() );
        if ( tablesInvolved.find( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() ) != tablesInvolved.end() ) {
          vtablesInvolved.push_back( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
          sparsingOutput += " AND " + pool::RelationalFileCatalog::RelationalFileCatalogNames::lfnTable() + "." + pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInLfnTable();
          sparsingOutput += " = " + pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() + "." + pool::RelationalFileCatalog::RelationalFileCatalogNames::fidFieldInPfnTable();
        }
      }
      else {
        vtablesInvolved.push_back( pool::RelationalFileCatalog::RelationalFileCatalogNames::pfnTable() );
      }
    }
  }

  return std::make_pair( sparsingOutput, vtablesInvolved );
}
