#include <iostream>
#include <stdexcept>
#include <memory>
#include <cstdlib>

#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/URIParser.h"
#include "POOLCore/POOLContext.h"

void resetCatalog( const std::string& databaseConnection );

int main( int, char** )
{
  try {
    pool::POOLContext::loadComponent( "SEAL/Services/MessageService" );
    // pool::POOLContext::setMessageVerbosityLevel( coral::Warning );

    char my_arg1[] = "CORAL_AUTH_USER=ral_writer";
    char my_arg2[] = "CORAL_AUTH_PASSWORD=ral";

    ::putenv( my_arg1 );
    ::putenv( my_arg2 );

    const std::string databaseConnection = "oracle://devdb10/ral_writer";

    resetCatalog( databaseConnection );

    pool::IFileCatalog fileCatalog;
    pool::URIParser p( "relationalcatalog_" + databaseConnection );
    p.parse();

    fileCatalog.setWriteCatalog( p.contactstring() );
    fileCatalog.connect();
    fileCatalog.start();

    std::string fileType = "default";
    std::string fileName1 = "my.first.file";
    std::string logicalFileName1 = "/my/first/file";
    std::string fileName2 = "my.second.file";
    std::string fileName3 = "my.third.file";
    std::string logicalFileName3 = "/my/third/file";

    pool::FCregister registerAction;
    fileCatalog.setAction( registerAction );

    std::string fid;
    registerAction.registerPFN( fileName1, fileType, fid );
    registerAction.registerPFN( fileName2, fileType, fid );
    registerAction.registerPFN( fileName3, fileType, fid );
    registerAction.registerLFN( fileName1, logicalFileName1 );
    registerAction.registerLFN( fileName3, logicalFileName3 );

    fileCatalog.commit();
    fileCatalog.disconnect();
  }
  catch ( std::exception& e ) {
    std::cerr << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unknown exception..." << std::endl;
    return 1;
  }
  return 0;
} 
