#ifndef RELATIONALSTORAGESERVICE_OBJECTWRITER_H
#define RELATIONALSTORAGESERVICE_OBJECTWRITER_H

#include "DbIOHandler.h"
#include "Reflex/Type.h"
#include <vector>

#include "RVersion.h"
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

namespace coral {
  
  class ISchema;
}

namespace pool {

  class ObjectRelationalMappingElement;
  class DataCallBack;

  namespace RelationalStorageService {

    class ObjectRelationalDataBuffer;
    class MappingManager;

    class ObjectWriter : virtual public DbIOHandler {
    public:
      /// Constructor
      ObjectWriter( coral::ISchema& schema,
                    const Reflex::Type& dictionary,
                    MappingManager& mappingManager,
                    pool::DbDatabase* databaseHandle );

      /// Destructor
      virtual ~ObjectWriter();

      /// Writes an object
      bool write( int oid,
                  const void* data,
                  pool::DataCallBack& callback );

      /// Writes an array
      bool writeArray( const std::vector<int>& identity,
                       ObjectRelationalDataBuffer& buffer,
                       const void* address,
                       pool::DataCallBack& callback );
      
      /// Writes a pool pointer
      bool writePoolPointer( int oid,
                             ObjectRelationalDataBuffer& buffer,
                             const void* address,
                             pool::DataCallBack& callback );
      
      /// Writes a pool polymorphic pointer
      bool writePoolPolymorphicPointer( int oid,
                                        ObjectRelationalDataBuffer& buffer,
                                        const void* address,
                                        pool::DataCallBack& callback );

      ObjectRelationalDataBuffer& dataBuffer();

      const Reflex::Type& refDictionary();
      
    private:

      bool write( int oid,
                  ObjectRelationalDataBuffer& dataBuffer,
                  const void* data,
                  pool::DataCallBack& callback );

      std::string getDataForPolyPtr(const std::string& typeName, const void* address);
      
      private:
      /// The relational schema
      coral::ISchema& m_schema;
      /// The map of mapping elements to element entries
      ObjectRelationalDataBuffer* m_dataBuffer;
      /// The dictionary for a pool reference
      const Reflex::Type m_refDict;
      /// The container mapping manager
      MappingManager& m_mappingManager;
    };

  }

}

#endif
