#ifndef PVECTORCONTAINER_HANDLER_H
#define PVECTORCONTAINER_HANDLER_H

#include "IContainerHandler.h"
#include "Reflex/Reflex.h"
#include "Reflex/Builder/CollectionProxy.h"
#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
 using namespace ROOT;
#endif

namespace pool {
 
  namespace RelationalStorageService {

    class PVectorContainerHandler : virtual public IContainerHandler {

    public:
      /// Constructor
      explicit PVectorContainerHandler( const Reflex::Type& dictionary );

      /// Destructor
      virtual ~PVectorContainerHandler();

      /// Returns the size of the container
      size_t size( const void* address );

      /// Returns the index of the first element
      size_t startElementIndex( const void* address );

      /// Returns an initialized iterator
      IContainerIteratorHandler* iterate( const void* address );

      /// Appends a new element and returns its address of the object reference
      void appendNewElement( void* address, void* data );

      /// Clear the content of the container
      void clear( const void* address );
      
      /// Returns the iterator return type
      Reflex::Type iteratorReturnType() const;

      /// Returns the associativeness of the container
      bool isAssociative() const { return m_isAssociative; }

      /// Returns the persistent size of the container
      size_t persistentSize( const void* address );
      
      /// execute the ending procedure for the container
      void finalize( void* address );
      
    private:
      /// The dictionary information
      Reflex::Type m_type;

      /// The iterator return type
      Reflex::Type m_iteratorReturnType;

      /// Flag indicating whether the container is associative
      bool m_isAssociative;

      /// Structure containing parameters of the collection instance  
      Reflex::Environ<long> m_collEnv;

      /// Proxy of the generic collection
      Reflex::CollFuncTable* m_collProxy;

      size_t m_persistentSizeAttributeOffset;
      size_t m_vecAttributeOffset;

    };

  }

}

#endif
