#ifndef PVECTORCONTAINER_ITERATOR_HANDLER_H
#define PVECTORCONTAINER_ITERATOR_HANDLER_H

#include "Reflex/Type.h"
#include "Reflex/Member.h"
#include "Reflex/Builder/CollectionProxy.h"
#include "IContainerIteratorHandler.h"
#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

namespace pool {

  namespace RelationalStorageService {

    class PVectorContainerIteratorHandler : virtual public IContainerIteratorHandler {
    public:
      /// Constructor
      PVectorContainerIteratorHandler( const Reflex::Environ<long>& collEnv,
                                       Reflex::CollFuncTable* collProxy,
                                       const TypeH& iteratorReturnType,
                                       size_t startElement );

      /// Destructor
      virtual ~PVectorContainerIteratorHandler();

      /// Increments itself
      void increment();

      /// Returns the current object
      void* object();

      /// Returns the return type of the iterator dereference method
      Reflex::Type returnType() const;

    private:

      /// The return type of the iterator dereference method
      TypeH m_returnType;
      
      /// Structure containing parameters of the collection instance  
      Reflex::Environ<long> m_collEnv;

      /// Proxy of the generic collection
      Reflex::CollFuncTable* m_collProxy;

      /// Current element object pointer
      void* m_currentElement;

      size_t m_startElement;
    };

  }

}

#endif
