#include "PoolBaseContainerManager.h"
#include "RelationalStorageServiceNames.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeList.h"
#include <sstream>

pool::RelationalStorageService::PoolBaseContainerManager::PoolBaseContainerManager( int containerId,
                                                                                    const std::string& containerType,
                                                                                    coral::ISchema& schema ):
  m_containerType(containerType),
  m_schema( schema ),
  m_dataForId( 0 ),
  m_whereClauseForId( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInContainerHeaderTable() +
                      " = :" + pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInContainerHeaderTable() ),
  m_setClauseForIncrementingOid( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable() + 
                                 " = " + pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable() +
                                 " + 1" ),
  m_lockedContainerRow( false ),
  m_readNumberOfObjectsWritten( false ),
  m_numberOfObjectsWritten( 0 ),
  m_readContainerSize( false ),
  m_containerSize( 0 ),
  m_readNumberOfObjectsDeleted( false ),
  m_setClauseForIncrementingDeletedObjects( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfDeletedObjectsVariableNameInContainerHeaderTable() + 
                                            " = " + pool::RelationalStorageService::RelationalStorageServiceNames::numberOfDeletedObjectsVariableNameInContainerHeaderTable() +
                                            " + 1" )
{
  m_dataForId = new coral::AttributeList;
  m_dataForId->extend<int>( pool::RelationalStorageService::RelationalStorageServiceNames::idVariableNameInContainerHeaderTable() );
  m_dataForId->begin()->data<int>() = containerId;
}


pool::RelationalStorageService::PoolBaseContainerManager::~PoolBaseContainerManager()
{
  delete m_dataForId;
}

bool
pool::RelationalStorageService::PoolBaseContainerManager::createContainer()
{
  return true;
}

int
pool::RelationalStorageService::PoolBaseContainerManager::id() const
{
  return m_dataForId->begin()->data<int>();
}

std::string
pool::RelationalStorageService::PoolBaseContainerManager::containerType() const{
  return m_containerType;
}
std::string
pool::RelationalStorageService::PoolBaseContainerManager::mappingVersion() const{
  return std::string("");
}

std::string
pool::RelationalStorageService::PoolBaseContainerManager::className() const{
  return std::string("");
}

unsigned long
pool::RelationalStorageService::PoolBaseContainerManager::size()
{
  if ( ! m_readContainerSize ) {
    coral::IQuery* query = m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).newQuery();
    std::string expression = pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable() +
      " - " + pool::RelationalStorageService::RelationalStorageServiceNames::numberOfDeletedObjectsVariableNameInContainerHeaderTable();
    query->addToOutputList( expression, "CONTAINER_SIZE" );
    query->defineOutputType( "CONTAINER_SIZE", coral::AttributeSpecification::typeNameForType< unsigned long >() );
    query->setCondition( m_whereClauseForId, *m_dataForId );
    coral::ICursor& cursor = query->execute();
    if ( cursor.next() ) {
      m_containerSize = cursor.currentRow().begin()->data< unsigned long >();
    }
    m_readContainerSize = true;
    delete query;
  }
  return m_containerSize;
}

unsigned long
pool::RelationalStorageService::PoolBaseContainerManager::numberOfObjectsWritten()
{
  return this->retrieveNumberOfObjectsWritten( true );
}

void
pool::RelationalStorageService::PoolBaseContainerManager::incrementNumberOfObjectsWritten()
{
  if ( ! m_lockedContainerRow ) {
    this->retrieveNumberOfObjectsWritten( true );
  }
  ++m_numberOfObjectsWritten;
  ++m_containerSize;
  m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).dataEditor().updateRows( m_setClauseForIncrementingOid, m_whereClauseForId, *m_dataForId );
}


void
pool::RelationalStorageService::PoolBaseContainerManager::incrementNumberOfObjectsDeleted()
{
  if ( ! m_readNumberOfObjectsDeleted ) {
    coral::IQuery* query = m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).newQuery();
    query->addToOutputList( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable() );
    query->defineOutputType( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable(), coral::AttributeSpecification::typeNameForType< unsigned long >() );
    query->addToOutputList( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfDeletedObjectsVariableNameInContainerHeaderTable() );
    query->defineOutputType( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfDeletedObjectsVariableNameInContainerHeaderTable(), coral::AttributeSpecification::typeNameForType< unsigned long >() );

    query->setCondition( m_whereClauseForId, *m_dataForId );
    if ( ! m_lockedContainerRow ) query->setForUpdate();
    coral::ICursor& cursor = query->execute();

    if ( cursor.next() ) {
      coral::AttributeList::const_iterator iAttribute = cursor.currentRow().begin();
      m_numberOfObjectsWritten = iAttribute->data<unsigned long>();
      ++iAttribute;
      unsigned long numberOfObjectsDeleted = iAttribute->data<unsigned long>();
      m_containerSize = m_numberOfObjectsWritten - numberOfObjectsDeleted;
    }
    delete query;
    m_readNumberOfObjectsDeleted = true;
    m_lockedContainerRow = true;
    m_readContainerSize = true;
  }

  m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).dataEditor().updateRows( m_setClauseForIncrementingDeletedObjects, m_whereClauseForId, *m_dataForId );
  --m_containerSize;
}

bool
pool::RelationalStorageService::PoolBaseContainerManager::updateNumberOfObjects(unsigned long writtenObjects,
                                                                                unsigned long deletedObjects)
{
  if(writtenObjects >= deletedObjects){

    m_numberOfObjectsWritten = writtenObjects;
    m_containerSize = writtenObjects - deletedObjects;

    std::ostringstream setClauseForNumberOfWrittenObjects;
    setClauseForNumberOfWrittenObjects << pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable()<< " = " << m_numberOfObjectsWritten; 

    m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).dataEditor().updateRows( setClauseForNumberOfWrittenObjects.str(), m_whereClauseForId, *m_dataForId );

    std::ostringstream setClauseForNumberOfDeletedObjects;
    setClauseForNumberOfDeletedObjects << pool::RelationalStorageService::RelationalStorageServiceNames::numberOfDeletedObjectsVariableNameInContainerHeaderTable() << " = " << deletedObjects;

    m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).dataEditor().updateRows( setClauseForNumberOfDeletedObjects.str(), m_whereClauseForId, *m_dataForId );
    return true;
  }
  return false;
}

unsigned long
pool::RelationalStorageService::PoolBaseContainerManager::retrieveNumberOfObjectsWritten( bool setForUpdate )
{
  if ( ( ! m_readNumberOfObjectsWritten ) ||
       ( m_readNumberOfObjectsWritten && setForUpdate && ( ! m_lockedContainerRow ) ) ) {
    coral::IQuery* query = m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).newQuery();
      
    query->addToOutputList( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable() );
    query->defineOutputType( pool::RelationalStorageService::RelationalStorageServiceNames::numberOfWrittenObjectsVariableNameInContainerHeaderTable(), coral::AttributeSpecification::typeNameForType< unsigned long >() );

    query->setCondition( m_whereClauseForId, *m_dataForId );
    if ( setForUpdate ) query->setForUpdate();
    coral::ICursor& cursor = query->execute();
    if ( cursor.next() ) {
      m_numberOfObjectsWritten = cursor.currentRow().begin()->data<unsigned long>();
    }
    delete query;
    m_readNumberOfObjectsWritten = true;
    if ( setForUpdate )  m_lockedContainerRow = true;
  }
  return m_numberOfObjectsWritten;
}

bool
pool::RelationalStorageService::PoolBaseContainerManager::removeContainerEntry()
{
  m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).dataEditor().deleteRows( m_whereClauseForId, *m_dataForId );
  return true;
}

/**
std::string
pool::RelationalStorageService::PoolBaseContainerManager::getClassName()
{
  coral::IQuery* query = m_schema.tableHandle( pool::RelationalStorageService::RelationalStorageServiceNames::containerHeaderTableName() ).newQuery();
  query->addToOutputList( pool::RelationalStorageService::RelationalStorageServiceNames::classNameVariableNameInContainerHeaderTable() );
  query->setCondition( m_whereClauseForId, *m_dataForId );
  coral::ICursor& cursor = query->execute();
  std::string className = "";
  if ( cursor.next() ) {
    className = cursor.currentRow().begin()->data<std::string>();
  }
  delete query;
  return className;
}
**/

void
pool::RelationalStorageService::PoolBaseContainerManager::endTransaction()
{
  m_lockedContainerRow = false;
  m_readNumberOfObjectsWritten = false;
  m_readNumberOfObjectsDeleted = false;
  m_readContainerSize = false;
}


