#ifndef POOLHOMOGENEOUSCONTAINERMANAGER_H
#define POOLHOMOGENEOUSCONTAINERMANAGER_H

#include "Reflex/Type.h"
#include "PoolBaseContainerManager.h"
#include "MappingManager.h"
#include <set>

#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

namespace pool {

  class ObjectRelationalMappingElement;

  namespace RelationalStorageService {

    class ObjectUpdater;
    class ObjectReader;

    class PoolHomogeneousContainerManager : public PoolBaseContainerManager
    {
    public:
      /// Constructor
      PoolHomogeneousContainerManager( int containerId,
                                       const std::string& containerName,
                                       const std::string& className,
                                       coral::ISchema& schema );

      /// Destructor
      ~PoolHomogeneousContainerManager();

      std::string className() const;
      
      std::string mappingVersion() const;

      const std::string& tableName() const;

      bool createContainer();

      /// Deletes all objects from the container and deletes the container
      bool dropContainer();

      /// Writes an object
      bool writeObject( const void* data,
                        int oid,
                        pool::DataCallBack& callback,
                        const std::string& type_name,
                        const Reflex::Type& dictionary,
                        pool::DbDatabase* databaseHandle );

      /// Updates an existing object
      bool updateObject( const void* data,
                         int oid,
                         pool::DataCallBack& callback,
                         const std::string& type_name,
                         const Reflex::Type& dictionary,
                         pool::DbDatabase* databaseHandle );

      /// Reads an object
      bool readObject( void** data,
                       int oid,
                       pool::DataCallBack& callback,
                       const std::string& type_name,
                       const Reflex::Type& dictionary,
                       pool::DbDatabase* databaseHandle );

      /// Reads an object
      bool readAnyObject( void** data,
                          int oid,
                          pool::DataCallBack& callback,
                          const std::string& type_name,
                          const Reflex::Type& dictionary,
                          pool::DbDatabase* databaseHandle );

      /// Deletes an object
      bool deleteObject( int oid );

      /// Force the update of the container contents
      bool updateContent(const std::vector<std::pair<std::string,std::string> >& classInfo, 
                                                                            bool recreate, bool dry);

      /// Checks the existence of an OID
      bool existsOid( int oid,
                      int& oidOut,
                      Reflex::Type& dictionary );

      /// Ends a transaction
      void endTransaction();
      
      private:

      /// set up the container data structure
      void initialize() const;
      
      /// Deletes an object (recursive method)
      bool deleteAnObject( const pool::ObjectRelationalMappingElement& mapping,
                           const std::string& parentTableName,
                           int oid );

      const pool::ObjectRelationalMappingElement& mappingElement() const;

      private:
      /// The container name
      std::string m_containerName;

      /// The container class name
      std::string m_className;

      /// The dictionary information of the class
      Reflex::Type m_dictionary;

      // The mapping provider
      mutable MappingManager m_mappingManager;

      // The initialized flag
      mutable bool m_initialized;

      /// The required object/relational mapping
      mutable const pool::ObjectRelationalMapping* m_mapping;

      /// The object writer
      ObjectUpdater* m_updater;

      /// The object reader
      ObjectReader* m_reader;

      /// The minimum oid
      int m_oidMin;

      /// The maximim oid
      int m_oidMax;

      /// The set of valid oids
      std::set< int > m_validOids;
    };
  }

}

#endif
