#ifndef POOLPOLYMORPHICCONTAINERMANAGER_H
#define POOLPOLYMORPHICCONTAINERMANAGER_H

#include "PoolBaseContainerManager.h"
#include "MappingManager.h"
#include <map>
#include "RVersion.h"
#include "Reflex/Type.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

namespace coral {

  class IRelationalTable;
  class AttributeList;
}

namespace pool {

  class ObjectRelationalMappingElement;

  namespace RelationalStorageService {

    class MappingSequence;
    class ObjectWriter;
    class ObjectReader;
    class ExistingDataScanner;

    class PoolPolymorphicContainerManager : public PoolBaseContainerManager
    {
    public:
      /// Constructor
      PoolPolymorphicContainerManager( int containerId,
                                       const std::string& containerName,
                                       coral::ISchema& schema,
                                       const std::string& tableName,
                                       MappingSequence& mappingSequences );

      /// Destructor
      ~PoolPolymorphicContainerManager();

      /// Deletes all objects from the container and deletes the container
      bool dropContainer();

      const std::string& tableName() const;

      /// Writes an object
      bool writeObject( const void* data,
                        int oid,
                        pool::DataCallBack& callback,
                        const std::string& type_name,
                        const Reflex::Type& dictionary,
                        pool::DbDatabase* databaseHandle );

      /// Updates an existing object
      bool updateObject( const void* data,
                         int oid,
                         pool::DataCallBack& callback,
                         const std::string& type_name,
                         const Reflex::Type& dictionary,
                         pool::DbDatabase* databaseHandle );

      /// Reads an object
      bool readObject( void** data,
                       int oid,
                       pool::DataCallBack& callback,
                       const std::string& type_name,
                       const Reflex::Type& dictionary,
                       pool::DbDatabase* databaseHandle );

      /// Reads an object
      bool readAnyObject( void** data,
                          int oid,
                          pool::DataCallBack& callback,
                          const std::string& type_name,
                          const Reflex::Type& dictionary,
                          pool::DbDatabase* databaseHandle );

      /// Deletes an object
      bool deleteObject( int oid );

      /// Force the update of the container contents
      bool updateContent(const std::vector<std::pair<std::string,std::string> >& classInfo,
                         bool recreate, 
                         bool dry);

      /// Checks the existence of an OID
      bool existsOid( int oid,
                      int& oidOut,
                      Reflex::Type& dictionary );

    private:
      /// Deletes an object (recursive method)
      bool deleteAnObject( const pool::ObjectRelationalMappingElement& mapping,
                           const std::string& parentTableName,
                           int oid );

      bool updateEntries( const ExistingDataScanner* dataScanner, int& lastOid);

    private:
      /// Helper structure holding the meta-data of a container entry
      class Entry {
      public:
        Entry( const Reflex::Type& theDictionary = Reflex::Type(),
               std::string theMappingVersion = "",
               int theTargetOid = 0 ):
          dictionary( theDictionary ),
          mappingVersion( theMappingVersion ),
          targetOid( theTargetOid )
        {}
        Reflex::Type dictionary;
        std::string mappingVersion;
        int targetOid;
      };

      private:

      std::string m_containerName;
      
      /// The schema associated
      coral::ISchema& m_schema;

      /// The container table name
      std::string m_containerTableName;
      
      /// The mapping manager
      MappingManager m_mappingManager;

      /// The mapping sequences
      MappingSequence& m_mappingSequences;

      /// The container row data buffer
      coral::AttributeList* m_containerRow;

      /// The map of writers
      std::map< Reflex::Type, ObjectWriter* > m_writers;

      /// The map of readers
      std::map< Reflex::Type, ObjectReader* > m_readers;

      /// The cached meta-data of a container entry
      std::map< int, Entry > m_cachedEntries;

      /// The minimum oid
      int m_oidMin;

      /// The maximim oid
      int m_oidMax;

      /// The mapping information (empty)
      std::string m_mappingInfo;

    };
  }

}

#endif
