#include "RelationalOODbImplementation.h"
#include "RelationalDomain.h"
#include "RelationalPoolDatabase.h"
#include "RelationalPoolHomogeneousContainer.h"
#include "RelationalPoolPolymorphicContainer.h"
#include "StorageSvc/DbInstanceCount.h"
#include "StorageSvc/DbType.h"
#include "POOLCore/Context.h"

pool::RelationalStorageService::RelationalOODbImplementation::RelationalOODbImplementation( void* context ):
  pool::OODatabaseImp(  context, pool::POOL_RDBMS_StorageType ),
  m_context( 0 )
{
  pool::DbInstanceCount::increment(this);
  if( context ) {
    m_context = static_cast<pool::Context*>(context);
  } 
}

pool::RelationalStorageService::RelationalOODbImplementation::RelationalOODbImplementation( void* context, const pool::DbType& type ):
  pool::OODatabaseImp(  context, type ),
  m_context( 0 )
{
  pool::DbInstanceCount::increment(this);
  if( context ) m_context = static_cast<pool::Context*>(context);
}

pool::RelationalStorageService::RelationalOODbImplementation::~RelationalOODbImplementation()
{
  pool::DbInstanceCount::decrement(this);
}


const char*
pool::RelationalStorageService::RelationalOODbImplementation::catalogLabel()
{
  return "POOL_RDBMS";
}


pool::IDbDomain*
pool::RelationalStorageService::RelationalOODbImplementation::createDomain()
{
  return new pool::RelationalStorageService::RelationalDomain( this, m_context );
}

pool::IDbDatabase*
pool::RelationalStorageService::RelationalOODbImplementation::createDatabase()
{
  return new pool::RelationalStorageService::RelationalPoolDatabase( this, m_context );
}


pool::IDbContainer*
pool::RelationalStorageService::RelationalOODbImplementation::createContainer( const pool::DbType& type )  
{
  if ( type.match( pool::POOL_RDBMS_HOMOGENEOUS_StorageType) ) {
    return new pool::RelationalStorageService::RelationalPoolHomogeneousContainer( this );
  }
  else if ( type.match(POOL_RDBMS_POLYMORPHIC_StorageType) ) {
    return new pool::RelationalStorageService::RelationalPoolPolymorphicContainer( this );
  }
  else if ( type.match(POOL_RDBMS_StorageType) ) { // The default container type is the homogeneous one.
    return new pool::RelationalStorageService::RelationalPoolHomogeneousContainer( this );
  }
  return 0;
}
