#include "SpecialContainerHandler.h"
#include "BaseContainerHandler.h"
#include "RelationalStorageServiceException.h"
#include "ObjectRelationalAccess/ObjectRelationalClassUtils.h"
#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

pool::RelationalStorageService::SpecialContainerHandler::SpecialContainerHandler( const TypeH& dictionary ):
  m_containerHandler( 0 ),
  m_containerOffset( 0 )
{
  // update dictionary to include base classes members
  dictionary.UpdateMembers(); 
  for ( unsigned int i=0;i<dictionary.DataMemberSize();i++){

    Reflex::Member field = dictionary.DataMemberAt(i);    
    Reflex::Type field_type = field.TypeOf();
    if ( ! field_type ) {
      throw pool::RelationalStorageService::RelationalStorageServiceException( "The dictionary of the underlying container of \"" +
                                                                               dictionary.Name(Reflex::SCOPED|Reflex::FINAL) + "\" is not available",
                                                                               "SpecialContainerHandler" );
    }
    if ( ObjectRelationalClassUtils::isTypeContainer(field_type) ) {
      m_containerHandler = new pool::RelationalStorageService::BaseContainerHandler( field_type );
      m_containerOffset = field.Offset();
      break;
    }
  }
  if ( m_containerHandler == 0 ) {
    throw pool::RelationalStorageService::RelationalStorageServiceException( "Could not retrieve the underlying container of \"" +
                                                                             dictionary.Name(Reflex::SCOPED|Reflex::FINAL) + "\" is not available",
                                                                             "SpecialContainerHandler" );
  }
}


pool::RelationalStorageService::SpecialContainerHandler::~SpecialContainerHandler()
{
  if ( m_containerHandler ) delete m_containerHandler;
}


size_t
pool::RelationalStorageService::SpecialContainerHandler::size( const void* address )
{
  return m_containerHandler->size( static_cast< const char* >( address ) + m_containerOffset );
}


pool::RelationalStorageService::IContainerIteratorHandler*
pool::RelationalStorageService::SpecialContainerHandler::iterate( const void* address )
{
  return m_containerHandler->iterate( static_cast< const char* >( address ) + m_containerOffset );
}


void
pool::RelationalStorageService::SpecialContainerHandler::appendNewElement( void* address, void* data )
{
  m_containerHandler->appendNewElement( static_cast< char* >( address ) + m_containerOffset, data );
}

void 
pool::RelationalStorageService::SpecialContainerHandler::clear( const void* address )
{
  m_containerHandler->clear( static_cast< const char* >( address ) + m_containerOffset );
}

TypeH
pool::RelationalStorageService::SpecialContainerHandler::iteratorReturnType() const
{
  return m_containerHandler->iteratorReturnType();
}
