#include "POOLCore/Exception.h"
#include "POOLCore/CommandOptions.h"
#include "RelationalStorageService/RelationalPoolDatabaseUtilities.h"
#include <iostream>

int main (int argc, char** argv)
{
  std::vector<pool::CommandOptions::Option> ops;
  pool::CommandOptions::Option opt;
  opt.flag = "-f";
  opt.property = "file";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the setup driver xml file name";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-o";
  opt.property = "outputFile";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the setup driver xml output file name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-d";
  opt.property = "dictionary";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the dictionary libraries";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-c";
  opt.property = "connectionString";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the connection string";
  opt.required = true;
  ops.push_back(opt);
  opt.flag = "-u";
  opt.property = "user";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user name";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-p";
  opt.property = "password";
  opt.type = pool::CommandOptions::Option::STRING;
  opt.helpEntry = "the user password";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-e";
  opt.property = "evolveSchema";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "allow schema changes for class evolution";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-dry";
  opt.property = "dryRun";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "only show the schema changes involved in the command ";
  opt.required = false;
  ops.push_back(opt);
  opt.flag = "-debug";
  opt.property = "debug";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the debug verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-info";
  opt.property = "info";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.required = false;
  opt.helpEntry = "enable the info verbosity mode";
  ops.push_back(opt);   
  opt.flag = "-h";
  opt.property = "help";
  opt.type = pool::CommandOptions::Option::BOOLEAN;
  opt.helpEntry = "display the help";
  opt.required = false;  
  ops.push_back(opt);
  pool::CommandOptions cmd(ops);
  try{
    cmd.parse(argc,argv);
    const std::map<std::string,std::string>& ops = cmd.userOptions();
    std::map<std::string,std::string>::const_iterator iter = ops.find("help");
    if(iter!=ops.end() || ops.size()==0){
      cmd.help();
    } else {
      pool::RelationalPoolDatabaseUtilities utility;
    
      bool evolve = false;
      iter = ops.find("evolve");
      if(iter!=ops.end()){
        evolve = true;
      } 
      bool dry = false;
      iter = ops.find("dryRun");
      if(iter!=ops.end()){
        dry = true;
      } 
      iter = ops.find("debug");
      if(iter!=ops.end()){
        utility.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                            pool::ObjectRelationalUtilities::debugVerbosityPropertyValue());
      } else {
        iter = ops.find("info");
        if(iter!=ops.end()){
          utility.setProperty(pool::ObjectRelationalUtilities::verbosityProperty(),
                              pool::ObjectRelationalUtilities::infoVerbosityPropertyValue());
        }
      }
      iter = ops.find("file");
      if(iter==ops.end()){
        throw pool::Exception("Driver file not provided", "", "pool_setup_database" );
      } 
      std::string theDriverFile = (*iter).second;
      std::string theOutputFile("");
      iter = ops.find("outputFile");
      if(iter!=ops.end()){
        theOutputFile = (*iter).second;
      } 
      iter = ops.find("dictionary");
      if(iter!=ops.end()){
        std::string theDictionaryLibraries = (*iter).second;
        utility.loadDictionaries(theDictionaryLibraries);
      }
      std::string theConnectionString("");
      iter = ops.find("connectionString");
      if(iter!=ops.end()){
        theConnectionString = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::connectionStringProperty(),theConnectionString);
      } 
      iter = ops.find("user");
      if(iter!=ops.end()){
        std::string theUserName = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::userNameProperty(),theUserName);
      } 
      iter = ops.find("password");
      if(iter!=ops.end()){
        std::string thePassword = (*iter).second;
        utility.setProperty(pool::ObjectRelationalUtilities::passwordProperty(),thePassword);
      }
      utility.setUpDatabase(theDriverFile,theOutputFile,evolve,dry);
      std::cout << "Set up of Pool Database \""<<theConnectionString<<"\" completed."<<std::endl;
    }
    return 0;
  } catch (const std::exception& e){
    std::cout << "ERROR: " << e.what() << std::endl;
    return 1;
  }
}

