#include <string>
#include <iostream>
#include <cstdio>
#include <cstdlib>
#include "src/RootCollection.h"
#include "src/RootCollectionMetadata.h"
#include "AttributeList/AttributeListSpecification.h"
#include "AttributeList/AttributeList.h"
#include "POOLCore/Token.h"
#include "POOLCore/Exception.h"

using namespace std;
using namespace pool;


void printAttributes( const AttributeList& attrList )
{
  const AttributeListSpecification& attrListSpec = attrList.attributeListSpecification();
  for( unsigned int i = 0; i < attrListSpec.size(); ++i )
  {
    const AttributeSpecification& attrSpec = attrListSpec[ i ];
    string attrName = attrSpec.name();
    string attrValue = attrList[ attrName ].getValueAsString();
    cout << attrName << " = " << attrValue << "\t";
  }

}


int main(){
  string connection( "" );
  try{
    cout << "Openning collection 'test_collection' in CREATE_AND_OVERWRITE mode." << endl;
    RootCollection collection( 0, connection, "test_collection", ICollection::CREATE_AND_OVERWRITE );
    cout << "Openning the collection." << endl;
    
    cout << "Specifying attribute list: 'attr1' of type 'int', 'attr2' of type 'std::string' "
         << "and attr3 of type 'double'." << endl;
    AttributeListSpecification& spec = collection.attributeListSpecification();
    spec.push_back( "attr1", "int" );
    spec.push_back( "attr2", "string" );
    spec.push_back( "attr3", "double" );
    spec.push_back( "attr4", "bool" );
    
    cout << "Adding 30 elements to the collection." << endl;
    for( int i=100; i<130; i++ )
      {
      AttributeList attrList( spec );
      attrList[ "attr1" ].setValue( i );
      char s[20];
      sprintf( s, "%s_%d", "testString", i );
      attrList[ "attr2" ].setValue( string( s ) );
      attrList[ "attr3" ].setValue( double( i ) / 3 );
      attrList[ "attr4" ].setValue( bool(i%3) );
      sprintf( s, "%s_%d","testToken", i );
      Token* t = new Token();
      t->fromString( string( "" ) );
      t->setDb( s );
      collection.add( *t, attrList );

      cout << "Token: " << t->toString();
      cout << endl;
      cout << "Attributes: ";
      printAttributes( attrList );
      cout << endl;
      t->release();
    }
    cout << "Done." << endl;
    cout << "Commiting the collection." << endl;
    collection.commit();
    cout << "Closing the collection." << endl;
    collection.close();
    
    cout << "[OVAL] finished hopefully" << endl;
    
  } catch ( std::exception& exception )
  {
    cerr << "EXCEPTION: " << exception.what() << endl;
  } catch ( ... )
  {
    cout << "Unknown exception" << endl;
  }


}




