//====================================================================
//
//        Package    : RootStorageSvc (The POOL project)
//
//        Author     : M.Frank
//====================================================================
#ifndef POOL_ROOTSTREAMERPROXYPATCH_H
#define POOL_ROOTSTREAMERPROXYPATCH_H 1

#include "Reflex/Reflex.h"
#include "TGenCollectionStreamer.h"

/*
 *  namespace declaration
 */
namespace pool  {
    enum CONTAINER  {
      VECTOR,
      LIST,
      DEQUE,
      SET,
      MULTISET,
      QUEUE,
      STACK,
      HASHSET,
      HASHMULTISET,
      MAP,
      MULTIMAP,
      HASHMAP,
      HASHMULTIMAP,
      NOCONTAINER
    };
  /**@class RootStreamerProxyPatch RootStreamerProxyPatch.h RootStorageSvc/RootStreamerProxyPatch.h
    *
    *  @author  M.Frank
    *  @version 3.0
    *  @date    20/11/2004
    */
  class RootStreamerProxyPatch : public TGenCollectionStreamer  {
  public:
    typedef ROOT::Reflex::Type       TypeH;
  protected:
    typedef TGenCollectionStreamer   Proxy;
    typedef TCollectionClassStreamer Stream;
    Proxy*     m_proxy;
    TypeH      m_type;
    TypeH      m_comp;
    bool       m_is_ref[2];
    bool       m_is_cont[2];
    bool       m_is_string[2];
    bool       m_optCheckCount;
    CONTAINER  m_cntType;
    TClass *m_class, *m_compClass, *m_keyClass;
    static bool install(const TypeH& clH);
    static bool install(TClass* rcl, TClass* cl, const TypeH& clH);
    static bool installDeque(const TypeH& clH);

    /// Stream I/O: Map input streamer
    void ReadMap2(int nElements, TBuffer &b);
    /// Stream I/O: Object input streamer
    void ReadObjects2(int nElements, TBuffer &b);
    /// Stream I/O: Map output streamer
    void WriteMap2(int nElements, TBuffer &b);
    /// Stream I/O: Object output streamer
    void WriteObjects2(int nElements, TBuffer &b);
    /// Primitive input streamer
    void ReadPrimitives2(int nElements, TBuffer &b);
    /// Primitive output streamer
    void WritePrimitives2(int nElements, TBuffer &b);
  public:
    /// Virtual copy constructor
    virtual TVirtualCollectionProxy* Generate() const  {
      return new RootStreamerProxyPatch(m_proxy, m_type);
    }
    RootStreamerProxyPatch(Proxy* p, const TypeH& clH);
    virtual ~RootStreamerProxyPatch();
    /// Streamer for I/O handling
    void Streamer(TBuffer &refBuffer, void *pObject, int siz) {
      return TGenCollectionStreamer::Streamer(refBuffer, pObject, siz);
    }
    virtual void Streamer(TBuffer &b);
    virtual void Streamer2(TBuffer &b);
    static bool patchClass(const TypeH& clH);
    static bool patchContainer(const TypeH& clH);
    static bool failure(const TypeH& clH, const char* msg="");
  };

  CONTAINER      contType(const RootStreamerProxyPatch::TypeH&);
  TClass*        getClass(const RootStreamerProxyPatch::TypeH&);
}      // namespace pool
#endif // POOL_ROOTSTREAMERPROXYPATCH_H
