#include <stdexcept>
#include "TClass.h"
#include "TBuffer.h"
#include "StorageSvc/DbReflex.h"
#include "RootStorageSvc/StreamerBase.h"
using namespace pool;

StreamerBase::StreamerBase(const TypeH& typ, TClass* cl)
: m_type(typ), m_root(cl) 
{
}

void StreamerBase::operator()(TBuffer &b, void *obj)  {
  try {
    if ( b.IsReading() )   {
      UInt_t start, count;
      Version_t version = b.ReadVersion(&start, &count, m_root);
      m_root->ReadBuffer(b, obj, version, start, count);
    }
    else  {
      m_root->WriteBuffer(b, obj);
    }
  }
  catch( const std::exception& e )    {
    std::string err = "Class:" + DbReflex::fullTypeName(m_type) + "> Exception in object I/O";
    err += e.what();
    throw std::runtime_error(err);
  }
  catch( ... )    {
    std::string err = "Class:" + DbReflex::fullTypeName(m_type) + "> Exception in object I/O";
    throw std::runtime_error(err);
  }
}
