#include "TMemFile.h"
#include "TROOT.h"
#include <iostream>
#include <fcntl.h>
#include "StorageSvc/DbMemoryIO.h"
typedef pool::DbMemoryIO _MEM;

TMemFile::TMemFile() : TFile()  {
  std::cout << "ctor called" << std::endl;
}


TMemFile::TMemFile(const char *url, Option_t *opt, const char *tit, Int_t compress)
: TFile(url,"NET",tit,compress)
{
  m_offset = 0;
  fOption = opt;
  fOption.ToUpper();

  if (fOption == "NEW")
    fOption = "CREATE";

  Bool_t create   = (fOption == "CREATE")   ? kTRUE : kFALSE;
  Bool_t recreate = (fOption == "RECREATE") ? kTRUE : kFALSE;
  Bool_t update   = (fOption == "UPDATE")   ? kTRUE : kFALSE;
  Bool_t read     = (fOption == "READ")     ? kTRUE : kFALSE;
  if (!create && !recreate && !update && !read) {
    read    = kTRUE;
    fOption = "READ";
  }
  m_memory = _MEM::instance();
  if (recreate) {
    if (m_memory->access(url, _MEM::kFileExists) == 0)
        m_memory->unlink(url);
    recreate = kFALSE;
    create   = kTRUE;
    fOption  = "CREATE";
  }
  if (create && m_memory->access(url, _MEM::kFileExists) == 0) {
    Error("TMemFile", "file %s already exists", url);
    goto zombie;
  }
  if (update) {
    if (m_memory->access(url, kFileExists) != 0) {
        update = kFALSE;
        create = kTRUE;
    }
    if (update && m_memory->access(url, _MEM::kWritePermission) != 0) {
        Error("TMemFile", "no write permission, could not open file %s", url);
        goto zombie;
    }
  }
  if (read) {
    if (m_memory->access(url, _MEM::kFileExists) != 0) {
        Error("TMemFile", "file %s does not exist", url);
        goto zombie;
    }
    if (m_memory->access(url, _MEM::kReadPermission) != 0) {
        Error("TMemFile", "no read permission, could not open file %s", url);
        goto zombie;
    }
  }

  // Connect to file system stream
  fRealName = url;

  if (create || update) {
#ifndef WIN32
    fD = SysOpen(url, O_RDWR | O_CREAT, 0644);
#else
    fD = SysOpen(url, O_RDWR | O_CREAT | O_BINARY, S_IREAD | S_IWRITE);
#endif
    if (fD == -1) {
      SysError("TMemFile", "file %s can not be opened", url);
      goto zombie;
    }
    fWritable = kTRUE;
  } else {
#ifndef WIN32
    fD = SysOpen(url, O_RDONLY, 0644);
#else
    fD = SysOpen(url, O_RDONLY | O_BINARY, S_IREAD | S_IWRITE);
#endif
    if (fD == -1) {
      SysError("TMemFile", "file %s can not be opened for reading", url);
      goto zombie;
    }
    fWritable = kFALSE;
  }
  Init(create);
  return;

zombie:
  // error in file opening occured, make this object a zombie
  MakeZombie();
  gDirectory = gROOT;
}

/// Mem file dtor. Close and flush directory structure.
TMemFile::~TMemFile()   {
  Close();
}

int TMemFile::dumpFile(const char* url)   {
  if ( url )  {
    return m_memory->dump(url);
  }
  else  {
    _MEM::Pathes::iterator i;
    for(i = m_memory->m_pathMap.begin(); i != m_memory->m_pathMap.end(); ++i)  {
      m_memory->dump((*i).first);
    }
  }
  return 0;
}

int TMemFile::listFiles()  {
  _MEM::Pathes::iterator i;
  for(i=m_memory->m_pathMap.begin(); i != m_memory->m_pathMap.end(); ++i)  {
    std::cout << "File [" << (*i).second << "]:"  << (*i).first << std::endl;
  }
  return 0;
}

int TMemFile::destroyFile(const char* url)  {
  if ( url )  {
    return m_memory->unlink(url);
  }
  return m_memory->unlinkAll();
}

int TMemFile::clearFiles()  {
  return m_memory->unlinkAll();
}

/// Interface to system open. All arguments like in POSIX open.
Int_t TMemFile::SysOpen(const char *pathname, Int_t flags, UInt_t mode) {
  int ret = m_memory->open(pathname, flags, mode);
  if (ret < 0)    {
    gSystem->SetErrorStr("Failed to open memory file");
  }
  return ret;
}

/// Interface to system close. All arguments like in POSIX close.
Int_t TMemFile::SysClose(Int_t fd)  {
  int ret = m_memory->close(fd);
  if (ret < 0)    {
    gSystem->SetErrorStr("Failed to close memory file.");
  }
  return ret;
}

/// Interface to system read. All arguments like in POSIX read.
Int_t TMemFile::SysRead(Int_t fd, void *buf, Int_t len)   {
  m_offset += len;
  int ret = int(m_memory->read(fd, buf, len));
  if (ret < 0)  {
    gSystem->SetErrorStr("Filed to read from memory file.");
  }
  return ret;
}

/// Interface to system write. All arguments like in POSIX write.
Int_t TMemFile::SysWrite(Int_t fd, const void *buf, Int_t len)    {
  m_offset += len;
  int ret = int(m_memory->write(fd, buf, len));
  if (ret < 0)  {
    gSystem->SetErrorStr("Filed to write memory file");
  }
  return ret;
}

// Interface to system lseek.
Long64_t TMemFile::SysSeek(Int_t fd, Long64_t offset, Int_t whence) {
  if (whence == SEEK_SET && offset == m_offset) return offset;
  Long64_t ret = m_memory->seek(fd, offset, whence);
  if (ret < 0)  {
    gSystem->SetErrorStr("Filed to seek memory file");
  }
  else  {
    m_offset = ret;
  }
  return ret;
}

// Interface to TSystem:GetPathInfo().
Int_t TMemFile::SysStat(Int_t fd, Long_t *id, Long64_t *size, Long_t *flags,
                        Long_t *modtime)
{
  struct stat statbuf;
  if (m_memory->stat(fd, &statbuf) >= 0) {
    if (id)  {
      *id = (statbuf.st_dev << 24) + statbuf.st_ino;
    }
    if (size)  {
      *size = statbuf.st_size;
    }
    if (modtime) {
      *modtime = statbuf.st_mtime;
    }
    if (flags) {
      *flags = 0;
      if (statbuf.st_mode & ((S_IEXEC)|(S_IEXEC>>3)|(S_IEXEC>>6)))
        *flags |= 1;
      if ((statbuf.st_mode & S_IFMT) == S_IFDIR)
        *flags |= 2;
      if ((statbuf.st_mode & S_IFMT) != S_IFREG &&
          (statbuf.st_mode & S_IFMT) != S_IFDIR)
        *flags |= 4;
    }
    return 0;
  }
  gSystem->SetErrorStr("Filed to stat memory file");
  return 1;
}

/// Create helper class that allows directory access via Memd.
TMemSystem::TMemSystem() : TSystem("-Mem", "Mem Helper System") {
  // name must start with '-' to bypass the TSystem singleton check
  SetName("Mem");
  m_memory = _MEM::instance();
}

// Make a directory via Memd.
Int_t TMemSystem::MakeDirectory(const char* /*dir */)  {
  return 0;
}

void *TMemSystem::OpenDirectory(const char* /* dir */)  {
  return 0;
}

// Free directory via Memd.
void TMemSystem::FreeDirectory(void* /* dirp */)  {
}

/// Get directory entry via Memd. Returns 0 in case no more entries.
const char *TMemSystem::GetDirEntry(void* /*dirp */) {
  return 0;
}

Int_t TMemSystem::GetPathInfo(const char *path, Long_t *id, Long64_t *size,
                              Long_t *flags, Long_t *modtime)   {
  struct stat statbuf;
  if (path && m_memory->stat(path, &statbuf) >= 0) {
    if (id)
        *id = 0;
    if (size)
        *size = statbuf.st_size;
    if (modtime)
        *modtime = statbuf.st_mtime;
    if (flags) {
        *flags = 0;
        if (statbuf.st_mode & ((S_IEXEC)|(S_IEXEC>>3)|(S_IEXEC>>6)))
          *flags |= 1;
        if ((statbuf.st_mode & S_IFMT) == S_IFDIR)
          *flags |= 2;
        if ((statbuf.st_mode & S_IFMT) != S_IFREG &&
            (statbuf.st_mode & S_IFMT) != S_IFDIR)
          *flags |= 4;
    }
    return 0;
  }
  return 1;
}

Bool_t TMemSystem::AccessPathName(const char *path, EAccessMode mode) {
   // Returns FALSE if one can access a file using the specified access mode.
   // Mode is the same as for the Unix access(2) function.
   // Attention, bizarre convention of return value!!
   if (m_memory->access(path, mode) == 0)
      return kFALSE;
   gSystem->SetErrorStr(TSystem::GetErrorStr());
   return kTRUE;
}


