#ifndef POOL_ROOTSTORAGESVC_TMEMFILE_H
#define POOL_ROOTSTORAGESVC_TMEMFILE_H

#include "TFile.h"
#include "TSystem.h"

namespace pool  { class DbMemoryIO; }

/** @class TMemFile TMemFile.h plugins/memfile/TMemFile.h
  *
  *                                                                  
  * A TMemFile is like a normal TFile except that it reads and writes   
  * its data via a Memd server.                                         
  *
  * @author  M.Frank
  * @version 1.0
  * @date    25/09/2004
  */
class TMemFile : public TFile {
private:
  pool::DbMemoryIO*  m_memory;
  Long64_t   m_offset;
  // Interface to basic system I/O routines
  Int_t      SysOpen(const char *pathname, Int_t flags, UInt_t mode);
  Int_t      SysClose(Int_t fd);
  Int_t      SysRead(Int_t fd, void *buf, Int_t len);
  Int_t      SysWrite(Int_t fd, const void *buf, Int_t len);
  Long64_t   SysSeek(Int_t fd, Long64_t offset, Int_t whence);
  Int_t      SysStat(Int_t fd, Long_t *id, Long64_t *size, Long_t *flags, Long_t *modtime);
  Int_t      SysSync(Int_t) { /* no fsync for Mem */ return 0; }

public:

  TMemFile();
  TMemFile(const char *url, Option_t *opt="",const char *tit="", Int_t compress=1);
  virtual ~TMemFile();
  int destroyFile(const char* name);
  int dumpFile(const char* name);
  int listFiles();
  int clearFiles();
};


class TMemSystem : public TSystem {
private:
  pool::DbMemoryIO*  m_memory;
public:
  TMemSystem();
  virtual ~TMemSystem() { }
  Int_t       MakeDirectory(const char *name);
  void       *OpenDirectory(const char *name);
  void        FreeDirectory(void *dirp);
  const char *GetDirEntry(void *dirp);
  int         GetPathInfo(const char *path, Long_t *id, Long64_t *size,
                          Long_t *flags, Long_t *modtime);
  int         GetPathInfo(const char *path, FileStat_t &buf)
  {   return TSystem::GetPathInfo(path, buf);                   }
  Bool_t      AccessPathName(const char *path, EAccessMode mode);
};

#endif // POOL_ROOTSTORAGESVC_TMEMFILE_H
