//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id: RulesDTDResolver.cpp,v 1.2 2008/02/29 10:02:36 witoldp Exp $
//
// Description:
//	Class RulesDTDResolver
//
// Environment:
//	This software was developed for the ATLAS collaboration.
//
// Author List:
//      Andy Salnikov
//
//------------------------------------------------------------------------

//-----------------------
// This Class's Header --
//-----------------------
#include "RulesDTDResolver.h"

//-----------------
// C/C++ Headers --
//-----------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "xercesc/framework/MemBufInputSource.hpp"
#include "xercesc/util/XMLString.hpp"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

using namespace xercesc ;

namespace {

  XMLByte dtd[] = "\
    <!ELEMENT rules_file_catalog (data_sources | rules*) > \
    <!ELEMENT data_sources (parent_catalog+) > \
    <!ATTLIST data_sources algorithm CDATA \"first-available\" > \
    <!ELEMENT parent_catalog EMPTY > \
    <!ATTLIST parent_catalog contact CDATA #REQUIRED > \
    <!ELEMENT rules (pfn_rule*) > \
    <!ATTLIST rules \
              host CDATA #IMPLIED \
              host_numbers CDATA #IMPLIED \
              xrootd CDATA #IMPLIED \
              pfn_regexp CDATA #IMPLIED \
              replace CDATA #IMPLIED > \
    <!ELEMENT pfn_rule EMPTY > \
    <!ATTLIST pfn_rule \
              host CDATA #IMPLIED \
              host_numbers CDATA #IMPLIED \
              xrootd CDATA #IMPLIED \
              pfn_regexp CDATA #IMPLIED \
              replace CDATA #IMPLIED > \
  ";

}


//		----------------------------------------
// 		-- Public Function Member Definitions --
//		----------------------------------------

// Constructors
pool::RulesDTDResolver::RulesDTDResolver() 
  : xercesc::EntityResolver()
{
}

// Destructor
pool::RulesDTDResolver::~RulesDTDResolver() 
{
}

// implementation of the resolver method
xercesc::InputSource* 
pool::RulesDTDResolver::resolveEntity ( const XMLCh* const /*publicId*/, 
                                                    const XMLCh* const systemId ) 
{
  XMLCh* dtdName = XMLString::transcode ( "rules-file-catalog-dtd" );
  bool match = XMLString::compareString( systemId, dtdName ) == 0 ;
  XMLString::release ( &dtdName ) ;
  if ( match ) {
    MemBufInputSource* memBufIS = 
      new MemBufInputSource ( ::dtd, strlen((const char*)::dtd), systemId, false ) ;
    return memBufIS;
  } else {
    return 0 ;
  }
}
