#ifndef POOL_RULESFCHOSTMATCHER_H
#define POOL_RULESFCHOSTMATCHER_H

//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id: RulesFCHostMatcher.h,v 1.1 2008/02/27 17:25:03 salnikov Exp $
//
// Description:
//	Class RulesFCHostMatcher.
//
// Environment:
//      This software was developed for the ATLAS collaboration.  If you
//      use all or part of it, please give an appropriate acknowledgement.
//
//------------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------
#include <string>

//----------------------
// Base Class Headers --
//----------------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------

//		---------------------
// 		-- Class Interface --
//		---------------------


/**
 *  This class matches the host specification in the rule file
 *  agains the current host name (or IP).
 *
 *  This software was developed for the ATLAS collaboration.  If you
 *  use all or part of it, please give an appropriate acknowledgement.
 *
 *  @see RulesFileCatalog
 *
 *  @version $Id: RulesFCHostMatcher.h,v 1.1 2008/02/27 17:25:03 salnikov Exp $
 *
 *  @author Andy Salnikov
 */

namespace pool {

class RulesFCHostMatcher {

public:

  // Constructor
  explicit RulesFCHostMatcher( const std::string& hostname = std::string() ) ;

  // Destructor
  ~RulesFCHostMatcher() ;

  // Match the given XML element agains the current host name
  bool match ( const std::string& hostRe, const std::string& hostNums ) const ;

protected:

  // Helper functions

private:

  // Friends

  // Data members
  std::string m_hostname ;

  // Note: if your class needs a copy constructor or an assignment operator, 
  //  make one of the following public and implement it.
  RulesFCHostMatcher( const RulesFCHostMatcher& );                // Copy Constructor
  RulesFCHostMatcher& operator= ( const RulesFCHostMatcher& );    // Assignment op

};

} // namespace pool

#endif // POOL_RULESFCHOSTMATCHER_H
