#ifndef POOL_RULESPFNRULE_H
#define POOL_RULESPFNRULE_H

//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id: RulesPFNRule.h,v 1.1 2008/02/27 17:25:03 salnikov Exp $
//
// Description:
//	Class Template.
//
// Environment:
//      This software was developed for the ATLAS collaboration.  If you
//      use all or part of it, please give an appropriate acknowledgement.
//
//------------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------
#include <string>
#include <boost/regex.hpp>

//----------------------
// Base Class Headers --
//----------------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------

//		---------------------
// 		-- Class Interface --
//		---------------------


/**
 *
 *  This software was developed for the ATLAS collaboration.  If you
 *  use all or part of it, please give an appropriate acknowledgement.
 *
 *  @see RulesFileCatalog
 *
 *  @version $Id: RulesPFNRule.h,v 1.1 2008/02/27 17:25:03 salnikov Exp $
 *
 *  @author Andy Salnikov
 */

namespace pool {

class RulesPFNRule {

public:

  /**
   *  Constructor
   *
   *  @param pfnRe    regular expression for PFN matching, can be empty
   *  @param replace  replacement expression, can be empty
   *  @param xrootd   host name of xrootd server, can be empty
   */
  RulesPFNRule( const std::string& pfnRe, const std::string& replace, const std::string& xrootd ) ;

  // Destructor
  ~RulesPFNRule();

  // Process PFN and return modified (or unchanged) PFN
  std::string process ( const std::string& pfn ) const ;

protected:

  // Helper functions

private:

  // Friends

  // Data members
  boost::regex m_re ;
  std::string m_replacement ;
  std::string m_xrootd ;

  // Note: if your class needs a copy constructor or an assignment operator, 
  //  make one of the following public and implement it.
  RulesPFNRule( const RulesPFNRule& );                // Copy Constructor
  RulesPFNRule& operator= ( const RulesPFNRule& );    // Assignment op

};

} // namespace pool

#endif // POOL_RULESPFNRULE_H
