//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id: FCHostMatcherTest.cpp,v 1.2 2009/02/18 13:33:04 witoldp Exp $
//
// Description:
//	Class PFNRuleTest...
//
// Author List:
//      $author
//
//------------------------------------------------------------------------

//-----------------------
// This Class's Header --
//-----------------------

//-----------------
// C/C++ Headers --
//-----------------
#include <iostream>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include <cppunit/extensions/HelperMacros.h>
#include "src/RulesFCHostMatcher.h"
#include "POOLCore/Exception.h"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

using namespace std ;
using namespace pool ;

//		----------------------------------------
// 		-- Public Function Member Definitions --
//		----------------------------------------

class FCHostMatcherTest : public CppUnit::TestFixture {

  CPPUNIT_TEST_SUITE( FCHostMatcherTest );
  CPPUNIT_TEST( testCtor );
  CPPUNIT_TEST_EXCEPTION( testExc1, pool::Exception );
  CPPUNIT_TEST_EXCEPTION( testExc2, pool::Exception );
  CPPUNIT_TEST( testMatch1 );
  CPPUNIT_TEST( testMatch2 );
  CPPUNIT_TEST( testMatch3 );
  CPPUNIT_TEST( testMatch4 );
  CPPUNIT_TEST_SUITE_END();

public:

  void setUp() {}
  void tearDown() {}
  
  void testCtor() ;
  void testExc1() ;
  void testExc2() ;
  void testMatch1() ;
  void testMatch2() ;
  void testMatch3() ;
  void testMatch4() ;

  void testMatch( const std::string& host, 
                const std::string& hostRe, 
                const std::string& hostNum,
                bool expect ) ;
};

void
FCHostMatcherTest::testCtor() 
{
  cout << "[OVAL] >>> test constructors" << endl ;

  // All of these are legitimate constructors
  RulesFCHostMatcher m1 () ;
  RulesFCHostMatcher m2 ( "atl-tdaq01" ) ;
}

void
FCHostMatcherTest::testExc1() 
{
  // wrong regexp should throw pool::Exception
  cout << "[OVAL] >>> test exception 1" << endl ;
  testMatch ( "atl-tdaq01", "*atl-tdaq]*", "", false ) ;
}

void
FCHostMatcherTest::testExc2() 
{
  // wrong host number list should throw pool::Exception
  cout << "[OVAL] >>> test exception 2" << endl ;
  testMatch ( "atl-tdaq01", "", "1-*", false ) ;
}

void
FCHostMatcherTest::testMatch1() 
{
  // test matching host name patters
  cout << "[OVAL] >>> test match 1" << endl ;
  testMatch ( "atl-tdaq01", "atl-tdaq.*", "", true ) ;
  testMatch ( "atl-tdaq01.slac.stanford.edu", "atl-tdaq.*edu", "", true ) ;
}

void
FCHostMatcherTest::testMatch2() 
{
  // Various combinations for matching the host number
  cout << "[OVAL] >>> test match 2" << endl ;
  testMatch ( "atl-tdaq01", "atl-tdaq%N", "1-10", true ) ;
  testMatch ( "atl-tdaq01", "atl-tdaq%N", "0,1,3,5-12", true ) ;
  testMatch ( "atl-tdaq01", "atl-tdaq.*", "1-10", true ) ;
  testMatch ( "atl-tdaq01", ".*", "1-10", true ) ;
  testMatch ( "atl-tdaq01", "", "1-10", true ) ;
}

void
FCHostMatcherTest::testMatch3() 
{
  // non-matching host number
  cout << "[OVAL] >>> test match 3" << endl ;
  testMatch ( "atl-tdaq01", "atl-tdaq%N", "11-20", false ) ;
  testMatch ( "atl-tdaq01", "%N", "11-20", false ) ;
  testMatch ( "atl-tdaq01", ".*", "11-20", false ) ;
  testMatch ( "atl-tdaq01", "", "11-20", false ) ;
}

void
FCHostMatcherTest::testMatch4() 
{
  // non-matching regular expression
  cout << "[OVAL] >>> test match 4" << endl ;
  testMatch ( "atl-tdaq01", "tdaq.*", "", false ) ;
}

void
FCHostMatcherTest::testMatch( const std::string& host, 
                        const std::string& hostRe, 
                        const std::string& hostNum,
                        bool expect ) 
{
  // my presumed host name
  RulesFCHostMatcher m ( host ) ;
  
  bool result = m.match ( hostRe, hostNum ) ;
  
  cout << "[OVAL] ... host: \"" << host << "\", hostRe: \"" << hostRe 
       << ", hostNum: \"" << hostNum 
       << "\", expect: " << ( expect ? "true" : "false" ) 
       << ", result: " << ( result ? "true" : "false" ) << endl ;

  CPPUNIT_ASSERT ( result == expect );
}

CPPUNIT_TEST_SUITE_REGISTRATION(FCHostMatcherTest);

#include<CppUnit_testdriver.cpp>
