//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id: PFNRuleTest.cpp,v 1.3 2009/02/06 09:42:21 witoldp Exp $
//
// Description:
//	Class PFNRuleTest...
//
// Author List:
//      $author
//
//------------------------------------------------------------------------


//-----------------------
// This Class's Header --
//-----------------------

//-----------------
// C/C++ Headers --
//-----------------
#include <iostream>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include <cppunit/extensions/HelperMacros.h>
#include "src/RulesPFNRule.h"
#include "POOLCore/Exception.h"


//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

using namespace std ;
using namespace pool ;

//		----------------------------------------
// 		-- Public Function Member Definitions --
//		----------------------------------------

class PFNRuleTest : public CppUnit::TestFixture {

  CPPUNIT_TEST_SUITE( PFNRuleTest );
  CPPUNIT_TEST( testCtor );
  CPPUNIT_TEST_EXCEPTION( testCtorExc1, pool::Exception );
  CPPUNIT_TEST_EXCEPTION( testCtorExc2, pool::Exception );
  CPPUNIT_TEST( testXrootd1 );
  CPPUNIT_TEST( testNoSubs );
  CPPUNIT_TEST( testSubs );
  CPPUNIT_TEST( testXrootd2 );
  CPPUNIT_TEST_SUITE_END();

public:

  void setUp() {}
  void tearDown() {}
  
  void testCtor() ;
  void testCtorExc1() ;
  void testCtorExc2() ;
  void testXrootd1() ;
  void testNoSubs() ;
  void testSubs() ;
  void testXrootd2() ;

};

void
PFNRuleTest::testCtor() 
{
  cout << "[OVAL] >>> test constructors" << endl ;

  // All of these are legitimate constructors
  RulesPFNRule r1 ( ".*", "",            "" ) ;
  RulesPFNRule r2 ( ".*", "replacement", "" ) ;
  RulesPFNRule r3 ( ".*", "",            "xrootd-host" ) ;
  RulesPFNRule r4 ( ".*", "replacement", "xrootd-host" ) ;
}

void
PFNRuleTest::testCtorExc1() 
{
  cout << "[OVAL] >>> test constructor exceptions 1" << endl ;
  
  // This should throw
  RulesPFNRule r ( "", "replacement", "" ) ;
}

void
PFNRuleTest::testCtorExc2() 
{
  cout << "[OVAL] >>> test constructor exceptions 2" << endl ;
  
  // This should throw
  RulesPFNRule r ( "***", "", "" ) ;
}

void
PFNRuleTest::testXrootd1() 
{
  cout << "[OVAL] >>> test xrootd 1" << endl ;
  
  RulesPFNRule r ( "", "", "xrootd-server" ) ;
  
  // input for the rule
  std::string input = "/full/path/name" ;
  // what we expect on output
  std::string expect = "root://xrootd-server//full/path/name" ;
  
  const std::string& tr = r.process ( input ) ;
  
  cout << "[OVAL] ... input: " << input << ", expect: " << expect << ", result: " << tr << endl ;

  CPPUNIT_ASSERT ( tr == expect );
}

void
PFNRuleTest::testNoSubs() 
{
  cout << "[OVAL] >>> test no replacement" << endl ;
  
  RulesPFNRule r ( "", "", "" ) ;
  
  // input for the rule
  std::string input = "/full/path/name" ;
  // what we expect on output
  std::string expect = input ;
  
  const std::string& tr = r.process ( input ) ;
  
  cout << "[OVAL] ... input: " << input << ", expect: " << expect << ", result: " << tr << endl ;

  CPPUNIT_ASSERT ( tr == expect );
}

void
PFNRuleTest::testSubs() 
{
  cout << "[OVAL] >>> test replacement" << endl ;
  
  RulesPFNRule r ( "/full/(.*)/name", "/better/$1/2file.root", "" ) ;
  
  // input for the rule
  std::string input = "/full/path/name" ;
  // what we expect on output
  std::string expect = "/better/path/2file.root" ;
  
  const std::string& tr = r.process ( input ) ;
  
  cout << "[OVAL] ... input: " << input << ", expect: " << expect << ", result: " << tr << endl ;

  CPPUNIT_ASSERT ( tr == expect );
}

void
PFNRuleTest::testXrootd2() 
{
  cout << "[OVAL] >>> test replacement+xrootd" << endl ;
  
  RulesPFNRule r ( "/full/(.*)", "/better/$1.root", "xrootd-server" ) ;

  // input for the rule
  std::string input = "/full/path/name" ;
  // what we expect on output
  std::string expect = "root://xrootd-server//better/path/name.root" ;
  
  const std::string& tr = r.process ( input ) ;
  
  cout << "[OVAL] ... input: " << input << ", expect: " << expect << ", result: " << tr << endl ;

  CPPUNIT_ASSERT ( tr == expect );
}


CPPUNIT_TEST_SUITE_REGISTRATION(PFNRuleTest);

#include<CppUnit_testdriver.cpp>
