// $Id: DataHandler.h,v 1.17 2008/05/13 14:39:31 witoldp Exp $
//====================================================================
//  DataHandler interface definition
//--------------------------------------------------------------------
//
//  Package    : StorageSvc (The pool framework)
//
//  Description: Interface of the Db persistenty service
//
//  @author      M.Frank
//
//====================================================================
#ifndef POOL_DATAHANDLER_H
#define POOL_DATAHANDLER_H
// $Header: /cvs/PF/pool/StorageSvc/StorageSvc/DataHandler.h,v 1.17 2008/05/13 14:39:31 witoldp Exp $

// Framework include files
#include "StorageSvc/DbStatus.h"
#include "RVersion.h"

// Forward declarations
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
namespace ROOT { namespace Reflex {  class Type;   }}
#else
namespace Reflex {  class Type;   }
#endif

/*
 *   POOL namespace declaration
 */
namespace pool {

  /** @class DataHandler DataHandler.h StorageSvc/DataHandler.h

      Define general base for object handling during persistency operations.

      @author Markus Frank 
  */
  class DataHandler    {
  public:
    ///Reflection information
#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
    typedef const ROOT::Reflex::Type& TypeH;
#else
    typedef const Reflex::Type& TypeH;
#endif

    /// Destructor (called only by sub-classes)
    virtual ~DataHandler()   {     }

    /// Quick identifier
    virtual unsigned int quickCast()  const  {   return 0; }
  
    /// Callback to handle on the fly object manipulation during save operations.
    /** @param data_type      [IN] DataType of the object to be processed
      * @param reflection     [IN] Reflection class information if availible
      * @param object_ptr     [IN] Pointer to the transient object.
      * @param data       [IN/OUT] Optional data for processing the request.
      *
      * @return DbStatus indicating success or failure.
      */
    virtual DbStatus onSave(int       data_type,      // data type
                            TypeH     reflection,     // Reflection information
                            void*     object_ptr,     // object ptr
                            void*     data) = 0;      // data pointer

    /// Callback to handle on the fly object manipulation during load operations.
    /** @param data_type      [IN] DataType of the object to be processed
      * @param reflection     [IN] Reflection class information if availible
      * @param object_ptr     [IN] Pointer to the transient object.
      * @param data       [IN/OUT] Optional data for processing the request.
      *
      * @return DbStatus indicating success or failure.
      */
    virtual DbStatus onLoad(int       data_type,      // data type
                            TypeH     reflection,     // Reflection information
                            void*     object_ptr,     // object ptr
                            void*     data) = 0;      // data pointer
  };
}       // End namespace pool
#endif  // POOL_DATAHANDLER_H
