// $Id: DbMemoryIO.h,v 1.8 2007/11/20 16:43:06 frankb Exp $
//====================================================================
//  DbHeap class definitions
//--------------------------------------------------------------------
//
//  Package    : StorageSvc  (The POOL project)
//  @author      M.Frank
//====================================================================
#ifndef POOL_DBMEMORYIO_H
#define POOL_DBMEMORYIO_H

// Framework include files
#include "StorageSvc/pool.h"

// C/C++ headers
#include <sys/stat.h>
#include <map>

/*
 *  POOL namespace declaration
 */
namespace pool  {

  // Forward declarations

  /**@class DbMemoryIO DbMemoryIO.h StorageSvc/DbMemoryIO.h
    *
    * Description:
    * Opaque file writing to memory buffer.
    *
    * @author  M.Frank
    * @version 1.0
    */
  class DbMemoryIO  {
  public:
    typedef struct stat IOStat;
    class Stat  {
    public:
      std::string    st_name;
      unsigned short st_mode;
      long long int     st_size;
      long long int     st_length;
      char*          st_fptr;
      char*          st_end;
      char*          st_buff;
      bool           st_open;
    };
    typedef std::map<std::string, int>  Pathes;
    typedef std::map<int, Stat>         Files;

    Pathes m_pathMap;
    Files  m_fdMap;
  public:

    enum AccessMode {
      kFileExists        = 0,
      kExecutePermission = 1,
      kWritePermission   = 2,
      kReadPermission    = 4
    };

    /// Singleton accessor
    static DbMemoryIO* instance();
    /// Standard constructor
    DbMemoryIO();
    /// Standard destructor
    virtual ~DbMemoryIO();
    /// POSIX like open statement
    int open(const std::string& url, int mode, unsigned int flag);
    /// POSIX like write statement
    long long int write(int fd, const void* buff, long long int siz);
    /// POSIX like read statement
    long long int read(int fd, void* buff, long long int siz);
    /// POSIX like close statement
    int close(int fd);
    /// Unlink single file
    int unlink(const std::string& url);
    /// Access single file
    int access(const std::string& url, int mode);
    /// POSIX like fstat statement
    int stat(const std::string& url, IOStat* buf);
    /// POSIX like lstat statement
    int stat(int fd, IOStat* buf);
    /// POSIX like lseek statement
    long long int seek(int fd, long long int offset, int origin);
    /// Unlink entire file system
    int unlinkAll();
    /// Dump single file information by name
    int dump(const std::string& url);
    /// Dump single file information by file descriptor
    int dump(int fd);
    /// Access file information
    Stat* info(int fd);
    /// Access file information
    Stat* info(const std::string& fd);
  };
}       // End namespace pool
#endif  // POOL_DBMEMORYIO_H
