//====================================================================
//  DbSequence class definition
//--------------------------------------------------------------------
//
//  Package    : poolDb (The POOL project)
//
//  Description: Sequence object
//
//  @author      M.Frank
//====================================================================
#ifndef POOL_DBSEQUENCE_H
#define POOL_DBSEQUENCE_H
#include "StorageSvc/DbObject.h"


/*
 * POOL namespace declaration
 */
namespace pool    {

  /** class DbSequence, DbSequence.h StorageSvc/DbSequence.h
    *
    * Class definition of the sequence object.
    *
    * @author  M.Frank
    * @version 1.0
    *
    */
  class DbSequence POOL_BASE {
  private:
    /// Sequence value
    int         m_value;
    /// Sequence name
    std::string m_name;
    /// Internal flag
    bool        m_needNew;

  public:
    /// Standard Constructor (Usage results in core-dump. NEVER use)
    DbSequence();
    /// Standard Destructor
    virtual ~DbSequence();
    /// Access next unique value
    virtual DbStatus next();
    /// Identifier of the next sequence number
    virtual int uniqueID()                  {  return m_value;      }
    /// Access Sequence name
    const std::string& name() const         {  return m_name;       }
    /// Set value context (If used, do not expect sequence to work!)
    void setUniqueID(int val)               {  m_value = val;       }
    /// Set Sequence name     (Internal use ONLY)
    void setName(const std::string& nam)    {  m_name = nam;        }
    /// Boolean internal flag (Internal use ONLY)
    bool needNew()    const                 {  return m_needNew;    }
    /// Boolean internal flag (Internal use ONLY)
    void setNew(bool val)                   {  m_needNew = val;     }

  };
}       // End namespace pool
#endif  // POOL_DBSEQUENCE_H
