//====================================================================
// $Id: IDbCursor.h,v 1.12 2007/11/28 16:19:32 frankb Exp $
//
//  Package    : pool/StorageSvc (The pool framework)
//
//  @author      M.Frank
//
//====================================================================
#ifndef POOL_IDBCURSOR_H
#define POOL_IDBCURSOR_H
// $Header: /cvs/PF/pool/StorageSvc/StorageSvc/IDbCursor.h,v 1.12 2007/11/28 16:19:32 frankb Exp $

// Framework include files
#include "StorageSvc/pool.h"

/*
 *  POOL namespace declaration
 */
namespace pool    {

  // Forward declarations
  class DbContainer;
  class DataCallBack;
  template <class T> class DbObjectHandle;

  /** @class IDbCursor IDbCursor.h StorageSvc/IDbCursor.h
    *
    * Definition of the IDbCursor class.
    *
    * Interface of a cursor to access all objects within a container.
    *
    * @author  Markus Frank                                               
    * @version 1                                                         
    */
  class IDbCursor  {
  protected:
    /// Destructor (called only by sub-classes)
    virtual ~IDbCursor()   {     }

  public:
    /// Release resource
    virtual void release() = 0;

    /// Access to cursor token
    virtual const Token* token() const = 0;

    /// Get next element of iteration using object handles
    /** @param cntH     [IN]  Handle to valid container object
      *                       (validity ensured by upper levels).
      * @param objH     [IN]  Reference to object handle receiving data.
      * @param mode     [IN]  Desired access mode.
      *
      * @return DbStatus code indicating success or failure.  
      */
    virtual DbStatus next(DbContainer&              cntH, 
                        DbObjectHandle<DbObject>& objH,
                        DbAccessMode              mode) = 0;

    /// Get next element of iteration using data callbacks
    /** @param cntH     [IN]  Handle to valid container object
      *                       (validity ensured by upper levels).
      * @param call     [IN]  Pointer to callback handling the data I/O.
      * @param mode     [IN]  Desired access mode.
      *
      * @return DbStatus code indicating success or failure.  
      */
    virtual DbStatus next(DbContainer&              cntH,
                        DataCallBack*             call, 
                        DbAccessMode              mode) = 0;
  };
}      // End namespace pool
#endif // POOL_IDBCURSOR_H
