//====================================================================
//
//  Package    : StorageSvc (The POOL project)
//
//  Author     : M.Frank
//====================================================================
#ifndef POOL_STORAGESVC_IIOHANDLER_H
#define POOL_STORAGESVC_IIOHANDLER_H

/*
 *   POOL namespace
 */
namespace pool {

  /** @class IIOHandler IIOHandler.h StorageSvc/IIOHandler.h
    *
    *   Interface declaration for IOHandler object
    *
    *   @author  M.Frank
    *   @version 1.0
    */
  class IIOHandler {
  protected:
    /// Destructor (called only by sub-classes)
    virtual ~IIOHandler()   {     }

  public:
    /// Category label
    static const char* category()  {  return "IIOHandler";  }

    /// Release call: destroy all internally allocated resources.
    virtual void release() = 0;

    /// Callback for reading the object
    /** @param obj  [IN]   Pointer to user object.
      */
    virtual void onReadUpdate(void* obj) = 0;

    /// Callback for writing the object
    /** @param obj  [IN]   Pointer to user object.
      */
    virtual void onWriteUpdate(void* obj) = 0;
  };
}      // End namespace pool
#endif // POOL_STORAGESVC_IIOHANDLER_H
