// $Header: /cvs/PF/pool/StorageSvc/StorageSvc/IStorageSvcEx.h,v 1.10 2007/11/28 16:19:33 frankb Exp $
#ifndef POOL_ISTORAGESVCEX_H
#define POOL_ISTORAGESVCEX_H

// Framework include files
#include "POOLCore/Token.h"
#include "StorageSvc/IStorageSvc.h"

// STL include files
#include <string>

/*
 *     pool namespace declaration
 */
namespace pool  {

  // Forward declarations
  class DbDatabase;

  /** @class IStorageSvcEx IStorageSvcEx.h StorageSvc/IStorageSvcEx.h

      This interface is only for direct handling of references.

      @author  Markus Frank
      @version 1.0
  */
  class IStorageSvcEx : virtual public IStorageSvc   {
  protected:
    /// Destructor (called only by sub-classes)
    virtual ~IStorageSvcEx()   {     }

  public:
    /// Retrieve interface ID
    static const Guid& interfaceID();

    /// IInterface implementation: Query interfaces of Interface
    virtual DbStatus queryInterface(const Guid& riid, void** ppvUnkn) = 0;

    /// IInterface implementation: Reference Interface instance               
    virtual unsigned int addRef() = 0;

    /// IInterface implementation: Release Interface instance                 
    virtual unsigned int release() = 0;

    /// Resolve reference OID to "real" token.
    /**
        @param   dbH       [IN] Reference to Database descriptor 
        @param   oid       [IN] Reference to the object ID of the target object
        @param   refpTok  [OUT] Reference to location for storing the
                                pointer of the resolved object token.

        @return                  DbStatus code indicating success or failure.
    */
    virtual DbStatus resolveRef(  const DbDatabase&   dbH,
                                  const Token::OID_t& oid,
                                  Token*&             refpTok) = 0;

    /// Create a reference for an object identified by its token.
    /** Fills a valid oid into the user supplied buffer given the
        token of the containing object and the token to the object
        the contained objects has a reference to.

        @param   dbH       [IN] Reference to Database descriptor 
        @param   toToken   [IN] Pointer to the token of the object the 
                                link should point to.
        @param   oid      [OUT] Reference to the object ID of the target object

        @return                 DbStatus code indicating success or failure.
    */
    virtual DbStatus makeRef(     const DbDatabase&   dbH,
                                  const Token*        toToken,
                                  Token::OID_t&       oid) = 0;
  };
}       // End namespace pool
#endif  // POOL_ISTORAGESVCEX_H
