# ifndef RCS_util_h__included__
# define RCS_util_h__included__

/// @file 
/// @author Jakub.Moscicki@cern.ch (kuba)

# include <string>
# include <sstream>

namespace Util 
{

/**
 * convert value to string using stream operator <<.
 * consequently, type T must have operator << for ostream
 */

template<class T>
std::string to_string(const T& val)
{
  std::ostringstream buf;
  buf << val;
  std::string ret = buf.str();
  return ret;
}


template<>
inline std::string to_string<std::string>(const std::string& s)
{
  return s;
}

/**
 * convert string to value using stream operator >>.
 * consequently, type T must have operator >> for istream
 */

template<class T>
bool to_value(std::string s, T& val)
{
  std::istringstream buf(s.c_str());
  return buf >> val;
}


template<>
inline bool to_value<std::string>(std::string s, std::string& dest)
{
  dest = s;
  return true;
}

}

# endif
