#include "POOLCore/Token.h"
#include "PersistencySvc/Placement.h"
#include "PersistencySvc/ITransaction.h"
#include "PersistencySvc/ISession.h"
#include "DataSvc/IDataSvc.h"
#include "DataSvc/Ref.h"
#include "DataSvc/RefException.h"
#include "TNamed.h"
#include "TestRefClasses/TestClass.h"
#include "DataSvc_AppendRead.h"

#ifndef WIN32
# include <unistd.h>
#endif

pool::AppendReadTest::AppendReadTest(
                       std::string& dbName1, const DbType& storage_type1,
                       std::string& dbName2, const DbType& storage_type2)
		       :ObjectStorageTest(dbName1,storage_type1,
                                          dbName2,storage_type2){
  m_sequence_type = 0;
}

bool pool::AppendReadTest::compareExpected(unsigned int i,const TestClass& obj){
  if(obj.id()!=(long)i){
    i=obj.id();
  }
  OId oid(i);
  TestClass t(i,oid);   
  if((int)i!=-1){
    char txt1[256], txt2[256];
    ::sprintf(txt1,"Name1 %d", i);
    ::sprintf(txt2,"Title %d", i);
    TNamed* nam = new TNamed(txt1,txt2);
    t.setTNamed(nam); 
    t.setLabel("WRITE");       
  }
  bool comp = (t== obj);
  if(!comp){
    printError("test ERROR: object retrieved different from the expected one");
    printM("Expected:-->");
    t.dump();
    printM("Retrieved:-->");
    obj.dump();
  }
  return comp;
}

bool pool::AppendReadTest::compareExpected(std::string,unsigned int i,
                                           const TestClass& obj){
  return compareExpected(i,obj);
}
void pool::AppendReadTest::reWrite(unsigned int NC){

  Placement place;
  place.setDatabase(m_dbName2,pool::DatabaseSpecification::PFN );
  place.setContainerName("test2_cont_1");
  place.setTechnology(m_type2.type());

  // Start an update transaction
  if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::UPDATE ) ) ) {
    printError("test ERROR: Could not start an UPDATE transaction");
  }  

  std::vector<pool::Ref<TestClass> > refTst(NC);
  std::vector<Ref<const TNamed> > refNams(NC);

  unsigned int i;
  if(NC>tokens().size()){
    printError("test ERROR: trying to read more objects than available");
    NC = tokens().size();
  }
  for (i = 0; i < NC; ++i )   {

    std::string tok = tokens()[i];

    // Read main object
    pool::Ref<TestClass> refObjRead(m_dataSvc,tok);

    int read_id = -1;
    if(refObjRead){
      read_id = refObjRead->id();
    }
    OId id(read_id);
    pool::Ref<TestClass> reftmp(m_dataSvc,new TestClass(i,id));        
        
    refTst[i] = reftmp;               

    try{
      refTst[i].markWrite(place);
      tokens().push_back(refTst[i].toString());
    } catch (const RefException& e){
      printException(e);
    }

    // this is only a work-around for avoid the cache location to be
    // removed from the cache after the Ref goes out of scope
    refNams[i].setCacheSvc(&m_dataSvc->cacheSvc());
    refNams[i] = refObjRead->named();  // the cache location is not there yet!
    refNams[i].token(); // ok, now the cache location has been assigned!    

    refTst[i]->setTNamed(refNams[i]);  
    refTst[i]->setLabel("WRITE");  
  }
  printM("Commit transaction...");
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the WRITE transaction." );
  }

  printM("Acid read-back test:");
  if ( ! m_dataSvc->transaction().start( pool::ITransaction::READ ) ) {
    printError("test ERROR: Could not start a READ transaction." );
  }
  for(i=0;i<NC;i++){
    if(!compareExpected(i,*refTst[i])) break;
  }
  if ( ! m_dataSvc->transaction().commit() ) {
    printError("test ERROR: Could not commit the READ transaction." );
  }

  // disconnect 
  m_dataSvc->session().disconnectAll();

  printM("Clearing cache");
  m_dataSvc->clearCache();
  return;
}

void pool::AppendReadTest::selectTestSequence(SequenceType type){
  m_sequence_type = type;
}

void pool::AppendReadTest::execute(){
  switch(m_sequence_type){
    case WRITE: {
      write(500);
      read(500);
      break;
    }
    case APPEND: {
      reWrite(500);
      read(1000);
      break;
    }
    case READ: {
      read(1000);
      break;
    }
    case FULL: {
      write(500);
      read(500);
      reWrite(500);
      read(1000);
      break;
    }
  }
}

int main(int argc, char** argv)     {
  typedef pool::AppendReadTest Test;
  std::string f1 = "test_AppendRead1.root";
  std::string f2 = "test_AppendRead2.root";
  Test theTest(f1, pool::ROOTKEY_StorageType,
               f2, pool::ROOTKEY_StorageType);
  if(argc > 1){
    if(argv[1][0]=='F'){
      theTest.selectTestSequence(Test::FULL);
    } else if(argv[1][0]=='W'){
      theTest.selectTestSequence(Test::WRITE);
    } else if(argv[1][0]=='U'){
      theTest.selectTestSequence(Test::APPEND);
    } else if(argv[1][0]=='R'){
      theTest.selectTestSequence(Test::READ);
    }
  } else {
    theTest.selectTestSequence(Test::FULL);
  }
  theTest.run();
  return 0;
}
