#include "PluginManager/PluginManager.h"
#include "POOLCore/Token.h"
#include "FileCatalog/URIParser.h"
#include "FileCatalog/IFileCatalog.h"
#include "StorageSvc/DbType.h"
#include "PersistencySvc/DatabaseConnectionPolicy.h"
#include "PersistencySvc/ISession.h"
#include "PersistencySvc/ITransaction.h"
#include "PersistencySvc/IDatabase.h"
#include "PersistencySvc/Placement.h"
#include "DataSvc/DataSvcFactory.h"
#include "DataSvc/IDataSvc.h"
#include "DataSvc/ICacheSvc.h"
#include "DataSvc/Ref.h"
#include "DataSvc/SharedAnyPtr.h"
#include <map>
#include<utility>
#include<vector>
#include<string>
#include "ABC.h"
#include <stdexcept>

#include<iostream>
#include<iomanip>

#include "SealBase/SharedLibrary.h"
#include "SealBase/SharedLibraryError.h"

#include "TestBase/TestKernel.h"

int go(int argc);
int main(int argc) {

  try { return go(argc);}
  catch(const seal::Exception& er){
    std::cout << "catched seal exception " << std::endl;
    er.printOut(std::cerr);
    return 4;
  }
}

int go(int){
  pool::TestKernel testLog; 

  seal::PluginManager::get()->initialise();

  std::string library = "test_DataSvc_CMSMultiCacheDict"; 
  std::cout << "Loading libraries " << library << std::endl;
  try {  
    seal::SharedLibrary::load( seal::SharedLibrary::libname( library ) );
  } catch ( seal::SharedLibraryError *error) {
    std::cout << "test ERROR: "<< error->explainSelf() << std::endl;
    testLog.printError("test ERROR: failure in loading shared libraries");
  }

  pool::URIParser p;
  p.parse();
  
  // get hold of a factory for file catalogs
//   pool::IFileCatalog* cat = pool::FCSystemTools::createCatalog(p.prefix());
  pool::IFileCatalog* cat = new pool::IFileCatalog();
  cat->setWriteCatalog( p.contactstring() );
  
  if (!cat) {
    throw std::runtime_error( "Could not create a file catalog" );
  }

//   cat->connect(p.url());

  cat->connect();
  cat->start();
  
  pool::IDataSvc *svc1 = pool::DataSvcFactory::instance(cat);
  pool::IDataSvc *svc2 = pool::DataSvcFactory::instance(cat);
  NavigationTests::B::c_dsvc=svc1;
  // Define the policy for the implicit file handling
  pool::DatabaseConnectionPolicy policy;
  policy.setWriteModeForNonExisting(pool::DatabaseConnectionPolicy::CREATE);
  // policy.setWriteModeForExisting(pool::DatabaseConnectionPolicy::OVERWRITE);
  policy.setWriteModeForExisting(pool::DatabaseConnectionPolicy::UPDATE);
  svc1->session().setDefaultConnectionPolicy(policy);
  svc2->session().setDefaultConnectionPolicy(policy);


  svc1->transaction().start(pool::ITransaction::UPDATE);
  pool::Ref<NavigationTests::CC> c1(svc1, new NavigationTests::CC);
  {

    pool::Placement place;
    place.setDatabase("CN",pool::DatabaseSpecification::PFN);
    place.setContainerName("L");
    place.setTechnology(pool::ROOTKEY_StorageType.type()); 
    // This will also register the file. For this to occur, the placement object must use a PFN.
    c1.markWrite(place);
    std::cout << "marked CC - N marked = " << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  if(svc1->cacheSvc().numberOfMarkedObjects()!=1){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N=" << svc1->cacheSvc().numberOfMarkedObjects() <<std::endl;
  }
  std::cout << "c1 " << c1.toString() << std::endl;
  svc1->transaction().commit();
  if(svc1->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked after commit = " << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc1->session().disconnectAll();
  svc1->cacheSvc().resetCache();

  std::cout << "c done" << std::endl;

  svc1->transaction().start(pool::ITransaction::READ);
  svc2->transaction().start(pool::ITransaction::UPDATE);
  pool::Ref<NavigationTests::B> b1(svc2, new NavigationTests::B(4));
  {
    pool::Placement place;
    place.setDatabase("BN", pool::DatabaseSpecification::PFN);
    place.setContainerName("A");
    place.setTechnology(pool::ROOTKEY_StorageType.type()); 
    // This will also register the file. For this to occur, the placement object must use a PFN.
    b1.markWrite(place);
    std::cout << "marked B - N marked = " << svc2->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  b1->c = c1;
  // b1->c =  pool::Ref<NavigationTests::CC>(svc2,c1.toString());
  if(svc2->cacheSvc().numberOfMarkedObjects()!=1){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N=" << svc2->cacheSvc().numberOfMarkedObjects() <<std::endl;
  }
  std::cout << "b1->c " << b1->c.toString() << std::endl;
  if(svc1->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked svc1 = " << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc2->transaction().commit();
  if(svc2->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked after commit svc2 = " << svc2->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc2->session().disconnectAll();
  svc2->cacheSvc().resetCache();
  svc1->transaction().commit();
  if(svc1->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked after commit svc1 = " << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
 svc1->session().disconnectAll();
  svc1->cacheSvc().resetCache();
  std::cout << pool::TestKernel::OVALTAG << "Number of entries in cache1: " 
            << svc1->cacheSvc().cacheSize() << std::endl;
  std::cout << pool::TestKernel::OVALTAG << "Number of entries in cache2: " 
            << svc2->cacheSvc().cacheSize() << std::endl;

  std::cout << "b done" << std::endl;

  svc1->transaction().start(pool::ITransaction::READ);
  svc2->transaction().start(pool::ITransaction::READ);

  std::cout << "C cache is " <<  &NavigationTests::B::c_dsvc->cacheSvc() << std::endl;
  // first explicit
  if(!c1.token()){
    testLog.printError("test ERROR: the token of c1 has been deleted after reset (1)");
  }
  std::cout << "c1 " << c1.toString() << std::endl;
  if(c1.isOpen()){
    testLog.printError("test ERROR: c1.isOpen() returns true after reset (1)");
  }
  //std::cout << "c1 o " << c1.isOpen() <<std:: endl;
  if(c1.isNull()){
    testLog.printError("test ERROR: c1.isNull() returns true after write on DB (1)");
  }
  //std::cout << "c1 n " << c1.isNull() << std::endl;
  //std::cout << "c1 t " << c1.token() << std::endl;
  std::cout << "c1 p " << &(*c1) << std::endl;
  std::cout << "c1 p " << (*c1).me() << std::endl;
  if(&(*c1) != (*c1).me()) {
    testLog.printError("test ERROR: c1 pointer != c1->me() (1)");
  }
  if(!c1.cacheSvc()){
    testLog.printError("test ERROR: c1 cacheSvc is null (1)");
  }
  //std::cout << "c1 c " << c1.cacheSvc() << std::endl;
  {
    pool::Ref<NavigationTests::A> c2(*c1.cacheSvc(),&(*c1));
    if(c2.cacheEntry()!=c1.cacheEntry()){
      testLog.printError("test ERROR: c2 cache entry not shared with c1 (1)"); 
    }
    //std::cout << "c2 " << c2.toString() << std::endl;
    std::cout << "c2 p " << &(*c2) << std::endl;
  }
  std::cout << "after c1 p " << &(*c1) << std::endl;
  {
    pool::Ref<NavigationTests::A> c2(svc1,const_cast<NavigationTests::AV*>((*c1).me()));
    if(c2.cacheEntry()!=c1.cacheEntry()){
      testLog.printError("test ERROR: c2 cache entry not shared with c1 (2)"); 
    }
    //std::cout << "c2 " << c2.toString() << std::endl;
    std::cout << "c2 p " << &(*c2) << std::endl;
  }

  {
    pool::Ref<NavigationTests::A> c4(svc2,c1.toString());
    if(c4.cacheEntry()==c1.cacheEntry()){
      testLog.printError("test ERROR: c4 cache entry is shared with c1 (1)"); 
    }
    //std::cout << "c4 " << c4.toString() << std::endl;
    std::cout << "c4 p " << &(*c4) << std::endl;
    pool::Ref<NavigationTests::A> c5(svc1,c1.toString());
    if(c5.cacheEntry()!=c1.cacheEntry()){
      testLog.printError("test ERROR: c5 cache entry not shared with c1 (1)"); 
    }
    //std::cout << "c4 " << c4.toString() << std::endl;
    std::cout << "c5 p " << &(*c5) << std::endl;
  }

  struct Bha {
    Bha(const pool::Ref<NavigationTests::A>& c, const std::string & n) : cm(c){
      std::cout << "bha " << n << " " << cm.toString() << std::endl;
      std::cout << "bha " << n << " p " << &(*cm) << std::endl;
    }
 
    pool::Ref<NavigationTests::A> cm;
  };
  std::cout << "after c1 p " << &(*c1) << std::endl;
  {
    pool::Ref<NavigationTests::A> c3;
    {
      pool::Ref<NavigationTests::AV> c2(svc1,const_cast<NavigationTests::AV*>((*c1).me()));
      if(c2.cacheEntry()!=c1.cacheEntry()){
        testLog.printError("test ERROR: c4 cache entry not shared with c1 (3)"); 
      }
      Bha aa(c2,"c2");
      c3 = c2;
    }
    if(c3.ptr() != c1.ptr()){
      testLog.printError("test ERROR c3.ptr() != c1.ptr() (1)");
    }
    std::cout << "after c3 p " << &(*c3) << std::endl;
  }
  std::cout << "after c1 p " << &(*c1) << std::endl;

  // then implicit
  if(b1){
    if(c1 != b1->c){
      testLog.printError("test ERROR: c1 != b1->c (1)");
    }
    std::cout << "b1->c " << b1->c.toString() << std::endl;
    if(!b1->c.isOpen()){
      testLog.printError("test ERROR: b1->c is not open after c opening ");
    }
    //std::cout << "b1->c o " << b1->c.isOpen() <<std:: endl;
    if(b1->c.isNull()){
      testLog.printError("test ERROR: b1->c is null (2)");
    }
    //std::cout << "b1->c n " << b1->c.isNull() << std::endl;
    if(b1->c.cacheEntry() != c1.cacheEntry()){
      testLog.printError("test ERROR: b1->c and c1 doesn't share the same cache entry");
    }
    //std::cout << "b1->c t " << b1->c.token() << std::endl;
    std::cout << "b1->c p " << &(*b1->c) << std::endl;
    if(b1->c.cacheEntry() != c1.cacheEntry()){
      testLog.printError("test ERROR: b1->c and c1 doesn't share the same cache entry");
    }    
    if(b1->c.cacheSvc() != c1.cacheSvc()){
      testLog.printError("test ERROR: b1->c and c1 doesn't share the same cache");
    }    
    //std::cout << "b1->c c " << b1->c.cacheSvc() << std::endl;
    if(b1->c){
      std::cout << "b1->c p " << &(*(b1->c)) << std::endl;
    } else {
      testLog.printError("test ERROR: b1->c read back is null");
    }
  } else {
    testLog.printError("test ERROR: b1 read back is null (1)");
  }

  //std::cout << "refs " << std::setiosflags(std::ios::boolalpha)
  //	    << (c1 == b1->c) << std::endl;

  if(!(*c1.token() == *b1->c.token())){
    testLog.printError("test ERROR: c1 token != b1->c token (1)");
  } 
  //std::cout << "tokens " << std::setiosflags(std::ios::boolalpha)
  //	    << (*c1.token() == *b1->c.token()) << std::endl;
  
  if(c1.toString() != b1->c.toString()){
    testLog.printError("test ERROR: c1 toString != b1->c toString (1)");
  } 
  //std::cout << "strings " << std::setiosflags(std::ios::boolalpha)
  //	    << (c1.toString() == b1->c.toString()) << std::endl;

  if(svc1->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object in reading"); 
    std::cout << "N marked svc1 = " << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  if(svc2->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object in reading"); 
    std::cout << "N marked svc2 = " << svc2->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc2->transaction().commit();
  if(svc2->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked svc2 after commit = " << svc2->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc2->session().disconnectAll();
  svc2->cacheSvc().resetCache();
  svc1->transaction().commit();
  if(svc1->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked svc1 after commit = " << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  svc1->session().disconnectAll();
  svc1->cacheSvc().resetCache();

  svc1->transaction().start(pool::ITransaction::READ);
  svc2->transaction().start(pool::ITransaction::READ);

  // first implicit
  if(b1){
    if(!b1->c.token()){
      testLog.printError("test ERROR: b1->c token is null after reset cache");
    }
    std::cout << "b1->c " << b1->c.toString() << std::endl;
    if(b1->c.isOpen()){
      testLog.printError("test ERROR: b1->c is Open after reset cache (3)");
    }
    //std::cout << "b1->c o " << b1->c.isOpen() <<std:: endl;
    if(b1->c.isNull()){
      testLog.printError("test ERROR: b1->c is null  (3)");
    }
    //std::cout << "b1->c n " << b1->c.isNull() << std::endl;
    //std::cout << "b1->c t " << b1->c.token() << std::endl;
    std::cout << "b1->c p " << &(*b1->c) << std::endl;
    if(c1.cacheEntry() != b1->c.cacheEntry()){
      testLog.printError("test ERROR: b1->c doesn't share cache entry with c1 (2)");
    }
  } else {
    testLog.printError("test ERROR: b1 read back is null (2)");
  }
  // then explicit
  if(!c1.token()){
      testLog.printError("test ERROR: c1 token is null after reset cache");
  }     
  std::cout << "c1 " << c1.toString() << std::endl;
  if(!c1.isOpen()){
    testLog.printError("test ERROR: c1 is Open after reset cache (3)");
  }
  //std::cout << "c1 o " << c1.isOpen() <<std:: endl;
  if(c1.isNull()){
    testLog.printError("test ERROR: c1 is null  (3)");
  }
  //std::cout << "c1 n " << c1.isNull() << std::endl;
  //std::cout << "c1 t " << c1.token() << std::endl;
  std::cout << "c1 p " << &(*c1) << std::endl;

  svc2->transaction().commit();
  svc2->session().disconnectAll();
  svc2->cacheSvc().resetCache();
  svc1->transaction().commit();
  svc1->session().disconnectAll();
  svc1->cacheSvc().resetCache();


  std::cout << "in a loop" << std::endl;
  for (int i=0;i<100;i++) {
    svc1->transaction().start(pool::ITransaction::UPDATE);
    {
      pool::Ref<NavigationTests::A> c3;
      {
	pool::Ref<NavigationTests::AV> c2(svc1,const_cast<NavigationTests::AV*>((*c1).me()));
	Bha aa(c2,"c2");
	c3 = c2;
        if(! (c2 == c3)){
          testLog.printError("test ERROR: c2 == c3 is false (2)");
        }
        if(! (c2 == c1)){
          testLog.printError("test ERROR: c2 == c1 is false (2)");
        }
     }
      std::cout << "after c3 p " << &(*c3) << std::endl;
    }

    pool::Ref<NavigationTests::B> bn(svc1, new NavigationTests::B(4));
    {
      pool::Placement place;
      place.setDatabase("BN", pool::DatabaseSpecification::PFN);
      place.setContainerName("B");
      place.setTechnology(pool::ROOTKEY_StorageType.type()); 
      // This will also register the file. For this to occur, the placement object must use a PFN.
      bn.markWrite(place);
    }
    bn->c = c1;
    
    std::cout << i << " after c1 p " << &(*c1) << " " << (*c1).me() << std::endl;
    if(svc1->cacheSvc().numberOfMarkedObjects()!=1){
      testLog.printError("test ERROR: wrong number of Marked Object"); 
      std::cout << "N marked svc1 = " 
                << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
    }

    svc1->transaction().commit();
    svc1->session().disconnectAll();
    svc1->cacheSvc().resetCache();
    
  }
  if(svc1->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked svc1 after commit = " 
                << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }

  std::cout << "new style loop" << std::endl;
  svc1->transaction().start(pool::ITransaction::READ);
  for (int i=0;i<10;i++) {
    svc2->transaction().start(pool::ITransaction::UPDATE);
    {
      pool::Ref<NavigationTests::A> c3;
      {
	pool::Ref<NavigationTests::AV> c2(svc1,const_cast<NavigationTests::AV*>((*c1).me()));
	Bha aa(c2,"c2");
	c3 = c2;
      }
    }

    pool::Ref<NavigationTests::B> bn1(svc2, new NavigationTests::B(4));
    pool::Ref<NavigationTests::B> bn2(svc2, new NavigationTests::B(4));
    {
      pool::Placement place;
      place.setDatabase("BN", pool::DatabaseSpecification::PFN);
      place.setContainerName("B");
      place.setTechnology(pool::ROOTKEY_StorageType.type());
      // This will also register the file. For this to occur, the placement object must use a PFN.
      bn1.markWrite(place);
      bn2.markWrite(place);
    }
    bn1->c = c1;
    bn2->c = &(*c1);

    if(c1.ptr() != bn1->c.ptr()){
      testLog.printError("test ERROR: bn1->c != c1 (1)");
    }
    if(c1.ptr() != bn2->c.ptr()){
      testLog.printError("test ERROR: bn1->c != c1 (1)");
    }
    if(bn1->c.cacheEntry() == bn2->c.cacheEntry()){
      testLog.printError("test ERROR: bn1->c and bn2->c share the same cache entry.");
    }
    if(c1.cacheEntry() != bn1->c.cacheEntry()){
      testLog.printError("test ERROR: c1 and bn2->c don't share the same cache entry.");
    }

    if(svc2->cacheSvc().numberOfMarkedObjects()!=2){
      testLog.printError("test ERROR: wrong number of Marked Object"); 
      std::cout << "N marked svc2  = " 
                << svc2->cacheSvc().numberOfMarkedObjects() << std::endl;
    }
    svc2->transaction().commit();
    svc2->session().disconnectAll();
    svc2->cacheSvc().resetCache();

  }
  if(svc1->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked svc1  = " 
                << svc1->cacheSvc().numberOfMarkedObjects() << std::endl;
  }

  svc1->transaction().commit();
  svc1->session().disconnectAll();
  svc1->cacheSvc().resetCache();
  if(svc2->cacheSvc().numberOfMarkedObjects()!=0){
    testLog.printError("test ERROR: wrong number of Marked Object"); 
    std::cout << "N marked svc2 after loop  = " 
                << svc2->cacheSvc().numberOfMarkedObjects() << std::endl;
  }
  cat->commit();

  delete svc2;
  delete svc1;
  delete cat;

  std::cout << "Test completed." << std::endl;
  unsigned int nobj = pool::SharedAnyPtr::numberOfInstances();
  std::cout << pool::TestKernel::OVALTAG << "Number of object instances: " 
            << nobj << std::endl;
  if(nobj){
    pool::SharedAnyPtr::dumpRegister();
  }
  testLog.dumpError();

  return 0;

}

