#include "PersistencySvc/Placement.h"
#include "PersistencySvc/ITransaction.h"
#include "PersistencySvc/ISession.h"
#include "POOLCore/Token.h"
#include "DataSvc/IDataSvc.h"
#include "DataSvc/Ref.h"
#include "DataSvc/RefException.h"
#include "TestBase/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/poolDb.h"
#include "TestRefClasses/TestClass.h"
#include <stdexcept>

#ifndef WIN32
# include <unistd.h>
#endif

namespace pool  {

  class SimpleDeleteTest : public TestBase {
    private:
    const DbType m_type;

    public: 
    SimpleDeleteTest(const DbType& storage_type)
                   :TestBase(),m_type(storage_type){
      std::vector< std::string > libraries;
      libraries.push_back( "test_TestRefClassesDict" );
      init( libraries );       
    };


    void erase(){
      m_dataSvc->transaction().start(pool::ITransaction::UPDATE);
        pool::Ref<TestClass> o1(m_dataSvc, new TestClass);
      {
        pool::Placement place;
        place.setDatabase("DN.root", pool::DatabaseSpecification::PFN);
        place.setContainerName("L"); 
        place.setTechnology(pool::ROOTKEY_StorageType.type()); 
        // This will also register the file. For this to occur, the placement object must use a PFN.
        o1.markWrite(place);
      }
      m_dataSvc->transaction().commit();
      m_dataSvc->session().disconnectAll();
      m_dataSvc->transaction().start(pool::ITransaction::UPDATE);
      o1.markDelete();  
      m_dataSvc->transaction().commit();
      m_dataSvc->session().disconnectAll();
      m_dataSvc->transaction().start(pool::ITransaction::UPDATE);
      m_dataSvc->transaction().commit();
      m_dataSvc->session().disconnectAll();
    }

    void execute(){
      erase();
    }
  };

}

int main(int, char**)     {
  typedef pool::SimpleDeleteTest Test;
  Test theTest(pool::ROOTTREE_StorageType);
  theTest.run();
  return 0;
}
