#include "PersistencySvc/Placement.h"
#include "PersistencySvc/ITransaction.h"
#include "POOLCore/Token.h"
#include "DataSvc/IDataSvc.h"
#include "DataSvc/Ref.h"
#include "DataSvc/RefException.h"
#include "TestBase/TestBase.h"
#include "StorageSvc/DbType.h"
#include "StorageSvc/poolDb.h"
#include "TestRefClasses/DerivedClass.h"
#include "TestRefClasses/DerivedTNamed.h"
#include <stdexcept>
#ifndef _WIN32
#include <unistd.h>
#endif

namespace pool  {

  class TypeCheckingTest : public TestBase {
  private:
    const std::string m_dbName1;
    const std::string m_dbName2;
    const DbType m_type1;
    const DbType m_type2;
    std::string m_fid1;
    std::string m_fid2;
    pool::Guid m_classGuid;

    bool compareExpected(unsigned int i,const DerivedClass& obj){
      OId oid(i);
      DerivedClass t(i,oid);   
      char txt1[256], txt2[256];
      ::sprintf(txt1,"Name1 %d", i);
      ::sprintf(txt2,"Title %d", i);
      TNamed* nam = new TNamed(txt1,txt2);
      t.setTNamed(nam);        
      bool comp = (t== obj);
      if(!comp){
        printError("test ERROR: object retrieved different from the expected one");
        printM("Expected:-->");
        t.dump();
        printM("Retrieved:-->");
        obj.dump();
      }
      return comp;
    }

  public: 
    TypeCheckingTest(std::string& dbName1, const DbType& storage_type1,
      std::string& dbName2, const DbType& storage_type2)
      :TestBase(),m_dbName1(dbName1),m_dbName2(dbName2),
      m_type1(storage_type1),m_type2(storage_type2),m_classGuid(){
        std::vector< std::string > libraries;
        libraries.push_back( "test_TestRefClassesDict" );
        init( libraries );       
        m_fid1 = fileId(m_dbName1,m_type1);
        m_fid2 = fileId(m_dbName2,m_type2);
        if(m_fid1 == "" || m_fid2 == ""){
          printError("test ERROR: cannot retrieve file id");
          return;
        }
      };

    /// write objects to the persistent store
    void write(unsigned int NC)       {

      printM("Start write procedure");  
      Placement place1, place2;
      place1.setDatabase(m_dbName1,pool::DatabaseSpecification::PFN );
      place1.setContainerName("test_cont_1");
      place1.setTechnology(m_type1.type());
      place2.setDatabase(m_dbName2,pool::DatabaseSpecification::PFN );
      place2.setContainerName("test_cont_2");
      place2.setTechnology(m_type2.type());

      // Start an update transaction
      if ( ! ( m_dataSvc->transaction().start( pool::ITransaction::UPDATE ) ) ) {
        printError("test ERROR: Could not start an UPDATE transaction");
      }  

      std::vector<pool::Ref<DerivedClass> > refTst(NC);
      unsigned int i;
      for (i = 0; i < NC; ++i )   {
        char txt1[256], txt2[256];
        ::sprintf(txt1,"Name1 %d", i);
        ::sprintf(txt2,"Title %d", i);

        //pool::Ref<DerivedTNamed> refNamed(m_dataSvc,new DerivedTNamed(txt1, txt2));
        pool::Ref<TNamed> refNamed(m_dataSvc,new TNamed(txt1, txt2));

        TNamed* nam = new TNamed(txt1, txt2);
        Ref<TNamed> nr(m_dataSvc->cacheSvc());

        if(nr==refNamed) printError("test ERROR: comparison 1 failed");
        if(nr!=0) printError("test ERROR: comparison 2 failed");

        nr = nam;
        if(nr==refNamed) printError("test ERROR: comparison 3 failed");
        if(refNamed.ptr()!=refNamed) printError("test ERROR: comparison 3.1 failed");
        if(nr==0) printError("test ERROR: comparison 4 failed");

        if(nr==refNamed || !(nr!=refNamed)){
          printError("test ERROR: refs equals operator (1) wrong!");

        }
        if(nam==refNamed || !(nam!=refNamed)){
          printError("test ERROR: ref-ptr equals operator (1) wrong!");
        } 

        OId id(i);
        pool::Ref<DerivedClass> reftmp(m_dataSvc,new DerivedClass(i,id));   

        if(nr==reftmp) printError("test ERROR: comparison 5 failed");
        if(0==reftmp) printError("test ERROR: comparison 6 failed");
        //if(nam==reftmp) printError("test ERROR: comparison 7 failed");

        //refTst[i] = reftmp;
        pool::Ref<TestClass> r0;
        if(r0==reftmp) printError("test ERROR: comparison 8 failed");
        r0.castDynamic(reftmp);
        if(r0!=reftmp) printError("test ERROR: comparison 9 failed");

        pool::Ref<DerivedClass> r1;
        r1.castDynamic(r0);
        if(r1!=reftmp) printError("test ERROR: comparison 10 failed");
        if(r1!=r0) printError("test ERROR: comparison 11 failed");

        Ref<DerivedClass> r2;
        r2.castDynamic(refNamed);
        if(r2!=0) printError("test ERROR: (2.0) cast dynamic error");
        if(!r2.isNull()) printError("test ERROR: (2.1) cast dynamic error");
        Ref<TestClass> r3(m_dataSvc->cacheSvc());
        r3 = new DerivedClass(i,id);
        r2.castDynamic(r3);
        if(r2==0) printError("test ERROR: (3.0) cast dynamic error");
        if(r2.isNull()) printError("test ERROR: (3.1) cast dynamic error");
        Ref<TestClass> r4;
        r4 = new DerivedClass(i,id);
        r2.castDynamic(r4);
        if(r2==0) printError("test ERROR: (4.0) cast dynamic error");
        if(r2.isNull()) printError("test ERROR: (4.1) cast dynamic error");
        Ref<TestClass> r5(m_dataSvc,new TestClass(i,id));
        r2.castDynamic(r5);

        if(r2!=0) printError("test ERROR: (5.0) 1 cast dynamic error");
        if(!r2.isNull()) printError("test ERROR: (5.1) 2 cast dynamic error");

        DerivedClass* tc = new DerivedClass(i,id);
        Ref<DerivedClass> rd(m_dataSvc,tc);
        if(r0==rd || !(r0!=rd)){
          printError("test ERROR: ref equals operator (2.0) wrong!");
        } 
        if(((TestClass*)tc)==r0 || !(((TestClass*)tc)!=r0)){
          printError("test ERROR: equals operator (2.1) wrong!");
        }

        refTst[i] = r1;

        try{
          refNamed.markWrite(place1);
        } catch (const RefException& e){
          printException(e);        
        }
        try{
          refTst[i].markWrite(place2);
          m_classGuid = refTst[i].token()->classID();
        } catch (const RefException& e){
          printException(e);
        }
        refTst[i]->setTNamed(refNamed);
        Token *tok = new Token;
        tok->fromString(refTst[i].toString());
        Ref<DerivedClass> rr1(m_dataSvc,*tok);
        tok->release();
        if(refTst[i]!=rr1) printError("test ERROR: comparison 12 failed");     
        Ref<DerivedClass> rr2;
        rr2 = rr1.ptr();
        if(refTst[i]!=rr2) printError("test ERROR: comparison 13 failed");             
      }

      printM("Commit transaction...");
      if ( ! m_dataSvc->transaction().commit() ) {
        printError("test ERROR: Could not commit the WRITE transaction." );
      }

      std::cout << OVALTAG << "Written on disk "<< NC << " objects\n";

      printM("Acid read-back test:");
      if ( ! m_dataSvc->transaction().start( pool::ITransaction::READ ) ) {
        printError("test ERROR: Could not start a READ transaction." );
      }
      for(i=0;i<NC;i++){
        if(!compareExpected(i,*refTst[i])) break;
      }
      if ( ! m_dataSvc->transaction().commit() ) {
        printError("test ERROR: Could not commit the READ transaction." );
      }

      printM("Clearing cache");
      m_dataSvc->clearCache();
      for(i=0;i<NC;i++){
        try{
          refTst[i]->named()->GetName();
          printError("test ERROR: Exception was not throw after cache clearing");
        }
        catch (const RefException& /* e */){  // expected behavor
        }
      }

      return;
    }

    /// read objects from the persistent store
    void read(unsigned int n_expected)        {
      // Starting a read transaction
      if ( ! m_dataSvc->transaction().start( pool::ITransaction::READ ) ) {
        printError("test ERROR: Could not start a READ transaction." );
      }
      unsigned int i;
      for ( i = 0;  ; ++i )   {
        Token *tok = new Token();
        // Since this example does not use collections,
        // we do it here by faking the tokens to be 
        // returned by the collections
        // 
        tok->setDb(m_fid2);
        tok->setCont("test_cont_2");
        tok->setTechnology(m_type2.type());
        tok->setClassID(m_classGuid);
        tok->oid().first  = 1;
        tok->oid().second = i;

        // testing failure for attempting to retrieve on a bad type 
        // (tok2 does not correspond to a TNamed!)
        try {
          pool::Ref<TNamed> badRef(m_dataSvc,*tok);
          badRef->GetName();
          printError("test ERROR: RefException was NOT thrown for invalid type!" );
        } catch (const RefException& /* e */){ // This is the correct behaviour.
        }

        // Read first object
        pool::Ref<TestClass> refObj(m_dataSvc,*tok);
        tok->release();

        Ref<DerivedClass> rd(m_dataSvc,*tok);
        if(refObj!=rd || !(refObj==rd)){
          printError("test ERROR: ref equals operator (3) failed");
        }

        if( !refObj ){
          std::cout << OVALTAG 
            << " End of reading after " << i << " Objects." << std::endl;
          break;
        }

        Ref<DerivedClass> r0;
        r0.castDynamic(refObj);
        r0->doNothing();
        if(!compareExpected(i,*r0)) break;      


        DerivedClass* deriv = dynamic_cast<DerivedClass*>(refObj.ptr());
        deriv->doNothing(); // only check that the call is possible

        if(!compareExpected(i,*deriv)) break;

        Ref<TestClass> r1;
        r1 = deriv;
        if(r1!=rd  || !(r1==rd)){
          printError("test ERROR: ref equals operator (4) failed");
        }
        if((TestClass*)deriv!=r1 || !((TestClass*)deriv==r1)){
          printError("test ERROR: ref-ptr equals operator (4) failed");
        }

        // testing Ref consistency
        pool::Ref<TestClass> reftmp(m_dataSvc);
        reftmp = refObj.ptr();
        if(reftmp.token()!=refObj.token()){
          printError("test ERROR: the token is not shared");
        }
        Ref<DerivedClass> r2;
        r2 = dynamic_cast<DerivedClass*>(refObj.ptr());
        reftmp = r2;
        r2 = rd;
        if(r2!=reftmp) printError("test ERROR: operator equals failed");       
        r2.clear();
        r2 = rd.ptr(); 
        if(r2.token()!=refObj.token()){
          printError("test ERROR: the token is not shared (2)");
        }
        Ref<TestClass> r3 = r2;
        if(r3!=reftmp) printError("test ERROR: operator equals failed");       

        refObj = 0;
      }

      if ( ! m_dataSvc->transaction().commit() ) {
        printError("test ERROR: Could not commit the READ transaction." );
      }

      if(i!=n_expected)
        printError("test ERROR: number of object retrieved different from expected");

      printM("Clear cache...");
      m_dataSvc->clearCache();
      return;
    }

    void execute(){ 
      write(1000);
      read(1000);
    }
  };

}

int main(int, char**)     {
  typedef pool::TypeCheckingTest Test;
  std::string f1 = "type_checking1.root";
  std::string f2 = "type_checking2.root";
  Test theTest(f1, pool::ROOT_StorageType,f2, pool::ROOT_StorageType );
  theTest.run();
  return 0;
}
