#ifndef POOL_EXPLICITECOLLECTION_TESTBASE_H
#define POOL_EXPLICITECOLLECTION_TESTBASE_H

#include <string>
#include <vector>
#include "POOLCore/Exception.h"

namespace pool  
{

  class IDataSvc;
  class IFileCatalog;
  class DbType;

  class TestBase
  {
    protected:
      IDataSvc* m_dataSvc;
      IFileCatalog* m_fileCatalog;
      bool m_init_test;

    private:
      unsigned int nerr;

    public:

      /// Standard Constructor
      TestBase();

      /// Standard Destructor
      virtual ~TestBase();

      std::string fileId( const std::string& pfn, const DbType& technology );

      /// Utility function to print exception messages
      void printException( pool::Exception& e );

      /// Loads dynamically a set of libraries
      static void init( const std::vector< std::string >& libraries );
 
      /// Method containing code of a test
      virtual void execute() = 0;

      /// Method that runs the test
      void run();
    

    public:

      static std::string OVALTAG;
      
      void printM( const char* str );

      void printE( pool::Exception& e );

      void dLine();

      void sLine();

      void printError( const char* str );

      void dumpError() const;

      unsigned int errors() const { return nerr; }

      void addError() { nerr++; }

      void resetError() { nerr = 0; }

  };

}

#endif // POOL_EXPLICITECOLLECTION_TESTBASE_H
