#include "FileCatalog/URIParser.h"
#include "SealKernel/Exception.h"
#include "FCPerformanceTestBase/FCPerformanceTestBase.h"



#include <iomanip>
#include <exception>
#include <string>
#include <iostream>

#include <ctime>
#include <sys/time.h>


pool::FCPerformanceTestBase::FCPerformanceTestBase( int argc, char* argv[] )
                : m_appName( "" ), m_noi( 1 ), m_noth( 1 ), m_nodeName( "single" ),
                  m_filePrefix( "very_very_long_physical_file_name_that_meets_better_real_requirements" ),
                  m_threads( 0 ), m_threadID( 0 )
{
  pool::POOLContext::loadComponent( "SEAL/Services/Message Service" );
  // pool::POOLContext::setMessageVerbosityLevel( coral::Info );

  if( ! parseArguments( argc, argv ) )
  {
    throw pool::FCPerformanceTestBase::BadArgumentsException();
  }

  m_parser.parse();

  if( m_parser.prefix() != "xmlcatalog" && m_parser.prefix() != "mysqlcatalog" )
  {
    throw pool::FCPerformanceTestBase::BadCatalogTypeException( m_parser.prefix() );
  }

  if( m_parser.prefix() == "xmlcatalog" && m_noth > 1 )
  {
    m_noth = 1;
    std::cout << "XML Catalog is not multithreaded! Threads number set to 1." << std::endl;
  }

  m_threads = new pthread_t[ m_noth ];
}


pool::FCPerformanceTestBase::~FCPerformanceTestBase()
{
  delete[] m_threads;
}


void pool::FCPerformanceTestBase::printSingleResults( std::string title, std::string action, TimeT* timeTable[] )
{
  TimeT sum[4];
  sum[0] = sum[1] = sum[2] = sum[3] = 0;

  std::cout << std::setfill( ' ' );
  std::cout << std::endl << title << " (all times in miliseconds):" << std::endl;
  std::cout.setf( std::ios::right );
  std::cout << std::setw( 15 ) << "Thread #" << std::setw( 15 ) << "start"
            << std::setw( 15 ) << action << std::setw( 15 ) << "commit"
            << std::setw( 15 ) << "disconnection" << std::endl;

  for( int i=0; i<m_noth; i++ )
  {
    std::cout << std::setw(15) << i;
    for( int j=0; j<4; j++ )
    {
      std::cout << std::setw(15) << timeTable[j][i];
      sum[j] += timeTable[j][i];
    }
    std::cout << std::endl;
  }
  std::cout << std::endl << std::setw(15) << "Average times:" << std::setw(15) << sum[0]/m_noth
            << std::setw(15) << sum[1]/m_noth << std::setw(15) << sum[2]/m_noth << std::setw(15) 
            << sum[3]/m_noth << std::endl << std::endl;

}


void pool::FCPerformanceTestBase::printException( seal::Exception& e )
{
  std::cerr << "SEAL exception: " << e.message() << "\n";
}


TimeT pool::FCPerformanceTestBase::universalTime()
{
  timeval now;
  gettimeofday( &now, 0 );
  return 1000000000LL*now.tv_sec + 1000LL*now.tv_usec;
}


void pool::FCPerformanceTestBase::printUserInfo( char* appName )
{
  std::cerr << "Usage: \n\t" << appName << " <number_of_iterations> [number_of_threads] [node_name]";
  std::cerr << std::endl << std::endl;
}


bool pool::FCPerformanceTestBase::parseArguments( int argc, char* argv[] )
{

  m_appName = std::string( argv[0] );

  switch ( argc )
  {
    case 2:
      if( atoi( argv[1] ) <= 0 ) return false;
      m_noi = atoi( argv[1] );
      break;
    case 3:
      if( ( atoi( argv[1] ) <= 0 ) || ( atoi( argv[2] ) <= 0 ) ) return false;
      m_noi = atoi( argv[1] );
      m_noth = atoi( argv[2] );
      break;
    case 4:
      if( ( atoi( argv[1] ) <= 0 ) || ( atoi ( argv[2] ) <= 0 ) ) return false;
      m_noi = atoi( argv[1] );
      m_noth = atoi( argv[2] );
      m_nodeName = std::string( argv[3] );
      break;
    default:
      return false;
  }
  return true;
}

