#ifndef PERFORMANCESTUDY_H
#define PERFORMANCESTUDY_H

#include <string>
#include <set>

namespace pool {
  class IFileCatalog;
  class IDataSvc;
}

class PerformanceStudy
{
public:
  // Constructor. Initializes services, loads the dictionary
  PerformanceStudy();

  // Destructor.
  virtual ~PerformanceStudy();

  // Adds a library in the list of libraries to be loaded during runtime
  void addRunTimeLibrary( const std::string& libName );

  // Runs the application
  int run();

protected:
  // Prepares a study. All initialization steps are done here.
  virtual void prepare();

  // The main method of the application
  virtual void theMainMethod();

  // Finalized the study. Extra calculations may be computed here.
  virtual void finalize();

  // Prints memory usage
  unsigned int currentMemoryUsage() const;

  // Returns the ellapsed time
  double ellapsedTime() const;

  // Returns the initial memory usage
  unsigned int initialMemory() const;

  // Returns the final memory usage
  unsigned int finalMemory() const;

  // Returns the dataSvc object
  pool::IDataSvc& dataSvc();

  // Returns the file catalog object
  pool::IFileCatalog& fileCatalog();

private:
  // The file catalog
  pool::IFileCatalog* m_fileCatalog;
  // The data service
  pool::IDataSvc* m_dataSvc;
  // The libraries to be loaded.
  std::set< std::string > m_libs;
  // The elapsed time for the main method
  double m_ellapsedTime;
  // The initial memory usage
  unsigned int m_initialMemory;
  // The final memory usage
  unsigned int m_finalMemory;

private:
  // Loads the runtime libraries
  void loadLibraries();

  // No copy constructor or assignment operator
  PerformanceStudy( const PerformanceStudy& );
  PerformanceStudy& operator=( const PerformanceStudy& );
};


#endif
