// $Id: ProcessTime.h,v 1.5 2003/07/21 16:42:28 rado Exp $
#ifndef POOL_BENCH_PROCESSTIME_H
#define POOL_BENCH_PROCESSTIME_H 1

#include "SealBase/TimeInfo.h"

# include <iosfwd>

namespace pool
{

	/** 
	 * Class for process time measurements.
	 *
	 */
	class ProcessTime
	{
		public:
			typedef seal::TimeInfo::NanoSecs Clock_t;
			
		public:
			/// Constructor, will initialise to current time
			ProcessTime();
			/// Contructor from individual time components (in internal units).
			ProcessTime( Clock_t rt, Clock_t ut, Clock_t st );
			/// Construtor initializing to zero
			ProcessTime( int );

			/// @name Scaling factors corresponding to the units of time.
			/// @{
			static const double SECONDS;
			static const double MILLISEC;
			static const double MICROSEC;
			static const double NANOSEC;
			/// @}

			/// Return wall clock in internal units. 
			/// Multiply by SECONDS constant to get the time in seconds.
			Clock_t getRealTime() const {
				return m_real;
			}

			/// Return user mode clock in internal units.
			/// Multiply by SECONDS constant to get the time in seconds.
			Clock_t getUserTime() const {
				return m_user;
			}

			/// Return system mode clock in internal units.
			/// Multiply by SECONDS constant to get the time in seconds.
			Clock_t getSystemTime() const {
				return m_system;
			}
	  
			std::ostream& printPrettyOn(std::ostream &os, double units=MILLISEC) const;

			std::ostream& printTabsOn(std::ostream&os, double units=MILLISEC) const;
                         	                
			ProcessTime operator -(const ProcessTime &otherTime) const;
			ProcessTime &operator =(const ProcessTime &otherTime);

		private:
			static Clock_t s_tps;
			Clock_t m_user;
			Clock_t m_system;
			Clock_t m_real;
	};

  std::ostream& operator<<( std::ostream& os, const ProcessTime& ptv ) {
    return ptv.printPrettyOn( os );
  }
  
}


#endif // POOL_BENCH_PROCESSTIME_H
