// $Id: ProcessTime.cpp,v 1.5 2003/07/21 16:42:29 rado Exp $
#include "PoolBench/ProcessTime.h"

#include <iostream>
#include <iomanip>

namespace pool
{
  const double ProcessTime::SECONDS  = 1e-9;
  const double ProcessTime::MILLISEC = 1e-6;
  const double ProcessTime::MICROSEC = 1e-3;
  const double ProcessTime::NANOSEC  = 1.0;

	ProcessTime::Clock_t
	ProcessTime::s_tps = 1e-9 * seal::TimeInfo::resolution();
	
	ProcessTime::ProcessTime()
		: m_user(0),
			m_system(0),
			m_real(0)
	{
		seal::TimeInfo::init();
		seal::TimeInfo::processTimes( m_user, m_system, m_real );
	}
	
	ProcessTime::ProcessTime( Clock_t rt, Clock_t ut, Clock_t st )
		: m_user( ut ),
			m_system( st ),
			m_real( rt )
	{
		seal::TimeInfo::init();
	}
	
	ProcessTime::ProcessTime( int )
		: m_user(0),
			m_system(0),
			m_real(0)
	{
		seal::TimeInfo::init();
	}
	
  ProcessTime::ProcessTime
	ProcessTime::operator -(const ProcessTime::ProcessTime &otherTime) const
  {
    return ProcessTime(getRealTime()-otherTime.getRealTime(), 
											 getUserTime()-otherTime.getUserTime(), 
											 getSystemTime()-otherTime.getSystemTime());
  }

  ProcessTime::ProcessTime&
	ProcessTime::operator =(const ProcessTime::ProcessTime &otherTime)
  {
    m_user = otherTime.getUserTime();
    m_system = otherTime.getSystemTime();
    m_real = otherTime.getRealTime();
    return *this;
  }

  std::ostream&
	ProcessTime::printPrettyOn(std::ostream &os, double units) const
  {
    std::string ustr;

    if( units == SECONDS )
      ustr = "s";

    if( units == MILLISEC )
      ustr = "ms";

    if( units == MICROSEC )
      ustr = "us";
    
    if( units == NANOSEC )
      ustr = "ns";
    
    os << "Real: "   << std::setprecision(9)
       << getRealTime()*units   << " " << ustr << " ";
    os << "User: "   << std::setprecision(9)
       << getUserTime()*units   << " " << ustr << " ";
    os << "System: " << std::setprecision(9)
       << getSystemTime()*units << " " << ustr;

    return os;
  }

  std::ostream& ProcessTime::printTabsOn(std::ostream &os, double units) const
  {
    os << getRealTime()*units << " ";
    os << getUserTime()*units << " ";
    os << getSystemTime()*units;
    return os;
  }
}
