#include "SealBase/Signal.h"

#include "PoolBench/ProcessTime.h"

#include <iostream>
#include <iomanip>

namespace pool
{
	/** This class boots the timing system and records process start-up time
	 * It must be the first object in the main program.
	 */
	class BenchApp
	{
		public:
			BenchApp( int argc, char* argv[] )
				: m_real_start_cycles( 0 ),
					m_real_start_time( 0 )
			{
				// Initialize timing system
				seal::TimeInfo::init ();

				// Record process start-up times
				if (seal::TimeInfo::features () & seal::TimeInfo::FEATURE_REAL_COUNT_EPOCH)
				{
					m_real_start_cycles = seal::TimeInfo::realCycles ();
					m_real_start_time   = seal::TimeInfo::realNsecs ();
				}

				// Install trace-back facility
				seal::Signal::handleFatal( argv[0] );
			}

			/// Return process start-up CPU cycles
			seal::TimeInfo::NanoTicks processStartupCycles() const {
				return m_real_start_cycles;
			}
			/// Return process start-up time
			seal::TimeInfo::NanoSecs processStartupTime() const {
				return m_real_start_time;
			}

		private:
			/// Real CPU cycles
			seal::TimeInfo::NanoTicks m_real_start_cycles;
			/// Real start up time
			seal::TimeInfo::NanoSecs  m_real_start_time;
	};
	
}

int main( int argc, char* argv[] )
{
	pool::BenchApp bap( argc, argv );
	pool::ProcessTime pt;

	std::cout << "------------------------------------------------\n"
						<< "Start    "
						<< bap.processStartupCycles()     << " ticks, "
						<< bap.processStartupTime() * 1e-9 << " secs\n" 
						<< "Process  " << std::endl << pt << std::endl
            << "System   "
						<< std::setprecision(9) << seal::TimeInfo::time () * 1e-9
						<< " since epoch, "
						<< seal::TimeInfo::elapsedTime () * pool::ProcessTime::SECONDS
						<< "s elapsed; resolution: "
						<< seal::TimeInfo::resolution () * pool::ProcessTime::SECONDS
            << std::endl;

	return 0;
}
