#ifndef TESTCLASS_H
#define TESTCLASS_H

#include "DataSvc/Ref.h"
#include "TNamed.h"
#include <iostream>

// 
struct OId  {

  long id;  

  static long next() 
    { static long cnt=0; return ++cnt; }

  OId():id(0){  
    id = next();
  }

  OId(long anId): id(anId){
  }

  OId(const OId& aId): id(aId.id){}

  virtual ~OId(){}

};

class TestClass  {

 private:

  // primitive data members
   long a;
   int b;
   float c;
   double d;

   // string data member
   std::string m_label;

   // Reference to Root type data member
   pool::Ref<const TNamed > m_ref;

   // custom type data member
   OId m_id;  
 
 public:

   // seed constructor
   TestClass(int seed, const OId& anId): 
     a(seed),b(seed),c(float(seed)),d(seed),m_label("TestClass"),m_id(anId){ 
    }

   // empty (default) constructor 
   TestClass(): 
     a(0),b(0),c(0),d(0),m_label("TestClass"),m_id(){ 
    }

   // default destructor
    virtual ~TestClass() {
    }

    const TNamed* named() const {
      return m_ref;
    }
  
    void setTNamed(const TNamed* pObj)        {
      m_ref = pObj;
    }
    
    int id() const {
      return m_id.id;
    }

    void setLabel(const std::string& aLabel){
      m_label = aLabel;
    }
  
    const std::string label() const {
      return m_label;
    }
    
    bool operator==(const TestClass& t) const;

    void dump() const;

    static bool equals(const TNamed& t1,const TNamed& t2);

};

#endif
