#ifndef EVENTSERVICE_H
#define EVENTSERVICE_H

#include "Reflex/Type.h"
#include "IEventService.h"
#include "ITokenCache.h"
#include <map>

#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif


class OriginObject;

namespace pool {
  class Token;
  class IFileCatalog;
  class IPersistencySvc;
  class DataHandler;
  class Placement;
  class ITokenIterator;
}



class EventService: virtual public IEventService,
		    virtual private ITokenCache
{
public:
  EventService();
  ~EventService();
  IDataCache* cache();
  void writeObjectTree( const OriginObject* object );
  const OriginObject* readNextEvent();

private:
  void registerObject( TargetObject* object, const std::string& key );
  TargetObject* lookup( const std::string& key );
  std::string key( TargetObject* object ) const;

  int registerToken( pool::Token* token );
  void registerKey( int index, const std::string& key );
  pool::Token* saveObject( const TargetObject* object );

private:
  const Reflex::Type m_originClassType;
  const Reflex::Type m_targetClassType;
  pool::IFileCatalog*         m_fileCatalog;
  pool::IPersistencySvc*      m_persistencySvc;
  pool::DataHandler*          m_dataHandler;
  pool::Placement*            m_placementHintOrigin;
  pool::Placement*            m_placementHintTarget;
  OriginObject*               m_originObject;
  pool::ITokenIterator*       m_tokenIterator;

  std::map< std::string, std::pair< pool::Token*, TargetObject* > > m_linksAndTokens;
  std::map< int, pool::Token* > m_tokens;

  void clearCache();
};

#endif
