#include "TestDriver.h"
#include <stdexcept>
#include <iostream>
#include <sstream>
#include <memory>

#include "Reflex/SharedLibrary.h"
#include "libname.h"



#include "EventService.h"
#include "OriginObject.h"
#include "TargetObject.h"

namespace pool  {
  int numTokenInstances();
}

pool::TestDriver::TestDriver():
  m_events( 100 )
{
  
  // pool::POOLContext::setMessageVerbosityLevel( coral::Info );
}

pool::TestDriver::~TestDriver()
{
  std::cout << "[OVAL] Number of floating tokens : " << pool::numTokenInstances() << std::endl;
}

void
pool::TestDriver::loadLibraries( const std::vector<std::string>& libraries )
{
  for ( std::vector<std::string>::const_iterator iLibrary = libraries.begin();
	iLibrary != libraries.end(); ++iLibrary ) {
    const std::string& library = *iLibrary;
    std::cout << "Loading library " << library << std::endl;
    
    Reflex::SharedLibrary libloader( libname(library) );
    
    if(!libloader.Load()){
      std::cout << libloader.Error() << std::endl;
    }
  }
}

void
pool::TestDriver::write()
{
  std::auto_ptr< IEventService > eventService( new EventService() );
  for ( int iEvent = 0; iEvent < m_events; ++iEvent ) {
    // Creating the top object
    OriginObject* topObject = new OriginObject();
    std::ostringstream osLabel;
    osLabel << "Event_" << iEvent;
    topObject->label = osLabel.str();
    for ( int iTargetObject = 0; iTargetObject < iEvent; ++iTargetObject ) {
      // Creating a target object
      TargetObject* targetObject = new TargetObject();
      int targetData = m_events * iEvent + iTargetObject;
      targetObject->targetData = targetData;

      // Registering it in the cache
      std::ostringstream osKey;
      osKey << "Key_" << targetData;
      eventService->cache()->registerObject( targetObject, osKey.str() );

      // Setting the link
      topObject->data.push_back( DataLink( eventService->cache(),
					   targetObject ) );
    }
    // Writing the top object
    eventService->writeObjectTree( topObject );
  }
}


void
pool::TestDriver::read()
{
  std::auto_ptr< IEventService > eventService( new EventService() );

  int numberOfEventsRead = 0;
  for ( const OriginObject* topObject = eventService->readNextEvent(); topObject != 0; ++numberOfEventsRead, topObject = eventService->readNextEvent() ) {
    if ( numberOfEventsRead > m_events ) break;
    std::ostringstream osLabel;
    osLabel << "Event_" << numberOfEventsRead;
    const std::string expectedLabel = osLabel.str();
    if ( topObject->label != expectedLabel ) {
      std::ostringstream error;
      error << topObject->label << " read back instead of " << expectedLabel;
      throw std::runtime_error( error.str() );
    }
    const std::vector< DataLink >& data = topObject->data;
    if ( data.size() != static_cast< unsigned int>( numberOfEventsRead ) ) {
      std::ostringstream error;
      error << data.size() << " elements found instead of " << numberOfEventsRead;
      throw std::runtime_error( error.str() );
    }
    for ( int iElement = 0; iElement < numberOfEventsRead; ++iElement ) {
      const int readTargetData = data[iElement]->targetData;
      const int expectedData = m_events * numberOfEventsRead + iElement;
      if ( expectedData != readTargetData ) {
	std::ostringstream error;
	error << readTargetData << " read back instead of " << expectedData;
	throw std::runtime_error( error.str() );
      }
    }
  }

  if ( numberOfEventsRead != m_events ) {
    std::ostringstream error;
    error << numberOfEventsRead << " events read back instead of " << m_events;
    throw std::runtime_error( error.str() );
  }
}
