#include "MyDataCallBack.h"
#include "StorageSvc/DataHandler.h"
#include "StorageSvc/Shape.h"
#include "StorageSvc/DbColumn.h"
#include "MyTestClasses.h"
#include <vector>

MyDataCallBack::MyDataCallBack():
  m_transientType( Reflex::Type::ByName( "MyTransientClass" ) ),
  m_dataHandler( 0 ),
  m_shape( 0 ),
  m_transientObjectData( 0 ),
  m_persistentObjectData( 0 )
{}

MyDataCallBack::~MyDataCallBack()
{}

pool::DataCallBack*
MyDataCallBack::clone() const
{
  MyDataCallBack* p = new MyDataCallBack();
  p->m_dataHandler = m_dataHandler;
  p->m_shape = m_shape;
  return static_cast< pool::DataCallBack* >( p );
}


Reflex::Type
MyDataCallBack::transientType() const
{
  return m_transientType;
}

void
MyDataCallBack::release()
{
  pool::DataCallBack::release();
}

void
MyDataCallBack::setObject( const void* ptr )
{
  m_transientObjectData = ptr;
}

const void*
MyDataCallBack::object() const
{
  return m_transientObjectData;
}

void
MyDataCallBack::setShape( const pool::Shape* shape )
{
  m_shape = shape;
}

const pool::Shape*
MyDataCallBack::shape() const
{
  return m_shape;
}

void
MyDataCallBack::setHandler( pool::DataHandler* pSvc )
{
  m_dataHandler = pSvc;
}

pool::DataHandler*
MyDataCallBack::handler() const
{
  return m_dataHandler;
}


pool::DbStatus
MyDataCallBack::start( pool::DataCallBack::CallType action_type,
		       void* data,
		       void** context)
{
  switch(action_type) {
  case GET:
    {
      // Do not allocate space for the persistent data buffer.
      // The back end implementation will do that.
      m_persistentObjectData = 0;
    }
    break;
  case PUT:
    // Check if the data corresponds to the transient data pointer
    if ( data != m_transientObjectData ) {
      // Input data is different from the transient data
      return pool::Error;
    }
    else {
      // allocate space for the persistent data buffer
      MyPersistentClass* pc = new MyPersistentClass();
      if ( ! pc ) return pool::Error;
      // Copy the relevant data from the transient class to the persistent one.
      const MyTransientClass* tc = static_cast< const MyTransientClass* >( m_transientObjectData );
      pc->a = tc->a;
      pc->b = tc->b;
      pc->d = tc->d;
      m_persistentObjectData = pc;
    }
    break;
  default:
    return pool::Error;
  };
  *context = m_persistentObjectData;
  return pool::Success;
}


pool::DbStatus
MyDataCallBack::bind( pool::DataCallBack::CallType action_type,
		      const pool::DbColumn* col_identifier,
		      int /* col_number */,
		      void* context,
		      void** data )
{
  switch(action_type)  {
  case GET:
    *data = &m_persistentObjectData;
    break;
  case PUT:
    /* This works for ROOT Trees but not for ROOT keys !!!
    if ( context != m_persistentObjectData ) {
      std::cerr << "Context different than persistent data" << std::endl;
      return pool::Error;
    }
    */
    *data = static_cast< char* >( context ) + col_identifier->offset();
    break;
  default:
    return pool::Error;
  };
  return pool::Success;
}

pool::DbStatus
MyDataCallBack::end( pool::DataCallBack::CallType action_type,
		     void* /* context */ )
{
  switch(action_type)  {
  case GET:
    {
      // Create and fill the corresponding transient object
      MyTransientClass* tc = new MyTransientClass();
      MyPersistentClass* pc = static_cast< MyPersistentClass* >( m_persistentObjectData );
      tc->a = pc->a;
      tc->b = float(pc->b);
      tc->c = 123;
      tc->d = pc->d;
      m_transientObjectData = tc;
      // free the persistent data buffer
      delete pc;
      m_persistentObjectData = 0;
      break;
    }
  case PUT:
    // free the persistent data buffer
    delete static_cast< MyPersistentClass* >( m_persistentObjectData );
    m_persistentObjectData = 0;
    break;
  default:
    return pool::Error;
  };
  return pool::Success;
}


pool::DbStatus
MyDataCallBack::notify( pool::DataCallBack::CallType action_type,
			int data_type,
			Reflex::Type reflection,
			void* object_ptr,
			void* data )
{
  if ( m_dataHandler ) {
    switch( action_type )   {
    case GET:
      return m_dataHandler->onLoad(data_type, reflection, object_ptr, data);
    case PUT:
      return m_dataHandler->onSave(data_type, reflection, object_ptr, data);
    default:
      return pool::Error;
    };
  }
  return pool::Success;
}
