#ifndef TESTCLASSES_H
#define TESTCLASSES_H

#include <string>
#include <vector>
#include <sstream>
#include <iostream>
#include <boost/cstdint.hpp>
#if defined(WIN32) || defined(_WIN32)
typedef unsigned int uint32_t;
#endif

struct Metadata{
  float EtMin,EtMax;
    int signaltks;
    Metadata():EtMin(0),EtMax(0),signaltks(0){}
    Metadata(const Metadata& rhs):EtMin(rhs.EtMin),EtMax(rhs.EtMax),signaltks(rhs.signaltks){}
    Metadata& operator=(const Metadata& rhs){
      EtMin = rhs.EtMin;
      EtMax = rhs.EtMax;
      signaltks = rhs.signaltks;
      return *this;
    }
    bool operator==(const Metadata& rhs) const {
      if(EtMin!=rhs.EtMin) return false;
      if(EtMax!=rhs.EtMax) return false;
      if(signaltks!=rhs.signaltks) return false;
      return true;
    }
  bool operator!=(const Metadata& rhs) const {
    return !operator==(rhs);
  }
    
};

struct Entry{
    Metadata metadata;
    std::vector<double> m_histogram;
    Entry():metadata(),m_histogram(){}
    Entry(const Entry& rhs):metadata(rhs.metadata),m_histogram(rhs.m_histogram){}
    Entry& operator=(const Entry& rhs){
      metadata = rhs.metadata;
      m_histogram = rhs.m_histogram;
      return *this;
    }
    bool operator==(const Entry& rhs) const {
      if(metadata!=rhs.metadata) return false;
      if(m_histogram!=rhs.m_histogram) return false;
      return true;
    }
  bool operator!=(const Entry& rhs) const {
    return !operator==(rhs);
  }
};

class SimpleMember {

 public:
  SimpleMember():m_flag(false),m_id(0){}
  SimpleMember(const SimpleMember& mem):m_flag(mem.m_flag),m_id(mem.m_id){}
  SimpleMember& operator=(const SimpleMember& mem){
    m_flag = mem.m_flag;
    m_id = mem.m_id;
    return *this;
  }
  bool m_flag;
  long m_id;

};


class SimpleClass {

 public:

  SimpleClass():m_intData(0),m_stringData(""),m_objectData(),m_arrayData(),m_nestedArrayData(),m_entries(){}
  SimpleClass(unsigned int id):m_intData(id),m_stringData(""),m_objectData(),m_arrayData(),m_nestedArrayData(),m_entries(){
    m_intData = id;
    std::ostringstream os;
    os << "SimpleClass_"<<id; 
    m_stringData = os.str();
    m_objectData.m_flag = true;
    m_objectData.m_id = id;
    for(int i=0;i<(int)id+1;i++){
      m_arrayData.push_back(i);
    }
    for(int j=0;j<(int)id+1;j++){
      std::vector<double> vec;
      for(int i=0;i<100;i++){
        double v = 0.+i;
        vec.push_back(v);
      }
      m_nestedArrayData.push_back(vec);
    }
    for(int i=0;i<(int)id+1;i++){
      Entry entry;
      entry.metadata.EtMax = float(0.+i);
      entry.metadata.EtMin = entry.metadata.EtMax-float(1.);
      if(entry.metadata.EtMin < 0.) entry.metadata.EtMin = float(0.);
      entry.metadata.signaltks = i;
      for(int j=0;j<10;j++){
        double v = 0.+j;
        entry.m_histogram.push_back(v);
      }
      m_entries.push_back(entry);
    }
    for(int j=0;j<2;j++){
      for(int i=0;i<20;i++){
        float v = float(0.+i);
        m_carrayData[j].push_back(v);
      }
    }
    
  }

  SimpleClass(const SimpleClass& obj):m_intData(obj.m_intData),m_stringData(obj.m_stringData),m_objectData(obj.m_objectData),m_arrayData(obj.m_arrayData),m_entries(obj.m_entries){}
  SimpleClass& operator=(const SimpleClass& obj){
    m_intData = obj.m_intData;
    m_stringData = obj.m_stringData;
    m_objectData = obj.m_objectData;
    m_arrayData = obj.m_arrayData;
    m_nestedArrayData = obj.m_nestedArrayData;
    m_entries = obj.m_entries;
    for(int i=0;i<2;++i){
      m_carrayData[i] = obj.m_carrayData[i];
    }
    return *this;
  }

  bool operator==(const SimpleClass& rhs){
    if(m_intData!=rhs.m_intData) return false;
    if(m_stringData!=rhs.m_stringData) return false;
    if(m_objectData.m_flag!=rhs.m_objectData.m_flag) return false;
    if(m_objectData.m_id!=rhs.m_objectData.m_id) return false;
    if(m_arrayData!=rhs.m_arrayData) return false;
    if(m_nestedArrayData!=rhs.m_nestedArrayData) {
      std::cout << "Size expected="<<m_nestedArrayData.size()<<" - found="<<rhs.m_nestedArrayData.size()<<std::endl;
      return false;
    }
    if(m_entries!=rhs.m_entries) return false;
    for(int i=0;i<2;++i){
      if(m_carrayData[i]!=rhs.m_carrayData[i]) return false;
    }
    return true;
  }
  bool operator!=(const SimpleClass& rhs){
    return !operator==(rhs);
  }

  unsigned int m_intData;
  std::string m_stringData;
  SimpleMember m_objectData;
  std::vector<int> m_arrayData;
  std::vector<std::vector<double> > m_nestedArrayData;
  std::vector<Entry> m_entries;
  std::vector<float> m_carrayData[2];
};


class OtherClass {

 public:

  OtherClass():m_intData(0),m_stringData(""),m_objectData(),m_arrayData(){}
  OtherClass(unsigned int id):m_intData(id),m_stringData(""),m_objectData(),m_arrayData(){
    m_intData = id;
    std::ostringstream os;
    os << "OtherClass_"<<id; 
    m_stringData = os.str();
    m_objectData.m_flag = true;
    m_objectData.m_id = id;
    for(int i=0;i<(int)id+1;i++){
      m_arrayData.push_back(i);
    }
  }

  OtherClass(const OtherClass& obj):m_intData(obj.m_intData),m_stringData(obj.m_stringData),m_objectData(obj.m_objectData),m_arrayData(obj.m_arrayData){}
  OtherClass& operator=(const OtherClass& obj){
    m_intData = obj.m_intData;
    m_stringData = obj.m_stringData;
    m_objectData = obj.m_objectData;
    m_arrayData = obj.m_arrayData;
    return *this;
  }

  bool operator==(const OtherClass& rhs){
    if(m_intData!=rhs.m_intData) return false;
    if(m_stringData!=rhs.m_stringData) return false;
    if(m_objectData.m_flag!=rhs.m_objectData.m_flag) return false;
    if(m_objectData.m_id!=rhs.m_objectData.m_id) return false;
    if(m_arrayData!=rhs.m_arrayData) return false;
    return true;
  }
  bool operator!=(const OtherClass& rhs){
    return !operator==(rhs);
  }

  unsigned int m_intData;
  std::string m_stringData;
  SimpleMember m_objectData;
  std::vector<int> m_arrayData;
};

class DataClass {
 public:
  DataClass():m_f(0.),m_i(0){}
  DataClass(unsigned int id) : m_f(float(0.+id)),m_i(int(id)){}
  virtual ~DataClass(){}
  DataClass(const DataClass& cp):m_f(cp.m_f),m_i(cp.m_i){}
  DataClass& operator=(const DataClass& cp){
    m_f = cp.m_f;
    m_i = cp.m_i;
    return *this;
  }
  bool operator==(const DataClass& rhs){
    if(m_f!=rhs.m_f)return false;
    if(m_i!=rhs.m_i)return false;
    return true;
  }
  bool operator!=(const DataClass& rhs){
    return !operator==(rhs);
  }
  float m_f;
  int m_i;

};

class SiStripNoises {
  public:
  SiStripNoises():v_noises(),indexes(){}
  SiStripNoises(unsigned int id):v_noises(),indexes(){
    short id_s = (short)id;
    for(short i=0;i<id_s+1;i++) v_noises.push_back(i);
    for(unsigned int i=0;i<id+1;i++) {
      DetRegistry reg;
      reg.detid = i;
      reg.ibegin = i;
      reg.iend = i;
      indexes.push_back(reg);
    }
  }
  struct DetRegistry{
    uint32_t detid;
    uint32_t ibegin;
    uint32_t iend;
    bool operator==(const DetRegistry& rhs) const {
      if(detid!=rhs.detid)return false;
      if(ibegin!=rhs.ibegin)return false;
      if(iend!=rhs.iend)return false;
      return true;
    }
    bool operator!=(const DetRegistry& rhs) const {
      return !operator==(rhs);
    }
  };
  bool operator==(const SiStripNoises& rhs) const {
    if(v_noises!=rhs.v_noises){
      return false;
    }
    if(indexes!=rhs.indexes){
      return false;
    }
    return true;
  }
  bool operator!=(const SiStripNoises& rhs) const {
    return !operator==(rhs);
  }
  //std::vector<int16_t> v_noises;
  std::vector<short> v_noises;
  std::vector<DetRegistry> indexes;
};

namespace {
   std::vector<double>::iterator dummy1;
   std::vector<int>::iterator dummy2;
#ifdef _WIN32
   typedef short int16_t;
#endif
   std::vector<int16_t>::iterator dummy3;
   std::vector<short>::iterator dummy4;
   std::vector<float>::iterator dummy5;
   std::vector<Entry>::iterator dummy6;
   std::vector<SiStripNoises::DetRegistry>::iterator dummy7;
   std::vector<std::vector<double> >::iterator dummy8;

}




#endif
