//$Id: FCdeleteEntry.cpp,v 1.20 2008/03/26 16:50:31 witoldp Exp $
/**FCdeleteEntry.cpp -- FileCatalog command line tool to delete all entries associated with given file.
   @author: Zhen Xie
   @date: 02/03/2005 Z.X.
   set default logging to Warning if no POOL_OUTMSG_LEVEL is set; 
   separate logging stream to std::cerr, output stream to std::cout.
   @date: 07/04/2005 Z.X.
   adopt to split catalog interface
*/
#include "FileCatalog/CommandLine.h"
#include "FileCatalog/IFileCatalog.h"
#include "FileCatalog/FCException.h"
#include "FileCatalog/IFCAction.h"
#include "FileCatalog/FCEntry.h"
#include "FileCatalog/IFCContainer.h"
#include "POOLCore/Exception.h"
#include "CoralBase/MessageStream.h"
#include "CoralBase/MessageStream.h"
#include <memory>
using namespace pool;
void printUsage(){
  std::cout<<"usage: FCdeleteEntry [-q query -l lfname -p pfname -u contactstring -h]" <<std::endl;
}
int main(int argc, char** argv) {
  if(!::getenv( "POOL_OUTMSG_LEVEL" )){ //if not set, default to warning
    //    // pool::POOLContext::setMessageVerbosityLevel(coral::Warning);
  }else{
    coral::MessageStream pms("get threshold");
    //    // pool::POOLContext::setMessageVerbosityLevel( pms.threshold() );
  }
  /*
  if( mesgsvc ){
    //all logging go to cerr
    mesgsvc->setOutputStream( std::cerr, seal::Msg::Nil );
    mesgsvc->setOutputStream( std::cerr, coral::Verbose );
    mesgsvc->setOutputStream( std::cerr, coral::Debug );
    mesgsvc->setOutputStream( std::cerr, coral::Info );
    mesgsvc->setOutputStream( std::cerr, coral::Fatal );
    mesgsvc->setOutputStream( std::cerr, coral::Error );
    mesgsvc->setOutputStream( std::cerr, coral::Warning );
  } 
  */
  std::string  myuri;
  std::string  mylfn;
  std::string  mypfn;
  std::string  myquery;
  //  IFileCatalog::FileID myguid;
  try{
    CommandLine commands(argc,argv);
    if( commands.Exists("u") ){
      myuri=commands.GetByName("u");
    }    
    if( commands.Exists("q") ){
      myquery=commands.GetByName("q");
    }
    if( commands.Exists("l") ){
      mylfn=commands.GetByName("l");
    }
    if( commands.Exists("p") ){
      mypfn=commands.GetByName("p");
    }
    
    if( commands.Exists("h") ){
      printUsage();
      exit(0);
    }
  }catch(std::string& strError){
    std::cerr << "Error: command parsing error "<<strError<<std::endl;
    exit(0);
  }
  
  if( mylfn.empty() && mypfn.empty() && myquery.empty() ){
    printUsage();
    std::cerr<<"Error: must specify lfname using -l , pfname using -p, query using -q"<<std::endl;
    exit(0);
  }

  if( !mylfn.empty() && !mypfn.empty() ){
    printUsage();
    std::cerr<<"Error: can only delete either PFN or LFN"<<std::endl;
    exit(0);
  }
  try{  
    std::auto_ptr<IFileCatalog> mycatalog(new IFileCatalog);
    mycatalog->setWriteCatalog(myuri);
    FCAdmin a;
    mycatalog->setAction(a);
    FClookup l;
    mycatalog->setAction(l);
    mycatalog->connect();
    mycatalog->start();
    if( !myquery.empty() ){
      PFNContainer pfns(mycatalog.get());
      FileCatalog::FileID fid;
      l.lookupPFNByQuery(myquery,pfns);
      while(pfns.hasNext()){
        PFNEntry pentry=pfns.Next();
        a.deleteEntry(pentry.guid());
      }
    }else if( !mylfn.empty() ){
      FileCatalog::FileID fid;
      a.lookupFileByLFN(mylfn,fid);
      a.deleteEntry(fid);
    }else if( !mypfn.empty() ) {
      FileCatalog::FileID fid;
      std::string ftype;
      a.lookupFileByPFN(mypfn,fid,ftype);
      a.deleteEntry(fid);
    }
    mycatalog->commit();  
    mycatalog->disconnect();
  }catch (const pool::Exception& er){
    //er.printOut(std::cerr);
    //std::cerr << std::endl;
    std::cerr<<er.what()<<std::endl;
    exit(1);
  }catch (const std::exception& er){
    std::cerr<<er.what()<<std::endl;
    exit(1);
  }
}


