/*
  a test driver to test performance for XML Catalog.
  Author: Maria Girone 
*/
#include <ctime>
#ifdef _WIN32
#include <windows.h>
#include <process.h>
#define getpid _getpid
#undef ERROR
#else
#include <sys/time.h>
#include <unistd.h>
#endif
#include <cstdlib>
#include "Stat.h"


#include <string>
#include <iostream>
#include <iomanip>
#include <exception>

#include "XMLCatalog/XMLFileCatalog.h"

#include "FileCatalog/IFCAction.h"

#include "FileCatalog/CommandLine.h"


#include "FileCatalog/FCSystemTools.h"
#include "FileCatalog/URIParser.h"

typedef long long TimeT;
// return nanosec since the Epoc

inline TimeT  universalTime() { 
#ifdef _WIN32
  SYSTEMTIME now;
  FILETIME   fnow;
  GetSystemTime(&now);
  SystemTimeToFileTime(&now,&fnow);
  return 100LL * (*(unsigned long long int*)&fnow);
#else
  timeval now; 
  gettimeofday(&now,0);
  return 1000000000LL*now.tv_sec + 1000LL* now.tv_usec;
#endif
}

using namespace pool;



int main(int argc, char** argv){

  CommandLine commands(argc,argv);

  int nins=0;
  int ncom=1;
  int procid = getpid();
  if( commands.Exists("n") ){
    std::string myn=commands.GetByName("n");
    nins = atoi(myn.c_str());
    std::cout << " n_insert ="<<nins<<std::endl;
  } else{
    std::cout<< "I need the number of file to insert"<<std::endl;
    exit(1);
  } 
  if( commands.Exists("c") ){
    std::string myn=commands.GetByName("c");
    ncom = atoi(myn.c_str());
    std::cout << " n_commit ="<<ncom<<std::endl;
  }


  IFileCatalog* mycatalog = new IFileCatalog;      

  try{
    URIParser p;
    p.parse();
    p.dump();



    mycatalog->setWriteCatalog( p.url() );
    mycatalog->connect();
    FCregister r;
    mycatalog->setAction(r);

    Stat mystat;
    std::string message;    
    std::cout << " Starting inserting "<<nins<<" lines"<<std::endl;
    
    TimeT t0;
    TimeT t1;
    for (int icom = 0; icom<ncom ; icom++){
      TimeT Tparse=universalTime();
      mycatalog->start();
      TimeT delt = universalTime();
      message = "Parsing_Time";
      mystat.setInfo(message, (delt-Tparse)/1000 );


      std::string pfn;
      char name[200];
      FileCatalog::FileID fid;
      t0=delt;    
      for (int i = 0; i<nins; i++){
        sprintf(name,"myp:/host.domain/mpfname_4_%086d_%066d_%020d",i,procid,icom);
        pfn = name;
        
        delt = universalTime();
        r.registerPFN(pfn,"ascii",fid);
        t1=universalTime();
        if (i==1 || i==10 || i==50 || i==100 || i==500 || 
            i%1000 == 0){
          std::cout<<" File inserted: "<<pfn;
          std::cout<<" File ID: "<<fid;
          std::cout<<" Delta time: "<<((t1-delt)/1000)<<" usec";
          std::cout<<" Total time: "<<std::setw(12)<<((t1-t0)/1000)<<" usec";
          std::cout<<" size of catalog "<<sizeof(*mycatalog);
          std::cout<<std::endl;
          //          system("grep -B1 Mem: /proc/meminfo");
          message = "File_inserted";
        }
        mystat.setInfo(message, (t1-delt)/1000 );
        delt=t1;
      }

      delt = t1;
      //      system("grep -B1 Mem: /proc/meminfo");
      t0 = universalTime();
      mycatalog->commit(pool::FileCatalog::ONHOLD);
      t1 = universalTime();
      message = "Commit time spent";
      mystat.setInfo(message , (t1-t0)/1000 );
    }


    FClookup l;
    FCAdmin d;
    mycatalog->setAction(l);
    mycatalog->setAction(d);
    mycatalog->start();
    for (int i=0; i<100; i++){
      std::string fakepfn="fake";
      std::string fakelfn="fake";
      std::string fid="";      
      std::string ftype="";
      r.registerPFN(fakepfn,"ascii",fid);
      t0 = universalTime();
      l.lookupFileByPFN(fakepfn, fid,ftype);
      t1 = universalTime();
      message="lookup_PFN";
      mystat.setInfo(message, (t1-t0)/1000);

      t0 = universalTime();
      r.registerLFN(fakepfn,fakelfn);
      t1 = universalTime();
      message="Add_alias";
      mystat.setInfo(message, (t1-t0)/1000);
      t0 = universalTime();
      l.lookupFileByLFN(fakelfn, fid);
      t1 = universalTime();
      message="lookup_LFN";
      mystat.setInfo(message, (t1-t0)/1000);
      t0 = universalTime();
      std::cout << "Iteration "<<i<<" filename "<<fakepfn<<std::endl;
      d.deletePFN(fakepfn);
      t1 = universalTime();
      message="delete_PFN";
      mystat.setInfo(message, (t1-t0)/1000);
      
    }
    mycatalog->commit();

    //system("grep -B1 Mem: /proc/meminfo");
    //delete mycatalog;
    t0 = universalTime();
    //    manager.closeFileCatalog();
    t1 = universalTime();
    //    system("grep -B1 Mem: /proc/meminfo");
    message = "Close File Catalog";
    mystat.setInfo(message , (t1-t0)/1000 );
    
    
    mystat.print();
    mycatalog->disconnect();
  }

  catch( std::exception& er )
  {
    std::cerr << er.what() << std::endl;
    exit( 2 );
  }
  catch ( ... )
  {
    std::cerr << "Caught unknown exception!" << std::endl;
  }
  
  delete mycatalog; 
  
}
